﻿-----------------------------------------------------------------
/* Модули веб-клиента */
if not exists ( Select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules' ))
	Create table dbo.WO_InstalledModules
	(
		id uniqueidentifier,
		code nvarchar(200) Primary key,
		name nvarchar(200) collate database_default,
		descr nvarchar(1000) collate database_default,
		ver nvarchar(50) collate database_default,
		appver nvarchar(50) collate database_default,
		dtcreation datetime,
		creatorid uniqueidentifier,
		dbname nvarchar(200) collate database_default,
		dbuserlogin nvarchar(200) collate database_default,
		dbuserpass nvarchar(256) collate database_default,
		dblastupdatefile nvarchar(300) collate database_default,
		wwwrelativepath nvarchar(300) collate database_default,
		manifest ntext collate database_default,
		isenabled bit
	)
GO

/* Типы связанных объектов правовой политики с модулями веб-клиента */
if not exists ( Select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_ObjectTypes' ))
	Create table dbo.WO_InstalledModules_ObjectTypes
	(		
		modulecode nvarchar(200) collate database_default,
		objectcode nvarchar(200) collate database_default,
        objectcodefull nvarchar(200) collate database_default
	)
GO

/* Кастомные привилегии модулей. Привязки кодов к идентификаторам */
if not exists ( Select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_CustomCodeHash' ))
    Create table dbo.WO_InstalledModules_CustomCodeHash
    (
        modulecode nvarchar(200) collate database_default,
        basiccode nvarchar(200) collate database_default,
        customcode nvarchar(400) collate database_default,
        customid uniqueidentifier,
        type int
    )
GO

if not exists (select * from dbo.sysindexes where name = N'idx_wo_codehash' and id = object_id(N'dbo.WO_InstalledModules_CustomCodeHash'))
    CREATE INDEX idx_wo_codehash ON dbo.WO_InstalledModules_CustomCodeHash([modulecode], [customcode]) WITH FILLFACTOR = 80 ON [PRIMARY]
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_Get' ))
	Drop procedure dbo.WO_InstalledModules_Get
GO

Create procedure dbo.WO_InstalledModules_Get
	@modulecode nvarchar(200) = null
as
begin
	
	Select id, code, name, descr, ver, appver, dtcreation, creatorid, dbname, dbuserlogin, dbuserpass, dblastupdatefile, wwwrelativepath, manifest, isenabled
	From WO_InstalledModules
	Where (isenabled = 1) 
		and ((code like @modulecode) or (@modulecode is null))
	
end
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_Add' ))
	Drop procedure dbo.WO_InstalledModules_Add
GO

Create procedure dbo.WO_InstalledModules_Add
	@modulecode nvarchar(200),
	@name nvarchar(200),
	@descr nvarchar(1000) = '',
	@ver nvarchar(50),
	@appver nvarchar(50),
	@creatorid uniqueidentifier,
	@dbname nvarchar(200),
	@dbuserlogin nvarchar(200),
	@dbuserpass nvarchar(256),
	@dblastupdatefile nvarchar(300),
	@wwwrelativepath nvarchar(300),
	@manifest ntext
as
begin
	
	if exists ( select code from dbo.WO_InstalledModules where code like @modulecode ) begin
		
		Select 0
		
	end else begin 
	
		Insert into dbo.WO_InstalledModules ( id, code, name, descr, ver, appver, dtcreation, creatorid, dbname, dbuserlogin, dbuserpass, dblastupdatefile, wwwrelativepath, manifest, isenabled )
		Values ( NEWID(), @modulecode, @name, @descr, @ver, @appver, GetDate(), @creatorid, @dbname, @dbuserlogin, @dbuserpass, @dblastupdatefile, @wwwrelativepath, @manifest, 1 )
		
		Select 1
		
	end
	
end
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_Update' ))
	Drop procedure dbo.WO_InstalledModules_Update
GO

Create procedure dbo.WO_InstalledModules_Update
	@modulecode nvarchar(200),
	@name nvarchar(200) = NULL,
	@descr nvarchar(1000) = NULL,
	@ver nvarchar(50) = NULL,
	@appver nvarchar(50) = NULL,
	@creatorid uniqueidentifier = NULL,
	@dbname nvarchar(200) = NULL,
	@dbuserlogin nvarchar(200) = NULL,
	@dbuserpass nvarchar(256) = NULL,
	@dblastupdatefile nvarchar(300) = NULL,
	@wwwrelativepath nvarchar(300) = NULL,
	@manifest ntext = NULL,
	@isenabled bit = NULL
as
begin
	
	if exists ( select code from dbo.WO_InstalledModules where code = @modulecode ) begin
		
		Update dbo.WO_InstalledModules
		Set name = IsNull ( @name, name ),
			descr = IsNull ( @descr, descr ),
			ver = IsNull ( @ver, ver ),
			appver = IsNull ( @appver, appver ),
			dbname = IsNull ( @dbname, dbname ),
			dbuserlogin = IsNull ( @dbuserlogin, dbuserlogin ),
			dbuserpass = IsNull ( @dbuserpass, dbuserpass ),
			dblastupdatefile = IsNull ( @dblastupdatefile, dblastupdatefile ),
			wwwrelativepath = IsNull ( @wwwrelativepath, wwwrelativepath ),
			manifest = IsNull ( @manifest, manifest ),
			isenabled = IsNull ( @isenabled, isenabled )			
		Where code = @modulecode
		
		Select 1
		
	end else begin 
	
		Select 0
		
	end
	
end
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_Remove' ))
	Drop procedure dbo.WO_InstalledModules_Remove
GO

Create procedure dbo.WO_InstalledModules_Remove
	@modulecode nvarchar(200)
as
begin

	if exists ( select code from dbo.WO_InstalledModules where code = @modulecode ) begin
	
        -----------------------------------------------------------
		-- Модуль
		Delete 
		From dbo.WO_InstalledModules
		Where code = @modulecode


        -----------------------------------------------------------
        -- Ролевые привилегии
        Delete 
        From dbo.A_UserRoles_Rights
        Where Id in ( Select CustomId From dbo.WO_InstalledModules_CustomCodeHash Where modulecode = @modulecode )


		-----------------------------------------------------------
		-- Объекты из правовой политики (автоматом все)
		Declare tcursor Cursor For
			Select objectcode, objectcodefull
			From dbo.WO_InstalledModules_ObjectTypes
			Where modulecode = @modulecode	

		Declare @objectcode nvarchar(200), @objectcodefull nvarchar(200)

		Open tcursor
		Fetch Next From tcursor Into @objectcode, @objectcodefull

		While ( @@fetch_status = 0 ) begin			
			exec A_Policy_Modules_ObjectTypes_Remove @objectcodefull
			
			Fetch Next From tcursor Into @objectcode, @objectcodefull
		end

        Close tcursor
        Deallocate tcursor
		
        -----------------------------------------------------------
		-- Привязки типов объектов к модулю
		Delete 
		From dbo.WO_InstalledModules_ObjectTypes
		Where modulecode = @modulecode


        -----------------------------------------------------------
        -- Кастомные связки строковых кодов с идентификаторами
        Delete
        From dbo.WO_InstalledModules_CustomCodeHash
        Where modulecode = @modulecode
        
		
		Select 1
	
	end else begin
		
		Select 0
		
	end	

end
GO

--------------------------------------------------------------

if exists ( Select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_RemoveData' ))
	Drop procedure dbo.WO_InstalledModules_RemoveData
GO

Create procedure [dbo].[WO_InstalledModules_RemoveData]
as 
begin

	-- Truncate module data
	Truncate Table dbo.WO_InstalledModules
	Truncate Table dbo.WO_InstalledModules_ObjectTypes
	Truncate Table dbo.WO_InstalledModules_CustomCodeHash

	-- Remove roles, rights and policies of modules
	Delete FROM A_UserRoles_Rights where code like 'wo.%' or code like 'wo_general'
	Delete FROM A_Policy_ObjectTypes where id >= 500
	Delete FROM A_Policy_Objects where type >= 500
	Delete FROM A_Policy_Privileges where objType >= 500
	Delete From A_Policy_PrivilegeDefaults where not idprivilege in (Select id from A_Policy_Privileges)	
	
	-- Drop databases of modules
	Declare @dbname nvarchar(200)
	Declare @sql nvarchar(4000)
	
	Declare tcursor Cursor For 
		Select name
		From master..sysdatabases 
		Where name like 'WO_Module_%'
	
	Open tcursor
	Fetch Next From tcursor Into @dbname

	While ( @@fetch_status = 0 ) Begin

		Set @sql = ' DROP DATABASE ' + @dbname
		Exec sp_executesql @stmt =@sql
		Print @sql

		Fetch Next From tcursor Into @dbname
	End

	Close tcursor
	Deallocate tcursor
	
end
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_CheckExists' ))
	Drop procedure dbo.WO_InstalledModules_CheckExists
GO

Create procedure dbo.WO_InstalledModules_CheckExists
	@modulecode nvarchar(200)
as
begin

	if exists ( Select code, name From dbo.WO_InstalledModules Where code like @modulecode )
		Select 1 as result, @modulecode as code
	else
		Select 0 as result, @modulecode as code
	
end
GO

--------------------------------------------------------------
/* Remove Module DB and module user*/
If object_id ('dbo.WO_Database_ModuleRemove') is not null
    Drop procedure [dbo].[WO_Database_ModuleRemove]
GO

Create procedure dbo.WO_Database_ModuleRemove
    @moduledbuser nvarchar(250), 
    @moduledbpass nvarchar(250), 
    @s_moduledbname nvarchar(250), 
    @moduledbname nvarchar(250), 
    @oktelldb nvarchar(250), 
    @oktellccdb nvarchar(250), 
    @oktellsettingsdb nvarchar(250),
    @moduleid nvarchar(200)
as
begin

    Declare @Statement nvarchar(4000)
    Declare @pre_Statement nvarchar(4000)
    
    Set @pre_Statement = '
		If exists (Select * From sysusers Where ([name] = ''' + @moduledbuser + ''') AND (issqluser = 1))
			Exec sp_dropuser ' + @moduledbuser + '
    '
    
    Set @Statement = N'
		/* Remove module role privileges */
		Exec WO_UserRoles_RemoveModuleRights
			@modulecode = ''' + @moduleid + '''


		/* Remove user */
		Use [' + @oktelldb + ']
		' + @pre_Statement + '
		
		Use [' + @oktellccdb + ']
		' + @pre_Statement + '
		
		Use [' + @oktellsettingsdb + ']
		' + @pre_Statement + '
    
    
		/* Remove module DB */
		Use [master]
		
		If exists (Select 1 From sysdatabases Where [name] = ''' + @s_moduledbname + ''') begin
			
			Alter Database [' + @moduledbname + '] 
			Set single_user 
			With rollback immediate
			
			If exists (Select * From sysusers  where [name] = ''' + @moduledbuser + ''' AND issqluser = 1) 
                Exec sp_dropuser ' + @moduledbuser + '
			
			Drop Database [' + @moduledbname + ']
		end

		
		/* Remove login */
		Use [master]
		If exists (Select * From syslogins Where [name] = ''' + @moduledbuser + ''' ) 
			Exec sp_droplogin ' + @moduledbuser
			
        
    Exec sp_executesql @Statement 
    
end
GO

--------------------------------------------------------------
/* Create DB and access for user */
IF OBJECT_ID ('dbo.WO_Database_ModuleCreate') IS NOT NULL
   DROP PROCEDURE dbo.WO_Database_ModuleCreate
GO
        
CREATE procedure dbo.WO_Database_ModuleCreate
    @moduledbuser nvarchar(250), 
    @moduledbpass nvarchar(250), 
    @s_moduledbname nvarchar(250), 
    @moduledbname nvarchar(250), 
    @oktelldb nvarchar(250), 
    @oktellccdb nvarchar(250), 
    @oktellsettingsdb nvarchar(250),
    @moduleid nvarchar(200),
    @moduledbfilename nvarchar(200) = NULL
as
begin

    Declare @Statement nvarchar(4000),
            @pre_Statement nvarchar(4000)
    
    DECLARE @db_path varchar (4000)
    
    DECLARE @collate_oktell  varchar (500)
    
   SET @collate_oktell = cast(DATABASEPROPERTYEX(@oktelldb, 'Collation') as VARCHAR(500))

    IF (SELECT cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  ) > 8
    BEGIN
        SELECT @db_path = physical_name FROM sys.master_files WHERE database_id = DB_ID(N'master') AND type_desc = 'ROWS'
    END
    ELSE
    BEGIN
        SELECT @db_path = rtrim(ltrim([filename])) FROM [master]..sysfiles WHERE [name] = 'master' 
    END
    
    SET @db_path = REVERSE(RIGHT(REVERSE(@db_path),(LEN(@db_path)-CHARINDEX('\', REVERSE(@db_path),1))+1))
    
    /* DB & login creation */
    Set @Statement = N'
		/* DB master */
		USE [master]
		
		If not exists (Select 1 From sysdatabases Where [name] = ''' + @s_moduledbname + ''')
			Create Database ' + @moduledbname + 
				CASE WHEN @moduledbfilename IS NULL OR len(@moduledbfilename) = 0
					 THEN ''
					 ELSE '
						on ( Name = N''' + @moduledbfilename + ''', Filename = ''' + @db_path + @moduledbfilename + '.mdf'') 
						log on  ( Name = N''' + @moduledbfilename + '_log'', Filename = ''' + @db_path + @moduledbfilename + '.ldf'',SIZE = 10MB)'
				END + N'
                COLLATE ' + @collate_oktell + '
			
		If not exists (Select * From syslogins Where [name] = ''' + @moduledbuser + ''')
			Exec sp_addlogin 
				@loginame=''' + @moduledbuser + ''',
				@passwd=''' + @moduledbpass + ''',
				@defdb=''' + @s_moduledbname + '''
	'
	
    Exec sp_executesql @Statement
    
    
    /* DBs access */
    Set @pre_Statement = N'
			If not exists (Select 1 From sysusers Where name = ''db_select_and_executor'') begin
				Exec sp_addrole ''db_select_and_executor''
			end
	    
	        Exec sp_addrolemember ''db_datareader'', ''db_select_and_executor'' 
			' + 
     CASE WHEN cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  > 8 
     THEN N' GRANT EXEC TO db_select_and_executor '
     ELSE N'
			SET @sql1 = ''''
		    
			Declare curs Cursor For
				Select [name] 
				From sysobjects 
				Where xtype in (''P'', ''FN'')

			Open curs

			Fetch Next From curs Into @sql1
		    
			While (@@FETCH_STATUS = 0) begin
			
				Set @sql1 = ''GRANT EXEC ON ''  + @sql1 +  '' TO db_select_and_executor ''
				Exec sp_executesql @sql1
				Fetch Next From curs Into @sql1
				
			end
	    
			Close curs
			Deallocate curs
		'
	END + N'
			If not exists (Select * From sysusers Where ([name] = ''' + @moduledbuser + ''') and (issqluser = 1))
				Exec sp_adduser 
					@loginame=''' + @moduledbuser + ''', 
					@name_in_db=''' + @moduledbuser + ''' , 
					@grpname = ''db_select_and_executor'' 
	'
    
    Set @Statement = N'
		/* User to access DBs */
		Use [' + @moduledbname + ']
		
		If not exists (Select * From sysusers Where ([name] = ''' + @moduledbuser + ''') and (issqluser = 1))
			Exec sp_adduser 
				@loginame=''' + @moduledbuser + ''', 
				@name_in_db=''' + @moduledbuser + ''' , 
				@grpname = ''db_owner''  
    
		Declare @sql1 nvarchar(4000)
    
		USE [' + @oktelldb + ']
		' + @pre_Statement + '
		
		USE [' + @oktellccdb + ']
		' + @pre_Statement + '
		
		USE [' + @oktellsettingsdb + ']
		' + @pre_Statement

    Exec sp_executesql @Statement
    
end
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_ObjectTypes_Add' ))
	Drop procedure dbo.WO_InstalledModules_ObjectTypes_Add
GO

if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_ObjectTypes_AddUpdate' ))
	Drop procedure dbo.WO_InstalledModules_ObjectTypes_AddUpdate
GO

Create procedure dbo.WO_InstalledModules_ObjectTypes_AddUpdate
	@modulecode nvarchar(200),
	@objectcode nvarchar(200),
    @objectcodefull nvarchar(200),
    @useproject bit = 0
as
begin
    
    if exists ( Select *
			    From dbo.WO_InstalledModules_ObjectTypes
			    Where (modulecode = @modulecode)
				    and (objectcode = @objectcode))
    begin

        Declare @id int
        
        Select @id = id
        From A_Policy_ObjectTypes
        Where (name like @objectcodefull)
	        and (id >= 500)
            and (isBasicType = 0)

        if ( not @id is NULL ) begin
            Select @id as id, 1 as alreadyexists
            return
        end

    end


	-- Привязка типов объектов к модулям
	Insert into dbo.WO_InstalledModules_ObjectTypes ( modulecode, objectcode, objectcodefull )
	Values ( @modulecode, @objectcode, @objectcodefull )

	-- Фиксация числового типа объекта в правовой политике Oktell
	exec A_Policy_Modules_ObjectTypes_Create @objectcodefull, @useproject
	
    -- Надо вернуть сгенерированный ид для типа объекта
    Declare @idnew int
	
    Select @idnew = id
	From dbo.A_Policy_ObjectTypes
	Where (name like @objectcodefull)
		and (isBasicType = 0)

    Select @idnew as id, 0 as alreadyexists

    -- Привязка к проектам должна или не должна быть
    

end
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_GetPolicyObjectType' ))
	Drop procedure dbo.WO_InstalledModules_GetPolicyObjectType
GO

Create procedure dbo.WO_InstalledModules_GetPolicyObjectType
	@modulecode nvarchar(200),
	@objectcode nvarchar(200),
	@objecttype int output
as
begin

	Declare @objectcodefull nvarchar(200)

	Select @objectcodefull = objectcodefull
	From WO_InstalledModules_ObjectTypes
	Where (modulecode=@modulecode)
		and (objectcode=@objectcode)
		
	if ( @objectcodefull is NULL )
		Set @objectcodefull = @objectcode
		
	Select @objecttype = id
	From A_Policy_ObjectTypes
	Where name = @objectcodefull	
	
end
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_GetCustomIdByCode' ))
	Drop procedure dbo.WO_InstalledModules_GetCustomIdByCode
GO

Create procedure dbo.WO_InstalledModules_GetCustomIdByCode
    @modulecode nvarchar(200),
    @customcode nvarchar(400)
as
begin

    Select customid, type
    From dbo.WO_InstalledModules_CustomCodeHash
    Where (modulecode = @modulecode)
        and (customcode = @customcode)

end
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_InstalledModules_GetCustomIdByCode_Param' ))
	Drop procedure dbo.WO_InstalledModules_GetCustomIdByCode_Param
GO

Create procedure dbo.WO_InstalledModules_GetCustomIdByCode_Param
    @modulecode nvarchar(200),
    @customcode nvarchar(400),
    @customid uniqueidentifier OUTPUT,
    @type int OUTPUT
as
begin

    Select @customid = customid, @type = type
    From dbo.WO_InstalledModules_CustomCodeHash
    Where (modulecode = @modulecode)
        and (customcode = @customcode)

end
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_Policy_ObjectPrivileges_Save' ))
	Drop procedure dbo.WO_Policy_ObjectPrivileges_Save
GO

Create procedure dbo.WO_Policy_ObjectPrivileges_Save
    @idPrivilege uniqueidentifier,
    @objType int,
    @knot nvarchar(50),
    @name nvarchar(1000),
    @basiccode nvarchar(200),
    @fullcode nvarchar(400),
    @levelNesting int,
    @isBasicType bit,
    @isPolicyModify bit,
    @modulecode nvarchar(200)
as
begin

    exec dbo.A_Policy_ObjectPrivileges_Save
        @id = @idPrivilege,
        @objType = @objType,
        @knot = @knot,
        @name = @name,
        @code = @fullcode,
        @levelNesting = @levelNesting,
        @isBasicType = @isBasicType,
        @isPolicyModify = @isPolicyModify

    if ( not exists ( select * from dbo.WO_InstalledModules_CustomCodeHash where modulecode=@modulecode and customcode=@fullcode ))
        Insert into dbo.WO_InstalledModules_CustomCodeHash ( modulecode, basiccode, customcode, customid, type )
        Values ( @modulecode, @basiccode, @fullcode, @idPrivilege, 2 )
end
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_UserRoles_AddRight' ))
	Drop procedure dbo.WO_UserRoles_AddRight
GO

Create procedure dbo.WO_UserRoles_AddRight
    @idright uniqueidentifier,
    @knot nvarchar(50),
    @name nvarchar(1000),
    @basiccode nvarchar(200),
    @fullcode nvarchar(400),
    @modulecode nvarchar(200)
as
begin

    Declare @xid uniqueidentifier
    Declare @xtype int
    exec dbo.WO_InstalledModules_GetCustomIdByCode_Param 
        @modulecode = @modulecode, 
        @customcode = @fullcode, 
        @customid = @xid OUTPUT, 
        @type = @xtype OUTPUT

    if ( @xid is NULL ) begin

        Set @xid = @idright

        if (( @xid is null ) or ( @xid = '00000000-0000-0000-0000-000000000000' ))
            Set @xid = NEWID()

        exec dbo.A_UserRoles_AddRight
            @id = @xid,
            @knot = @knot,
            @name = @name,
            @code = @fullcode

        if ( not exists ( select * from dbo.WO_InstalledModules_CustomCodeHash where modulecode=@modulecode and customcode=@fullcode ))
            Insert into dbo.WO_InstalledModules_CustomCodeHash ( modulecode, basiccode, customcode, customid, type )
            Values ( @modulecode, @basiccode, @fullcode, @xid, 1 )
    end

end
GO

-------------------------------------------------------
if exists ( Select * from sysobjects where id= object_id ( 'dbo.WO_UserRoles_EnableModuleRights' ))
	Drop procedure dbo.WO_UserRoles_EnableModuleRights
GO

Create procedure dbo.WO_UserRoles_EnableModuleRights
    @modulecode nvarchar(200),
    @moduleknot nvarchar(50),
    @userid uniqueidentifier
as
begin

    DECLARE rcur cursor LOCAL FAST_FORWARD For 
	    Select Id
        From dbo.A_UserRoles_Rights
        Where code like 'wo.' + @modulecode + '.r%'
	
    OPEN rcur
	
    Declare @rid uniqueidentifier	
    Fetch Next From rcur into @rid	

    While ( @@FETCH_STATUS = 0 ) begin
	
	    exec dbo.A_UserRoles_AddRightForRole
            @idrole = 'c39d1c39-a413-42b7-ba83-dbee9fa5fb12', /*basic admin*/
            @idright = @rid,
            @value = 1

        exec dbo.A_UserRoles_AddRightForRole
		    @idrole=@userid, /*dominate*/
		    @idright=@rid,
		    @value=1

	    Fetch Next From rcur into @rid	
    end

    CLOSE rcur
    DEALLOCATE rcur

end
GO

-------------------------------------------------------
if exists ( Select * from sysobjects where id= object_id ( 'dbo.WO_UserRoles_GetEmptyKnotLevel' ))
	Drop procedure dbo.WO_UserRoles_GetEmptyKnotLevel
GO

Create procedure dbo.WO_UserRoles_GetEmptyKnotLevel
as
begin

	Declare @woknot nvarchar(50), @maxmodule int, @curmodule int

	Select @woknot = knot
	From A_UserRoles_Rights 
	Where code like 'wo_general'

	if ( @woknot is null ) begin

		Select null
		
	end else begin

		Select @maxmodule = Max (Cast ( Replace ( knot, @woknot + '.', '' ) as int ))
		From A_UserRoles_Rights 
		Where (knot like @woknot + '.%') 
			and (not knot like @woknot + '.%.%')

		if (( @maxmodule is NULL ) or ( @maxmodule < 1000 ))
			set @curmodule = 1000
		else 
			set @curmodule = @maxmodule + 1
			
		Select @woknot + '.' + cast ( @curmodule as nvarchar(20)) as knot	
		
	end
	
end
GO

-------------------------------------------------------
if exists ( Select * from sysobjects where id= object_id ( 'dbo.WO_UserRoles_GetModuleKnotLevel' ))
	Drop procedure dbo.WO_UserRoles_GetModuleKnotLevel
GO

Create procedure dbo.WO_UserRoles_GetModuleKnotLevel
    @modulecode nvarchar(200)
as
begin

	Declare @woknot nvarchar(50), @moduleknot nvarchar(50)

	Select @woknot = knot
	From A_UserRoles_Rights 
	Where Code like 'wo_general'

	if ( @woknot is null ) begin

		Select null
		
	end else begin

		Select @moduleknot = knot
		From A_UserRoles_Rights 
		Where (knot like @woknot + '.%') 
			and (not knot like @woknot + '.%.%')
			and (Code = 'wo.' + @modulecode + '.r')
			
		Select @moduleknot as knot
		
	end
	
end
GO

-------------------------------------------------------
if exists ( Select * from sysobjects where id= object_id ( 'dbo.WO_UserRoles_GetModuleSubRights' ))
	Drop procedure dbo.WO_UserRoles_GetModuleSubRights
GO

Create procedure dbo.WO_UserRoles_GetModuleSubRights
    @modulecode nvarchar(200)
as
begin
	
    Select Id, Knot, Code 
    From A_UserRoles_Rights
    Where code like 'wo.' + @modulecode + '.r.%'

end
GO

-------------------------------------------------------
if exists ( Select * from sysobjects where id= object_id ( 'dbo.WO_UserRoles_ClearModuleSubRights' ))
	Drop procedure dbo.WO_UserRoles_ClearModuleSubRights
GO

Create procedure dbo.WO_UserRoles_ClearModuleSubRights
    @modulecode nvarchar(200)
as
begin
	
    Delete
    From A_UserRoles_Rights
    Where code like 'wo.' + @modulecode + '.r.%'

    Delete
	From WO_InstalledModules_CustomCodeHash
	Where customcode like 'wo.' + @modulecode + '.r.%'

    Select 1 as result

end
GO

---------------------------------------------------
if exists ( Select * from sysobjects where id= object_id ( 'dbo.WO_UserRoles_RemoveModuleRights' ))
	Drop procedure dbo.WO_UserRoles_RemoveModuleRights
GO

Create procedure dbo.WO_UserRoles_RemoveModuleRights
    @modulecode nvarchar(200)
as
begin

	Declare @woknot nvarchar(50), @moduleknot nvarchar(50)

	Select @woknot = knot
	From A_UserRoles_Rights 
	Where code like 'wo_general'

	Select @moduleknot = knot
	From A_UserRoles_Rights 
	Where (knot like @woknot + '.%') 
		and (not knot like @woknot + '.%.%') 
		and (code like 'wo.' + @modulecode + '.r')

	Delete 
	From A_UserRoles_Rights
	Where (code like 'wo.' + @modulecode + '.r.%')
        or (code like 'wo.' + @modulecode + '.r')
	
end
GO

--------------------------------------------------------------
if exists ( Select * from sysobjects where id= object_id ( 'dbo.WO_UserRoles_GetModuleRights' ))
	Drop procedure dbo.WO_UserRoles_GetModuleRights
GO

Create procedure dbo.WO_UserRoles_GetModuleRights
	@modulecode nvarchar(200)
as
begin

	Select Id, Knot, Name, Code 
	From A_UserRoles_Rights
	Where code like 'wo.' + @modulecode + '.r.%'
	
end
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_Policy_GetSubjectList' ))
	Drop procedure dbo.WO_Policy_GetSubjectList
GO

Create procedure dbo.WO_Policy_GetSubjectList
    @modulecode nvarchar(200),
    @objecttype int = -1,
    @objecttypecode nvarchar(200)
as 
begin
    
    Declare @type int    
    if ( @objecttype >= 0 )
        Set @type = @objecttype
    else
        exec dbo.WO_InstalledModules_GetPolicyObjectType
            @modulecode = @modulecode,
	        @objectcode = @objecttypecode,
	        @objecttype = @type OUTPUT

    exec dbo.A_Policy_SubjectGroups_GetList
        @objecttype=@type

end
GO

--------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.WO_Policy_ObjectPrivileges_GetAll' ))
	Drop procedure dbo.WO_Policy_ObjectPrivileges_GetAll
GO

Create procedure dbo.WO_Policy_ObjectPrivileges_GetAll
    @modulecode nvarchar(200),
    @objecttype int = -1,
    @objecttypecode nvarchar(200)
as 
begin
    
    Declare @type int    
    if ( @objecttype >= 0 )
        Set @type = @objecttype
    else
        exec dbo.WO_InstalledModules_GetPolicyObjectType
            @modulecode = @modulecode,
	        @objectcode = @objecttypecode,
	        @objecttype = @type OUTPUT

    exec dbo.A_Policy_ObjectPrivileges_GetAll
        @objecttype=@type

end
GO

----------------------------------------------------------------
if exists ( select * FROM sysobjects where id = object_id ('dbo.WO_Policy_ObjectPrivileges_DefaultSave' ))
	Drop procedure dbo.WO_Policy_ObjectPrivileges_DefaultSave
GO

Create procedure dbo.WO_Policy_ObjectPrivileges_DefaultSave
	@modulecode nvarchar(200),
	@objtype int,
	@subjtype int,
	@subjid uniqueidentifier = NULL,
	@subjeid uniqueidentifier = NULL,
	@privilegecode nvarchar(200) = NULL
as
begin

	if ( @privilegecode is NULL ) begin	
		-- Убрать субъекту все привилегии
		
		Insert into dbo.A_Policy_PrivilegeDefaults ( idPrivilege, idSubject, subjType, subjExtraId, value )
			Select p.id, 
				IsNull(@subjid, st.groupId), @subjtype, @subjeid, 
				0
			From dbo.A_Policy_Privileges p
				Inner join dbo.A_Policy_SubjectTypes st
                    on ((st.equalMode = 1) 
							and (st.id = @subjtype))
					or ((st.equalMode = 2) 
						and (st.id = @subjtype) 
						and (st.groupId = @subjId) and (not st.groupid is null))
					or ((st.equalMode = 4) 
						and (st.id = @subjtype) 
						and (st.groupId = @subjEId) and (not st.groupid is null))
					--or ((st.equalMode = 3) 
					--	and (st.id = @subjtype) 
					--	and (st.groupId = @subjId) and (not st.groupid is null)
					--	and (st.groupId = @subjEid) and (not st.groupid is null))
			Where objType = @objtype				
		
	end else begin
		-- Поставить субъекту привилегию, а также в соответствии с levelNesting остальные привилегии	

		-- Ид привилегии находим по коду
		Declare @rightid uniqueidentifier, @xtype int
		exec dbo.WO_InstalledModules_GetCustomIdByCode_Param 
			@modulecode = @modulecode, 
			@customcode = @privilegecode, 
			@customid = @rightid OUTPUT, 
			@type = @xtype OUTPUT
			
		if ( not @rightid is NULL ) begin	        
		
			-- Уровень вложенности привилегии смотрим
			Declare @levelnesting int
			Select @levelnesting = levelNesting
			From dbo.A_Policy_Privileges p
			Where id = @rightid
					
			--exec dbo.[A_Policy_ObjectPrivileges_DefaultSave]
			--	@idPrivilege uniqueidentifier,
			--	@idSubject uniqueidentifier,	
			--	@subjType int,
			--	@subjExtraId uniqueidentifier = NULL,
			--	@value int = 1
			
			-- Саму привилегию указанную
			Insert into dbo.A_Policy_PrivilegeDefaults ( idPrivilege, idSubject, subjType, subjExtraId, value )
				Select p.id, 
					IsNull(@subjid, st.groupId), @subjtype, @subjeid, 
					1
				From dbo.A_Policy_Privileges p
					Inner join dbo.A_Policy_SubjectTypes st
                        on ((st.equalMode = 1) 
							and (st.id = @subjtype))
						or ((st.equalMode = 2) 
							and (st.id = @subjtype) 
							and (st.groupId = @subjId) and (not st.groupid is null))						
						or ((st.equalMode = 4) 
							and (st.id = @subjtype) 
							and (st.groupId = @subjEId) and (not st.groupid is null))
						--or ((st.equalMode = 3) 
						--	and (st.id = @subjtype) 
						--	and (st.groupId = @subjId) and (not st.groupid is null)
						--	and (st.groupId = @subjEid) and (not st.groupid is null))				
				Where (p.objType = @objtype)
					and (p.Id = @rightid)
			
			-- Те что ниже - в 1, те что выше - в 0. С тем же level не трогаем			
			Insert into dbo.A_Policy_PrivilegeDefaults ( idPrivilege, idSubject, subjType, subjExtraId, value )
				Select p.id, 
					IsNull(@subjid, st.groupId), @subjtype, @subjeid, 
					case 
						when ( p.levelNesting < @levelnesting ) then 1 
						when ( p.levelNesting > @levelnesting ) then 0
					end
				From dbo.A_Policy_Privileges p
					Inner join dbo.A_Policy_SubjectTypes st
                        on ((st.equalMode = 1) 
							and (st.id = @subjtype))
						or ((st.equalMode = 2) 
							and (st.id = @subjtype) 
							and (st.groupId = @subjId) and (not st.groupid is null))						
						or ((st.equalMode = 4) 
							and (st.id = @subjtype) 
							and (st.groupId = @subjEId) and (not st.groupid is null))
						--or ((st.equalMode = 3) 
						--	and (st.id = @subjtype) 
						--	and (st.groupId = @subjId) and (not st.groupid is null)
						--	and (st.groupId = @subjEid) and (not st.groupid is null))
				Where (p.objType = @objtype) 
					and (levelNesting != @levelnesting)
		end
	end
		
end	
GO

------------------------------------------------------

/* Текущий язык сервера */
Declare @curlang nvarchar(10)
Select @curlang = valuestr
From A_Settings 
Where name like 'CurrentLang'

Select @curlang = case 
        when (( @curlang is NULL ) or ( @curlang = 'rus' ) or ( @curlang = 'ru' )) 
            then 
                'ru' 
        else 
                'en'
    end

/* Базовые ролевые привилегии */
Declare @lang table 
( 
	id uniqueidentifier, 
	code nvarchar(200) collate database_default, 
	subknot nvarchar(10) collate database_default, 
	lang nvarchar(10) collate database_default, 
	name nvarchar(500) collate database_default
)
Insert into @lang ( id, code, subknot, lang, name ) Values ( 'FFBFEF2B-0D4D-4283-9573-769C152E5D18', 'wo_general', '', 'ru', 'Web-client' )
Insert into @lang ( id, code, subknot, lang, name ) Values ( 'FFBFEF2B-0D4D-4283-9573-769C152E5D18', 'wo_general', '', 'en', 'Web-client' )
Insert into @lang ( id, code, subknot, lang, name ) Values ( '5CDB1299-8D30-46B8-983D-9BD9F6289ADD', 'wo_general.installModule', '1', 'ru', 'Установка приложений' )
Insert into @lang ( id, code, subknot, lang, name ) Values ( '5CDB1299-8D30-46B8-983D-9BD9F6289ADD', 'wo_general.installModule', '1', 'en', 'Install App' )
Insert into @lang ( id, code, subknot, lang, name ) Values ( 'A8D316EF-C174-4525-90BD-64F79D7B1846', 'wo_general.updateModule', '2', 'ru', 'Обновление приложений' )
Insert into @lang ( id, code, subknot, lang, name ) Values ( 'A8D316EF-C174-4525-90BD-64F79D7B1846', 'wo_general.updateModule', '2', 'en', 'Update App' )
Insert into @lang ( id, code, subknot, lang, name ) Values ( '82A4570B-590F-4D24-9863-8FB6BC7ED5F8', 'wo_general.removeModule', '3', 'ru', 'Удаление приложений' )
Insert into @lang ( id, code, subknot, lang, name ) Values ( '82A4570B-590F-4D24-9863-8FB6BC7ED5F8', 'wo_general.removeModule', '3', 'en', 'Remove App' )
Insert into @lang ( id, code, subknot, lang, name ) Values ( '7E948BED-CC30-44BA-A54D-0FF891022FDB', 'wo_general.installFromFile', '4', 'ru', 'Установка/обновление приложений из файла' )
Insert into @lang ( id, code, subknot, lang, name ) Values ( '7E948BED-CC30-44BA-A54D-0FF891022FDB', 'wo_general.installFromFile', '4', 'en', 'Install/update App from file' )
Insert into @lang ( id, code, subknot, lang, name ) Values ( '21AB014E-1250-4857-9E2C-FB9870E4DB97', 'wo_general.updateApplication', '5', 'ru', 'Обновление веб-клиента' )
Insert into @lang ( id, code, subknot, lang, name ) Values ( '21AB014E-1250-4857-9E2C-FB9870E4DB97', 'wo_general.updateApplication', '5', 'en', 'Update web-client' )


/* Уровень в дереве для плагина */
Declare @maxknot int, @woknotint int, @woknot nvarchar(50)

Select @woknot = knot
From A_UserRoles_Rights 
Where code like 'wo_general'

if ( @woknot is NULL ) begin

    Select @maxknot = Max ( cast ( knot as int ))
    From A_userroles_rights 
    Where not knot like '%.%'

    if ( @maxknot < 500 )
	    set @woknotint = 500
    else
	    set @woknotint = @maxknot + 1;

    Set @woknot = cast ( @woknotint as nvarchar(50))
    
    Insert into A_UserRoles_Rights ( Id, Knot, Name, Code )
        Select id, @woknot, name, code
        From @lang
        Where (lang = @curlang) and (code = 'wo_general')    

end else begin

    Delete 
	From A_UserRoles_Rights
    Where (knot like @woknot + '.%.%') and (code not like 'wo_general%')
		and code not in ( Select customcode 
						  From WO_InstalledModules_CustomCodeHash cch
							Inner join WO_InstalledModules im on im.code = cch.modulecode
						  Where cch.customcode like 'wo.%.r%')

end

/* Все дополнительные привилегии в общий список */
Delete 
From A_UserRoles_Rights
Where id in (Select id from @lang where code != 'wo_general')

Insert into A_UserRoles_Rights ( Id, Knot, Name, Code )
    Select id, @woknot + '.' + cast ( subknot as nvarchar(10)), name, code
    From @lang l
    Where (lang = @curlang) and (code != 'wo_general')    
        and not exists ( Select id From A_UserRoles_Rights Where id = l.id )

/* Все новые плагинские привилегии в роль администратора */
DECLARE rcur cursor LOCAL FAST_FORWARD For 
	Select Distinct id 
	From @lang	
	
OPEN rcur
	
Declare @id uniqueidentifier	
Fetch Next From rcur into @id	

While ( @@FETCH_STATUS = 0 ) begin
	
	exec dbo.A_UserRoles_AddRightForRole
		@idrole='c39d1c39-a413-42b7-ba83-dbee9fa5fb12',
		@idright=@id,
		@value=1

	Fetch Next From rcur into @id	
end

CLOSE rcur
DEALLOCATE rcur
    
GO

-------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------
------------- DB Change Configurations for correct work Procedures on Upd, Ins, Del wihout this user-privileges ---------------
-------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------

if exists ( select * FROM sysobjects where id = object_id ('dbo.WO_Database_SetTRUSTWORTHY' ))
	Drop procedure dbo.WO_Database_SetTRUSTWORTHY
GO

Create procedure dbo.WO_Database_SetTRUSTWORTHY
	@dbname nvarchar(500)
as
begin

    IF ((SELECT cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  ) > 8) begin
		
		Declare @sql nvarchar(4000)
		Set @sql = N'
	    if exists (Select * FROM Master..sysdatabases where name = ''' + @dbname + ''')
		    ALTER DATABASE [oktell] SET TRUSTWORTHY ON;
        '
        
        EXEC sp_executesql @sql

    end

end
GO

exec dbo.WO_Database_SetTRUSTWORTHY 'oktell'
exec dbo.WO_Database_SetTRUSTWORTHY 'oktell_cc_temp'
exec dbo.WO_Database_SetTRUSTWORTHY 'oktell_settings'
GO

-------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------
--------- API Procedures WITH EXECUTE AS SELF ---------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------
-- WO_Policy_Objects_Save
-- WO_Policy_CheckPrivilege
-- WO_Policy_GetObjectsByCode
-- WO_UserRoles_CheckModulePrivilege
-------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------
IF OBJECT_ID ('WO_Policy_Objects_Save') IS NOT NULL
    DROP PROCEDURE [dbo].[WO_Policy_Objects_Save];
GO

DECLARE @SQL_text1 nVARCHAR(4000);
                
SET @SQL_text1 = '
CREATE PROCEDURE dbo.WO_Policy_Objects_Save
    @modulecode nvarchar(200),
    @objecttype int = -1,
    @objecttypecode nvarchar(200),
    @idObject uniqueidentifier,	
	@name nvarchar(1000),
	@idParentProject uniqueidentifier = NULL,
	@actionUserId uniqueidentifier = NULL 
' +
	CASE WHEN cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  > 8 
		 THEN ' WITH EXECUTE AS SELF '
		 ELSE ''
	END
+ '
as 
begin
    
    Declare @type int    
    if ( @objecttype >= 0 )
        Set @type = @objecttype
    else
        exec dbo.WO_InstalledModules_GetPolicyObjectType
            @modulecode = @modulecode,
	        @objectcode = @objecttypecode,
	        @objecttype = @type OUTPUT

    exec dbo.A_Policy_Objects_Save
        @idObject=@idObject,
        @type=@type,
        @name=@name,
        @idParentProject=@idParentProject,
        @actionUserId=@actionUserId

end
'

EXEC sp_executesql @SQL_text1
GO


-------------------------------------------------------------------------------------------------------------------------------
IF OBJECT_ID ('WO_Policy_Objects_Remove') IS NOT NULL
    DROP PROCEDURE [dbo].[WO_Policy_Objects_Remove];
GO

DECLARE @SQL_text1 nVARCHAR(4000);
                
SET @SQL_text1 = '
CREATE PROCEDURE dbo.WO_Policy_Objects_Remove
    @idObject uniqueidentifier
' +
	CASE WHEN cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  > 8 
		 THEN ' WITH EXECUTE AS SELF '
		 ELSE ''
	END
+ '
as
begin

    EXEC dbo.A_Policy_Objects_Remove
        @idObject=@idObject

end
'

EXEC sp_executesql @SQL_text1
GO


------------------------------------------
IF OBJECT_ID ('WO_Policy_CheckPrivilege') IS NOT NULL
    DROP PROCEDURE [dbo].[WO_Policy_CheckPrivilege];
GO

DECLARE @SQL_text2 nVARCHAR(4000)

SET @SQL_text2 = '
CREATE PROCEDURE WO_Policy_CheckPrivilege
    @modulecode    NVARCHAR(200),
    @idObject      UNIQUEIDENTIFIER,	
    @privilegecode NVARCHAR(200),
	@authorizeduserid UNIQUEIDENTIFIER,
    @isprevileged BIT OUTPUT
' +
	CASE WHEN cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  > 8 
		 THEN ' WITH EXECUTE AS SELF '
		 ELSE ''
	END
+ '
AS
BEGIN
    DECLARE @privilegeid UNIQUEIDENTIFIER,
            @type INT,
            @rowcntd INT
    SET @rowcntd = 0
    
    EXEC [oktell]..WO_InstalledModules_GetCustomIdByCode_Param
        @modulecode = @modulecode,
        @customcode = @privilegecode,
        @customid   = @privilegeid OUTPUT,
        @type       = @type OUTPUT
        
    EXEC [oktell]..A_Policy_SubjectPrivileges_GetForUserByObject 
         @iduser = @authorizeduserid
		,@idobject = @idobject
		,@idprivilege = @privilegeid
		,@onlyreturncount = 1
		,@resultcount = @rowcntd OUTPUT
        
   SET @isprevileged = CASE WHEN @rowcntd = 0 THEN 0 ELSE 1 END
    
END
'
EXEC sp_executesql @SQL_text2
GO 


------------------------------------------
IF OBJECT_ID ('WO_Policy_GetObjectsByCode') IS NOT NULL
    DROP PROCEDURE [WO_Policy_GetObjectsByCode];
GO

DECLARE @SQL_text3 nVARCHAR(4000)

SET @SQL_text3 = '
CREATE PROCEDURE WO_Policy_GetObjectsByCode
    @modulecode       NVARCHAR(200) = NULL,
    @objecttypeid     INT = -1,
    @objecttypecode   NVARCHAR(200) = NULL,
    @privilegecode    NVARCHAR(200) = NULL,
    @privilegeid      UNIQUEIDENTIFIER = NULL,
	@authorizeduserid UNIQUEIDENTIFIER,
    @idproject        UNIQUEIDENTIFIER = NULL
' +
	CASE WHEN cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  > 8 
		 THEN ' WITH EXECUTE AS SELF '
		 ELSE ''
	END
+ '
AS
BEGIN

   DECLARE @objtype INT, 
                   @privid UNIQUEIDENTIFIER,
                   @type INT
                 
    IF (@objecttypeid >= 0)
        SET @objtype = @objecttypeid
    ELSE
        EXEC WO_InstalledModules_GetPolicyObjectType
            @modulecode = @modulecode,
	        @objectcode = @objecttypecode,
	        @objecttype = @objtype OUTPUT
    
    IF @privilegecode IS NOT NULL
        EXEC WO_InstalledModules_GetCustomIdByCode_Param
            @modulecode = @modulecode,
            @customcode = @privilegecode,
            @customid   = @privid OUTPUT,
            @type       = @type OUTPUT
    ELSE 
        SET @privid = @privilegeid
        
    EXEC [A_Policy_SubjectPrivileges_GetForUserByObjectType] @iduser = @authorizeduserid
        ,@objecttype = @objtype
        ,@idprivilege = @privid
        ,@idparentproject =	@idproject
END
'

EXEC sp_executesql @SQL_text3
GO 


------------------------------------------
IF OBJECT_ID ('WO_UserRoles_CheckModulePrivilege') IS NOT NULL
    DROP PROCEDURE [WO_UserRoles_CheckModulePrivilege];
GO

DECLARE @SQL_text4 nVARCHAR(4000)

SET @SQL_text4 = '
CREATE PROCEDURE [dbo].[WO_UserRoles_CheckModulePrivilege]
    @modulecode       NVARCHAR(200) = NULL,
    @privilegecode    NVARCHAR(200) = NULL,
	@authorizeduserid UNIQUEIDENTIFIER,
    @idprovided BIT OUTPUT
' +
	CASE WHEN cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  > 8 
		 THEN ' WITH EXECUTE AS SELF '
		 ELSE ''
	END
+ '
AS
BEGIN   
    
    DECLARE @privilegecode_in_module NVARCHAR(500)
    SET @privilegecode_in_module = COALESCE(''wo.'' + @modulecode + ''.r.'' + @privilegecode, @privilegecode)
    
    
    EXEC [oktell].dbo.A_UserRoles_CheckPrivilege @privilegecode = @privilegecode_in_module
      ,@authorizeduserid = @authorizeduserid
      ,@isenabled = @idprovided OUTPUT
END
'

EXEC sp_executesql @SQL_text4
GO

------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------
---- API for custom modules --------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------
IF (SELECT cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  ) > 8
BEGIN
	EXEC sp_executesql N'
    ALTER DATABASE [oktell] SET TRUSTWORTHY ON;

    ALTER DATABASE [oktell_cc_temp] SET TRUSTWORTHY ON;

    ALTER DATABASE [oktell_settings] SET TRUSTWORTHY ON;
    '
END
GO
--------- API Procedures

--- 1
IF OBJECT_ID('[dbo].WO_M_UserNotifyLog_SetShownById') IS NOT NULL
    DROP PROCEDURE [dbo].WO_M_UserNotifyLog_SetShownById
GO

DECLARE @SQL_text1 nVARCHAR(4000);
                
SET @SQL_text1 = '
CREATE PROCEDURE [dbo].WO_M_UserNotifyLog_SetShownById @id uniqueidentifier, @wasshown bit = 1
  ' +
CASE WHEN cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  > 8 
     THEN ' WITH EXECUTE AS SELF '
     ELSE ''
END
+'
AS 
BEGIN
    EXEC [oktell].[dbo].[A_UserNotifyLog_SetShownById] @id = @id,	@wasshown = 1
END
'
EXEC sp_executesql @SQL_text1
GO

--- 2
IF OBJECT_ID('[dbo].WO_M_CallBackOrders_SetShownById') IS NOT NULL
    DROP PROCEDURE [dbo].WO_M_CallBackOrders_SetShownById
GO

DECLARE @SQL_text2 nVARCHAR(4000);
                
SET @SQL_text2 = '
CREATE PROCEDURE [dbo].WO_M_CallBackOrders_SetShownById @id uniqueidentifier, @wasshown bit = 1
  ' +
CASE WHEN cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  > 8 
     THEN ' WITH EXECUTE AS SELF '
     ELSE ''
END
+'
AS 
BEGIN
    UPDATE [oktell].[dbo].A_CallBackOrders
      SET Viewed = 1
    WHERE IdConn = @id
END
'
EXEC sp_executesql @SQL_text2
GO

--- 3
IF OBJECT_ID('[dbo].WO_M_UserNotifyLog_Delete') IS NOT NULL
    DROP PROCEDURE [dbo].WO_M_UserNotifyLog_Delete
GO

DECLARE @SQL_text3 nVARCHAR(4000);
                
SET @SQL_text3 = '
CREATE PROCEDURE [dbo].WO_M_UserNotifyLog_Delete @id uniqueidentifier
  ' +
CASE WHEN cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  > 8 
     THEN ' WITH EXECUTE AS SELF '
     ELSE ''
END
+'
AS 
BEGIN
    EXEC [oktell].[dbo].A_UserNotifyLog_Delete @id
END
'
EXEC sp_executesql @SQL_text3
GO

--- 4
IF OBJECT_ID('[dbo].WO_M_Stat_Comments_SetCommentById') IS NOT NULL
    DROP PROCEDURE [dbo].WO_M_Stat_Comments_SetCommentById
GO

DECLARE @SQL_text4 nVARCHAR(4000);
                
SET @SQL_text4 = '
CREATE PROCEDURE [dbo].WO_M_Stat_Comments_SetCommentById @id uniqueidentifier, @comment nVARCHAR(4000)
  ' +
CASE WHEN cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  > 8 
     THEN ' WITH EXECUTE AS SELF '
     ELSE ''
END
+'
AS 
BEGIN
    EXEC [oktell].[dbo].A_Stat_Comments_SetCommentById @id, @comment
END
'
EXEC sp_executesql @SQL_text4
GO

--- 5
IF OBJECT_ID('[dbo].WO_M_VoiceMail_SetListened') IS NOT NULL
    DROP PROCEDURE [dbo].WO_M_VoiceMail_SetListened
GO

DECLARE @SQL_text5 nVARCHAR(4000);
                
SET @SQL_text5 = '
CREATE PROCEDURE [dbo].WO_M_VoiceMail_SetListened @userid uniqueidentifier,@mailid uniqueidentifier,@dtnow DateTime = null
  ' +
CASE WHEN cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  > 8 
     THEN ' WITH EXECUTE AS SELF '
     ELSE ''
END
+'
AS 
BEGIN
    EXEC [oktell].[dbo].A_VoiceMail_SetListened @userid, @mailid, @dtnow 
END
'
EXEC sp_executesql @SQL_text5
GO

--- 6
IF OBJECT_ID('[dbo].WO_M_CallBackOrders_Save') IS NOT NULL
    DROP PROCEDURE [dbo].WO_M_CallBackOrders_Save
GO

DECLARE @SQL_text6 nVARCHAR(4000);
                
SET @SQL_text6 = '
CREATE PROCEDURE [dbo].WO_M_CallBackOrders_Save @id	uniqueidentifier,@userid uniqueidentifier,@comment nvarchar(1000),@done bit
  ' +
CASE WHEN cast(replace(left(cast(SERVERPROPERTY('productversion') as VARCHAR),2),'.','') as INT)  > 8 
     THEN ' WITH EXECUTE AS SELF '
     ELSE ''
END
+'
AS 
BEGIN
    EXEC [oktell].[dbo].A_CallBackOrders_Save
            @id,
            @userid,
            @comment,
            @done
END
'
EXEC sp_executesql @SQL_text6
GO