
ALTER  procedure [dbo].[A_Clients__GetByContact]
	@contact	nvarchar ( 255 ),
	@ownerid 	uniqueidentifier,
	@id		uniqueidentifier output,
	@name		nvarchar ( 255 ) output
as
begin
	Declare @idclient uniqueidentifier
	Set @idclient = '00000000-0000-0000-0000-000000000000'

	Select @idclient = IdClient
	From A_ClientContacts
	Where ( Len ( Content ) >= 5 ) and (( Content like '%' + @contact ) or ( @contact like '%' + Content ))

	if ( @@rowcount = 0 )
		return;

	Select @id = cl.[Id], @name = cl.[Name]
	From A_Clients cl
		Left join A_ClientContacts cc on cl.Id = cc.IdClient
	Where cl.Id = @idclient
		And ( cl.OwnerId = '00000000-0000-0000-0000-000000000000' Or cl.OwnerId = @ownerid )
end

GO


ALTER   procedure [dbo].[A_Clients__GetJournal]
	@idclient	uniqueidentifier,
	@dstart		datetime,
	@dstop		datetime,
	@tstart		datetime,
	@tstop		datetime,
	@idoperator	uniqueidentifier,
	@iduser		uniqueidentifier
as
begin
	
	Declare @tstartfloat float
	Declare @tstopfloat float
	Set @tstartfloat =  Convert ( float, @tstart ) - FLOOR ( Convert ( float, @tstart ) )
	Set @tstopfloat =  Convert ( float, @tstop ) - FLOOR ( Convert ( float, @tstop ) )


	Declare @usersdenied table ( Id uniqueidentifier )
	Insert into @usersdenied
		Select Id 
		From A_Users
		Where 	Id != @iduser and 
			Id not in ( 	Select UserB 
					From A_UserControls 
					Where UserA = @iduser )


	Select * FROM 
	(
		Select sc.TimeStart, 
			sc.TimeStop - sc.TimeStart as Length, 
			AStr as OperatorStr, 
			case when ( u.Id is null ) then sc.IsRecorded else 0 end as IsRecorded, 
			sc.ARecFile, 
			sc.BRecFile,
			0 as Direction
		From A_Stat_Connections_1x1 sc 
			Inner join A_ClientContacts cc on ( ( len ( cc.Content ) >= 5 ) and ((cc.Content like '%' + sc.BOutNumber) or (sc.BOutNumber like '%' + cc.Content)) )
			Inner join A_Clients c on cc.IdClient = c.Id		
			Left join @usersdenied u on sc.AUserId = u.Id
		Where ( TimeStart >= @dstart )
			and ( TimeStop <= @dstop )
			and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) >= @tstartfloat )
			and ( Convert ( float, TimeStop ) - FLOOR ( Convert ( float, TimeStop ) ) <= @tstopfloat )
			and ( @idoperator = '00000000-0000-0000-0000-000000000000' or sc.AUserId = @idoperator )

		Union All
	
		Select sc.TimeStart, 
			sc.TimeStop - sc.TimeStart as Length,
			BStr as OperatorStr, 
			case when ( u.Id is null ) then sc.IsRecorded else 0 end as IsRecorded, 
			sc.ARecFile, 
			sc.BRecFile,
			1 as Direction
		From A_Stat_Connections_1x1 sc 
			Inner join A_ClientContacts cc on ( ( len ( cc.Content ) >= 5 ) and ((cc.Content like '%' + sc.AOutNumber) or (sc.AOutNumber like '%' + cc.Content)) )
			Inner join A_Clients c on cc.IdClient = c.Id		
			Left join @usersdenied u on sc.BUserId = u.Id
		Where ( TimeStart >= @dstart )
			and ( TimeStop <= @dstop )
			and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) >= @tstartfloat )
			and ( Convert ( float, TimeStop ) - FLOOR ( Convert ( float, TimeStop ) ) <= @tstopfloat )
			and ( @idoperator = '00000000-0000-0000-0000-000000000000' or sc.BUserId = @idoperator )
	) a

	Order By TimeStart Desc

		
end