if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_DB_ClearConstraints' ) )
DROP PROCEDURE dbo.A_DB_ClearConstraints
GO

create procedure [dbo].[A_DB_ClearConstraints]
	@simpletablename nvarchar ( 500 )
as
begin
	
declare @tablename varchar ( 500 )
declare @sql nvarchar(1000)
set @tablename='dbo.' + @simpletablename


DECLARE tcursor CURSOR FOR 
	select s.name, s.parent_obj, s1.name
	from sysobjects s
		inner join sysobjects s1 on s.parent_obj = s1.Id
	where s.type = 'F' and s.name like '%' + @simpletablename + '%' 

declare @cnstrname varchar(500)
declare @parent int
declare @subtab varchar ( 500 )


OPEN tcursor
FETCH NEXT FROM tcursor INTO @cnstrname, @parent, @subtab

WHILE @@FETCH_STATUS = 0
BEGIN

set @sql = ' ALTER TABLE ' + @subtab + 
	' DROP CONSTRAINT ' + @cnstrname
exec sp_executesql @stmt =@sql
print @sql

FETCH NEXT FROM tcursor INTO @cnstrname, @parent, @subtab
END

CLOSE tcursor
DEALLOCATE tcursor

end
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_DB_ClearIndexes' ) )
DROP PROCEDURE dbo.A_DB_ClearIndexes
GO

create procedure [dbo].[A_DB_ClearIndexes]
	@tablename nvarchar ( 500 )
as
begin
	
declare @sql nvarchar(1000)

DECLARE tcursor CURSOR FOR 
select name from sysindexes where id=object_id(@tablename) and impid >= 0 /*status!=0*/ and name not like '_WA%'

declare @ind_name varchar(500)

OPEN tcursor
FETCH NEXT FROM tcursor INTO @ind_name

WHILE @@FETCH_STATUS = 0
BEGIN

set @sql='drop index '+@tablename+'.'+@ind_name
exec sp_executesql @stmt =@sql

FETCH NEXT FROM tcursor INTO @ind_name
END

CLOSE tcursor
DEALLOCATE tcursor


end
GO



exec A_DB_ClearConstraints 'A_TaskManager_SubTasks'
GO

exec A_DB_ClearConstraints 'A_TaskManager_SubTaskEfforts'
GO

exec A_DB_ClearConstraints 'A_TaskManager_SubTaskNumbers'
GO

exec A_DB_ClearConstraints 'A_TaskManager_Tasks'
GO

exec A_DB_ClearConstraints 'A_TaskManager_TaskSupervisors'
GO

if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_TaskManager_SubTasks' ) )
DROP TABLE dbo.A_TaskManager_SubTasks
GO

CREATE TABLE dbo.A_TaskManager_SubTasks
	(
	IdInList int NOT NULL,
	TaskId uniqueidentifier NOT NULL,
	NResult int NOT NULL,
	IsDeleted nvarchar(500) NOT NULL,
	NextEffortTime datetime NULL
	)
GO

ALTER TABLE dbo.A_TaskManager_SubTasks ADD CONSTRAINT
	PK_A_TaskManager_SubTasks PRIMARY KEY CLUSTERED 
	(
	IdInList,
	TaskId
	)

GO

ALTER TABLE dbo.A_TaskManager_SubTasks WITH NOCHECK ADD CONSTRAINT
	FK_A_TaskManager_SubTasks_A_TaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_TaskManager_SubTaskNumber' ) )
DROP TABLE dbo.A_TaskManager_SubTaskNumber
GO


CREATE TABLE dbo.A_TaskManager_SubTaskNumber
	(
	IdInList int NOT NULL,
	TaskId uniqueidentifier NOT NULL,
	Number nvarchar(20) NOT NULL,
	CountBusy int NOT NULL,
	CountNotRespond int NOT NULL,
	CountQHangOff int NOT NULL,
	QTime int NOT NULL,
	NResult int NOT NULL,
	NextEffortTime datetime NULL	
	)
GO

ALTER TABLE dbo.A_TaskManager_SubTaskNumber ADD CONSTRAINT
	PK_A_TaskManager_SubTaskNumber PRIMARY KEY CLUSTERED 
	(
	IdInList,
	TaskId,
	Number
	)

GO
ALTER TABLE dbo.A_TaskManager_SubTaskNumber WITH NOCHECK ADD CONSTRAINT
	FK_A_TaskManager_SubTaskNumber_A_TaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_TaskManager_SubTaskEfforts' ) )
DROP TABLE dbo.A_TaskManager_SubTaskEfforts
GO

CREATE TABLE dbo.A_TaskManager_SubTaskEfforts
	(
	Id uniqueidentifier NOT NULL,
	IdInList int NOT NULL,
	TaskId uniqueidentifier NOT NULL,
	Number nvarchar(20) NOT NULL,
	EDateTime datetime NOT NULL,
	EResult int NOT NULL,
	EInfo nvarchar(500) NULL,
	ConnectionChainId uniqueidentifier NULL
	)
GO

ALTER TABLE dbo.A_TaskManager_SubTaskEfforts ADD CONSTRAINT
	PK_A_TaskManager_SubTaskEfforts PRIMARY KEY CLUSTERED 
	(
	Id
	)

GO
CREATE NONCLUSTERED INDEX idxSubTaskId ON dbo.A_TaskManager_SubTaskEfforts
	(
	TaskId
	) WITH FILLFACTOR = 80
GO
ALTER TABLE dbo.A_TaskManager_SubTaskEfforts WITH NOCHECK ADD CONSTRAINT
	FK_A_TaskManager_SubTaskEfforts_A_TaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

exec A_DB_ClearConstraints 'A_TaskManager_Tasks'
GO

if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_TaskManager_Tasks' ) )
drop table dbo.A_TaskManager_Tasks
GO


CREATE TABLE dbo.A_TaskManager_Tasks
	(
	Id uniqueidentifier NOT NULL,
	IdProject uniqueidentifier NOT NULL,
	Name nvarchar(500) NOT NULL,
	Active int NOT NULL,
	Done int NOT NULL,
	Type int NOT NULL,
	IvrName nvarchar(200) NOT NULL,
	StopReason int NOT NULL,
	TalkScript nvarchar(200) NOT NULL,
	PauseWhenBusy float(53) NOT NULL,
	PauseWhenNotRespond float(53) NOT NULL,
	MaxEffortCountBusy int NOT NULL,
	MaxEffortCountNotRespond int NOT NULL,
	MinTimeToCheckDone int NOT NULL,
	TryConnectTime int NOT NULL,
	ChoiceOperator int NOT NULL,
	DeleteAfterSuccess int NOT NULL,
	IsExtNumber int NOT NULL,
	IsOutputTask int NOT NULL,
	Resulted int NOT NULL,
	ResultType int NOT NULL,
	ResultInfo nvarchar(500) NOT NULL,
	ResultDateTime datetime NOT NULL,
	CreationUserId uniqueidentifier NOT NULL,
	CreationType int NOT NULL,
	CreationDateTime datetime NOT NULL,
	StartRunDateTime datetime NOT NULL,
	StopRunDateTime datetime NOT NULL,
	CanAddDynamically int NOT NULL,
	TagDouble float(53) NOT NULL,
	ListId uniqueidentifier NOT NULL,
	OperatorShowQuery bit NOT NULL,
	OperatorShowNotify bit NOT NULL,
	ProgressCall bit NOT NULL,
	ProgressCallMaxCount int NOT NULL,
	ProgressCallKeepQueue bit NOT NULL,
	PredictCall bit NOT NULL,
	PredictCallOffsetSec int NOT NULL,
	QueueUse bit NOT NULL,
	QueueMaxCount int NOT NULL,
	QueueMusic nvarchar(100) NOT NULL,
	QueueLoyality int NOT NULL,
	ChoiceAbonent int NOT NULL,
	ChoiceNumber int NOT NULL,
	ChoiceTable int NOT NULL,
	ChoiceProgress int NOT NULL,
	DontSetDone bit NOT NULL,
	PauseSyncListWaitingNewAbonent int NOT NULL,
	IncomingCallsMaxCount int NOT NULL,
	ServiceScript nvarchar ( 200 ) not null,
	DialCallerId nvarchar ( 20 ) not null,
	IncomingUpdateIfExists bit not null,
	ActiveOnHolidays bit not null,
	Code nvarchar ( 30 ) not null,
	PauseSyncListForcedSec int not null,
	QueueMaxTimeMinutes int not null
	)
GO



ALTER TABLE dbo.A_TaskManager_Tasks ADD CONSTRAINT
	PK_A_TaskManager_Tasks PRIMARY KEY CLUSTERED 
	(
	Id
	)

GO


ALTER TABLE dbo.A_TaskManager_SubTaskEfforts WITH NOCHECK ADD CONSTRAINT
	FK_A_TaskManager_SubTaskEfforts_A_TaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

ALTER TABLE dbo.A_TaskManager_SubTaskNumber WITH NOCHECK ADD CONSTRAINT
	FK_A_TaskManager_SubTaskNumber_A_TaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

ALTER TABLE dbo.A_TaskManager_RunPeriods WITH NOCHECK ADD CONSTRAINT
	FK_A_TaskManager_RunPeriods_A_TaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

ALTER TABLE dbo.C_TaskControlEvents WITH NOCHECK ADD CONSTRAINT
	FK_C_TaskControlEvents_A_TaskManager_Tasks FOREIGN KEY
	(
	IDtask
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

ALTER TABLE dbo.A_TaskManager_UserInTask WITH NOCHECK ADD CONSTRAINT
	FK_A_TaskManager_UserInTask_A_TaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

ALTER TABLE dbo.C_MonopolyForTaskChanging WITH NOCHECK ADD CONSTRAINT
	FK_C_MonopolyForTaskChanging_A_TaskManager_Tasks FOREIGN KEY
	(
	IdTask
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

ALTER TABLE dbo.A_TaskManager_SubTasks WITH NOCHECK ADD CONSTRAINT
	FK_A_TaskManager_SubTasks_A_TaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

ALTER TABLE dbo.A_TaskManager_TaskExtLines WITH NOCHECK ADD CONSTRAINT
	FK_A_TaskManager_TaskExtLines_A_TaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

ALTER TABLE dbo.A_TaskManager_Operators WITH NOCHECK ADD CONSTRAINT
	FK_A_TaskManager_Operators_A_TaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_TaskSupervisors' ) )
drop Table dbo.A_TaskManager_TaskSupervisors
GO

CREATE TABLE dbo.A_TaskManager_TaskSupervisors
	(
	TaskId uniqueidentifier NULL,
	SupervisorId uniqueidentifier NULL
	)  ON [PRIMARY]
GO

ALTER TABLE dbo.A_TaskManager_TaskSupervisors ADD CONSTRAINT
	FK_A_TaskManager_TaskSupervisors_A_TaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_TaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

ALTER TABLE dbo.A_TaskManager_TaskSupervisors ADD CONSTRAINT
	FK_A_TaskManager_TaskSupervisors_A_Users FOREIGN KEY
	(
	SupervisorId
	) REFERENCES dbo.A_Users
	(
	ID
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO



if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_TaskManager_AllTask_Get' ) )
DROP PROCEDURE dbo.A_TaskManager_AllTask_Get
GO

CREATE Procedure dbo.A_TaskManager_AllTask_Get
as

begin
	select 	
		Id,
		IdProject,
		Name,
		Active,
		Done,
		Type,
		IvrName,
		StopReason,
		TalkScript,
		PauseWhenBusy,
		PauseWhenNotRespond,
		MaxEffortCountBusy,
		MaxEffortCountNotRespond,
		MinTimeToCheckDone,
		TryConnectTime,
		ChoiceOperator,
		DeleteAfterSuccess,
		IsExtNumber,
		IsOutputTask,
		Resulted,
		ResultType,
		ResultInfo,
		ResultDateTime,
		CreationUserId,
		CreationType,
		CreationDateTime,
		StartRunDateTime,
		StopRunDateTime,
		CanAddDynamically,
		TagDouble,
		ListId,
		OperatorShowQuery,
		OperatorShowNotify,
		ProgressCall,
		ProgressCallMaxCount,
		ProgressCallKeepQueue,
		PredictCall,
		PredictCallOffsetSec,
		QueueUse,
		QueueMaxCount,
		QueueMusic,
		QueueLoyality,
		ChoiceAbonent,
		ChoiceNumber,
		ChoiceTable,
		ChoiceProgress,
		DontSetDone,
		PauseSyncListWaitingNewAbonent,
		IncomingCallsMaxCount,
		ServiceScript,
		DialCallerId,
		IncomingUpdateIfExists,
		ActiveOnHolidays,
		Code,
		PauseSyncListForcedSec,
		QueueMaxTimeMinutes
	
	from A_TaskManager_Tasks
end

GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_TaskManager_Task_Update' ) )
DROP PROCEDURE dbo.A_TaskManager_Task_Update
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_TaskManager_Task_Save' ) )
DROP PROCEDURE dbo.A_TaskManager_Task_Save
GO


Create Procedure dbo.A_TaskManager_Task_Save

	@id 				uniqueidentifier,
	@idproject			uniqueidentifier,
	@name				nvarchar(500),
	@active				int,
	@done				int,
	@type				int,
	@ivrname			nvarchar(500),
	@stopreason			int,
	@talkscript			nvarchar(500),
	@pausewhenbusy			float,
	@pausewhennotrespond		float,
	@maxeffortcountbusy		int,
	@maxeffortcountnotrespond	int,
	@mintimetocheckdone		int,
	@tryconnecttime			int,
	@choiceoperator			int,
	@deleteaftersuccess		int,
	@isextnumber			int,
	@isoutputtask			int,
	@resulted			int,
	@resulttype			int,
	@resultinfo			nvarchar(500),
	@resultdatetime			datetime,
	@creationuserid			uniqueidentifier,
	@creationtype			int,
	@creationdatetime		datetime,
	@startrundatetime		datetime,
	@stoprundatetime		datetime,
	@canadddynamically		int,
	@tagdouble			float,
	@listid				uniqueidentifier,
	@operatorshowquery		bit,
	@operatorshownotify		bit,
	@progresscall			bit,
	@progresscallmaxcount		int,
	@progresscallkeepqueue		bit,
	@predictcall			bit,
	@predictcalloffsetsec		int,
	@queueuse			bit,
	@queuemaxcount			int,
	@queuemusic			nvarchar ( 500 ),
	@queueloyality			int,
	@choiceabonents			int,
	@choicenumber			int,
	@choicetable			int,
	@choiceprogress			int,
	@dontsetdone			bit,
	@pausesynclistwaitingnewabonent int,
	@incomingcallsmaxcount		int,
	@servicescript			nvarchar ( 200 ),
	@dialcallerid			nvarchar ( 20 ),
	@incomingupdateifexists		bit,
	@activeonholidays		bit,
	@code				nvarchar ( 30 ),
	@pausesynclistforcedsec		int,
	@queuemaxtimeminutes		int

as

begin
	select name from A_TaskManager_Tasks where id = @id

	if ( @@rowcount > 0 )
	begin
		update A_TaskManager_Tasks
		   Set 
			IdProject			= @idproject,
			Name 				= @name, 			
			Active 				= @active, 
			Done 				= @done, 
			Type 				= @type, 
			IvrName 			= @ivrname, 
			StopReason 			= @stopreason, 
			TalkScript 			= @talkscript, 
			PauseWhenBusy 			= @pausewhenbusy, 
			PauseWhenNotRespond 		= @pausewhennotrespond, 
			MaxEffortCountBusy 		= @maxeffortcountbusy, 
			MaxEffortCountNotRespond 	= @maxeffortcountnotrespond, 
			MinTimeToCheckDone 		= @mintimetocheckdone, 
			TryConnectTime 			= @tryconnecttime, 
			ChoiceOperator			= @choiceoperator,
			DeleteAfterSuccess 		= @deleteaftersuccess, 
			IsExtNumber 			= @isextnumber, 
			IsOutputTask			= @isoutputtask,
			Resulted 			= @resulted, 
			ResultType 			= @resulttype, 
			ResultInfo 			= @resultinfo, 
			ResultDateTime 			= @resultdatetime, 
			CreationUserId 			= @creationuserid, 
			CreationType 			= @creationtype, 
			CreationDateTime 		= @creationdatetime,
			StartRunDateTime		= @startrundatetime,
			StopRunDateTime			= @stoprundatetime,
			CanAddDynamically		= @canadddynamically,	
			TagDouble			= @tagdouble,	
			ListId				= @listid,
			OperatorShowQuery		= @operatorshowquery,
			OperatorShowNotify		= @operatorshownotify,
			ProgressCall			= @progresscall,
			ProgressCallMaxCount		= @progresscallmaxcount,
			ProgressCallKeepQueue		= @progresscallkeepqueue,
			PredictCall			= @predictcall,
			PredictCallOffsetSec		= @predictcalloffsetsec,
			QueueUse 			= @queueuse,
			QueueMaxCount			= @queuemaxcount,
			QueueMusic			= @queuemusic,
			QueueLoyality			= @queueloyality,
			ChoiceAbonent			= @choiceabonents,
			ChoiceNumber			= @choicenumber,
			ChoiceTable			= @choicetable,
			ChoiceProgress			= @choiceprogress,
			DontSetDone			= @dontsetdone,
			PauseSyncListWaitingNewAbonent	= @pausesynclistwaitingnewabonent,
			IncomingCallsMaxCount		= @incomingcallsmaxcount,
			ServiceScript			= @servicescript,
			DialCallerId			= @dialcallerid,
			IncomingUpdateIfExists		= @incomingupdateifexists,
			ActiveOnHolidays		= @activeonholidays,
			Code				= @code,
			PauseSyncListForcedSec		= @pausesynclistforcedsec,
			QueueMaxTimeMinutes		= @queuemaxtimeminutes
 		   Where
			Id 				= @id
	end
	else
	begin
		insert into A_TaskManager_Tasks
		(
			Id, 
			IdProject,
			Name, 
			Active, 
			Done, 
			Type, 
			IvrName, 
			StopReason, 
			TalkScript, 
			PauseWhenBusy, 
			PauseWhenNotRespond, 
			MaxEffortCountBusy, 
			MaxEffortCountNotRespond, 
			MinTimeToCheckDone, 
			TryConnectTime, 
			ChoiceOperator, 
			DeleteAfterSuccess, 
			IsExtNumber, 
			IsOutputTask, 
			Resulted, 
			ResultType, 
			ResultInfo, 
			ResultDateTime, 
			CreationUserId, 
			CreationType, 
			CreationDateTime, 
			StartRunDateTime, 
			StopRunDateTime, 
			CanAddDynamically, 
			TagDouble, 
			ListId,
			OperatorShowQuery,
			OperatorShowNotify,
			ProgressCall,
			ProgressCallMaxCount,
			ProgressCallKeepQueue,
			PredictCall,
			PredictCallOffsetSec,
			QueueUse,
			QueueMaxCount,
			QueueMusic,
			QueueLoyality,
			ChoiceAbonent,
			ChoiceNumber,
			ChoiceTable,
			ChoiceProgress,
			DontSetDone,			
			PauseSyncListWaitingNewAbonent,
			IncomingCallsMaxCount,
			ServiceScript,
			DialCallerId,
			IncomingUpdateIfExists,
			ActiveOnHolidays,
			Code,
			PauseSyncListForcedSec,
			QueueMaxTimeMinutes
		) 
		Values  
		(
			@id,
			@idproject,
			@name,
			@active,
			@done,
			@type,
			@ivrname,
			@stopreason,
			@talkscript,
			@pausewhenbusy,
			@pausewhennotrespond,
			@maxeffortcountbusy,
			@maxeffortcountnotrespond,
			@mintimetocheckdone,
			@tryconnecttime,
			@choiceoperator,
			@deleteaftersuccess,
			@isextnumber,
			@isoutputtask,
			@resulted,
			@resulttype,
			@resultinfo,
			@resultdatetime,
			@creationuserid,
			@creationtype,
			@creationdatetime,
			@startrundatetime,
			@stoprundatetime,
			@canadddynamically,
			@tagdouble,
			@listid,
			@operatorshowquery,
			@operatorshownotify,
			@progresscall,
			@progresscallmaxcount,
			@progresscallkeepqueue,
			@predictcall,
			@predictcalloffsetsec,
			@queueuse,
			@queuemaxcount,
			@queuemusic,
			@queueloyality,
			@choiceabonents,
			@choicenumber,
			@choicetable,
			@choiceprogress,
			@dontsetdone,
			@pausesynclistwaitingnewabonent,
			@incomingcallsmaxcount,
			@servicescript,
			@dialcallerid,
			@incomingupdateifexists,
			@activeonholidays,
			@code,
			@pausesynclistforcedsec,
			@queuemaxtimeminutes
		)		
	end
end


GO





If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Lists' ) )
drop table dbo.A_TaskManager_Lists
GO


Create table [dbo].[A_TaskManager_Lists]
(
	Id		uniqueidentifier NOT NULL,
	Name		nvarchar ( 100 ) NOT NULL,
	Description 	nvarchar ( 255 ) NOT NULL,
	ProjectId	uniqueidentifier,
	Type		int NOT NULL,
	Query		nvarchar ( 1000 ),
	JoinedTable	nvarchar ( 100 ),
	ColumnId	nvarchar ( 100 ),
	ColumnsPhone 	nvarchar ( 1000 ),
	ColumnName	nvarchar ( 100 ),
	ColumnsViewed	nvarchar ( 1000 ),
	ColumnsViewedDescription	nvarchar ( 4000 ),
)
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_List_GetInfo' ) )
drop procedure dbo.A_TaskManager_List_GetInfo
GO


Create procedure [dbo].[A_TaskManager_List_GetInfo]
	@id uniqueidentifier
as
begin

	Select Id, Name, Description, ProjectId, Type, Query, JoinedTable, ColumnId, ColumnsPhone, ColumnName, ColumnsViewed, ColumnsViewedDescription
	From A_TaskManager_Lists
	Where Id = @id or ProjectId = @id

end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_List_GetTaskNumberInfo' ) )
drop procedure dbo.A_TaskManager_List_GetTaskNumberInfo
GO


Create procedure [dbo].[A_TaskManager_List_GetTaskNumberInfo]
	@idtask uniqueidentifier
as
begin

	Select IdInList, Number, CountBusy, CountNotRespond, CountQHangOff, QTime, NResult, isnull ( NextEffortTime, GetDate() ) as NextEffortTime
	From A_TaskManager_SubTaskNumber stn		
	Where TaskId = @idtask
end
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_List_GetTaskPointInfo' ) )
drop procedure dbo.A_TaskManager_List_GetTaskPointInfo
GO


Create procedure [dbo].[A_TaskManager_List_GetTaskPointInfo]
	@idtask uniqueidentifier
as
begin
	Select IdInList, NResult, isnull ( NextEffortTime, GetDate() ) as NextEffortTime
	From A_TaskManager_SubTasks st
	Where TaskId = @idtask
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_List_Save' ) )
drop procedure dbo.A_TaskManager_List_Save
GO


Create procedure [dbo].[A_TaskManager_List_Save]
	@id uniqueidentifier,	
	@name nvarchar ( 100 ),
	@description nvarchar ( 255 ),
	@idproject uniqueidentifier,
	@type int,
	@query nvarchar ( 1000 ),
	@joinedtable nvarchar ( 100 ),
	@columnid nvarchar ( 100 ),
	@columnsphone nvarchar ( 1000 ),
	@columnname nvarchar ( 100 ),
	@columnsviewed nvarchar ( 1000 ),
	@columnsvieweddescription nvarchar ( 4000 )
as
begin
	
	Declare @tp int
	Declare @jt nvarchar ( 100 )

	Select @tp = Type, @jt = JoinedTable
	From A_TaskManager_Lists 
	Where Id = @id

	declare @sql nvarchar ( 100 )
	set @sql = 'drop table ' + @jt
	if ( @tp = 1 ) and ( @jt != @joinedtable )
		if ( exists ( Select * From sysobjects where id = object_id ( @jt ) ) )
			exec sp_executesql @sql

	Update A_Taskmanager_Lists
	Set		
		Name 		= @name,
		Description 	= @description,
		ProjectId 	= @idproject,
		Type 		= @type,
		Query 		= @query,
		JoinedTable 	= @joinedtable,
		ColumnId 	= @columnid,
		ColumnsPhone 	= @columnsphone,
		ColumnName 	= @columnname,
		ColumnsViewed 	= @columnsviewed,
		ColumnsViewedDescription = @columnsvieweddescription
	Where Id = @id

	if ( @@rowcount = 0 )
		Insert into A_TaskManager_Lists ( Id, Name, Description, ProjectId, Type, Query, JoinedTable, ColumnId, ColumnsPhone, ColumnName, ColumnsViewed, ColumnsViewedDescription )
		Values ( @id, @name, @description, @idproject, @type, @query, @joinedtable, @columnid, @columnsphone, @columnname, @columnsviewed, @columnsvieweddescription )
end
GO




If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_List_Delete' ) )
drop procedure dbo.A_TaskManager_List_Delete
GO


Create procedure [dbo].[A_TaskManager_List_Delete]
	@id uniqueidentifier
as
begin

	Update A_TaskManager_Tasks
	Set ListId = '00000000-0000-0000-0000-000000000000'
	Where ListId = @id

	Declare @type int
	Declare @jt nvarchar ( 100 )
	Select @type = Type, @jt = JoinedTable From A_TaskManager_Lists
	Where id = @id

	Delete From A_TaskManager_Lists
	Where Id = @id

	declare @sql nvarchar ( 100 )
	set @sql = 'drop table ' + @jt
	if ( @type = 1 )
	begin
		if ( exists ( Select * From sysobjects where id = object_id ( @jt ) ) )
			exec sp_executesql @sql
	end		
end
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Projects' ) )
drop table dbo.A_TaskManager_Projects
GO


Create Table [dbo].A_TaskManager_Projects
(
	Id uniqueidentifier,
	Name nvarchar ( 100 ),
	Description nvarchar ( 255 ),
	DateCreation DateTime,
	IdCreator uniqueidentifier
)
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Project_Save' ) )
drop procedure dbo.A_TaskManager_Project_Save
GO


Create procedure [dbo].A_TaskManager_Project_Save
	@id 	uniqueidentifier,
	@name 	nvarchar ( 100 ),
	@description nvarchar ( 255 ),
	@datecreation DateTime,
	@idcreator uniqueidentifier
as
begin

Update A_TaskManager_Projects
Set	Name = @name,
	Description = @description
Where Id = @id
	

if ( @@rowcount = 0 )
	Insert into A_TaskManager_Projects
	Values ( @id, @name, @description, @datecreation, @idcreator )

end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Project_GetAll' ) )
drop procedure dbo.A_TaskManager_Project_GetAll
GO


Create procedure [dbo].A_TaskManager_Project_GetAll
as
begin

Select Id, Name, Description, DateCreation, IdCreator
From A_TaskManager_Projects

end
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Project_Delete' ) )
drop procedure dbo.A_TaskManager_Project_Delete
GO


Create procedure [dbo].A_TaskManager_Project_Delete
	@id 	uniqueidentifier	
as
begin

--  
Delete from A_TaskManager_Projects
Where Id = @id

--  
DECLARE tcursor CURSOR FOR 
	select id 
	from A_TaskManager_Lists
	where ProjectId = @id 

declare @idlist uniqueidentifier

OPEN tcursor
FETCH NEXT FROM tcursor INTO @idlist

WHILE @@FETCH_STATUS = 0
BEGIN

exec A_TaskManager_List_Delete @idlist


FETCH NEXT FROM tcursor INTO @idlist
END

CLOSE tcursor
DEALLOCATE tcursor



--  ,     
Delete From A_TaskManager_Tasks 
Where IdProject = @id



end
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_UpdateResultInfo' ) )
drop procedure dbo.A_TaskManager_Task_UpdateResultInfo
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_TaskOut_UpdateResult_Task' ) )
drop procedure dbo.A_TaskManager_TaskOut_UpdateResult_Task
GO

CREATE Procedure [dbo].A_TaskManager_TaskOut_UpdateResult_Task	
	@id 				uniqueidentifier,
	@active				int,
	@done				int,	
	@stopreason			int = 0,
	@resulted			int = 0,
	@resulttype			int = 0,
	@resultinfo			nvarchar(500) = '',
	@resultdatetime			datetime
as

begin
	Update A_TaskManager_Tasks
	Set 			
		Active 		= @active, 
		Done 		= @done,
		StopReason 	= @stopreason, 
		Resulted 	= @resulted, 
		ResultType 	= @resulttype, 
		ResultInfo 	= @resultinfo, 
		ResultDateTime 	= @resultdatetime
 	Where
		Id 		= @id
		

end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_UpdateSubTaskPredResultInfo' ) )
drop procedure dbo.A_TaskManager_Task_UpdateSubTaskPredResultInfo
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_TaskOut_UpdateResult_Point' ) )
drop procedure dbo.A_TaskManager_TaskOut_UpdateResult_Point
GO


CREATE Procedure dbo.A_TaskManager_TaskOut_UpdateResult_Point
	@idtask			uniqueidentifier,	
	@idinlist		int,
	@nextefforttime		datetime,
	@nresult		int
as

begin
	update A_TaskManager_SubTasks
	   set
		NextEffortTime 		= @nextefforttime,
		NResult			= @nresult
	   where		
		TaskId = @idtask
		and IdInList = @idinlist

	if ( @@rowcount = 0 )
		Insert Into A_TaskManager_SubTasks
			( IdInList, TaskId, NResult, IsDeleted, NextEffortTime )
		Values
			( @idinlist, @idtask, @nresult, 0, @nextefforttime )
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_TaskOut_UpdateResult_Number' ) )
drop procedure dbo.A_TaskManager_TaskOut_UpdateResult_Number
GO

CREATE procedure [dbo].A_TaskManager_TaskOut_UpdateResult_Number
	@idtask 		uniqueidentifier,
	@idinlist		int,
	@number			nvarchar ( 20 ),
	@countbusy		int,
	@countnotrespond 	int,
	@countqhangoff		int,
	@qtime			int,
	@nextefforttime 	DateTime,
	@nresult		int
as
begin

	update A_TaskManager_SubTaskNumber
	   set
		CountBusy		= @countbusy,
		CountNotRespond		= @countnotrespond,
		CountQHangOff		= @countqhangoff,
		QTime			= @qtime,
		NextEffortTime 		= @nextefforttime,
		NResult			= @nresult
	   where		
		TaskId = @idtask
		and IdInList = @idinlist
		and Number = @number

	if ( @@rowcount = 0 )
		Insert Into A_TaskManager_SubTaskNumber 
			( IdInList, TaskId, Number, CountBusy, CountNotRespond, CountQHangOff, QTime, NResult, NextEffortTime )
		Values
			( @idinlist, @idtask, @number, @countbusy, @countnotrespond, @countqhangoff, @qtime, @nresult, @nextefforttime )	
end
GO
					


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_SubTask_CreateEffort' ) )
drop procedure dbo.A_TaskManager_Task_SubTask_CreateEffort
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_TaskOut_UpdateResult_Effort' ) )
drop procedure dbo.A_TaskManager_TaskOut_UpdateResult_Effort
GO


CREATE Procedure dbo.A_TaskManager_TaskOut_UpdateResult_Effort
	@id 			uniqueidentifier,
	@idinlist		int,
	@idtask			uniqueidentifier,
	@number			nvarchar ( 20 ),
	@edatetime		datetime,
	@eresult		int,
	@einfo			nvarchar(500),
	@idchain		uniqueidentifier
as

begin
	Insert Into A_TaskManager_SubTaskEfforts
		( Id, IdInList, TaskId, Number, EDateTime, EResult, EInfo, ConnectionChainId )
	Values	
		( @id, @idinlist, @idtask, @number, @edatetime, @eresult, @einfo, @idchain )
end

GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_TaskOut_UpdateResult_PointSimple' ) )
drop procedure dbo.A_TaskManager_TaskOut_UpdateResult_PointSimple		     
GO


CREATE Procedure dbo.A_TaskManager_TaskOut_UpdateResult_PointSimple
	@idtask			uniqueidentifier,	
	@idinlist		int,
	@nresult		int
as

begin
	update A_TaskManager_SubTasks
	set
		NResult			= @nresult
   	where		
		TaskId = @idtask
		and IdInList = @idinlist
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_TaskOut_UpdateResult_NumberSimple' ) )
drop procedure dbo.A_TaskManager_TaskOut_UpdateResult_NumberSimple
GO

CREATE procedure [dbo].A_TaskManager_TaskOut_UpdateResult_NumberSimple
	@idtask 		uniqueidentifier,
	@idinlist		int,
	@number			nvarchar ( 20 ),
	@nresult		int
as
begin

	update A_TaskManager_SubTaskNumber
	set
		NResult			= @nresult
	where		
		TaskId = @idtask
		and IdInList = @idinlist
		and Number = @number
end
GO
			



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_UserInTask' ) )
drop table dbo.A_TaskManager_UserInTask
GO

Create Table dbo.A_TaskManager_UserInTask
(
	Id uniqueidentifier Not Null Primary Key,
	UserId uniqueidentifier not null,
	TaskId uniqueidentifier not null,
	EffortId uniqueidentifier not null,
	TimeStart DateTime not null,
	TimeStopPrior DateTime,
	TimeStopWaitCall DateTime,
	TimeStopBackCall DateTime,
	TimeStopDialog DateTime,
	TimeStopPost DateTime
)

GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_User_StopHandling' ) )
drop procedure dbo.A_TaskManager_User_StopHandling
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_User_StopPriorHandling' ) )
drop procedure dbo.A_TaskManager_User_StopPriorHandling
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_User_StartHandling' ) )
drop procedure dbo.A_TaskManager_User_StartHandling
GO


CREATE procedure [dbo].[A_TaskManager_User_StartHandling ]
	@id			uniqueidentifier,
	@userid			uniqueidentifier,
	@taskid			uniqueidentifier,
	@effortid		uniqueidentifier,
	@start			datetime

as
begin

	Insert Into A_TaskManager_UserInTask
			( Id, UserId, TaskId, EffortId, TimeStart )
		values
			( @id, @userid, @taskid, @effortid, @start )
	
end
GO




If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_User_UpdateHandling' ) )
drop procedure dbo.A_TaskManager_User_UpdateHandling
GO

CREATE procedure [dbo].[A_TaskManager_User_UpdateHandling ]
	@id			uniqueidentifier,
	@stopprior		datetime = NULL,
	@stopwaitcall		datetime = NULL,
	@stopbackcall		datetime = NULL,
	@stopdialog		datetime = NULL,
	@stoppost		datetime = NULL

as
begin

	Update A_TaskManager_UserInTask
	Set 	
		TimeStopPrior = IsNull ( TimeStopPrior, @stopprior ),
		TimeStopWaitCall = IsNull ( TimeStopWaitCall, @stopwaitcall ),
		TimeStopBackCall = IsNull ( TimeStopBackCall, @stopbackcall ),
		TimeStopDialog = IsNull ( TimeStopDialog, @stopdialog ),
		TimeStopPost = IsNull ( TimeStopPost, @stoppost )
	Where Id = @id
end
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_User_StopHandlingEffort' ) )
drop procedure dbo.A_TaskManager_User_StopHandlingEffort
GO


CREATE procedure [dbo].[A_TaskManager_User_StopHandlingEffort ]
	@effortid		uniqueidentifier,
	@stop			datetime

as
begin

	Update A_TaskManager_UserInTask
	Set TimeStopPost = @stop
	Where EffortId = @effortid
		and TimeStopPost is null

	Update A_TaskManager_UserInTask
	Set TimeStopDialog = @stop
	Where EffortId = @effortid
		and TimeStopDialog is null

	Update A_TaskManager_UserInTask
	Set TimeStopBackCall = @stop
	Where EffortId = @effortid
		and TimeStopBackCall is null

	Update A_TaskManager_UserInTask
	Set TimeStopWaitCall = @stop
	Where EffortId = @effortid
		and TimeStopWaitCall is null

	Update A_TaskManager_UserInTask
	Set TimeStopPrior = @stop
	Where EffortId = @effortid
		and TimeStopPrior is null
	
end

GO


if exists ( Select * From sysobjects where id = object_id ( 'dbo.C_UserParams' ) )
if not Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'C_UserParams' 
								and TABLE_SCHEMA like 'dbo'
								and COLUMN_NAME like 'IsCC' )
Alter Table C_UserParams
Add IsCC bit, IsLunch bit
GO

if exists ( Select * From sysobjects where id = object_id ( 'dbo.C_UserParams' ) )
Update C_UserParams
Set 	IsCC = 0, 
	IsLunch = 0
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_CallCenter_State_SetUserInCC' ) )
drop procedure dbo.A_CallCenter_State_SetUserInCC
GO

create procedure [dbo].[A_CallCenter_State_SetUserInCC]
	@iduser uniqueidentifier,
	@incc	bit
as
begin
	Update C_UserParams
	Set 	IsCC = @incc,
		IsLunch = 0
	Where IdUser = @iduser
end
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_GetSupervisors' ) )
drop procedure dbo.A_TaskManager_Task_GetSupervisors
GO

Create procedure A_TaskManager_Task_GetSupervisors
	@taskid uniqueidentifier
as
begin 
	Select SupervisorId 
	From A_Taskmanager_TaskSupervisors
	Where TaskId = @taskid
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_ClearExtLines' ) )
drop procedure dbo.A_TaskManager_Task_ClearExtLines
GO

Create procedure  A_TaskManager_Task_ClearExtLines
	@taskid uniqueidentifier
as
begin
	Delete From A_TaskManager_TaskExtLines
	Where Taskid = @taskid
end
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_ClearSupervisors' ) )
drop procedure dbo.A_TaskManager_Task_ClearSupervisors
GO

Create procedure  A_TaskManager_Task_ClearSupervisors
	@taskid uniqueidentifier
as
begin
	Delete From A_TaskManager_TaskSupervisors
	Where Taskid = @taskid
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_UpdateSubTaskPredResultInfo' ) )
drop procedure dbo.A_TaskManager_Task_UpdateSubTaskPredResultInfo
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_UpdateResultInfo' ) )
drop procedure dbo.A_TaskManager_Task_UpdateResultInfo
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_Update' ) )
drop procedure dbo.A_TaskManager_Task_Update
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_SubTask_GetEfforts' ) )
drop procedure dbo.A_TaskManager_Task_SubTask_GetEfforts
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_SubTask_ClearEfforts' ) )
drop procedure dbo.A_TaskManager_Task_SubTask_ClearEfforts
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_AddIncomingCall' ) )
drop procedure dbo.A_TaskManager_Task_AddIncomingCall
GO


Create Procedure dbo.A_TaskManager_Task_AddIncomingCall

	@id				uniqueidentifier,
	@taskid				uniqueidentifier,
	@number				nvarchar ( 50 ),
	@timestart			datetime,
	@timestop			datetime,
	@callresult			int,
	@callresultinfo			nvarchar ( 500 ),
	@connectionchainid		uniqueidentifier
as

begin
	
	if Exists ( select Id from A_TaskManager_Tasks where id = @taskid )
	begin
		Insert Into A_TaskManager_Task_IncomingCalls
			( Id, TaskId, Number, CallTimeStart, CallTimeStop, CallResult, CallInfo, ConnectionChainId )
		Values 
			( @id, @taskid, @number, @timestart, @timestop, @callresult, @callresultinfo, @connectionchainid )
	end
end
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_TaskAuto_List' ) )
drop table dbo.A_TaskManager_TaskAuto_List
GO


Create table dbo.A_TaskManager_TaskAuto_List
(
	Id	uniqueidentifier not null Primary key,
	Phone	nvarchar ( 20 ) not null
)
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_DB_FREEALL' ) )
drop procedure dbo.A_DB_FREEALL
GO


CREATE PROCEDURE [dbo].[A_DB_FREEALL] AS
begin
	Delete from dbo.A__Extra_OneLine_Settings
	Delete from dbo.A_ActualBase
	Delete from dbo.A_CallBackOrders
	Delete from dbo.A_CallBackRecievers
	Delete from dbo.A_CallCenter_Counters
	Delete from dbo.A_ClientContacts
	Delete from dbo.A_ClientContactTypes
	Delete from dbo.A_ClientGroups
	Delete from dbo.A_ClientHistory
	Delete from dbo.A_Clients
	Delete from dbo.A_Clients_Groups
	Delete from dbo.A_ExtNumbers
	Delete from dbo.A_Groups
	Delete from dbo.A_LineCodes
	Delete from dbo.A_LinesReactors
	Delete from dbo.A_MobileControlMaskDenied
	Delete from dbo.A_MobileControlMaskFree
	Delete from dbo.A_Monopoly
	Delete from dbo.A_NumberPlan
	Delete from dbo.A_NumberPlanAction
	Delete from dbo.A_Personal_ListInfo
	Delete from dbo.A_Redirect
	Delete from dbo.A_Redirect_Abonents
	Delete from dbo.A_Redirect_TimeTable
	Delete from dbo.A_Rules
	Delete from dbo.A_RuleRecords
	Delete from dbo.A_ServerExtLines
	Delete from dbo.A_ServerGSMLines
	Delete from dbo.A_ServerIntLines
	Delete from dbo.A_ServerNumbers
	Delete from dbo.A_ServerUSBLines
	Delete from dbo.A_Settings
	Delete from dbo.A_Streams
	Delete from dbo.A_User_NotifyLog
	Delete from dbo.A_UserControlLines
	Delete from dbo.A_UserControls
	Delete from dbo.A_UserOutcomingLines
	Delete from dbo.A_UserRights
	Delete from dbo.A_Users
	Delete from dbo.A_UserStateHistory
	Delete from dbo.A_VoiceMail
	Delete from dbo.A_WorkPlace_Address
	Delete from dbo.A_WorkPlaces

	Delete from dbo.A_TaskManager_Lists
	Delete from dbo.A_TaskManager_Operators
	Delete from dbo.A_TaskManager_Projects
	Delete from dbo.A_TaskManager_RunPeriods
	Delete from dbo.A_TaskManager_SubTaskEfforts
	Delete from dbo.A_TaskManager_SubTaskNumber
	Delete from dbo.A_TaskManager_SubTasks
	Delete from dbo.A_TaskManager_Task_IncomingCalls
	Delete from dbo.A_TaskManager_TaskAuto_List
	Delete from dbo.A_TaskManager_TaskExtLines
	Delete from dbo.A_TaskManager_Tasks
	Delete from dbo.A_TaskManager_TaskSupervisors
	Delete from dbo.A_TaskManager_UserInTask

	Delete from dbo.A_Stat_Connections_1x1
	Delete from dbo.A_Stat_Talk_Algorythms
	Delete from dbo.A_Stat_Talk_Card_Check
	Delete from dbo.A_Stat_Talk_Card_Radio
	Delete from dbo.A_Stat_Talk_Card_Text
	Delete from dbo.A_Stat_Talk_StopInfo
	
	Delete from dbo.C_GroupOperators
	Delete from dbo.C_Groups
	Delete from dbo.C_MonopolyForTaskChanging
	Delete from dbo.C_OperatorControlEvents
	Delete from dbo.C_OperatorTimeTable
	Delete from dbo.C_TaskControlEvents
	Delete from dbo.C_UserParams
	
	Delete from dbo.P_ListView
	Delete from dbo.T_LastCallPhoneNumber
	

	---------------------------------------------
	---   ------------------------
	---------------------------------------------
	DECLARE tcursor CURSOR FOR 
		Select name  FROM sysobjects 
		Where type = 'U' and name like 'A_TaskManager_LocalList%'
	declare @llname nvarchar ( 1000 )
	declare @sql nvarchar ( 500 )

	OPEN tcursor
	FETCH NEXT FROM tcursor INTO @llname

	WHILE @@FETCH_STATUS = 0
	BEGIN
		set @llname = 'dbo.[' + @llname + ']'
		set @sql = ' DROP TABLE ' + @llname
		exec sp_executesql @stmt =@sql
	
		FETCH NEXT FROM tcursor INTO @llname
	END

	CLOSE tcursor
	DEALLOCATE tcursor
	---------------------------------------------
	---------------------------------------------

end
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_List_JoinedTable_GetParams' ) )
drop procedure dbo.A_TaskManager_List_JoinedTable_GetParams
GO


Create procedure [dbo].[A_TaskManager_List_JoinedTable_GetParams]
	@idlist 	uniqueidentifier
as
begin
	
	Select JoinedTable, Query, ColumnId, ColumnsPhone 
	FROM A_TaskManager_Lists
	Where Id = @idlist
end
GO




If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_CallCenter_UserStateHistory' ) )
drop table dbo.A_CallCenter_UserStateHistory
GO

Create table dbo.[A_CallCenter_UserStateHistory]
(
	Id uniqueidentifier not null primary key,
	OperatorId uniqueidentifier not null,
	State int not null,
	TimeEnter datetime not null,
	TimeLeave datetime null,
	LengthSec float null,
	Info nvarchar ( 100 ) not null,
	SelfInfo bit not null	
)
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_CallCenter_UserStateChange' ) )
drop procedure dbo.A_CallCenter_UserStateChange
GO


CREATE  procedure dbo.[A_CallCenter_UserStateChange]
	@start bit,
	@idoperator uniqueidentifier,
	@state int,
	@info nvarchar ( 100 ) = null,
	@selfinfo bit = null
as
begin
	update A_CallCenter_UserStateHistory
	Set TimeLeave = GetDate ( ),
		LengthSec = Cast ( (GetDate() - TimeEnter) as float ) * 86400
	Where OperatorId = @idoperator 
		and State = @state
		and TimeLeave is null

	if ( @start > 0 )
	  begin
		if ( @state in ( 2, 3, 4 ) )
			update A_CallCenter_UserStateHistory
			Set 	TimeLeave = GetDate ( ),
				LengthSec = Cast ( (GetDate() - TimeEnter) as float ) * 86400
			Where OperatorId = @idoperator 
				and State in ( 2, 3, 4 )
				and TimeLeave is null

		Insert into A_CallCenter_UserStateHistory 
		Values ( NewId ( ), @idoperator, @state, GetDate ( ), null, null, @info, @selfinfo )

		if ( @state = 1 )
			Insert into A_CallCenter_UserStateHistory 
			Values ( NewId ( ), @idoperator, 2, GetDate ( ), null, null, '', 0 )
	  end
	else
	  begin
		if ( @state = 1 )
			update A_CallCenter_UserStateHistory
			Set 	TimeLeave = GetDate ( ),
				LengthSec = Cast ( (GetDate()- TimeEnter) as float ) * 86400
			Where OperatorId = @idoperator 
				and TimeLeave is null
	  end
end
GO




If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_CallCenter_State_SetUserInLunch' ) )
drop procedure dbo.A_CallCenter_State_SetUserInLunch
GO

create procedure [dbo].[A_CallCenter_State_SetUserInLunch]
	@iduser uniqueidentifier,
	@inlunch bit
as
begin
	Update C_UserParams
	Set IsLunch = @inlunch
	Where IdUser = @iduser
end
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UsersPermissionsLines' ) )
drop table dbo.A_UsersPermissionsLines
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UsersPermissions' ) )
drop table dbo.A_UsersPermissions
GO









-- 
If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Users_GetAllUsersWithoutOperators' ) )
drop procedure dbo.A_Users_GetAllUsersWithoutOperators
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.C_UserParams' ) )
drop table dbo.C_UserParams
GO

exec A_DB_ClearConstraints 'A_UserParams'

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UserParams' ) )
drop table dbo.A_UserParams
GO


Create Table dbo.[A_UserParams]
(
	IdUser uniqueidentifier not null primary key,
	IdRole uniqueidentifier not null,
	IsOperator bit not null,
	CallerId nvarchar ( 20 ) not null,
	IsRedirect bit not null,
	IsCC bit not null,
	IsLunch bit not null	
)
GO


ALTER TABLE dbo.A_UserParams ADD CONSTRAINT
	FK_A_UserParams_A_Users FOREIGN KEY
	(
	IdUser
	) REFERENCES dbo.A_Users
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UserRoles' ) )
drop table dbo.A_UserRoles
GO

Create Table dbo.[A_UserRoles]
(
	Id uniqueidentifier not null primary key,
	Name nvarchar ( 100 ) not null,
	CreationType int not null
)
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UserRolesExt' ) )
drop table dbo.A_UserRolesExt
GO

Create Table dbo.A_UserRolesExt
(
	IdRole 	uniqueidentifier not null,
	rKey int not null,
	rValue bit not null
)
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_UserGroup_Role_GetAll' ) )
drop procedure dbo.A_Admin_UserGroup_Role_GetAll
GO

create procedure [dbo].[A_Admin_UserGroup_Role_GetAll]
as
begin
	Select ur.Id, ur.Name, ur.CreationType, ure.rKey, ure.rValue 
	From A_UserRoles ur
		inner join A_UserRolesExt ure on ur.Id = ure.IdRole
end
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_UserGroup_Role_Save' ) )
drop procedure dbo.A_Admin_UserGroup_Role_Save
GO

create procedure [dbo].[A_Admin_UserGroup_Role_Save]
	@id uniqueidentifier,
	@name nvarchar ( 100 ),
	@creationtype int
as
begin
	Update A_UserRoles
	Set 	Name = @name,
		CreationType = @creationtype
	Where 	Id = @id

	if ( @@rowcount = 0 )
		Insert Into A_UserRoles
		Values ( @id, @name, @creationtype )

	Delete 
	From A_UserRolesExt
	Where IdRole = @id
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_UserGroup_RoleExt_Save' ) )
drop procedure dbo.A_Admin_UserGroup_RoleExt_Save
GO

create procedure [dbo].[A_Admin_UserGroup_RoleExt_Save]
	@roleid uniqueidentifier,
	@rkey int,
	@rval bit
as
begin
	Insert into A_UserRolesExt
	Values ( @roleid, @rkey, @rval )
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_UserGroup_User_GetAll' ) )
drop procedure dbo.A_Admin_UserGroup_User_GetAll
GO

CREATE Procedure [dbo].[A_Admin_UserGroup_User_GetAll]
As
Begin
	Select 	u.Id, 
		u.Name, 
		Info, 
		ParentGroupId, 
		Login, 
		Password, 
		Type, 
		isnull ( IsOperator, 0 ) as IsOperator,
		isnull ( up.IdRole, '00000000-0000-0000-0000-000000000000' ) as RoleId,
		isnull ( up.CallerId, '' ) as CallerId
	From A_Users u
		left join A_UserParams up on u.Id = up.IdUser
End
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_UserGroup_User_Save' ) )
drop procedure dbo.A_Admin_UserGroup_User_Save
GO



CREATE procedure [dbo].[A_Admin_UserGroup_User_Save]
	@id		uniqueidentifier,
	@name		nvarchar ( 200 ),
	@info		nvarchar ( 1000 ),
	@parentgroupid	uniqueidentifier,
	@login		nvarchar ( 50 ),
	@password	nvarchar ( 50 ),	
	@type		int,
	@isoperator	bit,
	@roleid		uniqueidentifier,
	@callerid	nvarchar ( 20 )

as
begin

	Update A_Users
	Set 	
		Name 		= @name,
		Info 		= @info,
		ParentGroupId 	= @parentgroupid,
		Login		= @login,
		Password 	= @password,
		Type		= @type
	Where 
		Id 		= @id
	
	if ( @@rowcount = 0 )
	begin
		Insert 
		Into A_Users 
			( Id, Name, Info, ParentGroupId, Login, Password, Type, Enabled )
		Values
			( @id, @name, @info, @parentgroupid, @login, @password, @type, 1 )
	end

	Update A_UserParams
	Set	IdRole = @roleid,
		CallerId = @callerid,
		IsOperator = @isoperator
	Where IdUser = @id

	if ( @@rowcount = 0 )
		Insert 
		Into A_UserParams ( IdUser, IdRole, IsOperator, CallerId, IsRedirect, IsCC, IsLunch  )
		Values	( @id, @roleid, @isoperator, @callerid, 0, 0, 0 )



	if ( @isoperator = 0 )
	  begin
		Delete 
		From C_GroupOperators
		Where IDoperator = @id
	  end


	exec A_Admin_UserGroup_Group_SiftEmpty
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_CallCenter_State_SetUserInCC' ) )
drop procedure dbo.A_CallCenter_State_SetUserInCC
GO

create procedure [dbo].[A_CallCenter_State_SetUserInCC]
	@iduser uniqueidentifier,
	@incc	bit
as
begin
	Update A_UserParams
	Set 	IsCC = @incc,
		IsLunch = 0
	Where IdUser = @iduser
end
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_CallCenter_State_SetUserInLunch' ) )
drop procedure dbo.A_CallCenter_State_SetUserInLunch
GO

create procedure [dbo].[A_CallCenter_State_SetUserInLunch]
	@iduser uniqueidentifier,
	@inlunch bit
as
begin
	Update A_UserParams
	Set IsLunch = @inlunch
	Where IdUser = @iduser
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Redirect_SetUserState' ) )
drop procedure dbo.A_Redirect_SetUserState
GO


CREATE procedure [dbo].[A_Redirect_SetUserState]
	@userid uniqueidentifier,
	@isset bit
as
begin
	Update A_UserParams
	Set IsRedirect = @isset
	Where IdUser = @userid
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.C_GroupsOperators_GetAll' ) )
drop procedure dbo.C_GroupsOperators_GetAll
GO


CREATE Procedure [dbo].[C_GroupsOperators_GetAll]
As
begin
	Select u.id as uid, u.name as uname, g.id as gid, g.caption as gcaption
	FROM 	
		A_Users u inner join 
		A_UserParams up on u.id = up.iduser LEFT join
		C_GroupOperators go on go.IdOperator = up.IdUser LEFT join
		C_Groups g on g.id = go.IdGroup
		where up.IsOperator > 0 
	Union All
	Select null, null, g.id as gid, g.caption as gcaption
		From C_Groups g where not g.id in ( Select idgroup from C_GroupOperators )

end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_DB_FREEALL' ) )
drop procedure dbo.A_DB_FREEALL
GO


CREATE PROCEDURE [dbo].[A_DB_FREEALL] AS
begin
	Delete from dbo.A__Extra_OneLine_Settings
	Delete from dbo.A_ActualBase
	Delete from dbo.A_CallBackOrders
	Delete from dbo.A_CallBackRecievers
	Delete from dbo.A_CallCenter_Counters
	Delete from dbo.A_CallCenter_UserStateHistory
	Delete from dbo.A_ClientContacts
	Delete from dbo.A_ClientContactTypes
	Delete from dbo.A_ClientGroups
	Delete from dbo.A_ClientHistory
	Delete from dbo.A_Clients
	Delete from dbo.A_Clients_Groups
	Delete from dbo.A_ExtNumbers
	Delete from dbo.A_Groups
	Delete from dbo.A_LineCodes
	Delete from dbo.A_LinesReactors
	Delete from dbo.A_MobileControlMaskDenied
	Delete from dbo.A_MobileControlMaskFree
	Delete from dbo.A_Monopoly
	Delete from dbo.A_NumberPlan
	Delete from dbo.A_NumberPlanAction
	Delete from dbo.A_Personal_ListInfo
	Delete from dbo.A_Redirect
	Delete from dbo.A_Redirect_Abonents
	Delete from dbo.A_Redirect_TimeTable
	Delete from dbo.A_Rules
	Delete from dbo.A_RuleRecords
	Delete from dbo.A_ServerExtLines
	Delete from dbo.A_ServerGSMLines
	Delete from dbo.A_ServerIntLines
	Delete from dbo.A_ServerNumbers
	Delete from dbo.A_ServerUSBLines
	Delete from dbo.A_Settings
	Delete from dbo.A_Streams
	Delete from dbo.A_User_NotifyLog
	Delete from dbo.A_UserControlLines
	Delete from dbo.A_UserControls
	Delete from dbo.A_UserOutcomingLines
	Delete from dbo.A_UserRoles
	Delete from dbo.A_UserRolesExt
	Delete from dbo.A_Users
	Delete from dbo.A_UserStateHistory
	Delete from dbo.A_VoiceMail
	Delete from dbo.A_WorkPlace_Address
	Delete from dbo.A_WorkPlaces

	Delete from dbo.A_TaskManager_Lists
	Delete from dbo.A_TaskManager_Operators
	Delete from dbo.A_TaskManager_Projects
	Delete from dbo.A_TaskManager_RunPeriods
	Delete from dbo.A_TaskManager_SubTaskEfforts
	Delete from dbo.A_TaskManager_SubTaskNumber
	Delete from dbo.A_TaskManager_SubTasks
	Delete from dbo.A_TaskManager_Task_IncomingCalls
	Delete from dbo.A_TaskManager_TaskAuto_List
	Delete from dbo.A_TaskManager_TaskExtLines
	Delete from dbo.A_TaskManager_Tasks
	Delete from dbo.A_TaskManager_TaskSupervisors
	Delete from dbo.A_TaskManager_UserInTask

	Delete from dbo.A_Stat_Connections_1x1
	Delete from dbo.A_Stat_Talk_Algorythms
	Delete from dbo.A_Stat_Talk_Card_Check
	Delete from dbo.A_Stat_Talk_Card_Radio
	Delete from dbo.A_Stat_Talk_Card_Text
	Delete from dbo.A_Stat_Talk_StopInfo
	
	Delete from dbo.C_GroupOperators
	Delete from dbo.C_Groups
	Delete from dbo.C_MonopolyForTaskChanging
	Delete from dbo.C_OperatorControlEvents
	Delete from dbo.C_OperatorTimeTable
	Delete from dbo.C_TaskControlEvents
	
	Delete from dbo.P_ListView
	Delete from dbo.T_LastCallPhoneNumber
	

	---------------------------------------------
	---   ------------------------
	---------------------------------------------
	DECLARE tcursor CURSOR FOR 
		Select name  FROM sysobjects 
		Where type = 'U' and name like 'A_TaskManager_LocalList%'
	declare @llname nvarchar ( 1000 )
	declare @sql nvarchar ( 500 )

	OPEN tcursor
	FETCH NEXT FROM tcursor INTO @llname

	WHILE @@FETCH_STATUS = 0
	BEGIN
		set @llname = 'dbo.[' + @llname + ']'
		set @sql = ' DROP TABLE ' + @llname
		exec sp_executesql @stmt =@sql
	
		FETCH NEXT FROM tcursor INTO @llname
	END

	CLOSE tcursor
	DEALLOCATE tcursor
	---------------------------------------------
	---------------------------------------------

end
GO




if not Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_Stat_Connections_1x1' 
								and TABLE_SCHEMA like 'dbo'
								and COLUMN_NAME like 'FromCC' )
Alter table A_Stat_Connections_1x1
Add FromCC bit 
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_AddIncomingCall' ) )
drop procedure dbo.A_TaskManager_Task_AddIncomingCall
GO

Create Procedure dbo.A_TaskManager_Task_AddIncomingCall

	@id				uniqueidentifier,
	@taskid				uniqueidentifier,
	@number				nvarchar ( 50 ),
	@timestart			datetime,
	@timestop			datetime,
	@callresult			int,
	@callresultinfo			nvarchar ( 500 ),
	@connectionchainid		uniqueidentifier
as

begin
	
	if Exists ( select Id from A_TaskManager_Tasks where id = @taskid )
	begin
		--   
		Insert Into A_TaskManager_Task_IncomingCalls
			( Id, TaskId, Number, CallTimeStart, CallTimeStop, CallResult, CallInfo, ConnectionChainId )
		Values 
			( @id, @taskid, @number, @timestart, @timestop, @callresult, @callresultinfo, @connectionchainid )

		--      
		Update A_Stat_Connections_1x1
		Set FromCC = 1 Where IdChain = @connectionchainid
	end
end
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_TaskManager_TaskOut_UpdateResult_Effort' ) )
drop procedure dbo.A_TaskManager_TaskOut_UpdateResult_Effort
GO


Create Procedure dbo.A_TaskManager_TaskOut_UpdateResult_Effort
	@id 			uniqueidentifier,
	@idinlist		int,
	@idtask			uniqueidentifier,
	@number			nvarchar ( 20 ),
	@edatetime		datetime,
	@eresult		int,
	@einfo			nvarchar(500),
	@idchain		uniqueidentifier
as

begin
	-- 
	Insert Into A_TaskManager_SubTaskEfforts
		( Id, IdInList, TaskId, Number, EDateTime, EResult, EInfo, ConnectionChainId )
	Values	
		( @id, @idinlist, @idtask, @number, @edatetime, @eresult, @einfo, @idchain )

	--      
	Update A_Stat_Connections_1x1
	Set FromCC = 1 Where IdChain = @idchain
end
GO





If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Stat_Service_FillTempByFilter' ) )
drop procedure dbo.A_Stat_Service_FillTempByFilter
GO

CREATE PROCEDURE [dbo].[A_Stat_Service_FillTempByFilter]
	@iduser uniqueidentifier,
	@dstart datetime,
	@dstop datetime,
	@tstart datetime,
	@tstop datetime,
	@lentime datetime,
	@taskid uniqueidentifier = NULL, --Task defined
	@fromcc bit = NULL,
	@operatorid uniqueidentifier = NULL, --Operator defined
	@innerprefix nvarchar(100) = NULL, --Internal Number defined
	@outnumber nvarchar(100) = NULL  --Out Number defined
AS
begin

--Prepare @A_Stat_Connections_1x1_tmp
Declare @A_Stat_Connections_1x1_tmp1 table( Id uniqueidentifier PRIMARY KEY) 
Declare @A_Stat_Connections_1x1_tmp table( Id uniqueidentifier PRIMARY KEY, AUSerId uniqueidentifier, BUSerId uniqueidentifier, ALineId uniqueidentifier, BLineId uniqueidentifier) 
declare @tstartfloat float  declare @tstopfloat float 
set @tstartfloat = Convert ( float, @tstart )  
set @tstopfloat  = Convert ( float, @tstop )  
set @tstartfloat = @tstartfloat - FLOOR ( @tstartfloat )  
set @tstopfloat = @tstopfloat - FLOOR ( @tstopfloat )  

declare @lt float
set @lt = Convert ( float, @lentime ) - Floor ( Convert ( float, @lentime ) )


--Date & Time Intervals
Insert into @A_Stat_Connections_1x1_tmp 
select Id, AUSerId , BUSerId , ALineId , BLineId 
from A_Stat_Connections_1x1 sc
where 
	 ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
	and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) >= @tstartfloat )  
	and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) <= @tstopfloat ) 
	and ( CONVERT ( float, TimeStop ) - Convert ( float, TimeAnswer ) >= @lt )
	and (( FromCC is null ) or ( FromCC <= @fromcc ))
	
	--and not exists(select id from  @A_Stat_Connections_1x1_tmp where id=sc.ID)


--Permissions
--Users
declare @upermitted table ( Id uniqueidentifier Primary Key )
Insert into @upermitted
	Select UserB
	From A_UserControls auc
	Where UserA = @iduser

if not exists (select UserB from  A_UserControls where UserA = @iduser and UserB = @iduser)
	Insert into @upermitted (Id) Values (@iduser)

Declare @commpermitted table ( Id uniqueidentifier PRIMARY KEY)  	 -- Id Connection
Insert into @commpermitted
	Select  sc.ID --auc.UserB   	
	From @upermitted up
		inner join @A_Stat_Connections_1x1_tmp sc on up.Id = sc.AUserId

Insert into @commpermitted
	Select  sc.ID --auc.UserB   	
	From @upermitted up
		inner join @A_Stat_Connections_1x1_tmp sc on up.Id=sc.BUserId
	Where not exists(select id from @commpermitted where Id=sc.ID)


--Lines
print '------Lines'
declare @lpermitted table ( Id uniqueidentifier Primary Key )
Insert into @lpermitted
     Select IdLine
     From A_UserControlLines
     Where UserA = @iduser
          and IdLine != '11111111-1111-1111-1111-111111111111'       

declare @canlistenext int       
Select @canlistenext = Count ( * ) 
From A_UserControlLines 
Where UserA = @iduser and IdLine = '11111111-1111-1111-1111-111111111111'

if ( @canlistenext > 0 )
     Insert into @lpermitted
          Select Id 
          FROM A_ServerExtLines sel
          Where not exists(select id from @lpermitted where Id = sel.ID)
     
Insert into @commpermitted
     Select sc.ID            
     From @lpermitted l
          inner join @A_Stat_Connections_1x1_tmp sc on sc.ALineId=l.Id
     Where not exists(select id from @commpermitted where Id = sc.ID)

Insert into @commpermitted            
     Select sc.ID            
     From @lpermitted l
          inner join @A_Stat_Connections_1x1_tmp sc on  sc.BLineId=l.Id
     Where not exists(select id from @commpermitted where Id = sc.ID)

print '------Lines'

delete @A_Stat_Connections_1x1_tmp 
	where id not in (select Id from @commpermitted )

--Numbers 
--Inner
if @innerprefix is not null begin
	declare @t table ( nid uniqueidentifier, prefix nvarchar ( 50 ), addrtype int, reactid uniqueidentifier )
	declare @temp table ( id uniqueidentifier, prefix nvarchar ( 50 ), addrtype int, oldrid uniqueidentifier, newrid uniqueidentifier )

	Insert Into @t
	Select np.Id, np.Prefix, rr.InnerAddressType, rr.ReactId
	From A_NumberPlan np
		Inner join A_NumberPlanAction na on na.NumId = np.Id
		Inner join A_Rules r on na.ExtraId = r.Id
		Inner join A_RuleRecords rr on rr.RuleId = r.Id

		declare @cnt int
		set @cnt = 0

		while ( @@rowcount > 0 ) and ( @cnt < 10 )
		begin
			Set @cnt = @cnt + 1

			/*    @cnt - */
			Insert Into @temp
				Select t.nid, t.prefix, t.addrtype, t.reactid, t1.reactid
				From @t t
					Inner join @t t1 on t.Reactid = t1.nId
				Where t.ReactId != t.Nid

			/*    */
			Delete From @t
				From @temp
			Where nid = id and reactid = oldrid

			/* */
			Insert Into @t ( nid, prefix, addrtype, reactid )
				Select id, prefix, addrtype, newrid 
				From @temp

			/* */
			Delete From @temp	
		end --\MakePrefixTableAnyWay condition

/*
		declare @tusers table ( id uniqueidentifier ) 
		insert into @tusers 
			Select reactid FROM @t where addrtype = 0 and prefix like @innerprefix

		declare @tlines table ( id uniqueidentifier ) 
		insert into @tlines 
			Select reactid FROM @t where addrtype = 1 and prefix like @innerprefix
*/
	
		
		

		--Users
		Insert into @A_Stat_Connections_1x1_tmp1 
		select Id
		from @A_Stat_Connections_1x1_tmp sc
			inner join @t t on sc.AUserID=t.reactid
		where addrtype = 0 and prefix like @innerprefix

		Insert into @A_Stat_Connections_1x1_tmp1 
		select Id
		from @A_Stat_Connections_1x1_tmp sc
			inner join @t t on sc.BUserID=t.reactid
		where addrtype = 0 and prefix like @innerprefix
			and not exists(select id from  @A_Stat_Connections_1x1_tmp1 where id=sc.ID)

		--Lines
		Insert into @A_Stat_Connections_1x1_tmp1 
		select Id
		from @A_Stat_Connections_1x1_tmp sc
			inner join @t t on sc.AUserID=t.reactid
		where addrtype = 1 and prefix like @innerprefix
			and not exists(select id from  @A_Stat_Connections_1x1_tmp1 where id=sc.ID)

		Insert into @A_Stat_Connections_1x1_tmp1 
		select Id
		from @A_Stat_Connections_1x1_tmp sc
			inner join @t t on sc.BUserID=t.reactid
		where addrtype = 1 and prefix like @innerprefix
			and not exists(select id from  @A_Stat_Connections_1x1_tmp1 where id=sc.ID)
		
		delete @A_Stat_Connections_1x1_tmp 
			where id not in (select Id from @A_Stat_Connections_1x1_tmp1 )
		delete @A_Stat_Connections_1x1_tmp1
		
end --\NUmbers

--Outer NUMBVERS
if @OutNumber is not null begin

		Insert into @A_Stat_Connections_1x1_tmp1 
		select Id
		from A_Stat_Connections_1x1 sc
		where  AOutNumber like '%'+@OutNumber+'%'

		Insert into @A_Stat_Connections_1x1_tmp1 
		select Id
		from A_Stat_Connections_1x1 sc
		where  BOutNumber like '%'+@OutNumber+'%'
			and not exists(select id from  @A_Stat_Connections_1x1_tmp1 where id=sc.ID)

		delete @A_Stat_Connections_1x1_tmp 
			where id not in (select Id from @A_Stat_Connections_1x1_tmp1 )
		delete @A_Stat_Connections_1x1_tmp1

end
--\Outer NUMBVERS

--operatorid
if @operatorid is not null begin

		Insert into @A_Stat_Connections_1x1_tmp1 
		select Id
		from @A_Stat_Connections_1x1_tmp sc
		where  AUserId=@operatorid

		Insert into @A_Stat_Connections_1x1_tmp1 
		select Id 
		from @A_Stat_Connections_1x1_tmp sc
		where  BUserId=@operatorid
			and not exists(select id from  @A_Stat_Connections_1x1_tmp1 where id=sc.ID)

		delete @A_Stat_Connections_1x1_tmp 
			where id not in (select Id from @A_Stat_Connections_1x1_tmp1 )
		delete @A_Stat_Connections_1x1_tmp1
end

--\operatorid

--@taskid
/*
if @taskid is not null begin
		Declare @tasks table ( TaskId uniqueidentifier, ConnectionChainId uniqueidentifier PRIMARY KEY (ConnectionChainId) )  
		Insert into @tasks  
		Select  TaskId, ConnectionChainId
		FROM A_TaskManager_Task_IncomingCalls  
		Where TaskId = @taskid  
		
		Insert into @tasks  
		Select distinct TaskId, ConnectionId   
		from A_TaskManager_SubTasks st   
			inner join A_TaskManager_SubTaskEfforts ste on ste.SubTaskId = st.Id  
		Where TaskId = @taskid  

		Insert into @A_Stat_Connections_1x1_tmp1 
			select Id 
			from A_Stat_Connections_1x1 sc
				inner join @tasks  t on sc.IdChain = t.ConnectionChainId

		delete @A_Stat_Connections_1x1_tmp 
			where id not in (select Id from @A_Stat_Connections_1x1_tmp1 )
		delete @A_Stat_Connections_1x1_tmp1
end
*/
--Select * FROM A_Stat_Connections_1x1 Where TimeStart > '17.12.2006' and (AUserId = 'DA803F01-EA77-40FA-BC9D-E2EFB36FD5A8' or BUserId = 'DA803F01-EA77-40FA-BC9D-E2EFB36FD5A8' )

Select sc.*--.Id, IdChain, IdPrev, IdNext, TimeStart, TimeAnswer, TimeStop, ReasonStart, ReasonStop, StopSide, sc.ALineId, sc.BLineId, AOutNumber, BOutNumber, ALineNum, BLineNum, sc.AUserId, sc.BUserId, ANumberDialed, BNumberDialed, IsRecorded, ARecFile, BRecFile, ConnectionType, Astr, Bstr, TimeStop - TimeAnswer as Length, TimeAnswer - TimeStart as AnswerLength 
FROM A_Stat_Connections_1x1 sc--,@ulpermitted ul,@tasks t
		Inner join @A_Stat_Connections_1x1_tmp sct on sc.Id=sct.Id
		--Inner Join @commpermitted cp on sc.ID=cp.ID

end
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Stat_Office_TimeScale' ) )
drop procedure dbo.A_Stat_Office_TimeScale
GO

CREATE procedure [dbo].[A_Stat_Office_TimeScale]
	@iduser		uniqueidentifier,
	@dstart 	DateTime, 
	@dstop	 	DateTime, 
	@tstart 	DateTime, 
	@tstop	 	DateTime,
	@lentime 	DateTime,
	@conntype 	int 			= -1, 
	@taskid		uniqueidentifier 	= null,
	@fromcc		bit			= null,
	@operatorid 	uniqueidentifier 	= null,
	@innerprefix 	nvarchar ( 20 )		= null, 
	@outnumber 	nvarchar ( 20 )		= null,

	@minvalue 	int 			output, 
	@maxvalue 	int 			output
as
begin
/*
----------------------------------------------------------
------- debug --------------------------------------------
----------------------------------------------------------
	declare @dstart 	DateTime 
	declare @dstop	 	DateTime 
	declare @tstart 	DateTime 
	declare @tstop	 	DateTime
	declare @lentime 	DateTime
	declare @conntype 	int 
	declare @taskid		uniqueidentifier
	declare @fromcc		bit
	declare @operatorid 	uniqueidentifier
	declare @innerprefix	nvarchar ( 20 )
	declare @outnumber	nvarchar ( 20 )
	declare @iduser		uniqueidentifier
	declare @minvalue 	int
	declare @maxvalue 	int
	
	set dateformat dmy
	set @dstart = '25.12.2006' 
	set @dstop = '28.12.2006' 
	set @tstart = '8:00'
	set @tstop = '18:00' 
	set @lentime = '0:00:01' 
	set @conntype = -1
	set @taskid = N'00000000-0000-0000-0000-000000000000'
	set @operatorid = N'00000000-0000-0000-0000-000000000000'
	set @innerprefix = N'' 
	set @outnumber = N''
	set @iduser = 'DA803F01-EA77-40FA-BC9D-E2EFB36FD5A8'
	set @fromcc = 1
----------------------------------------------------------
------- end debug ----------------------------------------
----------------------------------------------------------
*/

	declare @tstartfloat float  declare @tstopfloat float 
	set @tstartfloat = Convert ( float, @tstart )  
	set @tstopfloat  = Convert ( float, @tstop )  
	set @tstartfloat = @tstartfloat - FLOOR ( @tstartfloat )  
	set @tstopfloat = @tstopfloat - FLOOR ( @tstopfloat )  


	if ( @taskid = '00000000-0000-0000-0000-000000000000' )
		set @taskid = null

	if ( @operatorid = '00000000-0000-0000-0000-000000000000' )
		set @operatorid = null

	if ( @innerprefix = '' )
		set @innerprefix = null

	if ( @outnumber = '' )
		set @outnumber = null;

	Select top 0 * into #t from A_Stat_Connections_1x1
	Insert into #t Exec A_Stat_Service_FillTempByFilter
		@iduser,
		@dstart,
		@dstop,
		'0:00',
		'23:59:59',
		@lentime,
		@taskid,
		@fromcc,
		@operatorid,
		@innerprefix,
		@outnumber
	



	Declare @min_ int, @max_ int;
	Declare @emptyguid uniqueidentifier
	Set @emptyguid = '00000000-0000-0000-0000-0000000000000000'



	Select @minvalue = isnull ( min ( c ), 0 ), @maxvalue = isnull ( max ( c ), 0 ) from
	(
		Select count ( * ) c 
		From #t
		Where (( @conntype = -1 ) or ( @conntype = 1 and connectiontype in (4,5) ) or (@conntype = 2 and connectiontype in (1,6,7)) or (@conntype = 3 and connectiontype in (2,3,8)))
		Group By convert ( varchar(10), timestart, 104 ), datepart ( hour, timestart )
	) a;


	declare @empty table ( cdate varchar ( 10 ), c0 int, c1 int, c2 int, c3 int, c4 int, c5 int, c6 int, c7 int, c8 int, c9 int, c10 int, 
		c11 int, c12 int, c13 int, c14 int, c15 int, c16 int, c17 int, c18 int, c19 int, c20 int, c21 int, c22 int, c23 int, cbottom int, ctop int )
	declare @tempdate datetime 
	set @tempdate = @dstart
	while @tempdate < @dstop
	begin
		Insert into @empty ( cdate, c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10, c11, c12, c13, c14, c15, c16, c17, c18, c19, c20, c21, c22, c23, cbottom, ctop )
		Values ( convert ( varchar(10), @tempdate, 126 ), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 )
			
		set @tempdate = DateAdd ( d, 1, @tempdate )
	end
	

	declare @ttt table ( cdate varchar ( 10 ), c0 int, c1 int, c2 int, c3 int, c4 int, c5 int, c6 int, c7 int, c8 int, c9 int, c10 int, 
		c11 int, c12 int, c13 int, c14 int, c15 int, c16 int, c17 int, c18 int, c19 int, c20 int, c21 int, c22 int, c23 int, cbottom int, ctop int )

	insert into @ttt

	Select 	cdate,
     		sum ( c0 ) as c0, 
    	 	sum ( c1 ) as c1,
     		sum ( c2 ) as c2,
	     	sum ( c3 ) as c3,
     		sum ( c4 ) as c4,
     		sum ( c5 ) as c5, 
     		sum ( c6 ) as c6,
     		sum ( c7 ) as c7,
     		sum ( c8 ) as c8,
     		sum ( c9 ) as c9,
     		sum ( c10 ) as c10, 
     		sum ( c11 ) as c11, 
     		sum ( c12 ) as c12,
     		sum ( c13 ) as c13,
     		sum ( c14 ) as c14,
     		sum ( c15 ) as c15,
     		sum ( c16 ) as c16,
     		sum ( c17 ) as c17,
     		sum ( c18 ) as c18,
	     	sum ( c19 ) as c19,
	     	sum ( c20 ) as c20,
	     	sum ( c21 ) as c21,
	     	sum ( c22 ) as c22,  
	     	sum ( c23 ) as c23,
	     	sum ( cbefore ) as cbottom,
	     	sum ( cafter ) as ctop  

	From 

	(

     		Select  
			convert ( varchar(10), timestart, 126 ) cdate,
          		case ( DatePart ( hour, timestart ) ) when 0 then 1 else 0 end as c0,
         		case ( DatePart ( hour, timestart ) ) when 1 then 1 else 0 end as c1,
          		case ( DatePart ( hour, timestart ) ) when 2 then 1 else 0 end as c2,
          		case ( DatePart ( hour, timestart ) ) when 3 then 1 else 0 end as c3,
          		case ( DatePart ( hour, timestart ) ) when 4 then 1 else 0 end as c4,
          		case ( DatePart ( hour, timestart ) ) when 5 then 1 else 0 end as c5,
          		case ( DatePart ( hour, timestart ) ) when 6 then 1 else 0 end as c6,
          		case ( DatePart ( hour, timestart ) ) when 7 then 1 else 0 end as c7,
          		case ( DatePart ( hour, timestart ) ) when 8 then 1 else 0 end as c8,
          		case ( DatePart ( hour, timestart ) ) when 9 then 1 else 0 end as c9,
          		case ( DatePart ( hour, timestart ) ) when 10 then 1 else 0 end as c10,
          		case ( DatePart ( hour, timestart ) ) when 11 then 1 else 0 end as c11,
          		case ( DatePart ( hour, timestart ) ) when 12 then 1 else 0 end as c12,
          		case ( DatePart ( hour, timestart ) ) when 13 then 1 else 0 end as c13,
	          	case ( DatePart ( hour, timestart ) ) when 14 then 1 else 0 end as c14,
	          	case ( DatePart ( hour, timestart ) ) when 15 then 1 else 0 end as c15,
	          	case ( DatePart ( hour, timestart ) ) when 16 then 1 else 0 end as c16,
	          	case ( DatePart ( hour, timestart ) ) when 17 then 1 else 0 end as c17,
	          	case ( DatePart ( hour, timestart ) ) when 18 then 1 else 0 end as c18,
	          	case ( DatePart ( hour, timestart ) ) when 19 then 1 else 0 end as c19,
	          	case ( DatePart ( hour, timestart ) ) when 20 then 1 else 0 end as c20,
	          	case ( DatePart ( hour, timestart ) ) when 21 then 1 else 0 end as c21,
	          	case ( DatePart ( hour, timestart ) ) when 22 then 1 else 0 end as c22,
	          	case ( DatePart ( hour, timestart ) ) when 23 then 1 else 0 end as c23,			
			case when Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) < @tstartfloat then 1 else 0 end as cbefore,
			case when Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) > @tstopfloat then 1 else 0 end as cafter


     		From 
		(
			Select Id, TimeStart		
			From #t
			Where ((@conntype = -1) or (@conntype = 1 and connectiontype in (4,5)) or (@conntype = 2 and connectiontype in (1,6,7)) or ( @conntype = 3 and connectiontype in (2,3,8)))
	          	Group By Id, TimeStart
		)b

		Union all

		Select *
		FROM @empty
	)a

	Group by cdate
	Order by cdate

	
	declare @mxv1 int
	declare @mxv2 int
	Select @mxv1 = Max ( ctop ) From @ttt
	Select @mxv2 = Max ( cbottom ) From @ttt
	if ( @maxvalue < @mxv1 )
		Set @maxvalue = @mxv1
	if ( @maxvalue < @mxv2 )
		Set @maxvalue = @mxv2

	Select * 
	From @ttt
end
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Stat_Office_GetJournal' ) )
drop procedure dbo.A_Stat_Office_GetJournal
GO

CREATE procedure [dbo].[A_Stat_Office_GetJournal]
	@iduser uniqueidentifier,
	@dstart datetime,
	@dstop datetime,
	@tstart datetime,
	@tstop datetime,
	@lentime datetime,
	@taskid uniqueidentifier = NULL, --Task defined
	@fromcc bit = NULL,
	@operatorid uniqueidentifier = NULL, --Operator defined
	@innerprefix nvarchar(100) = NULL, --Internal Number defined
	@outnumber nvarchar(100) = NULL  --Out Number defined
as
begin	
	Select top 0 * into #t from A_Stat_Connections_1x1
	Insert into #t Exec A_Stat_Service_FillTempByFilter
		@iduser,
		@dstart,
		@dstop,
		@tstart,
		@tstop,
		@lentime,
		@taskid,
		@fromcc,
		@operatorid,
		@innerprefix,
		@outnumber

	Select top 1000 t.*, TimeStop - TimeAnswer as Length, TimeStart - TimeStart as AnswerLength
	From #t t
	Order By TimeStart Desc
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Stat_Office_Diagramm_AllCalls' ) )
drop procedure dbo.A_Stat_Office_Diagramm_AllCalls
GO


CREATE procedure [dbo].[A_Stat_Office_Diagramm_AllCalls]
	@iduser uniqueidentifier,
	@dstart datetime,
	@dstop datetime,
	@tstart datetime,
	@tstop datetime,
	@lentime datetime,
	@taskid uniqueidentifier = null,
	@fromcc bit = null,
	@operatorid uniqueidentifier = null,
	@innerprefix nvarchar ( 100 ) = null,
	@outnumber nvarchar ( 100 ) = null,
	@mode int
as

begin

/*
-- ---------------------------------------------------
	declare @iduser uniqueidentifier
	declare @dstart datetime
	declare @dstop datetime
	declare @tstart datetime
	declare @tstop datetime
	declare @lentime datetime
	declare @taskid uniqueidentifier
	declare @operatorid uniqueidentifier
	declare @innerprefix nvarchar ( 100 )
	declare @outnumber nvarchar ( 100 )
	declare @mode int

	select @iduser = 'DA803F01-EA77-40FA-BC9D-E2EFB36FD5A8', 
		@dstart = '15.06.2006', 
		@dstop = '20.12.2006', 
		@tstart = '8:00', 
		@tstop = '20:00', 
		@lentime = '0:00:01',
		@mode = 4
--------------------------------------------------------------------
*/
		

	if not exists ( select * from tempdb..sysobjects where id = object_id('tempdb..#tempstatistics') )
		select top 0 * into #tempstatistics From A_Stat_Connections_1x1
	else
		delete from #tempstatistics
		
	Insert into #tempstatistics Exec A_Stat_Service_FillTempByFilter
		@iduser,
		@dstart,
		@dstop,
		@tstart,
		@tstop,
		@lentime,
		@taskid,
		@fromcc,
		@operatorid,
		@innerprefix,
		@outnumber

	
	--EStatisticsViewMode.svmByDay:
	if ( @mode = 0 )
 		begin						
			declare @k0 table ( idchain uniqueidentifier, timestop datetime )
			Insert into @k0
				Select IDChain, Max(TimeStop) as TimeStop
				From #tempstatistics							
				Group by IDChain

			Select Year ( timestop ) y, Month ( timestop ) m, Day ( timestop ) d, count ( * ) as Count_
			From @k0
			Group by Year ( timestop ), Month ( timestop ), Day ( timestop )							
		end

	--EStatisticsViewMode.svmByHour:
	else if ( @mode = 1 )
		begin
			declare @k1 table ( idchain uniqueidentifier, timestop datetime )
			Insert into @k1
				Select IDChain, Max(TimeStop) as TimeStop
				From #tempstatistics
				Group by IDChain

			Select DatePart ( hour, timestop ) h, count ( * ) as Count_
			From @k1
			Group by DatePart ( hour, timestop )
		end

	--EStatisticsViewMode.svmByOperator:
	else if ( @mode = 2 )
		begin							
			declare @lines2 table ( lineid uniqueidentifier )
			insert into @lines2
				Select ID from A_ServerIntLines
				union 
				Select ID from A_ServerUSBLines
				union 
				Select ID from A_ServerExtLines
				union 
				Select ID from A_ServerGSMLines
	

			Select uid, sum ( count_ ) count_ from
			(
				select AUserId as uid, count ( * ) as count_
				from #tempstatistics ts
					inner join A_Users u on u.Id = ts.AUserId
				Group by AUserId
			
				union all
			
				select BUserId as uid, count ( * ) as count_
				from #tempstatistics ts
					inner join A_Users u on u.Id = ts.BUserId
				Group By BUserId

				union

				select AUserId as uid, count ( * ) as count_
				from #tempstatistics ts
					inner join @lines2 l on l.lineid = ts.BLineId
				Where ( ALineNum like 'IVR' )
				Group by AUserId

				union all
					
				select BUserId as uid, count ( * ) as count_
				from #tempstatistics ts
					inner join @lines2 l on l.lineid = ts.ALineId
				Where ( BLineNum like 'IVR' )
				Group By BUserId
	
			)a
			group by uid
		end

	--EStatisticsViewMode.svmByLine:
	else if ( @mode = 3 )
		begin														
			declare @lines3 table ( lineid uniqueidentifier )
			insert into @lines3
				Select ID from A_ServerIntLines
				union 
				Select ID from A_ServerUSBLines
				union
				Select ID from A_ServerExtLines
				union
				Select ID from A_ServerGSMLines


			Select linenum, sum ( count_ ) count_ from
			(
				select ALineNum as linenum, count ( * ) as count_
				from #tempstatistics ts
					inner join @lines3 l on l.lineid = ts.ALineId	
				Group by ALineNum
			
				union all

				select BLineNum as linenum, count ( * ) as count_
				from #tempstatistics ts
					inner join @lines3 l on l.lineid = ts.BLineId
				Group By BLineNum

				union all

				select ALineNum as linenum, count ( * ) as count_
				from #tempstatistics ts 
					inner join @lines3 l on l.lineid = ts.BLineId
				where ALineNum like 'IVR' 
				Group by ALineNum

				union all

				select BLineNum as linenum, count ( * ) as count_
				from #tempstatistics ts
					inner join @lines3 l on l.lineid = ts.ALineId
				where BLineNum like 'IVR'
				Group by BLineNum
			)a
			group by linenum
		end

	--EStatisticsViewMode.svmByNumber:						
	else if ( @mode = 4 )
	begin
		declare @t table ( nid uniqueidentifier, prefix nvarchar ( 50 ), addrtype int, reactid uniqueidentifier )
		declare @temp table ( id uniqueidentifier, prefix nvarchar ( 50 ), addrtype int, oldrid uniqueidentifier, newrid uniqueidentifier )
				
		Insert Into @t
			Select np.Id, np.Prefix, rr.InnerAddressType, rr.ReactId
			From A_NumberPlan np
				Inner join A_NumberPlanAction na on na.NumId = np.Id
				Inner join A_Rules r on na.ExtraId = r.Id
				Inner join A_RuleRecords rr on rr.RuleId = r.Id


		declare @cnt int
		set @cnt = 0

		while ( @@rowcount > 0 ) and ( @cnt < 10 )
		begin
			Set @cnt = @cnt + 1
	
			/*    @cnt - */
			Insert Into @temp
				Select t.nid, t.prefix, t.addrtype, t.reactid, t1.reactid
				From @t t
					Inner join @t t1 on t.Reactid = t1.nId
				Where t.ReactId != t.Nid
	
			/*    */
			Delete From @t
				From @temp
			Where nid = id and reactid = oldrid

			/* */
			Insert Into @t ( nid, prefix, addrtype, reactid )
				Select id, prefix, addrtype, newrid From @temp

			/* */
			Delete From @temp	
		end

		declare @lines4 table ( lineid uniqueidentifier )
		insert into @lines4
			Select ID from A_ServerIntLines
			union 
			Select ID from A_ServerUSBLines

		Select prefix, sum ( count_ ) count_ from
		(
			Select t1.prefix, 0 as InnerAddrType, count ( * ) as count_
			From #tempstatistics t 
				inner join @t t1 on t1.reactid = t.BUserId and t1.addrtype = 0 
				inner join A_Users u on u.id = AUserId
			Group by prefix

			union all

			Select t1.prefix, 0 as InnerAddrType, count ( * ) as count_
			From #tempstatistics t 
				inner join @t t1 on t1.reactid = t.BUserId and t1.addrtype = 0 
				inner join A_Users u on u.id = BUserId
			Group by prefix

			union all

			Select t1.prefix, 1 as InnerAddrType, count ( * ) as count_
			From #tempstatistics t 
				inner join @t t1 on t1.reactid = t.ALineId and t1.addrtype = 1 
				inner join @lines4 l on l.lineid = ALineId
			Group by prefix

			union all
		
			Select t1.prefix, 1 as InnerAddrType, count ( * ) as count_
			From #tempstatistics t 
				inner join @t t1 on t1.reactid = t.BLineId and t1.addrtype = 1 
				inner join @lines4 l on l.lineid = BLineId
			Group by prefix
		)a
		Group by prefix
	end	

	delete from #tempstatistics

end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Stat_Office_Diagramm_AverageCallTime' ) )
drop procedure dbo.A_Stat_Office_Diagramm_AverageCallTime
GO


CREATE procedure [dbo].[A_Stat_Office_Diagramm_AverageCallTime]
	@iduser uniqueidentifier,
	@dstart datetime,
	@dstop datetime,
	@tstart datetime,
	@tstop datetime,
	@lentime datetime,
	@taskid uniqueidentifier = null,
	@fromcc bit = null,
	@operatorid uniqueidentifier = null,
	@innerprefix nvarchar ( 100 ) = null,
	@outnumber nvarchar ( 100 ) = null,
	@mode int
as

begin

/*
-- ---------------------------------------------------
	declare @iduser uniqueidentifier
	declare @dstart datetime
	declare @dstop datetime
	declare @tstart datetime
	declare @tstop datetime
	declare @lentime datetime
	declare @taskid uniqueidentifier
	declare @operatorid uniqueidentifier
	declare @innerprefix nvarchar ( 100 )
	declare @outnumber nvarchar ( 100 )
	declare @mode int

	select @iduser = 'DA803F01-EA77-40FA-BC9D-E2EFB36FD5A8', 
		@dstart = '15.06.2006', 
		@dstop = '20.12.2006', 
		@tstart = '8:00', 
		@tstop = '20:00', 
		@lentime = '0:00:01',
		@mode = 2
--------------------------------------------------------------------
*/

	if not exists ( select * from tempdb..sysobjects where id = object_id('tempdb..#tempstatistics') )
		select top 0 * into #tempstatistics From A_Stat_Connections_1x1
	else
		delete from #tempstatistics
		
	Insert into #tempstatistics 
	Exec A_Stat_Service_FillTempByFilter
		@iduser,
		@dstart,
		@dstop,
		@tstart,
		@tstop,
		@lentime,
		@taskid,
		@fromcc,
		@operatorid,
		@innerprefix,
		@outnumber


	--EStatisticsViewMode.svmByDay:
	if ( @mode = 0 )
 		begin						
			if not exists ( select * from tempdb..sysobjects where id = object_id('tempdb..#tempstatisticsYMDdateInt') )
				select top 0 Year( Timestart ) y, Month ( TimeStart ) m, Day ( TimeStart ) D, TimeStop - TimeAnswer length, 1 as count_
				into #tempstatisticsYMDdateInt
				From A_Stat_Connections_1x1
			else
				delete from #tempstatisticsYMDdateInt

			insert into #tempstatisticsYMDdateInt
				Select Year( Timestart ) y, Month ( TimeStart ) m, Day ( TimeStart ) D, Cast ( sum ( CAST ( TimeStop - TimeAnswer as float ) ) as DateTime ) length, Count ( * ) as count_
				From #tempstatistics
				Group By Year( Timestart ), Month ( TimeStart ), Day ( TimeStart )

			Select y, m, d, ( ( ( ( DatePart ( dy, length ) - 1 ) * 24 + DatePart ( hh, length ) ) * 60 + DatePart ( mi, length ) ) * 60 + DatePart ( ss, length ) ) / count_ as AverageSec
			From #tempstatisticsYMDdateInt

			delete from #tempstatisticsYMDdateInt										
		end

	--EStatisticsViewMode.svmByHour:
	else if ( @mode = 1 )
		begin
			if not exists ( select * from tempdb..sysobjects where id = object_id('tempdb..#tempstatisticsYMDHdateInt') )
				select top 0 DatePart ( hh, TimeStart ) Hstart, TimeStop - TimeAnswer length, 1 as count_
				into #tempstatisticsYMDHdateInt
				From A_Stat_Connections_1x1
			else
				delete from #tempstatisticsYMDHdateInt

			insert into #tempstatisticsYMDHdateInt
				Select DatePart ( hh, TimeStart ) Hstart, Cast ( sum ( CAST ( TimeStop - TimeAnswer as float ) ) as DateTime ) length, count ( * ) as count_
				From #tempstatistics
				Group By DatePart ( hh, TimeStart )

			Select Hstart, ( ( ( ( DatePart ( dy, length ) - 1 ) * 24 + DatePart ( hh, length ) ) * 60 + DatePart ( mi, length ) ) * 60 + DatePart ( ss, length ) ) / count_ as AverageSec
			From #tempstatisticsYMDHdateInt

			delete from #tempstatisticsYMDHdateInt
		end

	--EStatisticsViewMode.svmByOperator:
	else if ( @mode = 2 )
		begin	
			Select uid, ( ( ( ( DatePart ( dy, length ) - 1 ) * 24 + DatePart ( hh, length ) ) * 60 + DatePart ( mi, length ) ) * 60 + DatePart ( ss, length ) ) / count_ as AverageSec
			From
			(
				Select uid, Cast ( sum ( CAST ( TimeStop - TimeAnswer as float ) ) as DateTime ) length, count ( * ) as count_ From
				(
					Select AUserId as uid, TimeAnswer, TimeStop
					From #tempstatistics
					Where AUserId != 'AB000000-0000-0000-0000-000000000000'
	
					union all
	
					Select BUserId as uid, TimeAnswer, TimeStop
					From #tempstatistics
					Where BUserId != 'AB000000-0000-0000-0000-000000000000'
				)a
				Group by uid
			)b
		end

	--EStatisticsViewMode.svmByLine:
	else if ( @mode = 3 )
		begin														
			declare @lines3 table ( lineid uniqueidentifier )
			insert into @lines3
				Select ID from A_ServerIntLines
				union 
				Select ID from A_ServerUSBLines
				union
				Select ID from A_ServerExtLines
				union
				Select ID from A_ServerGSMLines


			Select linenum, ( ( ( ( DatePart ( dy, length ) - 1 ) * 24 + DatePart ( hh, length ) ) * 60 + DatePart ( mi, length ) ) * 60 + DatePart ( ss, length ) ) / count_ as AverageSec
			From
			(
				Select linenum, Cast ( sum ( CAST ( TimeStop - TimeAnswer as float ) ) as DateTime ) length, count ( * ) as count_ From
				(
					Select ALineNum as linenum, TimeAnswer, TimeStop
					From #tempstatistics ts
					inner join @lines3 l on l.lineid = ts.ALineId
	
					union all
	
					Select BLineNum as linenum, TimeAnswer, TimeStop
					From #tempstatistics ts
					inner join @lines3 l on l.lineid = ts.BLineId
				)a
				Group by linenum
			)b

		end

	--EStatisticsViewMode.svmByNumber:						
	else if ( @mode = 4 )
		begin
			declare @t table ( nid uniqueidentifier, prefix nvarchar ( 50 ), addrtype int, reactid uniqueidentifier )
			declare @temp table ( id uniqueidentifier, prefix nvarchar ( 50 ), addrtype int, oldrid uniqueidentifier, newrid uniqueidentifier )
				
			Insert Into @t
				Select np.Id, np.Prefix, rr.InnerAddressType, rr.ReactId
				From A_NumberPlan np
					Inner join A_NumberPlanAction na on na.NumId = np.Id
					Inner join A_Rules r on na.ExtraId = r.Id
					Inner join A_RuleRecords rr on rr.RuleId = r.Id


			declare @cnt int
			set @cnt = 0

			while ( @@rowcount > 0 ) and ( @cnt < 10 )
			begin
				Set @cnt = @cnt + 1
	
				/*    @cnt - */
				Insert Into @temp
					Select t.nid, t.prefix, t.addrtype, t.reactid, t1.reactid
					From @t t
						Inner join @t t1 on t.Reactid = t1.nId
					Where t.ReactId != t.Nid
	
				/*    */
				Delete From @t
					From @temp
				Where nid = id and reactid = oldrid

				/* */
				Insert Into @t ( nid, prefix, addrtype, reactid )
					Select id, prefix, addrtype, newrid From @temp

				/* */
				Delete From @temp	
			end


			declare @lines4 table ( lineid uniqueidentifier )
			insert into @lines4
				Select ID from A_ServerIntLines
				union 
				Select ID from A_ServerUSBLines
				union 
				Select ID from A_ServerExtLines
				union 
				Select ID from A_ServerGSMLines


			Select prefix, ( ( ( ( DatePart ( dy, length ) - 1 ) * 24 + DatePart ( hh, length ) ) * 60 + DatePart ( mi, length ) ) * 60 + DatePart ( ss, length ) ) / count_ as AverageSec
			From
			(
				Select prefix, Cast ( sum ( CAST ( TimeStop - TimeAnswer as float ) ) as DateTime ) length, count ( * ) as count_ From
				(
					Select t1.prefix, 0 as InnerAddrType, TimeAnswer, TimeStop
					From #tempstatistics t inner join 
						@t t1 on t1.reactid = t.AUserId and t1.addrtype = 0 inner join 
						A_Users u on u.id = AUserId

					union all

					Select t1.prefix, 0 as InnerAddrType, TimeAnswer, TimeStop
					From #tempstatistics t inner join 
						@t t1 on t1.reactid = t.BUserId and t1.addrtype = 0 inner join 
						A_Users u on u.id = BUserId

					union all

					Select t1.prefix, 1 as InnerAddrType, TimeAnswer, TimeStop
					From #tempstatistics t inner join 
						@t t1 on t1.reactid = t.ALineId and t1.addrtype = 1 inner join
						@lines4 l on l.lineid = ALineId

					union all

					Select t1.prefix, 1 as InnerAddrType, TimeAnswer, TimeStop
					From #tempstatistics t inner join 
						@t t1 on t1.reactid = t.BLineId and t1.addrtype = 1 inner join
						@lines4 l on l.lineid = BLineId
				)a
				Group by prefix
			)b

		end	

	delete from #tempstatistics

end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Stat_Office_Diagramm_CallTime' ) )
drop procedure dbo.A_Stat_Office_Diagramm_CallTime
GO


CREATE procedure [dbo].[A_Stat_Office_Diagramm_CallTime]
	@iduser uniqueidentifier,
	@dstart datetime,
	@dstop datetime,
	@tstart datetime,
	@tstop datetime,
	@lentime datetime,
	@taskid uniqueidentifier = null,
	@fromcc bit = null,
	@operatorid uniqueidentifier = null,
	@innerprefix nvarchar ( 100 ) = null,
	@outnumber nvarchar ( 100 ) = null,
	@mode int
as

begin

/*
-- ---------------------------------------------------
	declare @iduser uniqueidentifier
	declare @dstart datetime
	declare @dstop datetime
	declare @tstart datetime
	declare @tstop datetime
	declare @lentime datetime
	declare @taskid uniqueidentifier
	declare @operatorid uniqueidentifier
	declare @innerprefix nvarchar ( 100 )
	declare @outnumber nvarchar ( 100 )
	declare @mode int

	select @iduser = 'DA803F01-EA77-40FA-BC9D-E2EFB36FD5A8', 
		@dstart = '15.06.2006', 
		@dstop = '20.12.2006', 
		@tstart = '8:00', 
		@tstop = '20:00', 
		@lentime = '0:00:01',
		@mode = 2
--------------------------------------------------------------------
*/

	if not exists ( select * from tempdb..sysobjects where id = object_id('tempdb..#tempstatistics') )
		select top 0 * into #tempstatistics From A_Stat_Connections_1x1
	else
		delete from #tempstatistics
		
	Insert into #tempstatistics 
	Exec A_Stat_Service_FillTempByFilter
		@iduser,
		@dstart,
		@dstop,
		@tstart,
		@tstop,
		@lentime,
		@taskid,
		@fromcc,
		@operatorid,
		@innerprefix,
		@outnumber


	--EStatisticsViewMode.svmByDay:
	if ( @mode = 0 )
 		begin						
			if not exists ( select * from tempdb..sysobjects where id = object_id('tempdb..#tempstatisticsYMDdate') ) 
				select top 0 Year( Timestart ) y, Month ( TimeStart ) m, Day ( TimeStart ) D, TimeStop - TimeAnswer length 
				into #tempstatisticsYMDdate 
				From A_Stat_Connections_1x1
			else
				delete from #tempstatisticsYMDdate

			insert into #tempstatisticsYMDdate
				Select Year( Timestart ) y, Month ( TimeStart ) m, Day ( TimeStart ) D, Cast ( sum ( CAST ( TimeStop - TimeAnswer as float ) ) as DateTime ) length
				From #tempstatistics
				Group By Year( Timestart ), Month ( TimeStart ), Day ( TimeStart )

			Select y, m, d, ( DatePart ( dy, length ) - 1 ) * 24 + DatePart ( hh, length ) H, DatePart ( mi, length ) MI, DatePart ( ss, length ) S, DatePart ( ms, length ) MS
			From #tempstatisticsYMDdate

			delete from #tempstatisticsYMDdate										
		end

	--EStatisticsViewMode.svmByHour:
	else if ( @mode = 1 )
		begin
			if not exists ( select * from tempdb..sysobjects where id = object_id('tempdb..#tempstatisticsYMDHdate') )
				select top 0 DatePart ( hh, TimeStart ) Hstart, TimeStop - TimeAnswer length 
				into #tempstatisticsYMDHdate 
				From A_Stat_Connections_1x1
			else
				delete from #tempstatisticsYMDHdate

			insert into #tempstatisticsYMDHdate
				Select DatePart ( hh, TimeStart ) Hstart, Cast ( sum ( CAST ( TimeStop - TimeAnswer as float ) ) as DateTime ) length
				From #tempstatistics
				Group By DatePart ( hh, TimeStart )

			Select Hstart, ( DatePart ( dy, length ) - 1 ) * 24 + DatePart ( hh, length ) H, DatePart ( mi, length ) MI, DatePart ( ss, length ) S, DatePart ( ms, length ) MS
			From #tempstatisticsYMDHdate

			delete from #tempstatisticsYMDHdate
		end

	--EStatisticsViewMode.svmByOperator:
	else if ( @mode = 2 )
		begin			
			Select uid, ( DatePart ( dy, length ) - 1 ) * 24 + DatePart ( hh, length ) H, DatePart ( mi, length ) MI, DatePart ( ss, length ) S, DatePart ( ms, length ) MS
			From
			(
				Select uid, Cast ( sum ( CAST ( TimeStop - TimeAnswer as float ) ) as DateTime ) length From
				(
					Select AUserId as uid, TimeAnswer, TimeStop
					From #tempstatistics
					Where AUserId != 'AB000000-0000-0000-0000-000000000000'
	
					union all
	
					Select BUserId as uid, TimeAnswer, TimeStop
					From #tempstatistics
					Where BUserId != 'AB000000-0000-0000-0000-000000000000'
				)a
				Group by uid
			)b			
		end

	--EStatisticsViewMode.svmByLine:
	else if ( @mode = 3 )
		begin														
			declare @lines table ( lineid uniqueidentifier )
			insert into @lines
				Select ID from A_ServerIntLines
				union 
				Select ID from A_ServerUSBLines
				union
				Select ID from A_ServerExtLines
				union
				Select ID from A_ServerGSMLines


			Select linenum, ( DatePart ( dy, length ) - 1 ) * 24 + DatePart ( hh, length ) H, DatePart ( mi, length ) MI, DatePart ( ss, length ) S, DatePart ( ms, length ) MS
			From
			(
				Select linenum, Cast ( sum ( CAST ( TimeStop - TimeAnswer as float ) ) as DateTime ) length From
				(
					Select ALineNum as linenum, TimeAnswer, TimeStop
					From #tempstatistics ts
					inner join @lines l on l.lineid = ts.ALineId
	
					union all
	
					Select BLineNum as linenum, TimeAnswer, TimeStop
					From #tempstatistics ts
					inner join @lines l on l.lineid = ts.BLineId
				)a
				Group by linenum
			)b
		end

	--EStatisticsViewMode.svmByNumber:						
	else if ( @mode = 4 )
		begin
			declare @t table ( nid uniqueidentifier, prefix nvarchar ( 50 ), addrtype int, reactid uniqueidentifier )
			declare @temp table ( id uniqueidentifier, prefix nvarchar ( 50 ), addrtype int, oldrid uniqueidentifier, newrid uniqueidentifier )
				
			Insert Into @t
				Select np.Id, np.Prefix, rr.InnerAddressType, rr.ReactId
				From A_NumberPlan np
					Inner join A_NumberPlanAction na on na.NumId = np.Id
					Inner join A_Rules r on na.ExtraId = r.Id
					Inner join A_RuleRecords rr on rr.RuleId = r.Id


			declare @cnt int
			set @cnt = 0

			while ( @@rowcount > 0 ) and ( @cnt < 10 )
			begin
				Set @cnt = @cnt + 1
	
				/*    @cnt - */
				Insert Into @temp
					Select t.nid, t.prefix, t.addrtype, t.reactid, t1.reactid
					From @t t
						Inner join @t t1 on t.Reactid = t1.nId
					Where t.ReactId != t.Nid
	
				/*    */
				Delete From @t
					From @temp
				Where nid = id and reactid = oldrid

				/* */
				Insert Into @t ( nid, prefix, addrtype, reactid )
					Select id, prefix, addrtype, newrid From @temp

				/* */
				Delete From @temp	
			end


			declare @lines4 table ( lineid uniqueidentifier )
			insert into @lines4
				Select ID from A_ServerIntLines
				union 
				Select ID from A_ServerUSBLines
				union 
				Select ID from A_ServerExtLines
				union 
				Select ID from A_ServerGSMLines
	

			Select prefix, ( DatePart ( dy, length ) - 1 ) * 24 + DatePart ( hh, length ) H, DatePart ( mi, length ) MI, DatePart ( ss, length ) S, DatePart ( ms, length ) MS
			From
			(
				Select prefix, Cast ( sum ( CAST ( TimeStop - TimeAnswer as float ) ) as DateTime ) length
				From
				(
					Select t1.prefix, 0 as InnerAddrType, TimeAnswer, TimeStop
					From #tempstatistics t inner join 
						@t t1 on t1.reactid = t.AUserId and t1.addrtype = 0 inner join 
						A_Users u on u.id = AUserId

					union all

					Select t1.prefix, 0 as InnerAddrType, TimeAnswer, TimeStop
					From #tempstatistics t inner join 
						@t t1 on t1.reactid = t.BUserId and t1.addrtype = 0 inner join 
						A_Users u on u.id = BUserId
	
					union all

					Select t1.prefix, 1 as InnerAddrType, TimeAnswer, TimeStop
					From #tempstatistics t inner join 
						@t t1 on t1.reactid = t.ALineId and t1.addrtype = 1 inner join
						@lines4 l on l.lineid = ALineId

					union all

					Select t1.prefix, 1 as InnerAddrType, TimeAnswer, TimeStop
					From #tempstatistics t inner join 
						@t t1 on t1.reactid = t.BLineId and t1.addrtype = 1 inner join
						@lines4 l on l.lineid = BLineId
				)b
				Group by prefix
			)c

		end	

	delete from #tempstatistics

end
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Stat_Office_Diagramm_FlashedCalls' ) )
drop procedure dbo.A_Stat_Office_Diagramm_FlashedCalls
GO


CREATE procedure [dbo].[A_Stat_Office_Diagramm_FlashedCalls]
	@iduser uniqueidentifier,
	@dstart datetime,
	@dstop datetime,
	@tstart datetime,
	@tstop datetime,
	@lentime datetime,
	@taskid uniqueidentifier = null,
	@fromcc bit = null,
	@operatorid uniqueidentifier = null,
	@innerprefix nvarchar ( 100 ) = null,
	@outnumber nvarchar ( 100 ) = null,
	@mode int
as

begin

/*
-- ---------------------------------------------------
	declare @iduser uniqueidentifier
	declare @dstart datetime
	declare @dstop datetime
	declare @tstart datetime
	declare @tstop datetime
	declare @lentime datetime
	declare @taskid uniqueidentifier
	declare @operatorid uniqueidentifier
	declare @innerprefix nvarchar ( 100 )
	declare @outnumber nvarchar ( 100 )
	declare @mode int

	select @iduser = 'DA803F01-EA77-40FA-BC9D-E2EFB36FD5A8', 
		@dstart = '15.06.2006', 
		@dstop = '20.12.2006', 
		@tstart = '8:00', 
		@tstop = '20:00', 
		@lentime = '0:00:01',
		@mode = 0
--------------------------------------------------------------------
*/	

	if not exists ( select * from tempdb..sysobjects where id = object_id('tempdb..#tempstatistics') )
		select top 0 * into #tempstatistics From A_Stat_Connections_1x1
	else
		delete from #tempstatistics
		
	Insert into #tempstatistics 
	Exec A_Stat_Service_FillTempByFilter
		@iduser,
		@dstart,
		@dstop,
		@tstart,
		@tstop,
		@lentime,
		@taskid,
		@fromcc,
		@operatorid,
		@innerprefix,
		@outnumber


	if not exists ( select * from tempdb..sysobjects where id = object_id('tempdb..#tempstatisticsflash') )
		select top 0 sc1.*, sc2.ALineId A2LineId, sc2.BLineId B2LineId into #tempstatisticsflash From A_Stat_Connections_1x1 sc1 inner join A_Stat_Connections_1x1 sc2 on sc1.IDNext = sc2.Id
	else
		delete from #tempstatisticsflash

	insert into #tempstatisticsflash
		Select ts1.*, ts2.ALineId A2LineId, ts2.BLineId B2LineId
		FROM #tempstatistics ts1 inner join 
			#tempstatistics ts2 on ts1.idnext = ts2.id
		where not ( ts1.ALineId = ts2.ALineId and ts1.BLineID = ts2.BLineId ) 
			and not ( ts1.ALineId = ts2.BLineId and ts1.BLineID = ts2.ALineId )
			and not ( ts1.AUserId = 'AB000000-0000-0000-0000-000000000000' )
			and not ( ts1.BUserId = 'AB000000-0000-0000-0000-000000000000' )


	--EStatisticsViewMode.svmByDay:
	if ( @mode = 0 )
 		begin						
			Select Year ( timestop ) y, Month ( timestop ) m, Day ( timestop ) d, count ( * ) as Count_
			From #tempstatisticsflash
			Group by Year ( timestop ), Month ( timestop ), Day ( timestop )											
		end

	--EStatisticsViewMode.svmByHour:
	else if ( @mode = 1 )
		begin
			Select DatePart ( hour, timestop ) h, count ( * ) as Count_
			From #tempstatisticsflash
			Group by DatePart ( hour, timestop )
		end

	--EStatisticsViewMode.svmByOperator:
	else if ( @mode = 2 )
		begin							
			select uid, count ( * ) as count_
			From
			( 
				Select AUserId as uid, TimeStop
				from #tempstatisticsflash
				where StopSide = 0 and 
					(AUserId != '00000000-0000-0000-0000-000000000000')
		
				union all

				Select BUserId as uid, TimeStop
				from #tempstatisticsflash
				where StopSide = 1 and
					(BUserId != '00000000-0000-0000-0000-000000000000')
			)a
			Group by uid								
		end

	--EStatisticsViewMode.svmByLine:
	else if ( @mode = 3 )
		begin														
			declare @lines table ( lineid uniqueidentifier )
			insert into @lines
				Select ID from A_ServerIntLines
				union 
				Select ID from A_ServerUSBLines

			select linenum, count ( * ) as count_
			from
			(
				select BLineNum linenum, TimeStop
				from #tempstatisticsflash ts 
				inner join @lines l on ts.BLineId = l.lineid

				union

				select ALineNum linenum, TimeStop
				from #tempstatisticsflash ts 
				inner join @lines l on ts.ALineId = l.lineid
			)a

			group by linenum
		end

	--EStatisticsViewMode.svmByNumber:						
	else if ( @mode = 4 )
		begin
			declare @t table ( nid uniqueidentifier, prefix nvarchar ( 50 ), addrtype int, reactid uniqueidentifier )
			declare @temp table ( id uniqueidentifier, prefix nvarchar ( 50 ), addrtype int, oldrid uniqueidentifier, newrid uniqueidentifier )
				
			Insert Into @t
				Select np.Id, np.Prefix, rr.InnerAddressType, rr.ReactId
				From A_NumberPlan np
					Inner join A_NumberPlanAction na on na.NumId = np.Id
					Inner join A_Rules r on na.ExtraId = r.Id
					Inner join A_RuleRecords rr on rr.RuleId = r.Id


			declare @cnt int
			set @cnt = 0

			while ( @@rowcount > 0 ) and ( @cnt < 10 )
			begin
				Set @cnt = @cnt + 1
	
				/*    @cnt - */
				Insert Into @temp
					Select t.nid, t.prefix, t.addrtype, t.reactid, t1.reactid
					From @t t
						Inner join @t t1 on t.Reactid = t1.nId
					Where t.ReactId != t.Nid
	
				/*    */
				Delete From @t
					From @temp
				Where nid = id and reactid = oldrid

				/* */
				Insert Into @t ( nid, prefix, addrtype, reactid )
					Select id, prefix, addrtype, newrid From @temp

				/* */
				Delete From @temp	
			end

			declare @lines4 table ( lineid uniqueidentifier )
			insert into @lines4
				Select ID from A_ServerIntLines
				union 
				Select ID from A_ServerUSBLines


			Select prefix, sum ( count_ ) count_ from
			(
				Select t1.prefix, 0 as InnerAddrType, count ( * ) as count_
				From #tempstatisticsflash t inner join 
					@t t1 on t1.reactid = t.AUserId and t1.addrtype = 0 inner join 
					A_Users u on u.id = AUserId
				Group by prefix
					
				union all

					Select t1.prefix, 0 as InnerAddrType, count ( * ) as count_
				From #tempstatisticsflash t inner join 
					@t t1 on t1.reactid = t.BUserId and t1.addrtype = 0 inner join 
					A_Users u on u.id = BUserId
				Group by prefix
					
				union all

					Select t1.prefix, 1 as InnerAddrType, count ( * ) as count_
				From #tempstatisticsflash t inner join 
					@t t1 on t1.reactid = t.ALineId and t1.addrtype = 1 inner join
					@lines l on l.lineid = ALineId
				Group by prefix
					
				union all

				Select t1.prefix, 1 as InnerAddrType, count ( * ) as count_
				From #tempstatisticsflash t inner join 
					@t t1 on t1.reactid = t.BLineId and t1.addrtype = 1 inner join
					@lines l on l.lineid = BLineId
				Group by prefix
			)a
			Group by prefix

		end	

	delete from #tempstatisticsflash
	delete from #tempstatistics

end
GO



If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Stat_Office_Diagramm_HandledCalls' ) )
drop procedure dbo.A_Stat_Office_Diagramm_HandledCalls
GO

CREATE procedure [dbo].[A_Stat_Office_Diagramm_HandledCalls]
	@iduser uniqueidentifier,
	@dstart datetime,
	@dstop datetime,
	@tstart datetime,
	@tstop datetime,
	@lentime datetime,
	@taskid uniqueidentifier = null,
	@fromcc bit = null,
	@operatorid uniqueidentifier = null,
	@innerprefix nvarchar ( 100 ) = null,
	@outnumber nvarchar ( 100 ) = null,
	@mode int
as

begin

/*
-- ---------------------------------------------------
	declare @iduser uniqueidentifier
	declare @dstart datetime
	declare @dstop datetime
	declare @tstart datetime
	declare @tstop datetime
	declare @lentime datetime
	declare @taskid uniqueidentifier
	declare @operatorid uniqueidentifier
	declare @innerprefix nvarchar ( 100 )
	declare @outnumber nvarchar ( 100 )
	declare @mode int

	select @iduser = 'DA803F01-EA77-40FA-BC9D-E2EFB36FD5A8', 
		@dstart = '15.06.2006', 
		@dstop = '20.12.2006', 
		@tstart = '8:00', 
		@tstop = '20:00', 
		@lentime = '0:00:01',
		@mode = 2
--------------------------------------------------------------------
*/		

	if not exists ( select * from tempdb..sysobjects where id = object_id('tempdb..#tempstatistics') )
		select top 0 * into #tempstatistics From A_Stat_Connections_1x1
	else
		delete from #tempstatistics
		
	Insert into #tempstatistics 
	Exec A_Stat_Service_FillTempByFilter
		@iduser,
		@dstart,
		@dstop,
		@tstart,
		@tstop,
		@lentime,
		@taskid,
		@fromcc,
		@operatorid,
		@innerprefix,
		@outnumber

	Delete from #tempstatistics
	Where not ConnectionType in ( 4, 5 )

	Delete From #tempstatistics
	Where ConnectionType = 4 and ( not IdNext is null )


	--EStatisticsViewMode.svmByDay:
	if ( @mode = 0 )
 		begin						
			declare @k0 table ( idchain uniqueidentifier, timestop datetime )
			Insert into @k0
				Select IDChain, Max(TimeStop) as TimeStop
				From #tempstatistics							
				Group by IDChain


			Select Year ( timestop ) y, Month ( timestop ) m, Day ( timestop ) d, count ( * ) as Count_
			From @k0
			Group by Year ( timestop ), Month ( timestop ), Day ( timestop )						
		end

	--EStatisticsViewMode.svmByHour:
	else if ( @mode = 1 )
		begin
			declare @k1 table ( idchain uniqueidentifier, timestop datetime )
			Insert into @k1
				Select IDChain, Max(TimeStop) as TimeStop
				From #tempstatistics
				Group by IDChain

			Select DatePart ( hour, timestop ) h, count ( * ) as Count_
			From @k1
			Group by DatePart ( hour, timestop )
		end

	--EStatisticsViewMode.svmByOperator:
	else if ( @mode = 2 )
		begin							
				select BUserId, count ( * ) as count_
				from #tempstatistics
				group by BUserId													
		end

	--EStatisticsViewMode.svmByLine:
	else if ( @mode = 3 )
		begin														
			select BLineNum, count ( * ) as count_
			from #tempstatistics
			group by BLineNum
		end

	--EStatisticsViewMode.svmByNumber:						
	else if ( @mode = 4 )
	begin
		declare @t table ( nid uniqueidentifier, prefix nvarchar ( 50 ), addrtype int, reactid uniqueidentifier )
		declare @temp table ( id uniqueidentifier, prefix nvarchar ( 50 ), addrtype int, oldrid uniqueidentifier, newrid uniqueidentifier )
				
		Insert Into @t
			Select np.Id, np.Prefix, rr.InnerAddressType, rr.ReactId
			From A_NumberPlan np
				Inner join A_NumberPlanAction na on na.NumId = np.Id
				Inner join A_Rules r on na.ExtraId = r.Id
				Inner join A_RuleRecords rr on rr.RuleId = r.Id


		declare @cnt int
		set @cnt = 0

		while ( @@rowcount > 0 ) and ( @cnt < 10 )
		begin
			Set @cnt = @cnt + 1
	
			/*    @cnt - */
			Insert Into @temp
				Select t.nid, t.prefix, t.addrtype, t.reactid, t1.reactid
				From @t t
					Inner join @t t1 on t.Reactid = t1.nId
				Where t.ReactId != t.Nid
	
			/*    */
			Delete From @t
				From @temp
			Where nid = id and reactid = oldrid

			/* */
			Insert Into @t ( nid, prefix, addrtype, reactid )
				Select id, prefix, addrtype, newrid From @temp

			/* */
			Delete From @temp	
		end

			declare @lines table ( lineid uniqueidentifier )
			insert into @lines
				Select ID from A_ServerIntLines
				union 
				Select ID from A_ServerUSBLines


			Select prefix, sum ( count_ ) count_ from
			(
				Select t1.prefix, 0 as InnerAddrType, count ( * ) as count_
				From #tempstatistics t inner join 
					@t t1 on t1.reactid = t.BUserId and t1.addrtype = 0 inner join 
					A_Users u on u.id = BUserId
				Group by prefix

				union all

				Select t1.prefix, 1 as InnerAddrType, count ( * ) as count_
				From #tempstatistics t inner join 
					@t t1 on t1.reactid = t.BLineId and t1.addrtype = 1 inner join
					@lines l on l.lineid = BLineId
				Group by prefix
			)a
			Group by prefix
	end	

	delete from #tempstatistics

end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Stat_Office_Records_Remove' ) )
drop procedure dbo.A_Stat_Office_Records_Remove
GO

CREATE PROCEDURE dbo.[A_Stat_Office_Records_Remove]
	@id uniqueidentifier
as
begin
	declare @t DateTime
	declare @r bit
	declare @a nvarchar ( 100 )
	declare @b nvarchar ( 100 )
	
	Select @t = TimeStart, @r = IsRecorded, @a = ARecFile, @b = BRecFile
	From A_Stat_Connections_1x1 
	Where Id = @id

	Update A_Stat_Connections_1x1
	Set IsRecorded = 0, ARecFile = '', BRecFile = ''
	Where Id = @id

	Select @t, @r, @a, @b	
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_CallCenter_Temp_OnLoad' ) )
drop procedure dbo.A_CallCenter_Temp_OnLoad
GO

CREATE PROCEDURE dbo.[A_CallCenter_Temp_OnLoad]
as
begin
	declare @i int
end
GO




If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_DB_RemoveTails' ) )
drop procedure dbo.A_DB_RemoveTails
GO


CREATE PROCEDURE [dbo].[A_DB_RemoveTails] AS
begin

		
		Delete From A_WorkPlaces where (type in (2,3,4)) and not (parentId in (Select [ID] From A_WorkPlaces))
		Delete From A_WorkPlaces where (type in (2,3,4)) and not (LineId in (Select [ID] From A_ServerIntLines union all Select ID From A_ServerUSBLines))
		Delete FROM A_ServerExtLines where not [ServerID] in (Select [id] from A_WORKPLACES)
		Delete FROM A_ServerIntLines where not [ServerID] in (Select [id] from A_WORKPLACES)
		Delete FROM A_ServerGSMLines where not [ServerID] in (Select [id] from A_WORKPLACES)
		Delete FROM A_ServerUSBLines where not [ServerID] in (Select [id] from A_WORKPLACES)
	
		Delete FROM A_ServerUSBLines where not [id] in (Select LineID from A_WORKPLACES)
		Delete FROM A_LinesReactors where not [WPID] in (Select [id] from A_WORKPLACES)
		Delete From A_LinesReactors where not LineID in (Select [ID] from A_ServerExtLines 
								union Select [ID] from A_ServerIntLines
								union Select [ID] from A_ServerGSMLines
								union Select [ID] from A_ServerUSBLines)
		delete from A_ServerNumbers where not [id] in (Select [ID] FROM A_WorkPlaces Where type=5)
		delete from A_ServerNumbers where [id] in (Select [ID] from A_WorkPlaces where hasdatabase>0)



		declare @t table(id_ uniqueidentifier)
		insert into @t
			Select [id] from A_WORKPLACES union 
			Select [id] from A_Users union
			Select [id] from A_ServerExtLines union
			Select [id] from A_ExtNumbers union
			Select [id] from A_ServerIntLines union
			Select [id] from A_ServerUSBLines 

		
		Delete From A_RuleRecords where not RuleID in (Select ID from A_Rules)

		Delete from A_RuleRecords where (InnerAddressType in (0,1)) and ( not ReactID in (Select id_ from @t ))

		Delete from A_NumberPlanAction where numtype=0 and extraid not in (Select ID from A_Rules)
		delete from A_NumberPlan where id not in (Select numid from A_NumberPlanAction)
		Delete From A_RuleRecords where InnerAddressType=2 and not ReactID in (Select ID from A_NumberPlan)
		Delete From A_Rules	Where not id in ( select ruleid from A_RuleRecords )		

		delete from A_TaskManager_TaskExtLines where not ExtLineId in ( Select id from A_ServerExtLines )
		delete from A_TaskManager_Operators where not OperatorId in ( Select id from A_Users union Select Id from C_Groups )


		-----------------------------------------------------
		--    -----------
		-----------------------------------------------------
		declare @lastpingtime datetime
		Set @lastpingtime = GetDate()
		select @lastpingtime = valueDateTime From A_Settings where [name] like 'PBXStatusServerPingTime'
		
		declare @t1 table ( uid  uniqueidentifier ,  tm datetime )
		Insert into @t1
			Select UserId, Max ( TimeChange ) as TS FROM A_UserStateHistory Group By UserId
	

		Insert into A_UserStateHistory

		Select 	newid(), ush.UserId, 0, @lastpingtime, '', 0
			From A_UserStateHistory ush inner join @t1 t on ush.UserId = t.uid and ush.TimeChange = t.tm
			Where ush.State != 0

		--  
		Delete From A_TaskManager_UserInTask
		Where not exists ( Select Id From A_Users Where A_Users.Id = UserId )

		Delete From A_TaskManager_UserInTask
		Where not exists ( Select Id From A_TaskManager_Tasks Where A_TaskManager_Tasks.Id = TaskId )

		Delete From A_TaskManager_Operators
		Where not exists ( Select Id from A_TaskManager_Tasks where A_TaskManager_Tasks.Id = TaskId )

		Delete From A_TaskManager_TaskExtLines
		Where not exists ( Select Id from A_TaskManager_Tasks where A_TaskManager_Tasks.Id = TaskId )

		Delete From A_TaskManager_Operators
		Where IsGroup = 1 and not exists ( Select Id from C_Groups where C_Groups.Id = OperatorId )

		Delete From A_TaskManager_Operators
		Where IsGroup = 1 and not Exists ( Select * From A_Users where A_Users.Id = OperatorId )

		-- User in Task -    
		Update A_TaskManager_UserInTask
		Set TimeStopPost = @lastpingtime 
		Where TimeStopPost is null

		Update A_TaskManager_UserInTask
		Set TimeStopDialog = @lastpingtime 
		Where TimeStopDialog is null

		Update A_TaskManager_UserInTask
		Set TimeStopPrior = @lastpingtime
		Where TimeStopPrior is null

		Update A_TaskManager_UserInTask
		Set TimeStopWaitCall = @lastpingtime 
		Where TimeStopWaitCall is null

		Update A_TaskManager_UserInTask
		Set TimeStopBackCall = @lastpingtime 
		Where TimeStopBackCall is null



		Delete From A_Monopoly
		Delete From C_MonopolyForTaskChanging

		declare @uidt table ( id uniqueidentifier )
		Insert into @uidt
			Select Id From A_Users
		Delete From A_UserControls
		Where not UserA in (Select id from @uidt)
			or not UserB in (Select id from @uidt)
		-----------------------------------------------------
		-----------------------------------------------------

		/*    =   -  */
		declare @svraddr nvarchar ( 100 )
		Select @svraddr = Address 
		From A_WorkPlaces
		Where HasFastEngine = 1
		Update A_WorkPlaces Set Address = '  ' 
		Where Address like @svraddr and HasFastEngine = 0

		/*Null  0  ID   */
		Update A_ServerIntLines
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where DefaultUserId is null

		/* 0    */
		Update A_ServerIntLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where not DefaultUserId in (Select id from A_Users)

		/*       */
		Declare @useddefusers table (id uniqueidentifier)
		Declare @idtozero table (id uniqueidentifier)
		Declare @lid uniqueidentifier
		Declare @uid uniqueidentifier
		Declare @tmp bit
		Declare defusercur Cursor For
			Select Id, DefaultUserId
			From A_ServerIntLines
			Where DefaultUserId != '00000000-0000-0000-0000-000000000000'
		Open defusercur
		Fetch Next From defusercur Into @lid, @uid
		While @@FETCH_STATUS = 0
		Begin		
			Select @tmp = 1 from @useddefusers where id = @uid
			if ( @tmp > 0 )
				Insert Into @idtozero (id) values (@lid)
			else
				Insert Into @useddefusers (id) values (@uid)
			Fetch Next From defusercur  Into @lid, @uid
		End
		Close defusercur
		Deallocate defusercur
		
		Update A_ServerIntLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		From A_ServerIntLines si, @idtozero iz
		Where si.Id = iz.Id

		--  ,   -    
		Delete 
		From A_TaskManager_Task_IncomingCalls
		Where ConnectionChainId = '00000000-0000-0000-0000-000000000000'


		--   
		Delete 
		From A_Redirect
		Where IdUser not in ( Select Id From A_Users )


		-- 
		exec A_Admin_UserGroup_Group_SiftEmpty

		-- A_TaskManager_UserInTask
		Delete 
		From A_TaskManager_UserInTask
		Where Not exists ( Select Id From A_Users Where A_Users.Id = UserId )

		Delete 
		From A_TaskManager_UserInTask
		Where Not exists ( Select Id From A_TaskManager_Tasks Where A_TaskManager_Tasks.Id = TaskId )

		-------------------------------------------
		--   --------------------
		-------------------------------------------
		Delete From A_UserRoles 
		Where creationtype != 0 
			and not exists ( Select IdRole FROM A_UserParams up Where up.IdRole = A_UserRoles.Id )
		Delete From A_UserRolesExt 
		Where not exists ( Select Id From A_UserRoles a where a.Id = IdRole )
		
		-------------------------------------------
		--    
		-------------------------------------------
		DECLARE tcursor CURSOR FOR 
			Select name, Id  FROM sysobjects 
			Where type = 'U' and name like 'A_TaskManager_LocalList%'
		declare @llid int
		declare @llname nvarchar ( 1000 )
		declare @ltid uniqueidentifier
		declare @ltids nvarchar ( 50 )
		declare @sql nvarchar ( 500 )

		OPEN tcursor
		FETCH NEXT FROM tcursor INTO @llname, @llid

		WHILE @@FETCH_STATUS = 0
		BEGIN
			set @llname = 'dbo.[' + @llname + ']'

			--   (id    Id  ) (      -       . N-1  )	
			--set @ltids = SUBSTRING ( @llname, 30, 36 )
			--set @ltid = cast ( @ltids as uniqueidentifier )
			--If not exists ( Select Id from A_Taskmanager_Lists Where Id = @ltid )
	
			--      JoinedTable  
			if not exists ( Select Id from A_TaskManager_Lists Where JoinedTable = @llname )
			begin
				set @sql = ' DROP TABLE ' + @llname
				--print @sql
				exec sp_executesql @stmt =@sql
			end

		FETCH NEXT FROM tcursor INTO @llname, @llid
		END

		CLOSE tcursor
		DEALLOCATE tcursor
		-------------------------------------------
		-------------------------------------------

		--       ,   
		Update A_CallCenter_UserStateHistory
		Set 	TimeLeave = @lastpingtime,
			LengthSec = Cast ( ( @lastpingtime - TimeEnter ) as float ) * 86400
		Where TimeLeave is NULL
		
		--        ( , ,  ,      )
		Exec A_CallCenter_Temp_OnLoad

end

GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_View_ServerIntLinesAll' ) )
drop view dbo.A_View_ServerIntLinesAll
GO

Create View [dbo].[A_View_ServerIntLinesAll]
as
	Select Id, Caption, SystemNum, Enabled From A_ServerIntLines
	Union
	Select Id, Caption, SystemNum, Enabled From A_ServerUsbLines
GO

if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Stat_Connections_1x1' ) )
if not Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_Stat_Connections_1x1' 
								and TABLE_SCHEMA like 'dbo'
								and COLUMN_NAME like 'IsBetweenUsers' )
Alter Table dbo.A_Stat_Connections_1x1
Add IsBetweenUsers bit default 0
GO


If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Stat_Connection_Start' ) )
drop procedure dbo.A_Stat_Connection_Start
GO

CREATE Procedure dbo.A_Stat_Connection_Start
	
	@id			uniqueidentifier,
	@idchain		uniqueidentifier,
	@connectiontype		int,
	@reasonstart		int,
	@timestart		datetime,
	@timeanswer		datetime,
	@alineid		uniqueidentifier,
	@alinenum		nvarchar ( 8 ),
	@anumberdialed		nvarchar ( 50 ),
	@aoutnumber		nvarchar ( 50 ),
	@astr			nvarchar ( 50 ),
	@auserid		uniqueidentifier,
	@blineid		uniqueidentifier,
	@blinenum		nvarchar ( 8 ),
	@bnumberdialed		nvarchar ( 50 ),
	@boutnumber		nvarchar ( 50 ),
	@bstr			nvarchar ( 50 ),
	@buserid		uniqueidentifier,
	@isrecorded		int,
	@arecfile		nvarchar ( 50 ),
	@brecfile		nvarchar ( 50 )
as

begin
	Declare @idprev uniqueidentifier

	Select top 1 @idprev = Id
		from A_Stat_Connections_1x1 
		where IdChain = @idchain 
		order by TimeStart desc

	
	if ( @@rowcount > 0 )	
		Update A_Stat_Connections_1x1
			Set IdNext = @Id
			Where Id = @idprev

	--  ?
	declare @betweenop bit
	Set @betweenop = 0
	if ( exists ( Select Id From A_Users Where Id = @auserid )
	 	and exists ( Select Id From A_Users Where Id = @buserid ) )
		Set @betweenop = 1
	else if ( exists ( Select Id From A_View_ServerIntLinesAll Where Id = @alineid ) 
		and exists ( Select Id From A_View_ServerIntLinesAll Where Id = @blineid ) )
		Set @betweenop = 1

	--  ? -    ,      -        (      )
	declare @fromcc bit
	Set @fromcc = 0
	if exists ( Select TimeStart From A_TaskManager_UserInTask
			Where ( UserId = @auserid or UserId = @buserid )
				And TimeStopDialog is null --     -     -   
			)	
		Set @fromcc = 1
	

	-- 
	Insert into A_Stat_Connections_1x1
	(
		Id,
		IdChain,
		IdPrev,
		ConnectionType,
		ReasonStart,	
		TimeStart,		
		TimeAnswer,		
		ALineId,	
		ALineNum,
		ANumberDialed,
		AOutNumber,	
		AStr,	
		AUserId,
		BLineId,	
		BLineNum,		
		BNumberDialed,
		BOutNumber,	
		BStr,	
		BUserId,
		IsRecorded,
		ARecFile,
		BRecFile,
		IsBetweenUsers,
		FromCC
	)

	Values

	(
		@id,
		@idchain,
		@idprev,
		@connectiontype,
		@reasonstart,
		@timestart,
		@timeanswer,
		@alineid,
		@alinenum,
		@anumberdialed,
		@aoutnumber,
		@astr,
		@auserid,
		@blineid,
		@blinenum,
		@bnumberdialed,
		@boutnumber,
		@bstr,
		@buserid,
		@isrecorded,
		@arecfile,
		@brecfile,
		@betweenop,
		@fromcc --0
	)

end
GO




If exists ( Select * From sysobjects where id = object_id ( 'A_CallCenter_Reports' ) )
drop table dbo.A_CallCenter_Reports
GO

Create Table dbo.[A_CallCenter_Reports]
(
	Id uniqueidentifier not null primary key,
	Stream image null
)
GO



If exists ( Select * From sysobjects where id = object_id ( 'A_CallCenter_Report_GetAll' ) )
drop procedure dbo.A_CallCenter_Report_GetAll
GO


Create Procedure dbo.[A_CallCenter_Report_GetAll]
as
begin
	Select Id, DataLength(Stream) as Length, Stream 
	From A_CallCenter_Reports
end
GO



If exists ( Select * From sysobjects where id = object_id ( 'A_CallCenter_Report_Save' ) )
drop procedure dbo.A_CallCenter_Report_Save
GO


Create Procedure dbo.[A_CallCenter_Report_Save]
	@id uniqueidentifier,
	@stream image
as
begin
	if exists ( Select id From A_CallCenter_Reports Where Id = @id )
		Update A_CallCenter_Reports
		Set Stream = @stream
		Where Id = @id
	else		 
		Insert into A_CallCenter_Reports ( Id, Stream )
		Values ( @id, @stream )
end
GO



If exists ( Select * From sysobjects where id = object_id ( 'A_CallCenter_Report_Delete' ) )
drop procedure dbo.A_CallCenter_Report_Delete
GO


Create Procedure dbo.[A_CallCenter_Report_Delete]
	@id uniqueidentifier	
as
begin
	Delete From A_CallCenter_Reports
	Where Id = @id
end
GO


If exists ( Select * From sysobjects where id = object_id ( 'A_TalkResults_Check_Simple' ) )
drop procedure dbo.A_TalkResults_Check_Simple
GO

CREATE procedure [dbo].[A_TalkResults_Check_Simple]
	@taskid uniqueidentifier,
	@cardid uniqueidentifier,

	@dstart 	datetime,
	@dstop 		datetime,
	@tstart 	datetime,
	@tstop 		datetime,
	@number 	nvarchar ( 50 ),
	@userid		uniqueidentifier
as

begin

declare @tstart2 DateTime
declare @tstop2 DateTime
set @tstart2 = Cast ( Cast ( @tstart as float ) - Floor ( Cast ( @tstart as float ) ) as DateTime )
set @tstop2 = Cast ( Cast ( @tstop as float ) - Floor ( Cast ( @tstop as float ) ) as DateTime )


declare @te table ( EffortId uniqueidentifier, IdChain uniqueidentifier, TaskId uniqueidentifier, Number nvarchar ( 50 ), CallTime DateTime, Result int, Info nvarchar ( 500 ) )

Insert into @te
	
	Select IdEffort, IdChain, IdTask, AbonentNumber, DateTimeStart, CallResult, CallResultInfo
	From [oktell_cc_temp].dbo.A_Cube_CC_EffortConnections ec
	Where ec.IdTask = @taskid
		and DateStart > @dstart and DateStart < @dstop
		and TimeStart > @tstart2 and TimeStart < @tstop2
		and AbonentNumber like '%' + @number + '%'


	Select I, Count ( * ) as cnt 	
	From A_Stat_Talk_Algorythms sta 
		inner join A_Stat_Talk_Card_Check tcc on tcc.IdAlgVisit = sta.IdAlgVisit
		inner join A_BigInt big on big.K & ChosenResult = big.K
		inner join @te te on sta.IdConnection = te.IdChain
	Where sta.IdTask = @taskid
		and tcc.IdCard = @cardid	
	Group By I
end
GO





If exists ( Select * From sysobjects where id = object_id ( 'A_TalkResults_Radio_Simple' ) )
drop procedure dbo.A_TalkResults_Radio_Simple
GO

CREATE procedure [dbo].[A_TalkResults_Radio_Simple]
	@taskid uniqueidentifier,
	@cardid uniqueidentifier,
	@dstart 	datetime,
	@dstop 		datetime,
	@tstart 	datetime,
	@tstop 		datetime,
	@number 	nvarchar ( 50 ),
	@userid	uniqueidentifier
as

begin


declare @tstart2 DateTime
declare @tstop2 DateTime
set @tstart2 = Cast ( Cast ( @tstart as float ) - Floor ( Cast ( @tstart as float ) ) as DateTime )
set @tstop2 = Cast ( Cast ( @tstop as float ) - Floor ( Cast ( @tstop as float ) ) as DateTime )


declare @te table ( EffortId uniqueidentifier, IdChain uniqueidentifier, TaskId uniqueidentifier, Number nvarchar ( 50 ), CallTime DateTime, Result int, Info nvarchar ( 500 ) )

Insert into @te

	Select IdEffort, IdChain, IdTask, AbonentNumber, DateTimeStart, CallResult, CallResultInfo
	From [oktell_cc_temp].dbo.A_Cube_CC_EffortConnections ec
	Where ec.IdTask = @taskid
		and DateStart > @dstart and DateStart < @dstop
		and TimeStart > @tstart2 and TimeStart < @tstop2
		and AbonentNumber like '%' + @number + '%'


	Select ChosenResult, Count ( * ) as cnt 
	From A_Stat_Talk_Algorythms sta
		inner join A_Stat_Talk_Card_Radio tcr on tcr.IdAlgVisit = sta.IdAlgVisit
		inner join @te te on sta.IdConnection = te.IdChain
	Where sta.IdTask = @taskid
		and tcr.IdCard = @cardid
	Group By ChosenResult


end
GO







If exists ( Select * From sysobjects where id = object_id ( 'A_TalkResults_Text_Simple' ) )
drop procedure dbo.A_TalkResults_Text_Simple
GO


CREATE procedure [dbo].[A_TalkResults_Text_Simple]
	@taskid uniqueidentifier,
	@cardid uniqueidentifier,@dstart 	datetime,
	@dstop 		datetime,
	@tstart 	datetime,
	@tstop 		datetime,
	@number 	nvarchar ( 50 ),
	@userid		uniqueidentifier
as

begin

declare @tstart2 DateTime
declare @tstop2 DateTime
set @tstart2 = Cast ( Cast ( @tstart as float ) - Floor ( Cast ( @tstart as float ) ) as DateTime )
set @tstop2 = Cast ( Cast ( @tstop as float ) - Floor ( Cast ( @tstop as float ) ) as DateTime )


declare @te table ( EffortId uniqueidentifier, IdChain uniqueidentifier, TaskId uniqueidentifier, Number nvarchar ( 50 ), CallTime DateTime, Result int, Info nvarchar ( 500 ) )

Insert into @te

	Select IdEffort, IdChain, IdTask, AbonentNumber, DateTimeStart, CallResult, CallResultInfo
	From [oktell_cc_temp].dbo.A_Cube_CC_EffortConnections ec
	Where ec.IdTask = @taskid
		and DateStart > @dstart and DateStart < @dstop
		and TimeStart > @tstart2 and TimeStart < @tstop2
		and AbonentNumber like '%' + @number + '%'


	Select ChosenResult
	From
	(
		Select ChosenResult
		From A_Stat_Talk_Algorythms sta
			inner join A_Stat_Talk_Card_Text tct on tct.IdAlgVisit = sta.IdAlgVisit
			inner join @te te on sta.IdConnection = te.IdChain
		Where sta.IdTask = @taskid
			and tct.IdCard = @cardid
			and not ltrim ( ChosenResult ) like ''
		Group By ChosenResult, tct.IdCard
	)a
end
GO





If exists ( Select * From sysobjects where id = object_id ( 'A_TalkResults_Filter_Journal' ) )
drop procedure dbo.A_TalkResults_Filter_Journal
GO

CREATE procedure [dbo].[A_TalkResults_Filter_Journal]
	@dstart 	datetime,
	@dstop 		datetime,
	@tstart 	datetime,
	@tstop 		datetime,
	@taskid 	uniqueidentifier,
	@number 	nvarchar ( 50 ),
	@operatorid	uniqueidentifier,
	@ResultValue int = NULL,
	@ResultMask int = NULL,
	@ResultText nvarchar(500) = NULL,
	@CardId uniqueidentifier,
	@showefforts	int
as
begin

declare @tstart2 DateTime
declare @tstop2 DateTime
set @tstart2 = Cast ( Cast ( @tstart as float ) - Floor ( Cast ( @tstart as float ) ) as DateTime )
set @tstop2 = Cast ( Cast ( @tstop as float ) - Floor ( Cast ( @tstop as float ) ) as DateTime )


declare @tmp_card_result table(TaskId  uniqueidentifier, IdChain uniqueidentifier)

if @ResultValue is not null
	Insert into @tmp_card_result(TaskId,IdChain)	
		Select IdTask, IdConnection
		From A_Stat_Talk_Algorythms sta
			inner join A_Stat_Talk_Card_Radio stcr on sta.IdAlgVisit=stcr.IdAlgVisit
		Where IdCard=@CardId and ChosenResult=@ResultValue


if @ResultMask is not null 
	delete 	@tmp_card_result
	Insert into @tmp_card_result(TaskId,IdChain)		
		Select IdTask, IdConnection
		From A_Stat_Talk_Algorythms sta
			inner join A_Stat_Talk_Card_Check stcc on sta.IdAlgVisit=stcc.IdAlgVisit
		Where IdCard=@CardId and ChosenResult=@ResultValue

if @ResultText is not null 
	delete 	@tmp_card_result
	Insert into @tmp_card_result(TaskId,IdChain)		
		Select IdTask, IdConnection
		From A_Stat_Talk_Algorythms sta
			inner join A_Stat_Talk_Card_Text stct on sta.IdAlgVisit=stct.IdAlgVisit
		Where IdCard=@CardId and ChosenResult=@ResultValue


declare @te table ( EffortId uniqueidentifier, IdChain uniqueidentifier, TaskId uniqueidentifier, Number nvarchar ( 50 ), CallTime DateTime, Result int, Info nvarchar ( 500 ) )

Insert into @te

	Select IdEffort, IdChain, IdTask, AbonentNumber, DateTimeStart, CallResult, CallResultInfo
	From [oktell_cc_temp].dbo.A_Cube_CC_EffortConnections ec
	Where ec.IdTask = @taskid
		and DateStart > @dstart and DateStart < @dstop
		and TimeStart > @tstart2 and TimeStart < @tstop2
		and AbonentNumber like '%' + @number + '%'


if ( @showefforts <= 0 )
	Delete 
	From @te 
	Where IdChain = '00000000-0000-0000-0000-000000000000'


declare @type int
Select @type = Type FROM A_TaskManager_Tasks Where Id = @taskid

if ( @type = 10 ) -- 1,2,3, 10 - IVR 
	Select Number, FullName, CallTime, Result, tStop - tStart as Length, Info, IsNull ( IsRecorded, 0 ) as IsRecorded, IsNull ( ARecFile, '' ) as ARecFile, IsNull ( BRecFile, '' ) as BRecFile
	From
	(
		Select te.Number ,'IVR' as FullName, CallTime, te.Result, te.Info, IsNull ( sc.TimeAnswer, 0 ) tStart, IsNull ( sc.TimeStop, 0 ) tStop, sc.IsRecorded, sc.ARecFile, sc.BRecFile
		From @te te
			left join A_Stat_Connections_1x1 sc on te.IdChain = sc.IdChain
	)a
	Order By CallTime

else 		-- 4,5, 20 - Operator
	Select Number, FullName, CallTime, Result, tStop - tStart as Length, Info, IsRecorded, ARecFile, BRecFile
	From
	(
		Select te.Number ,u.Name as FullName, CallTime, te.Result, te.Info, IsNull ( sc.TimeAnswer, 0 ) tStart, IsNull ( sc.TimeStop, 0 ) tStop, sc.IsRecorded, sc.ARecFile, sc.BRecFile
		From @te te
			inner join A_TaskManager_UserInTask ut on ut.EffortId = te.EffortId
			inner join A_Users u on ut.UserId = u.Id
			left join A_Stat_Connections_1x1 sc on te.IdChain = sc.IdChain and ( AUserId = ut.UserId or BUserId = ut.UserId )
	        Where sc.AUserId = @operatorid
	               or sc.BUserId = @operatorid 
	               or @operatorid = '00000000-0000-0000-0000-000000000000'
	)a
	Order By CallTime

end
GO




If exists ( Select * From sysobjects where id = object_id ( 'A_TaskManager_Operator_Get_TaskWork' ) )
drop procedure dbo.A_TaskManager_Operator_Get_TaskWork
GO

CREATE procedure [dbo].[A_TaskManager_Operator_Get_TaskWork]
	@idoperator		uniqueidentifier,
	@extraid		uniqueidentifier,
	@time			float OUTPUT
As
begin		
	Set @time = 0

	Select Sum ( Convert ( float, TimeStopPost - TimeStart ) )
	From A_TaskManager_UserInTask 
	Where UserId = @idoperator
		and TaskId = @extraid
		and Convert ( float, GetDate() - TimeStopPost ) < 1 
		and DatePart ( dd, GetDate() ) = DatePart ( dd, TimeStopPost )
end
GO



If exists ( Select * From sysobjects where id = object_id ( 'A_TaskManager_Task_GetSubTasks' ) )
drop procedure dbo.A_TaskManager_Task_GetSubTasks
GO 


If exists ( Select * From sysobjects where id = object_id ( 'A_TaskManager_Task_SubTask_GetEfforts' ) )
drop procedure dbo.A_TaskManager_Task_SubTask_GetEfforts
GO 

If exists ( Select * From sysobjects where id = object_id ( 'A_View_TaskManager_SubTaskEfforts' ) )
drop view dbo.A_View_TaskManager_SubTaskEfforts
GO 



If exists ( Select * From sysobjects where id = object_id ( 'A_Admin_Rule_SiftByNumbers' ) )
drop procedure dbo.A_Admin_Rule_SiftByNumbers
GO 


Create  Procedure dbo.A_Admin_Rule_SiftByNumbers
as
begin
	-- 	
	Delete 
	From A_Rules
	Where not id in ( 
		Select ExtraId
		From A_NumberPlanAction )


	Delete 
	From A_RuleRecords 
	Where not RuleID in ( 	
			Select ID 
			From A_Rules )

	Delete 
	From A_Rules	
	Where not id in ( 
			Select ruleid 
			From A_RuleRecords )
end
GO



If exists ( Select * From sysobjects where id = object_id ( 'A_WORKPLACE_CREATE' ) )
drop procedure dbo.A_WORKPLACE_CREATE
GO


If exists ( Select * From sysobjects where id = object_id ( 'C_CallCenter_GetOperatorJournal' ) )
drop procedure dbo.C_CallCenter_GetOperatorJournal
GO 



If exists ( Select * From sysobjects where id = object_id ( 'C_CallCenter_GetTaskJournal' ) )
drop procedure dbo.C_CallCenter_GetTaskJournal
GO 



If exists ( Select * From sysobjects where id = object_id ( 'C_CallCenter_GetTaskPercentsTableForIdGroupUser' ) )
drop procedure dbo.C_CallCenter_GetTaskPercentsTableForIdGroupUser
GO 



If exists ( Select * From sysobjects where id = object_id ( 'A_TaskManager_Task_ResourceSave_IncomingCallsCount' ) )
drop procedure dbo.A_TaskManager_Task_ResourceSave_IncomingCallsCount
GO 


Create  Procedure dbo.A_TaskManager_Task_ResourceSave_IncomingCallsCount
	@id uniqueidentifier,
	@incomingcallsmaxcount int
as
begin
	Update A_TaskManager_Tasks
	Set IncomingCallsMaxCount = @incomingcallsmaxcount
	Where Id = @id
end
GO
