exec A_DB_ClearConstraints 'A_SmsTaskManager_Tasks'
GO

exec A_DB_ClearConstraints 'A_SmsTaskManager_Operators'
GO

exec A_DB_ClearConstraints 'A_SmsTaskManager_Supervisors'
GO

exec A_DB_ClearConstraints 'A_SmsTaskManager_RunPeriods'
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Tasks' ) )
DROP Table dbo.A_SmsTaskManager_Tasks
GO

Create table dbo.A_SmsTaskManager_Tasks
(	
	Id uniqueidentifier Primary Key	,
	IdProject uniqueidentifier NOT NULL,
	Name nvarchar(500) NOT NULL,
	Code nvarchar ( 30 ) not null,
	Active int NOT NULL,
	Done int NOT NULL,
	Type int NOT NULL,
	SvcScript nvarchar(200) NOT NULL,
	StopReason int NOT NULL,
	ChoiceOperator int NOT NULL,
	DeleteAfterSuccess int NOT NULL,	
	IsOutputTask int NOT NULL,
	Resulted int NOT NULL,
	ResultType int NOT NULL,
	ResultInfo nvarchar(500) NOT NULL,
	ResultDateTime datetime NOT NULL,
	CreationUserId uniqueidentifier NOT NULL,
	CreationType int NOT NULL,
	CreationDateTime datetime NOT NULL,
	StartRunDateTime datetime NOT NULL,
	StopRunDateTime datetime NOT NULL,
	TagDouble float(53) NOT NULL,
	ListId uniqueidentifier NOT NULL,
	DontSetDone bit NOT NULL,
	ActiveOnHolidays bit not null,
	PauseSyncListWaitingNewAbonent int NOT NULL,
	PauseSyncListForcedSec int not null,
	IsMultiThread bit not null,
	ThreadCount int not null,
	IncomingUpdateIfExists bit not null
)
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Operators' ) )
DROP Table dbo.A_SmsTaskManager_Operators
GO

Create Table  dbo.A_SmsTaskManager_Operators
(
	TaskId uniqueidentifier not null,
	OperatorId uniqueidentifier not null,
	IsGroup int Null
)
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Supervisors' ) )
DROP Table dbo.A_SmsTaskManager_Supervisors
GO

Create Table  dbo.A_SmsTaskManager_Supervisors
(
	TaskId uniqueidentifier not null,
	SupervisorId uniqueidentifier not null
)
GO



if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_RunPeriods' ) )
DROP Table dbo.A_SmsTaskManager_RunPeriods
GO

Create Table  dbo.A_SmsTaskManager_RunPeriods
(
	TaskId uniqueidentifier not null,
	DayOfWeek int not null,
	StartTime datetime not null,
	FinishTime datetime not null
)
GO



ALTER TABLE dbo.A_SmsTaskManager_Operators WITH NOCHECK ADD CONSTRAINT
	FK_A_SmsTaskManager_Operators_A_SmsTaskManager_Tasks FOREIGN KEY
	(
		TaskId
	) REFERENCES dbo.A_SmsTaskManager_Tasks
	(
		Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO



ALTER TABLE dbo.A_SmsTaskManager_Supervisors ADD CONSTRAINT
	FK_A_SmsTaskManager_Supervisors_A_SmsTaskManager_Tasks FOREIGN KEY
	(
		TaskId
	) REFERENCES dbo.A_SmsTaskManager_Tasks
	(
		Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

ALTER TABLE dbo.A_SmsTaskManager_Supervisors ADD CONSTRAINT
	FK_A_SmsTaskManager_Supervisors_A_Users FOREIGN KEY
	(
		SupervisorId
	) REFERENCES dbo.A_Users
	(
		ID
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO


ALTER TABLE dbo.A_SmsTaskManager_RunPeriods WITH NOCHECK ADD CONSTRAINT
	FK_A_SmsTaskManager_RunPeriods_A_SmsTaskManager_Tasks FOREIGN KEY
	(
		TaskId
	) REFERENCES dbo.A_SmsTaskManager_Tasks
	(
		Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_AllTask_Get' ) )
DROP PROCEDURE dbo.A_SmsTaskManager_AllTask_Get
GO

CREATE Procedure dbo.A_SmsTaskManager_AllTask_Get
as

begin
	Select 	
		Id,
		IdProject,
		Name,
		Code,		
		Active,
		Done,
		Type,
		SvcScript,
		StopReason,
		ChoiceOperator,
		DeleteAfterSuccess,
		IsOutputTask,
		Resulted,
		ResultType,
		ResultInfo,
		ResultDateTime,
		CreationUserId,
		CreationType,
		CreationDateTime,
		StartRunDateTime,
		StopRunDateTime,
		TagDouble,
		ListId,
		DontSetDone,
		ActiveOnHolidays,
		PauseSyncListWaitingNewAbonent,
		PauseSyncListForcedSec,
		IsMultiThread,
		ThreadCount,
		IncomingUpdateIfExists	

	From A_SmsTaskManager_Tasks
end

GO




if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Task_Save' ) )
DROP PROCEDURE dbo.A_SmsTaskManager_Task_Save
GO


Create Procedure dbo.A_SmsTaskManager_Task_Save

	@id 				uniqueidentifier,
	@idproject			uniqueidentifier,
	@name				nvarchar(500),
	@code				nvarchar(30),
	@active				int,
	@done				int,
	@type				int,
	@svcscript			nvarchar(500),
	@stopreason			int,
	@choiceoperator			int,
	@deleteaftersuccess		int,
	@isoutputtask			int,
	@resulted			int,
	@resulttype			int,
	@resultinfo			nvarchar(500),
	@resultdatetime			datetime,
	@creationuserid			uniqueidentifier,
	@creationtype			int,
	@creationdatetime		datetime,
	@startrundatetime		datetime,
	@stoprundatetime		datetime,
	@tagdouble			float,
	@listid				uniqueidentifier,
	@dontsetdone			bit,
	@activeonholidays		bit,
	@pausesynclistwaitingnewabonent int,
	@pausesynclistforcedsec		int,
	@ismultithread			bit,
	@threadcount			int,
	@incomingupdateifexists		bit
	
as

begin
	Select name 
	From A_SmsTaskManager_Tasks 
	Where id = @id

	if ( @@rowcount > 0 )
	begin
		update A_SmsTaskManager_Tasks
		   Set 
			IdProject			= @idproject,
			Name 				= @name,
			Code				= @code,
			Active 				= @active, 
			Done 				= @done, 
			Type 				= @type, 
			SvcScript 			= @svcscript, 
			StopReason 			= @stopreason, 
			ChoiceOperator			= @choiceoperator,
			DeleteAfterSuccess 		= @deleteaftersuccess, 
			IsOutputTask			= @isoutputtask,
			Resulted 			= @resulted, 
			ResultType 			= @resulttype, 
			ResultInfo 			= @resultinfo, 
			ResultDateTime 			= @resultdatetime, 
			CreationUserId 			= @creationuserid, 
			CreationType 			= @creationtype, 
			CreationDateTime 		= @creationdatetime,
			StartRunDateTime		= @startrundatetime,
			StopRunDateTime			= @stoprundatetime,
			TagDouble			= @tagdouble,	
			ListId				= @listid,
			DontSetDone			= @dontsetdone,
			ActiveOnHolidays		= @activeonholidays,
			PauseSyncListWaitingNewAbonent	= @pausesynclistwaitingnewabonent,
			PauseSyncListForcedSec		= @pausesynclistforcedsec,
			IsMultiThread			= @ismultithread,
			ThreadCount			= @threadcount,
			IncomingUpdateIfExists		= @incomingupdateifexists

 		   Where
			Id 				= @id
	end
	else
	begin
		Insert into A_SmsTaskManager_Tasks
		(
			Id, 
			IdProject,
			Name, 
			Code,
			Active, 
			Done, 
			Type, 
			SvcScript, 
			StopReason, 
			ChoiceOperator, 
			DeleteAfterSuccess, 
			IsOutputTask, 
			Resulted, 
			ResultType, 
			ResultInfo, 
			ResultDateTime, 
			CreationUserId, 
			CreationType, 
			CreationDateTime, 
			StartRunDateTime, 
			StopRunDateTime, 
			TagDouble, 
			ListId,
			DontSetDone,
			ActiveOnHolidays,
			PauseSyncListWaitingNewAbonent,
			PauseSyncListForcedSec,
			IsMultiThread,
			ThreadCount,
			IncomingUpdateIfExists
		) 
		Values  
		(
			@id,
			@idproject,
			@name,
			@code,
			@active,
			@done,
			@type,
			@svcscript,
			@stopreason,
			@choiceoperator,
			@deleteaftersuccess,
			@isoutputtask,
			@resulted,
			@resulttype,
			@resultinfo,
			@resultdatetime,
			@creationuserid,
			@creationtype,
			@creationdatetime,
			@startrundatetime,
			@stoprundatetime,
			@tagdouble,
			@listid,
			@dontsetdone,
			@activeonholidays,
			@pausesynclistwaitingnewabonent,
			@pausesynclistforcedsec,
			@ismultithread,
			@threadcount,
			@incomingupdateifexists
		)		
	end
end


GO




if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Task_Delete' ) )
DROP Procedure dbo.A_SmsTaskManager_Task_Delete
GO

Create procedure dbo.A_SmsTaskManager_Task_Delete
	@id uniqueidentifier
as
begin
	Delete 
	From A_SmsTaskManager_Tasks 
	Where Id = @id
end
GO



if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_DB_FreeSmsTaskTables' ) )
DROP Procedure dbo.A_DB_FreeSmsTaskTables
GO

CREATE  PROCEDURE [dbo].[A_DB_FreeSmsTaskTables] AS
begin
	Delete 
	From dbo.A_SmsTaskManager_Tasks	
end
GO




if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Task_ClearOperators' ) )
DROP Procedure dbo.A_SmsTaskManager_Task_ClearOperators
GO

CREATE  PROCEDURE [dbo].[A_SmsTaskManager_Task_ClearOperators]
	@taskid uniqueidentifier
as
begin
	Delete 
	From A_SmsTaskManager_Operators 
	Where TaskId = @taskid
end
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Task_ClearSupervisors' ) )
DROP Procedure dbo.A_SmsTaskManager_Task_ClearSupervisors
GO

CREATE  PROCEDURE [dbo].[A_SmsTaskManager_Task_ClearSupervisors]
	@taskid uniqueidentifier
as
begin
	Delete 
	From A_SmsTaskManager_Supervisors 
	Where TaskId = @taskid
end
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Task_ClearPeriods' ) )
DROP Procedure dbo.A_SmsTaskManager_Task_ClearPeriods
GO

CREATE  PROCEDURE [dbo].[A_SmsTaskManager_Task_ClearPeriods]
	@taskid uniqueidentifier
as
begin
	Delete 
	From A_SmsTaskManager_RunPeriods 
	Where TaskId = @taskid
end
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Task_GetOperators' ) )
DROP Procedure dbo.A_SmsTaskManager_Task_GetOperators
GO

CREATE  PROCEDURE [dbo].[A_SmsTaskManager_Task_GetOperators]
	@taskid uniqueidentifier
as
begin
	Select OperatorId, IsGroup 
	From A_SmsTaskManager_Operators 
	Where TaskId = @taskid
end
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Task_GetSupervisors' ) )
DROP Procedure dbo.A_SmsTaskManager_Task_GetSupervisors
GO

CREATE  PROCEDURE [dbo].[A_SmsTaskManager_Task_GetSupervisors]
	@taskid uniqueidentifier
as
begin
	Select SupervisorId 
	From A_SmsTaskmanager_Supervisors
	Where TaskId = @taskid
end
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Task_GetPeriods' ) )
DROP Procedure dbo.A_SmsTaskManager_Task_GetPeriods
GO

CREATE  PROCEDURE [dbo].[A_SmsTaskManager_Task_GetPeriods]
	@taskid uniqueidentifier
as
begin
	Select	DayOfWeek, StartTime, FinishTime 
	From A_SmsTaskManager_RunPeriods 
	Where TaskId = @taskid
end
GO





if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Task_SetActivity' ) )
DROP Procedure dbo.A_SmsTaskManager_Task_SetActivity
GO

CREATE  PROCEDURE [dbo].[A_SmsTaskManager_Task_SetActivity]
	@id 	uniqueidentifier,
	@activityflag	int
as
begin
	Update A_SmsTaskManager_Tasks
	Set 	Active = @activityflag,
		Done = 0		
	Where	Id = @id
end
GO




exec A_DB_ClearConstraints 'C_MonopolyForTaskChanging'
GO


ALTER TABLE dbo.C_MonopolyForTaskChanging WITH NOCHECK ADD CONSTRAINT
	FK_C_MonopolyForTaskChanging_A_Users FOREIGN KEY
	(
		IdUser
	) REFERENCES dbo.A_Users
	(
		Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO



if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Task_ClearStatistics' ) )
DROP Procedure dbo.A_SmsTaskManager_Task_ClearStatistics
GO


CREATE Procedure dbo.A_SmsTaskManager_Task_ClearStatistics
	@id uniqueidentifier
as
begin

	--Delete 
	--From A_TaskManager_SubTaskEfforts
	--Where TaskId = @id


	Update A_SmsTaskManager_Tasks
	Set Done = 0
	Where Id = @id	
end
GO
