
exec A_DB_ClearConstraints 'A_SmsTaskManager_SubTasks'
GO

exec A_DB_ClearConstraints 'A_SmsTaskManager_SubTaskNumbers'
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_SubTasks' ) )
DROP TABLE dbo.A_SmsTaskManager_SubTasks
GO

CREATE TABLE dbo.A_SmsTaskManager_SubTasks
	(
	IdInList int NOT NULL,
	TaskId uniqueidentifier NOT NULL,
	NResult int NOT NULL,
	IsDeleted nvarchar(500) NOT NULL
	)
GO

ALTER TABLE dbo.A_SmsTaskManager_SubTasks ADD CONSTRAINT
	PK_A_SmsTaskManager_SubTasks PRIMARY KEY CLUSTERED 
	(
	IdInList,
	TaskId
	)

GO

ALTER TABLE dbo.A_SmsTaskManager_SubTasks WITH NOCHECK ADD CONSTRAINT
	FK_A_SmsTaskManager_SubTasks_A_SmsTaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_SmsTaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO

if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_SubTaskNumbers' ) )
DROP TABLE dbo.A_SmsTaskManager_SubTaskNumbers
GO


CREATE TABLE dbo.A_SmsTaskManager_SubTaskNumbers
	(
	IdInList int NOT NULL,
	TaskId uniqueidentifier NOT NULL,
	Number nvarchar(20) NOT NULL,
	NResult int NOT NULL,
	PushId int Not Null
	)
GO

ALTER TABLE dbo.A_SmsTaskManager_SubTaskNumbers ADD CONSTRAINT
	PK_A_SmsTaskManager_SubTaskNumbers PRIMARY KEY CLUSTERED 
	(
	IdInList,
	TaskId,
	Number
	)

GO
ALTER TABLE dbo.A_SmsTaskManager_SubTaskNumbers WITH NOCHECK ADD CONSTRAINT
	FK_A_SmsTaskManager_SubTaskNumbers_A_SmsTaskManager_Tasks FOREIGN KEY
	(
	TaskId
	) REFERENCES dbo.A_SmsTaskManager_Tasks
	(
	Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO



--A_SmsTaskManager_List_GetTaskPointInfo (id, nresult)
--A_SmsTaskManager_List_GetTaskNumberInfo (idpoint, num, nresult, pushid)

if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_List_GetTaskPointInfo' ) )
DROP PROCEDURE dbo.A_SmsTaskManager_List_GetTaskPointInfo
GO


Create procedure [dbo].[A_SmsTaskManager_List_GetTaskPointInfo]
	@idtask uniqueidentifier
as
begin
	Select IdInList, NResult
	From A_SmsTaskManager_SubTasks st
	Where TaskId = @idtask
end
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_List_GetTaskNumberInfo' ) )
DROP PROCEDURE dbo.A_SmsTaskManager_List_GetTaskNumberInfo
GO


Create procedure [dbo].[A_SmsTaskManager_List_GetTaskNumberInfo]
	@idtask uniqueidentifier
as
begin

	Select IdInList, Number, NResult, PushId
	From A_SmsTaskManager_SubTaskNumbers stn		
	Where TaskId = @idtask
end
GO




if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_TaskOut_UpdateResult_Task' ) )
DROP PROCEDURE dbo.A_SmsTaskManager_TaskOut_UpdateResult_Task
GO

Create procedure [dbo].A_SmsTaskManager_TaskOut_UpdateResult_Task
	@id 				uniqueidentifier,
	@active				int,
	@done				int,	
	@stopreason			int = 0,
	@resulted			int = 0,
	@resulttype			int = 0,
	@resultinfo			nvarchar(500) = '',
	@resultdatetime			datetime
as

begin
	Update A_SmsTaskManager_Tasks
	Set 			
		Active 		= @active, 
		Done 		= @done,
		StopReason 	= @stopreason, 
		Resulted 	= @resulted, 
		ResultType 	= @resulttype, 
		ResultInfo 	= @resultinfo, 
		ResultDateTime 	= @resultdatetime
 	Where
		Id 		= @id
		

end
GO

if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_TaskOut_UpdateResult_Point' ) )
DROP PROCEDURE dbo.A_SmsTaskManager_TaskOut_UpdateResult_Point
GO

Create procedure [dbo].A_SmsTaskManager_TaskOut_UpdateResult_Point
	@idtask		uniqueidentifier,
	@idinlist	int,
	@nresult	int
as
begin
	update A_SmsTaskManager_SubTasks
	   set
		NResult	= @nresult
	   where		
		TaskId = @idtask
		and IdInList = @idinlist

	if ( @@rowcount = 0 )
		Insert Into A_SmsTaskManager_SubTasks
			( IdInList, TaskId, NResult, IsDeleted )
		Values
			( @idinlist, @idtask, @nresult, 0 )
end
GO

if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_TaskOut_UpdateResult_Number' ) )
DROP PROCEDURE dbo.A_SmsTaskManager_TaskOut_UpdateResult_Number
GO

Create procedure [dbo].A_SmsTaskManager_TaskOut_UpdateResult_Number
	@idtask		uniqueidentifier,
	@idinlist	int,
	@number		nvarchar ( 20 ),
	@nresult	int,
	@pushid		int
as
begin

	update A_SmsTaskManager_SubTaskNumbers
	   set
		NResult	= @nresult,
		PushId	= @pushid
	   where		
		TaskId = @idtask
		and IdInList = @idinlist
		and Number = @number

	if ( @@rowcount = 0 )
		Insert Into A_SmsTaskManager_SubTaskNumbers 
			( IdInList, TaskId, Number, NResult, PushId )
		Values
			( @idinlist, @idtask, @number, @nresult, @pushid )	
end
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SmsTaskManager_Task_ClearStatistics' ) )
DROP Procedure dbo.A_SmsTaskManager_Task_ClearStatistics
GO


CREATE Procedure dbo.A_SmsTaskManager_Task_ClearStatistics
	@id uniqueidentifier
as
begin

	Delete 
	From A_SmsTaskManager_SubTasks
	Where TaskId = @id

	Delete 
	From A_SmsTaskManager_SubTaskNumbers
	Where TaskId = @id


	Update A_SmsTaskManager_Tasks
	Set Done = 0
	Where Id = @id	
end
GO

