exec A_DB_ClearConstraints 'A_SvcTaskManager_Tasks'
GO

exec A_DB_ClearConstraints 'A_SvcTaskManager_RunPeriods'
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SvcTaskManager_Tasks' ) )
DROP Table dbo.A_SvcTaskManager_Tasks
GO

Create table dbo.A_SvcTaskManager_Tasks
(	
	Id uniqueidentifier Primary Key,
	Active bit NOT NULL,
	StartType int Not Null,
	Name nvarchar(500) NOT NULL,	
	StartRunDateTime datetime NOT NULL,
	StopRunDateTime datetime NOT NULL,
	IntervalSec int Not null,
	SvcScript nvarchar(200) NOT NULL,
	TagString nvarchar ( 1000 ) NOT NULL,
	CreatorName nvarchar ( 200 ) NOT NULL,
	AutoDelete bit Not Null
)
GO


if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SvcTaskManager_RunPeriods' ) )
DROP Table dbo.A_SvcTaskManager_RunPeriods
GO

Create Table  dbo.A_SvcTaskManager_RunPeriods
(
	TaskId uniqueidentifier not null,
	DayOfWeek int not null,
	StartTime datetime not null,
	FinishTime datetime not null
)
GO


ALTER TABLE dbo.A_SvcTaskManager_RunPeriods WITH NOCHECK ADD CONSTRAINT
	FK_A_SvcTaskManager_RunPeriods_A_SvcTaskManager_Tasks FOREIGN KEY
	(
		TaskId
	) REFERENCES dbo.A_SvcTaskManager_Tasks
	(
		Id
	) ON UPDATE CASCADE
	 ON DELETE CASCADE
	
GO







if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SvcTaskManager_AllTask_Get' ) )
DROP PROCEDURE dbo.A_SvcTaskManager_AllTask_Get
GO

CREATE Procedure dbo.A_SvcTaskManager_AllTask_Get
as

begin
	Select 	
		Id,
		Active,
		StartType,
		Name,
		StartRunDateTime,
		StopRunDateTime,
		IntervalSec,
		SvcScript,
		TagString,
		CreatorName,
		AutoDelete

	From A_SvcTaskManager_Tasks
end

GO



if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SvcTaskManager_Task_Save' ) )
DROP PROCEDURE dbo.A_SvcTaskManager_Task_Save
GO


Create Procedure dbo.A_SvcTaskManager_Task_Save
	@id		uniqueidentifier,
	@active		bit,
	@starttype	int,
	@name		nvarchar ( 500 ),
	@startdt	datetime,
	@stopdt		datetime,
	@intervalsec	int,
	@svcscript	nvarchar ( 200 ),
	@tagstring	nvarchar ( 1000 ),
	@creatorname	nvarchar ( 200 ),
	@autodelete	bit
	
as

begin
	Select name 
	From A_SvcTaskManager_Tasks 
	Where id = @id

	if ( @@rowcount > 0 )
	begin
		update A_SvcTaskManager_Tasks
		   Set 
			Active =		@active,
			StartType =		@starttype,
			Name =			@name,
			StartRunDateTime =	@startdt,
			StopRunDateTime =	@stopdt,
			IntervalSec =		@intervalsec,
			SvcScript =		@svcscript,
			TagString =		@tagstring,
			CreatorName =		@creatorname,
			AutoDelete = 		@autodelete
 		   Where
			Id 			= @id
	end
	else
	begin
		Insert into A_SvcTaskManager_Tasks
		(
			Id,
			Active,
			StartType,
			Name,
			StartRunDateTime,
			StopRunDateTime,
			IntervalSec,
			SvcScript,
			TagString,
			CreatorName,
			AutoDelete
		) 
		Values  
		(
			@id,
			@active,
			@starttype,
			@name,
			@startdt,
			@stopdt,
			@intervalsec,
			@svcscript,
			@tagstring,
			@creatorname,
			@autodelete
		)		
	end
end
GO



if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SvcTaskManager_Task_Delete' ) )
DROP Procedure dbo.A_SvcTaskManager_Task_Delete
GO

Create procedure dbo.A_SvcTaskManager_Task_Delete
	@id uniqueidentifier
as
begin
	Delete 
	From A_SvcTaskManager_Tasks 
	Where Id = @id
end
GO



if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SvcTaskManager_Task_ClearPeriods' ) )
DROP Procedure dbo.A_SvcTaskManager_Task_ClearPeriods
GO

CREATE  PROCEDURE [dbo].[A_SvcTaskManager_Task_ClearPeriods]
	@taskid uniqueidentifier
as
begin
	Delete 
	From A_SvcTaskManager_RunPeriods 
	Where TaskId = @taskid
end
GO



if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SvcTaskManager_Task_GetPeriods' ) )
DROP Procedure dbo.A_SvcTaskManager_Task_GetPeriods
GO

CREATE  PROCEDURE [dbo].[A_SvcTaskManager_Task_GetPeriods]
	@taskid uniqueidentifier
as
begin
	Select	DayOfWeek, StartTime, FinishTime 
	From A_SvcTaskManager_RunPeriods 
	Where TaskId = @taskid
end
GO



if Exists ( Select * FROM sysobjects Where Id = object_id ( 'dbo.A_SvcTaskManager_Task_SetActivity' ) )
DROP Procedure dbo.A_SvcTaskManager_Task_SetActivity
GO

CREATE  PROCEDURE [dbo].[A_SvcTaskManager_Task_SetActivity]
	@id 	uniqueidentifier,
	@activityflag	bit
as
begin
	Update A_SvcTaskManager_Tasks
	Set 	Active = @activityflag
	Where	Id = @id
end
GO







