if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[A_UserMessages]'))
drop table [dbo].[A_UserMessages]
GO

Create Table dbo.A_UserMessages
(
	Id	uniqueidentifier primary key not null,
	IdUserA uniqueidentifier,
	IdUserB	uniqueidentifier,
	SentDate DateTime,
	Message text,
	Viewed bit,
	ViewedDate DateTime
)
GO

if not exists ( select name from sysindexes where id=object_id('dbo.A_UserMessages') and name = 'idxUserMessages_SentDate' )
	CREATE INDEX idxUserMessages_SentDate ON dbo.A_UserMessages
	(
		SentDate
	) WITH FILLFACTOR = 80
GO


if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[A_UserMessages_Add]'))
drop procedure [dbo].[A_UserMessages_Add]
GO

CREATE Procedure [dbo].A_UserMessages_Add
	@id		uniqueidentifier,
	@idusera	uniqueidentifier,
	@iduserb	uniqueidentifier,
	@sentdate	datetime,
	@message	text	
as
begin
	Insert into A_UserMessages
	Values ( @id, @idusera, @iduserb, @sentdate, @message, 0, null )
end
GO


if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[A_UserMessages_GetNew]'))
drop procedure [dbo].[A_UserMessages_GetNew]
GO

CREATE Procedure [dbo].A_UserMessages_GetNew
	@iduser uniqueidentifier

as
begin
	Select msg.Id, IdUserA, IsNull ( u1.Name, '???' ) as UserAName, IdUserB, IsNull ( u2.Name, '???' ) as UserBName, SentDate, Message, Viewed, ViewedDate 
	From A_UserMessages msg
		Left join A_Users u1 on IdUserA = u1.Id
		Left join A_Users u2 on IdUserB = u2.Id
	Where IdUserB = @iduser 
		and Viewed = 0
	Order by SentDate	
end
GO



if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[A_UserMessages_GetLastForUser]'))
drop procedure [dbo].[A_UserMessages_GetLastForUser]
GO

CREATE Procedure [dbo].A_UserMessages_GetLastForUser
	@idusera	uniqueidentifier,
	@iduserb	uniqueidentifier,
	@count		int
as
begin
	Declare @tmp table ( 
		Inc	int identity ( 1, 1 ),
		Id	uniqueidentifier,
		IdUserA uniqueidentifier,
		IdUserB	uniqueidentifier,
		SentDate DateTime,
		Message text collate database_default,
		Viewed bit,
		ViewedDate DateTime )

	Insert into @tmp ( Id, IdUserA, IdUserB, SentDate, Message, Viewed, ViewedDate )
		Select Top 100 * 
		FROM A_UserMessages 
		Where ( IdUserA = @idusera And IdUserB = @iduserb )
			or ( IdUserA = @iduserb And IdUserB = @idusera )
		Order By SentDate Desc


	Select tmp.Id, IdUserA, IsNull ( u1.Name, '???' ) as UserAName, IdUserB, IsNull ( u2.Name, '???' ) as UserBName, SentDate, Message, Viewed, ViewedDate
	FROM @tmp tmp
		Left join A_Users u1 on IdUserA = u1.Id
		Left join A_Users u2 on IdUserB = u2.Id
	Where Inc <= @count
	Order by SentDate
end
GO


if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[A_UserMessages_SetViewed]'))
drop procedure [dbo].[A_UserMessages_SetViewed]
GO

Create procedure dbo.A_UserMessages_SetViewed
	@idmessage uniqueidentifier
as
begin
	declare @ida uniqueidentifier
	declare @idb uniqueidentifier
	declare @dt datetime

	Select @ida = IdUserA, @idb = IdUserB, @dt = SentDate
	From A_UserMessages 
	Where Id = @idmessage

	if ( @@rowcount = 0 )
		return

	Update A_UserMessages
	Set Viewed = 1, ViewedDate = GetDate ( )
	Where IdUserA = @ida and IdUserB = @idb and SentDate <= @dt
	
end
GO




if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[A_UserMessages_GetHistory]'))
drop procedure [dbo].[A_UserMessages_GetHistory]
GO

Create procedure dbo.A_UserMessages_GetHistory
	@dtstart DateTime,
	@dtstop  DateTime,
	@searchtext nvarchar ( 100 ),
	@iduser uniqueidentifier,
	@idopponent uniqueidentifier
as
begin
	declare @ids table ( Id uniqueidentifier )

	if (( @idopponent != '00000000-0000-0000-0000-000000000000' ) and ( @searchtext != '' ))
		Insert into @ids
			Select Id
			From A_UserMessages
			Where SentDate > @dtstart 
				and SentDate < @dtstop 
				and ((IdUserA = @iduser and IduserB = @idopponent) or (IdUserB = @iduser and IdUserA = @idopponent))
				and Message like '%' + @searchtext + '%'
	else if ( @idopponent != '00000000-0000-0000-0000-000000000000' )
		Insert into @ids
			Select Id
			From A_UserMessages
			Where SentDate > @dtstart 
				and SentDate < @dtstop 
				and ((IdUserA = @iduser and IduserB = @idopponent) or (IdUserB = @iduser and IdUserA = @idopponent))
	else if ( @searchtext != '' )
		Insert into @ids
			Select Id
			From A_UserMessages
			Where SentDate > @dtstart 
				and SentDate < @dtstop 
				and (IdUserA = @iduser or IduserB = @iduser)
				and Message like '%' + @searchtext + '%'
	else
		Insert into @ids
			Select Id
			From A_UserMessages
			Where SentDate > @dtstart 
				and SentDate < @dtstop 
				and (IdUserA = @iduser or IduserB = @iduser)			
	



	Select msg.Id, IdUserA, IsNull ( u1.Name, '???' ) as UserAName, IdUserB, IsNull ( u2.Name, '???' ) as UserBName, SentDate, Message, Viewed, ViewedDate
	From A_UserMessages msg
		Inner join @ids ids on msg.Id = ids.Id
		Left join A_Users u1 on msg.IdUserA = u1.Id
		Left join A_Users u2 on msg.IdUserB = u2.Id
	Order by SentDate
end
GO