if exists ( Select * FROM sysobjects where id = object_id ( 'dbo.A_CallCenter_GetResourceInfo' ) )
	Drop procedure dbo.A_CallCenter_GetResourceInfo
GO

CREATE procedure [dbo].[A_CallCenter_GetResourceInfo]
	@minutes int,
	@dtnow datetime = null
as
begin

--  :
--  -   ( )
--  -   ()
--  -   ()
--  -    ()
--  -  ( )
--  -    ( )

--declare @minutes int
--set @minutes = 5

----------------------------------------------------------------------
--   ---------------------------------------------
----------------------------------------------------------------------
	Declare @intervalsec int 
	set @intervalsec = @minutes * 60

	if ( @dtnow is null )
		Set @dtnow = GetDate()

	Declare @dtstart DateTime
	Set @dtstart = DateAdd ( mi, - @minutes, @dtnow )

	Declare @operators table ( id uniqueidentifier )
	Insert into @operators
		Select IdUser FROM A_UserParams
		Where IsOperator = 1

	Declare @tasks table ( id uniqueidentifier, name nvarchar ( 200 ) collate database_default, IsOutput bit, projectname nvarchar ( 200 ) collate database_default)
	Insert into @tasks
		Select t.Id, t.Name, t.IsOutputTask, p.Name 
		From A_TaskManager_Tasks t
			Inner join A_TaskManager_Projects p on t.IdProject = p.Id
		Where Active = 1 
			and Done = 0 
			and StartRunDateTime < @dtnow
			and StopRunDateTime > @dtnow
			and CreationType = 2
		order by idproject, type

	
	Declare @taskoperators table (idtask uniqueidentifier, idoperator uniqueidentifier)
	Insert into @taskoperators
		Select TaskId, OperatorId 
		From A_TaskManager_Operators taskopers
			Inner join @operators opers on taskopers.OperatorId = opers.Id

	Declare @tasklinecount table (idtask uniqueidentifier, linecount int )
	Insert into @tasklinecount
		Select TaskId, Count ( * ) 
		From A_TaskManager_TaskExtLines el
			Inner join @tasks t on t.Id = el.TaskId
		Group By TaskId
		
		Union all
	
		Select Id, IncomingCallsMaxCount
		From A_TaskManager_Tasks
		Where IsOutputTask = 0	

	Declare @extlines table ( id uniqueidentifier, name nvarchar ( 100 ) collate database_default)
	Insert into @extlines
		Select Id, SystemNum From A_ServerExtLines


	declare @tempopers table ( idoperator uniqueidentifier, idtask uniqueidentifier, datetimestart datetime, datetimestop datetime, LenTime float )
	Insert into @tempopers
		Select idoperator, idtask, datetimestart, datetimestop, lentime
		From oktell_cc_temp.dbo.A_Cube_CC_OperatorStates os --With ( TabLock )
		Where DateTimeStop > @dtstart
		and not os.idtask is null

	declare @tempconn table ( IdConn uniqueidentifier, IdTask uniqueidentifier, IdExtLine uniqueidentifier, DateTimeStart datetime, DateTimeStop datetime, LenTime float )
	Insert into @tempconn
		Select IdConn, IdTask, IdExtLine, DateTimeStart, DateTimeStop, LenTime
		From oktell_cc_temp.dbo.A_Cube_CC_EffortConnections sc with ( INDEX ( idxCubeConnections_DateTimeStop ) ) --With ( TabLock )
		Where DateTimeStop > @dtstart


	Update @tempopers
	Set DateTimeStart = @dtstart
	Where DateTimeStart < @dtstart

	Update @tempopers
	Set DateTimeStop = @dtnow
	Where DateTimeStop > @dtnow

	Update @tempopers
	Set LenTime = Cast ( DateTimeStop - DateTimeStart as float ) * 86400

	Update @tempconn
	Set DateTimeStart = @dtstart
	Where DateTimeStart < @dtstart

	Update @tempconn
	Set DateTimeStop = @dtnow
	Where DateTimeStop > @dtnow

	Update @tempconn
	Set LenTime = Cast ( DateTimeStop - DateTimeStart as float ) * 86400


	declare @output table ( Type int, Id uniqueidentifier, Val float, Name nvarchar ( 100 ) collate database_default, ExtName nvarchar ( 100 ) collate database_default)
		

----------------------------------------------------------------------
--   -----------------------------------------------
----------------------------------------------------------------------

	Insert into @output ( Type, Id, Val )
		Select 1, IdOperator, Sum ( LenTime ) / @intervalsec * 100 as Val
		From
		(
			--   
			Select IdOperator, LenTime
			From @tempopers
	
			Union all
	
			--   
			Select id, 0
			From @operators o
		)a
			Inner join @operators o on a.IdOperator = o.Id
		Group By IdOperator


----------------------------------------------------------------------
--   ----------------------------------------------------
----------------------------------------------------------------------
	
	declare @temp51 table ( ALineId uniqueidentifier, BLineId uniqueidentifier, DateTimeStart datetime, DateTimeStop datetime, LenTime float )
	Insert into @temp51
		Select ALineId, BLineId, TimeStart, TimeStop, 0
		From A_Stat_Connections_1x1 sc --With ( TabLock )
		Where TimeStop > @dtstart or TimeStop is null

	Update @temp51
	Set DateTimeStart = @dtstart
	Where DateTimeStart < @dtstart

	Update @temp51
	Set DateTimeStop = @dtnow
	Where DateTimeStop > @dtnow

	Update @temp51
	Set LenTime = Cast ( DateTimeStop - DateTimeStart as float ) * 86400


	declare @temp52 table ( IdExtLine uniqueidentifier, DateTimeStart datetime, DateTimeStop datetime, LenTime float )
	Insert into @temp52
		Select el.Id, DateTimeStart, DateTimeStop, LenTime
		From @temp51 sc
			Inner join @extlines el on sc.ALineId = el.Id or sc.BLineId = el.Id



	declare @temp5 table ( IdExtLine uniqueidentifier, DateTimeStart datetime, DateTimeStop datetime, LenTime float )
	insert into @temp5
		Select IdExtLine, DateTimeStart, DateTimeStop, LenTime
		From @tempconn 
		Where IdConn is null	

		Union all
	
		Select IdExtLine, DateTimeStart, DateTimeStop, LenTime
		From @temp52


	Insert into @output ( Type, Id, Val )
		Select 2, IdExtLine, Sum ( LenTime ) / @intervalsec * 100 as Val
		From
		(
			Select IdExtLine, LenTime
			From @temp5

			Union all

			Select Id, 0
			From @extlines
		) a
			Inner join @extlines el on a.IdExtLine = el.Id
		Group By IdExtLine 

----------------------------------------------------------------------
--    -----------------------------------
----------------------------------------------------------------------

	declare @taskrescount table ( idtask uniqueidentifier, cnt int )
	Insert into @taskrescount 
		Select idtask, sum ( rescnt )
		From
		(
			Select idtask, linecount as rescnt
			From @tasklinecount
	
			Union all

			Select idtask, count ( * ) as rescnt
			From @taskoperators
			Group By Idtask
		)a
		Group By idtask


	Declare @tempres table ( IdTask uniqueidentifier, SumLen float )
	Insert into @tempres
		Select IdTask, Sum ( LenTime )
		From
		(
			Select IdTask, LenTime 
			From @tempopers
			
			Union all
		
			Select IdTask, LenTime
			From @tempconn
			
			Union all 
		
			Select Id, 0
			From @tasks
		)b
		Group By IdTask


	Insert into @output ( Type, Id, Val )
		Select 3, tr.IdTask, case when (cnt = 0) then 0 else SumLen / ( cnt * @intervalsec ) * 100 end as Val
		From @tempres tr
			Inner join @taskrescount trc on tr.IdTask = trc.IdTask
			Inner join @tasks t on tr.IdTask = t.Id

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

	Update o
	Set o.Name = u.Name
	From @output o, A_Users u
	Where o.Id = u.Id and o.Type = 1

	Update o
	Set o.Name = el.Name
	From @output o, @extlines el
	Where o.Id = el.Id and o.Type = 2

	Update o
	Set o.Name = t.Name,
	    o.ExtName = t.ProjectName
	From @output o, @tasks t
	Where o.Id = t.Id and o.Type = 3


	Select * 
	From @output
	Order By Type, ExtName, Name

end
GO


if exists ( Select * FROM sysobjects where id = object_id ( 'dbo.A_CallCenter_GetResourceTLCrossInfo' ) )
	Drop procedure dbo.A_CallCenter_GetResourceTLCrossInfo
GO

CREATE procedure [dbo].[A_CallCenter_GetResourceTLCrossInfo]
	@minutes int,
	@dtnow datetime = null
as
begin

--  :
--  -   ( )
--  -   ()
--  -   ()
--  -    ()
--  -  ( )
--  -    ( )

--declare @minutes int
--set @minutes = 5

----------------------------------------------------------------------
--   ---------------------------------------------
----------------------------------------------------------------------
	Declare @intervalsec int 
	set @intervalsec = @minutes * 60

	if ( @dtnow is null )
		Set @dtnow = GetDate()

	Declare @dtstart DateTime
	Set @dtstart = DateAdd ( mi, - @minutes, @dtnow )


	Declare @tasks table ( id uniqueidentifier, name nvarchar ( 200 ) collate database_default, IsOutput bit, projectname nvarchar ( 200 ) collate database_default)
	Insert into @tasks
		Select t.Id, t.Name, t.IsOutputTask, p.Name 
		From A_TaskManager_Tasks t
			Inner join A_TaskManager_Projects p on t.IdProject = p.Id
		Where Active = 1 
			and Done = 0 
			and StartRunDateTime < @dtnow
			and StopRunDateTime > @dtnow
			and CreationType = 2
		order by idproject, type


	declare @tasklines table ( idtask uniqueidentifier, idline uniqueidentifier )
	Insert into @tasklines
		Select TaskId, ExtLineId 
		From A_TaskManager_TaskExtLines el
			Inner join @tasks t on el.TaskId = t.Id
		Where t.IsOutput = 1


	declare @tempconn table ( IdConn uniqueidentifier, IdTask uniqueidentifier, IdExtLine uniqueidentifier, DateTimeStart datetime, DateTimeStop datetime, LenTime float )
	Insert into @tempconn
		Select IdConn, IdTask, IdExtLine, DateTimeStart, DateTimeStop, LenTime
		From oktell_cc_temp.dbo.A_Cube_CC_EffortConnections sc with ( INDEX ( idxCubeConnections_DateTimeStop ) ) --With ( TabLock )
		Where DateTimeStop > @dtstart

	Update @tempconn
	Set DateTimeStart = @dtstart
	Where DateTimeStart < @dtstart

	Update @tempconn
	Set DateTimeStop = @dtnow
	Where DateTimeStop > @dtnow

	Update @tempconn
	Set LenTime = Cast ( DateTimeStop - DateTimeStart as float ) * 86400


	declare @output table ( IdTask uniqueidentifier, IdLine uniqueidentifier, Val float )

----------------------------------------------------------------------
--   ----------------------------------------------------
----------------------------------------------------------------------

	declare @temp5 table ( IdTask uniqueidentifier, IdExtLine uniqueidentifier, DateTimeStart datetime, DateTimeStop datetime, LenTime float )
	insert into @temp5
		Select IdTask, IdExtLine, DateTimeStart, DateTimeStop, LenTime
		From @tempconn 
		Where not IdConn is null


	Insert into @output
		Select IdTask, IdExtLine, Sum ( LenTime ) / @intervalsec * 100 as Val
		From
		(
			Select IdTask, IdExtLine, LenTime
			From @temp5

			Union all

			Select IdTask, IdLine, 0
			From @tasklines tl
		) a
		Group By IdTask, IdExtLine
		Order by IdTask

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------


	Select * 
	From @output

end
GO

