if exists ( Select * FROM sysobjects where id = object_id ( 'dbo.A_DB_OptimizeRemove' ))
	drop procedure dbo.A_DB_OptimizeRemove
GO

Create procedure dbo.A_DB_OptimizeRemove
	@days int = 2
as
begin
	--    
	declare @dt DateTime
	Set @dt = DateAdd ( d, -@days, GetDate () )
	select @dt = Cast ( FLOOR ( Cast ( @dt as float ) ) as DateTime )

	-- 
	Declare @logname nvarchar(500)
	Declare @sz int, @msz int
	Declare @curdb nvarchar ( 500 )
	Set @curdb = DB_name()

	--      
	Select @logname = name from dbo.sysfiles WHERE (status & 0x40) <> 0 
	Select @sz = size, @msz = maxsize from dbo.sysfiles WHERE (status & 0x40) <> 0
	--  maxsize    1500
	if ( @msz != -1 and @msz < 1500 * 128 )
		exec ( N'alter database ' + @curdb + ' modify file(name='+@logname+', maxsize=1500)')
	exec ( N'alter database ' + @curdb + ' set recovery simple' )

	-- 
	If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
	begin
		--       
		Select @logname = name from oktell_cc_temp.dbo.sysfiles WHERE (status & 0x40) <> 0
		Select @sz = size, @msz = maxsize from oktell_cc_temp.dbo.sysfiles WHERE (status & 0x40) <> 0
		if ( @msz != -1 and @msz < 1500 * 128 )
			exec ( N'alter database oktell_cc_temp modify file(name='+@logname+', maxsize=1500)')
		exec ( N'alter database oktell_cc_temp set recovery simple' )

		--     
		DBCC DBREINDEX ( 'oktell_cc_temp.dbo.A_Cube_CC_EffortConnections' )
		DBCC DBREINDEX ( 'oktell_cc_temp.dbo.A_Cube_CC_OperatorStates' )
		DBCC DBREINDEX ( 'oktell_cc_temp.dbo.A_Cube_CC_CallQueue' )

		--   
		DBCC SHRINKDATABASE (oktell_cc_temp)
	
		--          
		Select @logname = name from oktell_cc_temp.dbo.sysfiles WHERE (status & 0x40) <> 0
		Select @sz = size, @msz = maxsize from oktell_cc_temp.dbo.sysfiles WHERE (status & 0x40) <> 0
		if ( @sz < 500 * 128 )
			exec ( N'alter database oktell_cc_temp modify file(name='+@logname+', size=500)')
		exec ( N'alter database oktell_cc_temp set recovery simple' )
	end

	--    
	Delete From dbo.A_CallBackOrders Where CalledDateTime < @dt
	Delete From dbo.A_CallCenter_UserStateHistory Where TimeEnter < @dt
	Delete From dbo.A_ICQMessages Where date_receive < @dt
	Delete From dbo.A_Stat_Connections_1x1 Where TimeStart < @dt
	Delete From dbo.A_Stat_Connections_Errors Where TimeStart < @dt
	Delete From dbo.A_Stat_Talk_Algorythms Where not exists ( Select IdChain From A_Stat_Connections_1x1 Where IdChain = IdConnection )
	Delete From dbo.A_Stat_Talk_StopInfo Where TimeStop < @dt
	Delete From dbo.A_TaskManager_Task_IncomingCalls Where CallTimeStart < @dt
	Delete From dbo.A_TaskManager_SubTaskEfforts Where EDateTime < @dt
	Delete From dbo.A_TaskManager_UserInTask Where TimeStart < @dt
	Delete From dbo.A_User_NotifyLog Where NotifyDateTime < @dt
	Delete From dbo.A_UserMessages Where SentDate < @dt
	Delete From dbo.A_UserStateHistory Where TimeChange < @dt
	Delete From dbo.A_VoiceMail Where SentTime < @dt

	--  
	DBCC DBREINDEX ( 'dbo.A_CallCenter_UserStateHistory' )
	DBCC DBREINDEX ( 'dbo.A_Stat_Connections_1x1' )
	DBCC DBREINDEX ( 'dbo.A_Stat_Connections_Errors' )
	DBCC DBREINDEX ( 'dbo.A_Stat_Talk_Algorythms' )
	DBCC DBREINDEX ( 'dbo.A_Stat_Talk_StopInfo' )
	DBCC DBREINDEX ( 'dbo.A_TaskManager_Task_IncomingCalls' )
	DBCC DBREINDEX ( 'dbo.A_TaskManager_SubTasks' )
	DBCC DBREINDEX ( 'dbo.A_TaskManager_SubTaskNumber' )
	DBCC DBREINDEX ( 'dbo.A_TaskManager_SubTaskEfforts' )
	DBCC DBREINDEX ( 'dbo.A_TaskManager_UserInTask' )
	DBCC DBREINDEX ( 'dbo.A_User_NotifyLog' )
	DBCC DBREINDEX ( 'dbo.A_UserMessages' )
	DBCC DBREINDEX ( 'dbo.A_UserStateHistory' )
	DBCC DBREINDEX ( 'dbo.A_VoiceMail' )
	DBCC DBREINDEX ( 'dbo.A_Stat_Connections_1x1' )
	
	--  
	DBCC SHRINKDATABASE (oktell)

	--        
	Select @logname = name from dbo.sysfiles WHERE (status & 0x40) <> 0 
	Select @sz = size, @msz = maxsize from dbo.sysfiles WHERE (status & 0x40) <> 0
	if ( @sz < 500 * 128 )
		exec ( N'alter database ' + @curdb + ' modify file(name='+@logname+', size=500)')
	exec ( N'alter database ' + @curdb + ' set recovery simple' )
end
GO




if exists ( Select * FROM sysobjects where id = object_id ( 'dbo.A_TaskManager_List_DistrDirectDb_IdNumCache_Create' ))
	drop procedure dbo.A_TaskManager_List_DistrDirectDb_IdNumCache_Create
GO

Create procedure dbo.A_TaskManager_List_DistrDirectDb_IdNumCache_Create
	@taskid uniqueidentifier,
	@table nvarchar ( 100 ) = '',
	@query nvarchar ( 4000 ) = '',
	@colid nvarchar ( 100 ),
	@colsphone nvarchar ( 1000 )
	
as
begin

------------------------------------------------------------
/*
declare @table nvarchar ( 100 )
declare @colid nvarchar ( 100 )
declare @taskid uniqueidentifier
declare @colsphone nvarchar ( 1000 )
declare @query nvarchar ( 4000 )

Select @taskid = '2528AE7C-5180-441B-8B08-BB3ABCCC7CA0', @colid = N'_', @colsphone = N' IsNull ( [], '''' )  + '', '' +  IsNull ( [2], '''' ) ', @table = N't2'
*/
----------------------------------------------------------

	declare @temptable nvarchar (100 )
	set @temptable = 'dbo.[t_cache1_' + cast ( @taskid as nvarchar (100 ) ) + ']'

	declare @temptable2 nvarchar (100 )
	set @temptable2 = 'dbo.[t_cache2_' + cast ( @taskid as nvarchar (100 ) ) + ']'

	declare @tableidx nvarchar (100), @tableidxconstr nvarchar ( 100 )
	set @tableidx = 'dbo.[A_TaskManager_dIdxn_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	set @tableidxconstr = 'A_TaskManager_dIdxn_' + cast ( @taskid as nvarchar ( 50 ) )
	


	declare @sql nvarchar ( 4000 )

	----------------------

	set @sql = '
		If not exists ( select * FROM sysobjects where id = object_id ( ''' + @tableidx + ''' ) )
			Create Table ' + @tableidx + ' ( id int not null, Phone nvarchar ( 50 ) not null )
		'

	exec sp_executesql @sql

	set @sql = '
		if not exists ( select s.name, s.parent_obj, s1.name from sysobjects s
			inner join sysobjects s1 on s.parent_obj = s1.Id where s.name like ''PK_' + @tableidxconstr + ''' )
			ALTER TABLE ' + @tableidx + ' ADD CONSTRAINT [PK_' + @tableidxconstr + '] PRIMARY KEY CLUSTERED 
			( [Id], [Phone] )  ON [PRIMARY] 
		'
			
	exec sp_executesql @sql

	set @sql = 'Truncate table ' + @tableidx

	exec sp_executesql @sql

	----------------------
	
	begin tran CreateIdNumCache

	----------------------

	set @sql = '
	
		if exists ( select * from sysobjects where id = object_id ( ''' + @temptable + ''' ) )
			drop table ' + @temptable + ' 
		'

	exec sp_executesql @sql
	
	----------------------

	set @sql = '
	
		if exists ( select * from sysobjects where id = object_id ( ''' + @temptable2 + ''' ) )
			drop table ' + @temptable2 + ' 
		'

	exec sp_executesql @sql
	
	-----------------------

	declare @dt datetime, @dt1 datetime
	set @dt = GetDate()

	set @sql = '
		Select ' + @colid + ' as Id,
			Replace ( Replace ( Replace (
			Replace ( Replace ( Replace ( 
			Replace ( Replace ( Replace (
			Replace ( Replace (
				''^'' + ' + @colsphone + ' + ''$'', 
				''^, '', ''^'' ), ''^, '', ''^'' ), ''^, '', ''^'' ),
				'', $'', ''$'' ), '', $'', ''$'' ), '', $'', ''$'' ),
				'', ,'', '', ''), '', ,'', '', ''), '', ,'', '', ''),
			''^'', '''' ), ''$'', '''' ) as Phone into ' + @temptable + '
		From '
	if ( Len ( @table ) > 0 )
		set @sql = @sql + @table
	else
		set @sql = @sql + '( ' + @query + ' )a'
		
	exec sp_executesql @sql

	----------------------

	declare @tt nvarchar ( 100 )
	set @tt = @temptable

	set @sql = '
		Insert into ' + @tableidx + '
			Select * From ' + @tt + ' 
			Where not Phone like ''%,%'' and not Phone like ''''
		'

	exec sp_executesql @sql


--	set @dt1 = GetDate() - @dt
--	print 'part3     ' + cast ( DatePart ( hh, @dt1 ) as nvarchar ( 10 ) ) + ':' + cast ( DatePart ( mi, @dt1 ) as nvarchar ( 10 ) ) + ':' + cast ( DatePart ( ss, @dt1 ) as nvarchar ( 10 ) ) + ':' + cast ( DatePart ( ms, @dt1 ) as nvarchar ( 10 ) )

	set @sql = '
		Declare ucursor Cursor LOCAL FAST_FORWARD For 
			Select Id, Phone From ' + @tt + ' Where Phone like ''%,%''

		declare @id int, @phone nvarchar ( 1000 )

		Open ucursor
		Fetch Next From ucursor Into @id, @phone

		While ( @@fetch_status = 0 )
		   begin	
			declare @i int, @j1 int, @j2 int, @ph nvarchar ( 50 )
	
			set @i = 0
			set @j1 = 1
			set @j2 = 0
			
			set @j2 = charindex ( '','', @phone, @j1 )
			while @j2 > 0 or @i = 0
			begin		
				if ( @j2 = 0 )
				begin
					set @j2 = len ( @phone ) + 1
					set @i = 1
				end
	
				select @ph = REPLACE ( REPLACE ( REPLACE ( REPLACE ( 
					LTRIM ( RTRIM ( substring ( @phone, @j1, @j2-@j1 ) ) ), 
					'' '', '''' ), ''('', '''' ), '')'', '''' ), ''-'', '''' )

				if ( Len ( @ph ) > 0 )
					if not exists ( select id from ' + @tableidx + ' where id = @id and Phone = @ph )
						Insert into ' + @tableidx + ' Values ( @id, @ph )

				select @j1 = @j2 + 1, @j2 = 0
				set @j2 = charindex ( '','', @phone, @j1 )
			end
	
	

			Fetch Next From ucursor Into @id, @phone
		   end

		Close ucursor
		Deallocate ucursor
	'

	exec sp_executesql @sql

--	set @dt1 = GetDate() - @dt
--	print 'part4     ' + cast ( DatePart ( hh, @dt1 ) as nvarchar ( 10 ) ) + ':' + cast ( DatePart ( mi, @dt1 ) as nvarchar ( 10 ) ) + ':' + cast ( DatePart ( ss, @dt1 ) as nvarchar ( 10 ) ) + ':' + cast ( DatePart ( ms, @dt1 ) as nvarchar ( 10 ) )

	
	----------------------

	set @sql = '	
		if exists ( select * from sysobjects where id = object_id ( ''' + @temptable + ''' ) )
			drop table ' + @temptable + ' 
		
		if exists ( select * from sysobjects where id = object_id ( ''' + @temptable2 + ''' ) )
			drop table ' + @temptable2 + ' 
		'

	exec sp_executesql @sql

	----------------------

	Commit tran CreateIdNumCache

end
GO
