if exists ( Select * FROM sysobjects where id = object_id ( 'dbo.A_CallCenter_ControlEvents' ) )
	drop table dbo.A_CallCenter_ControlEvents
GO

Create table dbo.A_CallCenter_ControlEvents 
(
	Id uniqueidentifier not null primary key,
	Name nvarchar ( 500 ),
	Type int,
	Type2 int,
	Active bit,
	Stream image,
	IsDeleted bit
)
GO

if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_CallCenter_ControlEvents_GetJournal' ))
	drop procedure dbo.A_CallCenter_ControlEvents_GetJournal
GO

Create procedure dbo.A_CallCenter_ControlEvents_GetJournal
	@iduser uniqueidentifier,
	@datestart datetime,
	@datestop datetime,
	@timestart datetime,
	@timestop datetime,
	@idsevent nvarchar ( 4000 ),
	@idstask nvarchar ( 4000 ),
	@idsoperator nvarchar ( 4000 )
as
begin
	Declare @dstart datetime, @tstart datetime, @dstop datetime, @tstop datetime
	Set @dstart = Cast ( Floor ( Cast ( @datestart as float ) ) as DateTime )
	Set @dstop = Cast ( Floor ( Cast ( @datestop as float ) + 1 ) as DateTime )
	Set @tstart = Cast ( Cast ( @timestart as float ) - Floor ( Cast ( @timestart as float ) ) as DateTime )
	Set @tstop = Cast ( Cast ( @timestop as float ) - Floor ( Cast ( @timestop as float ) ) as DateTime )

	declare @temp nvarchar ( 4000 )
	declare @tempid uniqueidentifier

	Declare @events table ( id uniqueidentifier )
	Declare @tasks table ( id uniqueidentifier )
	Declare @operators table ( id uniqueidentifier )

	set @temp = @idsevent
	if ( len ( @temp ) > 1 )
		while ( len ( @temp ) > 1 )
		begin
			select Substring ( @temp, 1, 36 )
			set @tempid = cast ( Substring ( @temp, 1, 36 )	 as uniqueidentifier )
			insert into @events values ( @tempid )
			set @temp = substring ( @temp, 38, len ( @temp ) - 37 )
		end	
	else
		Insert into @events
			Select Id 
			From A_CallCenter_ControlEvents
			Where IsDeleted = 0

	set @temp = @idstask
	if ( len ( @temp ) > 1 )
		while ( len ( @temp ) > 1 )
		begin
			select Substring ( @temp, 1, 36 )
			set @tempid = cast ( Substring ( @temp, 1, 36 )	 as uniqueidentifier )
			insert into @tasks values ( @tempid )
			set @temp = substring ( @temp, 38, len ( @temp ) - 37 )
		end
	else
	   begin
		Insert into @tasks
			Select Id 
			From A_TaskManager_Tasks
		Insert into @tasks ( id ) values ( '00000000-0000-0000-0000-000000000000' )			
	   end


	set @temp = @idsoperator
	if ( len ( @temp ) > 1 )
		while ( len ( @temp ) > 1 )
		begin
			select Substring ( @temp, 1, 36 )
			set @tempid = cast ( Substring ( @temp, 1, 36 )	 as uniqueidentifier )
			insert into @operators values ( @tempid )
			set @temp = substring ( @temp, 38, len ( @temp ) - 37 )
		end
	else
	   begin
		Insert into @operators
			Select Id 
			From A_Users
		Insert into @operators ( id ) values ( '00000000-0000-0000-0000-000000000000' )			
	   end
	

	Select j.Id, j.IdEvent, ce.Name as EventName, j.EventDateTime, j.ValStr, j.ValDec, j.IdTask, ti.Name as TaskName, j.IdOperator, oi.Name as OperatorName
	From A_CallCenter_ControlEventsJournal j
		Inner join A_CallCenter_ControlEvents ce on j.IdEvent = ce.Id
		Inner join @events e on j.IdEvent = e.Id
		Inner join @tasks t on j.IdTask = t.Id
		Inner join @operators o on j.IdOperator = o.Id
		Left join A_TaskManager_Tasks ti on ti.Id = t.Id
		Left join A_Users oi on oi.Id = o.Id
	Where ( EventDate between @dstart and @dstop )
		and ( EventTime > @tstart ) and ( EventTime < @tstop )
	
end
GO


if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_CallCenter_ControlEvents_AddExecutedData' ))
	drop procedure dbo.A_CallCenter_ControlEvents_AddExecutedData
GO

Create procedure dbo.A_CallCenter_ControlEvents_AddExecutedData
	@idevent uniqueidentifier,
	@valstr nvarchar ( 100 ),
	@valdec decimal,
	@idtask uniqueidentifier,
	@idoperator uniqueidentifier,
	@comment nvarchar ( 3000 ) = '',
	@dtnow datetime = null
as
begin
	if ( @dtnow is null )
		set @dtnow = GetDate()

	declare @time datetime, @date datetime
	set @date = Cast ( Floor ( Cast ( @dtnow as float ) ) as DateTime )
	set @time = Cast ( ( Cast ( @dtnow as float ) - cast ( @date as float ) ) as DateTime )

	Insert into A_CallCenter_ControlEventsJournal
	Values ( @idevent, @dtnow, @date, @time, @valstr, @valdec, @idtask, @idoperator, @comment )
end
GO