if Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_TaskManager_Projects' and TABLE_SCHEMA like 'dbo' )
if not Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_TaskManager_Projects' 
								and TABLE_SCHEMA like 'dbo'
								and COLUMN_NAME like 'Shared' )
begin
	Alter Table dbo.A_TaskManager_Projects
	Add Shared bit default 1
end
GO

Update A_TaskManager_Projects
Set Shared = 1
Where Shared is null
GO


if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Project_Save' ))
	Drop procedure A_TaskManager_Project_Save
GO 

Create procedure [dbo].A_TaskManager_Project_Save
	@id 	uniqueidentifier,
	@name 	nvarchar ( 100 ),
	@description nvarchar ( 255 ),
	@datecreation DateTime,
	@idcreator uniqueidentifier,
	@mainscript nvarchar ( 500 ) = null,
	@shared bit = 1
as
begin

	Update A_TaskManager_Projects
	Set	Name = @name,
		Description = @description,
		Shared = @shared
	Where Id = @id
	

	if ( @@rowcount = 0 )
		Insert into A_TaskManager_Projects ( Id, Name, Description, DateCreation, IdCreator, MainScript, Shared )
		Values ( @id, @name, @description, @datecreation, @idcreator, @mainscript, @shared )

	--      
	If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
	If Exists ( Select * FROM oktell_cc_temp..sysobjects where id = Object_Id ( 'oktell_cc_temp.dbo.A_Cube_CC_Cat_Project' ) )
		Update oktell_cc_temp.dbo.A_Cube_CC_Cat_Project
		Set Name = @name
		Where Id = @id
end
GO


if not exists ( Select s.name, s.parent_obj, s1.name
		From sysobjects s
			inner join sysobjects s1 on s.parent_obj = s1.Id
		Where s.name like 'PK_A_TaskManager_Projects' )
begin
	ALTER TABLE [dbo].[A_TaskManager_Projects] ALTER COLUMN [Id] uniqueidentifier not null
end
GO

if not exists ( Select s.name, s.parent_obj, s1.name
		From sysobjects s
		inner join sysobjects s1 on s.parent_obj = s1.Id
		Where s.name like 'PK_A_TaskManager_Projects' )
begin
	ALTER TABLE [dbo].[A_TaskManager_Projects] ADD CONSTRAINT [PK_A_TaskManager_Projects] PRIMARY KEY  CLUSTERED 
	(
		[ID]
	)  ON [PRIMARY]
end
GO


if exists ( select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_ProjectUsers' ) )
	Drop table dbo.A_TaskManager_ProjectUsers
GO

Create table dbo.A_TaskManager_ProjectUsers
( 	
	--Id uniqueidentifier primary key,	
	IdProject uniqueidentifier,
	IdUser uniqueidentifier
)
GO

	
ALTER TABLE [dbo].[A_TaskManager_ProjectUsers] ADD CONSTRAINT [FK_A_TaskManager_ProjectUsers_A_TaskManager_Projects] FOREIGN KEY 
	(
		[IdProject]
	) REFERENCES [A_TaskManager_Projects] (
		[ID]
	) ON DELETE CASCADE  ON UPDATE CASCADE 
GO


if exists ( select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_ProjectUsers_Clear' ) )
	Drop procedure dbo.A_TaskManager_ProjectUsers_Clear
GO

Create procedure dbo.A_TaskManager_ProjectUsers_Clear
	@idproject uniqueidentifier = '00000000-0000-0000-0000-000000000000'
as
begin
	if ( @idproject != '00000000-0000-0000-0000-000000000000' )
		Delete 
		From A_TaskManager_ProjectUsers	
		Where IdProject = @idproject
	else
		Delete 
		From A_TaskManager_ProjectUsers	
end
GO


/*
exec A_TaskManager_ProjectUsers_Clear

Declare @a uniqueidentifier
Declare curs Cursor For
	Select Id From A_TaskManager_Projects
Open curs
Fetch next from curs into @a
While @@fetch_status = 0
begin
	Insert into A_TaskManager_ProjectUsers ( IdProject, IdUser )
		Select @a, Id
		From A_Users

	Fetch next from curs into @a
end
Close curs
Deallocate curs
*/


if exists ( select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Project_GetAllViewed' ) )
	Drop procedure dbo.A_TaskManager_Project_GetAllViewed
GO

Create procedure dbo.A_TaskManager_Project_GetAllViewed
	@iduser uniqueidentifier
as
begin
	Declare @tpus table ( IdProject uniqueidentifier )

	Insert into @tpus (IdProject)
		Select distinct ( IdProject )
		From A_TaskManager_ProjectUsers
		Where IdUser = @iduser
	Insert into @tpus
		Select Id
		From A_TaskManager_Projects p
		Where Shared = 1 
			and not exists ( Select IdProject From @tpus Where IdProject = p.Id )

	Select tp.Id, tp.Name, tp.Description, tp.DateCreation, tp.IdCreator, tp.MainScript, tp.Shared
	From A_TaskManager_Projects tp
		Inner join @tpus filter on tp.Id = filter.IdProject
end
GO


if exists ( select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Project_GetAll' ) )
	Drop procedure dbo.A_TaskManager_Project_GetAll
GO

Create procedure [dbo].A_TaskManager_Project_GetAll
as
begin
	Select Id, Name, Description, DateCreation, IdCreator, MainScript, Shared
	From A_TaskManager_Projects
end
GO



if exists ( Select * from sysobjects Where id = object_id ( 'dbo.A_TaskManager_List_Delete' ) )
	Drop procedure dbo.A_TaskManager_List_Delete
GO

Create procedure [dbo].[A_TaskManager_List_Delete]
	@id uniqueidentifier,
	@droptable bit = 1
as
begin

	Update A_TaskManager_Tasks
	Set ListId = '00000000-0000-0000-0000-000000000000'
	Where ListId = @id

	Declare @type int
	Declare @jt nvarchar ( 100 )
	Select @type = Type, @jt = JoinedTable From A_TaskManager_Lists
	Where id = @id

	Delete From A_TaskManager_Lists
	Where Id = @id

	declare @sql nvarchar ( 100 )
	set @sql = 'drop table ' + @jt
	if ( @type in ( 1, 5, 6 ) ) and ( @droptable = 1 )
	begin
		if ( exists ( Select * From sysobjects where id = object_id ( @jt ) ) )
			exec sp_executesql @sql
	end		
end
GO

