
if exists ( Select * FROM sysobjects Where id = object_id ( 'dbo.A_Scheduler_Tasks' ) )
	Drop Table dbo.A_Scheduler_Tasks
GO

Create Table dbo.A_Scheduler_Tasks 
(
 	Id uniqueidentifier Primary Key Not Null,
	UDirector uniqueidentifier,
	UExecuter uniqueidentifier,
	Start DateTime,
	Stop DateTime,
	MyStart DateTime,
	MyStop DateTime,
	Title nvarchar (200),
	Comment text,
	Priority int,
	DoneProcess int,
	Done bit	
)
GO


CREATE INDEX idx_SchedulerTasks_Users
   ON A_Scheduler_Tasks (UDirector, UExecuter)
GO

if exists ( Select * FROM sysobjects Where id = object_id ( 'dbo.A_SchedulerTask_Save' ) )
	Drop Procedure dbo.A_SchedulerTask_Save
GO

Create Procedure dbo.A_SchedulerTask_Save
	@id uniqueidentifier,
	@udirector uniqueidentifier,
	@uexecuter uniqueidentifier,
	@start DateTime,
	@stop DateTime,
	@mystart DateTime,
	@mystop DateTime,
	@title nvarchar (200),
	@comment text,
	@priority int,
	@doneprocess int,
	@done bit	
as
begin

	Update dbo.A_Scheduler_Tasks
	Set
		UDirector = @udirector,
		UExecuter = @uexecuter,
		Start = @start,
		Stop = @stop,
		MyStart = @mystart,
		MyStop = @mystop,
		Title = @title,
		Comment = @comment,
		Priority = @priority,
		DoneProcess = @doneprocess,
		Done = @done 
	Where
		Id = @id

	if ( @@rowcount = 0 )
	begin
		Insert into dbo.A_Scheduler_Tasks
		(
			Id,
			UDirector,
			UExecuter,
			Start,
			Stop,
			MyStart,
			MyStop,
			Title,
			Comment,
			Priority,
			DoneProcess,
			Done
		)
		Values
		(
			@id,
			@udirector,
			@uexecuter,
			@start,
			@stop,
			@mystart,
			@mystop,
			@title,
			@comment,
			@priority,
			@doneprocess,
			@done
		)
	end
end
GO

if exists ( Select * FROM sysobjects Where id = object_id ( 'dbo.A_SchedulerTask_GetByUser' ) )
	Drop Procedure dbo.A_SchedulerTask_GetByUser
GO

Create Procedure dbo.A_SchedulerTask_GetByUser
	@iduser uniqueidentifier
as
begin

	--     
	Declare @controlled table ( Id uniqueidentifier )

	--        
	Insert into @controlled ( Id ) Values ( @iduser )
	Insert into @controlled ( Id )
		Select UserB 
		From A_UserControls
		Where UserA = @iduser

	--   
	Select t.Id, t.UDirector, t.UExecuter, t.Start, t.Stop, t.MyStart, t.MyStop, t.Title, t.Comment, t.Priority, t.DoneProcess, t.Done
	From dbo.A_Scheduler_Tasks t
		Inner join @controlled c on t.UExecuter = c.Id

end
GO

if exists ( Select * FROM sysobjects Where id = object_id ( 'dbo.A_SchedulerTask_Remove' ) )
	Drop Procedure dbo.A_SchedulerTask_Remove
GO

Create Procedure dbo.A_SchedulerTask_Remove
	@id uniqueidentifier
as
begin
	Delete from dbo.A_Scheduler_Tasks
	Where Id = @id
end
GO
