if exists ( Select * From sysobjects Where id = object_id ( 'dbo.A_PluginMenu' ) )
	Drop table dbo.A_PluginMenu
GO

Create table dbo.A_PluginMenu
(
	Id uniqueidentifier,
	OrderIndex int,
	IdModule uniqueidentifier,
	IdForm uniqueidentifier,
	HeaderText nvarchar ( 300 ),
	MenuText nvarchar ( 300 ),
	VisibleAllDefault bit,
	Description ntext,
	Type int,
	AddressString nvarchar ( 3000 )
)
GO

if exists ( Select * From sysobjects Where id = object_id ( 'dbo.A_PluginMenu_User' ) )
	Drop table dbo.A_PluginMenu_User
GO

Create table dbo.A_PluginMenu_User
(
	IdMenuLink uniqueidentifier,
	IdUser uniqueidentifier,
	Visible bit	
)
GO



if exists ( Select * from sysobjects where id=object_id('dbo.A_PluginMenu_GetAll'))
	drop procedure dbo.A_PluginMenu_GetAll
GO

Create procedure dbo.A_PluginMenu_GetAll
as
begin
	Select Id, OrderIndex, IdModule, IdForm, 
		HeaderText, MenuText, VisibleAllDefault, Description,
		Type, AddressString
	From A_PluginMenu
	Order by OrderIndex
end
GO


if exists ( Select * from sysobjects where id=object_id('dbo.A_PluginMenu_GetViewed'))
	drop procedure dbo.A_PluginMenu_GetViewed
GO

Create procedure dbo.A_PluginMenu_GetViewed
	@iduser uniqueidentifier
as
begin

	Select * 
	From (
	
		Select pm.Id, pm.OrderIndex, pm.IdModule, pm.IdForm, 
			pm.HeaderText, pm.MenuText, pm.VisibleAllDefault, pm.Description,
			pm.Type, pm.AddressString
		From A_PluginMenu pm 
			Inner join A_PluginMenu_User u 
				on ( u.IdMenuLink = pm.Id )
				and ( u.IdUser = @iduser )
				and ( u.Visible = 1 )
		Where pm.VisibleAllDefault = 0

		Union all

		Select pm.Id, pm.OrderIndex, pm.IdModule, pm.IdForm, 
			pm.HeaderText, pm.MenuText, pm.VisibleAllDefault, pm.Description,
			pm.Type, pm.AddressString	
		From A_PluginMenu pm 
		Where pm.VisibleAllDefault = 1
			And not exists ( 	
					Select IdMenuLink 
					From A_PluginMenu_User u 
					Where ( u.IdMenuLink = pm.Id ) 
						And ( u.IdUser = @iduser )
						And ( u.Visible = 0 )
					)
	)a
	Order By OrderIndex
end
GO


if exists ( Select * from sysobjects where id=object_id('dbo.A_PluginMenu_Delete'))
	drop procedure dbo.A_PluginMenu_Delete
GO

Create procedure dbo.A_PluginMenu_Delete
	@id uniqueidentifier	
as
begin
	Declare @oldorder int
	set @oldorder = -1

	--    
	Select @oldorder = OrderIndex 
	From A_PluginMenu
	Where Id = @id

	if ( @@rowcount > 0 )
	begin
		Delete From A_PluginMenu
		Where Id = @id

		Delete From A_PluginMenu_User
		Where IdMenuLink = @id

		--   
		Update A_PluginMenu
		Set OrderIndex = OrderIndex - 1
		Where OrderIndex > @oldorder
	end
end
GO



if exists ( Select * from sysobjects where id=object_id('dbo.A_PluginMenu_Save'))
	drop procedure dbo.A_PluginMenu_Save
GO

Create procedure dbo.A_PluginMenu_Save
	@id uniqueidentifier, 
	@orderindex int, 
	@idmodule uniqueidentifier, 
	@idform uniqueidentifier, 
	@headertext nvarchar ( 300 ),  
	@menutext nvarchar ( 300 ), 
	@visiblealldefault bit, 
	@description ntext,
	@type int, 
	@addressstring nvarchar ( 3000 )
as
begin
	Declare @oldorder int
	set @oldorder = -1

	--    
	Select @oldorder = OrderIndex 
	From A_PluginMenu
	Where Id = @id

	--    
	if ( @@rowcount > 0 )
	begin
		-- 
		Delete From A_PluginMenu
		Where Id = @id

		--   
		if ( @oldorder != @orderindex )		
			Update A_PluginMenu
			Set OrderIndex = OrderIndex - 1
			Where OrderIndex > @oldorder
	end

	--   
	if ( @oldorder != @orderindex )		
		Update A_PluginMenu
		Set OrderIndex = OrderIndex + 1
		Where OrderIndex >= @orderindex

	--    
	Insert 
	Into A_PluginMenu ( Id, OrderIndex, IdModule, IdForm, HeaderText, MenuText, VisibleAllDefault, Description, Type, AddressString )
	Values ( @id, @orderindex, @idmodule, @idform, @headertext, @menutext, @visiblealldefault, @description, @type, @addressstring )
end
GO



if exists ( Select * from sysobjects where id=object_id('dbo.A_PluginMenu_ClearUserLinks'))
	drop procedure dbo.A_PluginMenu_ClearUserLinks
GO

Create procedure dbo.A_PluginMenu_ClearUserLinks
	@idmenulink uniqueidentifier
as
begin
	Delete From A_PluginMenu_User
	Where IdMenuLink = @idmenulink
end
GO



if exists ( Select * from sysobjects where id=object_id('dbo.A_PluginMenu_SaveUserLink'))
	drop procedure dbo.A_PluginMenu_SaveUserLink
GO

Create procedure dbo.A_PluginMenu_SaveUserLink
	@idmenulink uniqueidentifier,
	@iduser uniqueidentifier,
	@visible bit
as
begin
	Insert into A_PluginMenu_User ( IdMenuLink, IdUser, Visible )
	Values ( @idmenulink, @iduser, @visible )
end
GO



if exists ( Select * from sysobjects where id=object_id('dbo.A_DB_RemoveTails'))
	drop procedure dbo.A_DB_RemoveTails
GO

Create Procedure [dbo].[A_DB_RemoveTails] as
begin
--      .  ,    ..
		-----------------------------------------------------
		--    ----------------------------
		-----------------------------------------------------
		Delete From A_WorkPlaces where (type in (2,3,4,6)) and not (parentId in (Select [ID] From A_WorkPlaces))
		Delete From A_WorkPlaces where (type in (2,3,4,6)) and not ( LineId in ( Select [ID] From A_View_ServerIntLinesAll ))
		Delete From A_ServerExtLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerIntLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerGSMLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerUSBLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerSipLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerUSBLines where not [id] in (Select LineID From A_WORKPLACES)
		Delete From A_ServerVirtualLines where not [ServerID] in (Select [id] From A_WORKPLACES)	
		
		Delete From A_LinesReactors where not [WPID] in ( Select [id] From A_WORKPLACES )
		Delete From A_LinesReactors where not LineID in ( Select [ID] From A_View_ServerLinesAll )
		delete From A_ServerNumbers where not [id] in ( Select [ID] From A_WorkPlaces Where type=5 )
		delete From A_ServerNumbers where [id] in ( Select [ID] From A_WorkPlaces where hasdatabase>0 )



		Declare @t table(id_ uniqueidentifier)
		insert into @t
			Select [id] From A_WORKPLACES Union 
			Select [id] From A_Users union
			Select [id] From A_ExtNumbers union
			Select [id] From A_View_ServerLinesAll			

		
		Delete From A_RuleRecords where not RuleID in (Select ID From A_Rules)

		-- ,     
		Delete From A_RuleRecords where (InnerAddressType in (0,1)) and ( not ReactID in (Select id_ From @t ))
		-- ,   
		Delete From A_RuleRecords where (InnerAddressType in (3)) and ( not ReactID in (Select [Id] From A_ServerStreams ))

		Delete From A_NumberPlanAction where numtype=0 and extraid not in (Select ID From A_Rules)
		delete From A_NumberPlan where id not in (Select numid From A_NumberPlanAction)
		Delete From A_RuleRecords where InnerAddressType=2 and not ReactID in (Select ID From A_NumberPlan)
		Delete From A_Rules	Where not id in ( Select ruleid From A_RuleRecords )		

		delete From A_TaskManager_TaskExtLines where not ExtLineId in ( Select id From A_ServerExtLines )
		delete From A_TaskManager_Operators where not OperatorId in ( Select id From A_Users Union Select Id From C_Groups )

		-----------------------------------------------------
		--     =   -   
		-----------------------------------------------------
		Declare @svraddr nvarchar ( 100 )
		Select @svraddr = Address 
		From A_WorkPlaces
		Where HasFastEngine = 1
		Update A_WorkPlaces Set Address = '  ' 
		Where Address like @svraddr and HasFastEngine = 0

		-----------------------------------------------------
		-- Null  0  ID    ----------------
		-----------------------------------------------------
		Update A_ServerIntLines
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where DefaultUserId is null

		Update A_ServerSipLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where DefaultUserId is null

		Update A_ServerVirtualLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where DefaultUserId is null

		-----------------------------------------------------
		--  0     -----------
		-----------------------------------------------------
		Update A_ServerIntLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where not DefaultUserId in (Select id From A_Users)

		Update A_ServerSipLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where not DefaultUserId in (Select id From A_Users)

		Update A_ServerVirtualLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where not DefaultUserId in (Select id From A_Users)

		-----------------------------------------------------
		--         --
		-----------------------------------------------------
		Declare @useddefusers table (id uniqueidentifier)
		Declare @idtozero table (id uniqueidentifier)
		Declare @lid uniqueidentifier
		Declare @uid uniqueidentifier
		Declare @tmp bit 		

		Declare defusercur Cursor For
			Select Id, DefaultUserId 
			From A_View_ServerIntLinesDefaultUsers
			Where DefaultUserId != '00000000-0000-0000-0000-000000000000'

		Open defusercur
		Fetch Next From defusercur Into @lid, @uid
		While @@FETCH_STATUS = 0
		Begin		
			Select @tmp = 1 From @useddefusers where id = @uid
			if ( @tmp > 0 )
				Insert Into @idtozero (id) values (@lid)
			else
				Insert Into @useddefusers (id) values (@uid)
			Fetch Next From defusercur  Into @lid, @uid
		End
		Close defusercur
		Deallocate defusercur
		
		Update A_ServerIntLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		From A_ServerIntLines si, @idtozero iz
		Where si.Id = iz.Id

		Update A_ServerSipLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		From A_ServerSipLines si, @idtozero iz
		Where si.Id = iz.Id

		Update A_ServerVirtualLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		From A_ServerVirtualLines si, @idtozero iz
		Where si.Id = iz.Id
		
		-----------------------------------------------------
		--    ------------------
		-----------------------------------------------------
		Begin Tran UIT

		Delete From A_TaskManager_UserInTask with ( TabLock, HoldLock )
		Where not exists ( Select Id From A_Users Where A_Users.Id = UserId )

		Delete From A_TaskManager_UserInTask --with ( TabLock )
		Where not exists ( Select Id From A_TaskManager_Tasks Where A_TaskManager_Tasks.Id = TaskId )
		
		Commit tran UIT


		Delete From A_TaskManager_Operators
		Where not exists ( Select Id From A_TaskManager_Tasks where A_TaskManager_Tasks.Id = TaskId )

		Delete From A_TaskManager_TaskExtLines
		Where not exists ( Select Id From A_TaskManager_Tasks where A_TaskManager_Tasks.Id = TaskId )

		Delete From A_TaskManager_Operators
		Where IsGroup = 1 and not exists ( Select Id From C_Groups where C_Groups.Id = OperatorId )

		Delete From A_TaskManager_Operators
		Where IsGroup = 1 and not Exists ( Select * From A_Users where A_Users.Id = OperatorId )
		

		Declare @uidt table ( id uniqueidentifier )
		Insert into @uidt
			Select Id From A_Users
		Delete From A_UserControls
		Where not UserA in (Select id From @uidt)
			or not UserB in (Select id From @uidt)

		-----------------------------------------------------
		--     --------------------
		-----------------------------------------------------
		Delete 
		From A_Redirect
		Where IdUser not in ( Select Id From A_Users )

		-----------------------------------------------------
		--   ------------------------------------
		-----------------------------------------------------
		exec A_Admin_UserGroup_Group_SiftEmpty		

		-----------------------------------------------------
		--   ------------------------------
		-----------------------------------------------------
		Delete From A_UserRoles 
		Where creationtype != 0 
			and not exists ( Select IdRole From A_UserParams up Where up.IdRole = A_UserRoles.Id )
		Delete From A_UserRolesExt 
		Where not exists ( Select Id From A_UserRoles a where a.Id = IdRole )

		-----------------------------------------------------
		--  - ---------------------
		-----------------------------------------------------
		Delete From A_PluginMenu_User
		Where not exists ( Select Id From A_Users u Where u.Id = A_PluginMenu_User.IdUser )

		-----------------------------------------------------
		--   ------------------------------
		-----------------------------------------------------	
		Delete from A_WorkPlace_Address

end
GO



