If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_settings' )
  If exists ( Select * From oktell_settings..sysobjects where id = object_id ( 'oktell_settings.dbo.A_Users' ))
    If not Exists ( Select * From oktell_settings.INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_Users' 
								and TABLE_SCHEMA like 'dbo'
								and COLUMN_NAME like 'IsDeleted' )
	Alter Table oktell_settings.dbo.A_Users
	Add [IsDeleted] bit Default 0
GO

If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_settings' )
  If exists ( Select * From oktell_settings..sysobjects where id = object_id ( 'oktell_settings.dbo.A_Users' ))
    If not Exists ( Select * From oktell_settings.INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_Users' 
								and TABLE_SCHEMA like 'dbo'
								and COLUMN_NAME like 'DeleteDT' )
	Alter Table oktell_settings.dbo.A_Users
	Add [DeleteDT] DateTime
GO

If exists ( select * From sysobjects where id = object_id ( 'dbo.A_Users' ) and objectproperty ( id, N'isview' ) = 1 )
	Drop view dbo.A_Users
GO

Create View dbo.A_Users
As
	Select *
	From oktell_settings.dbo.A_Users
GO

Update A_Users
Set IsDeleted = 0
Where IsDeleted is NULL
GO

if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_UserGroup_User_GetAll' ))
	Drop procedure dbo.A_Admin_UserGroup_User_GetAll
GO

Create Procedure [dbo].[A_Admin_UserGroup_User_GetAll]
As
Begin
	Select 	u.Id, 
		u.Name, 
		Info, 
		ParentGroupId, 
		Login, 
		Password, 
		Type, 
		isnull ( IsOperator, 0 ) as IsOperator,
		isnull ( up.IdRole, '00000000-0000-0000-0000-000000000000' ) as RoleId,
		isnull ( up.CallerId, '' ) as CallerId,
		isnull ( u.IsDeleted, 0 ) as IsDeleted,
		u.DeleteDT
	From A_Users u
		left join A_UserParams up on u.Id = up.IdUser
End
GO


if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Admin_UserGroup_User_Save' ))
	Drop procedure dbo.A_Admin_UserGroup_User_Save
GO

Create procedure [dbo].[A_Admin_UserGroup_User_Save]
	@id		uniqueidentifier,
	@name		nvarchar ( 200 ),
	@info		nvarchar ( 1000 ),
	@parentgroupid	uniqueidentifier,
	@login		nvarchar ( 50 ),
	@password	nvarchar ( 50 ),	
	@type		int,
	@isoperator	bit,
	@roleid		uniqueidentifier,
	@callerid	nvarchar ( 20 ) = '',
	@isdeleted	bit = 0,
	@deletedt	datetime = NULL
as
begin

	Declare @deletedtN DateTime
	Select @deletedtN = case when (( @isdeleted is null ) or ( @isdeleted = 0 )) then NULL else @deletedt end

	Update A_Users
	Set 	
		Name 		= @name,
		Info 		= @info,
		ParentGroupId 	= @parentgroupid,
		Login		= @login,
		Password 	= @password,
		Type		= @type,
		IsDeleted	= @isdeleted,
		DeleteDT	= @deletedtN
	Where 
		Id 		= @id
	
	if ( @@rowcount = 0 )
	begin
		Insert 
		Into A_Users 
			( Id, Name, Info, ParentGroupId, Login, Password, Type, Enabled, IsDeleted, DeleteDT )
		Values
			( @id, @name, @info, @parentgroupid, @login, @password, @type, 1, @isdeleted, @deletedt )
	end

	Update A_UserParams
	Set	IdRole = @roleid,
		CallerId = @callerid,
		IsOperator = @isoperator
	Where IdUser = @id

	if ( @@rowcount = 0 )
		Insert 
		Into A_UserParams ( IdUser, IdRole, IsOperator, CallerId, IsRedirect, IsCC, IsLunch  )
		Values	( @id, @roleid, @isoperator, @callerid, 0, 0, 0 )



	if ( @isoperator = 0 )
	  begin
		Delete 
		From C_GroupOperators
		Where IDoperator = @id
	  end


	exec A_Admin_UserGroup_Group_SiftEmpty


	--      
	If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
	If Exists ( Select * FROM oktell_cc_temp..sysobjects where id = Object_Id ( 'oktell_cc_temp.dbo.A_Cube_CC_Cat_OperatorInfo' ) )
		Update oktell_cc_temp.dbo.A_Cube_CC_Cat_OperatorInfo
		Set Name = @name
		Where Id = @id

end
GO


if exists ( select * from sysobjects where id = object_id ( 'dbo.C_GroupsOperators_GetAll' ))
	Drop procedure dbo.C_GroupsOperators_GetAll
GO

Create Procedure [dbo].[C_GroupsOperators_GetAll]
As
begin
	Select u.id as uid, u.name as uname, g.id as gid, g.caption as gcaption
	FROM 	
		A_Users u inner join 
		A_UserParams up on u.id = up.iduser LEFT join
		C_GroupOperators go on go.IdOperator = up.IdUser LEFT join
		C_Groups g on g.id = go.IdGroup
	Where ( up.IsOperator > 0 ) and 
		(( u.IsDeleted = 0 ) or ( u.IsDeleted is null ))
	Union All
	Select null, null, g.id as gid, g.caption as gcaption
		From C_Groups g where not g.id in ( Select idgroup from C_GroupOperators )

end
GO
