﻿if exists ( select * from sysobjects where id=object_id ('dbo.A_Chat_Messages_GetTopXByOldMapper' ))
	Drop procedure dbo.A_Chat_Messages_GetTopXByOldMapper
GO

Create procedure [dbo].[A_Chat_Messages_GetTopXByOldMapper]
	@authorizeduserid	uniqueidentifier,
	@opponentid uniqueidentifier,
	@start int,
	@count int,
	@search1 nvarchar(1000) = '',
	@search2 nvarchar(1000) = '',
	@dtstartutc datetime = null,
	@dtstoputc datetime = null
as
begin
		-- Возвращает список нескольких последних сообщений по диалогу для пользователя
	

		-- Нарезаем нужный интервал из сортированного по дате
		if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#interval' ))
			Drop table #interval

		Create table #interval ( IdMessage uniqueidentifier, IdDialog uniqueidentifier )
				
		-- Фильтр для даты
		Declare @filterdate nvarchar(4000)
		Set @filterdate = ''
		if ( @dtstartutc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC >= @dt1 )'
		if ( @dtstoputc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC <= @dt2 )'

		-- Фильтр для текста
		Declare @filtertext nvarchar(4000)
		Set @filtertext = ''	

		if (( @search1 = '' ) and ( @search2 = '' )) begin

			Set @filtertext = ''

		end else if (( @search1 = '' ) or ( @search2 = '' )) begin

			Declare @search nvarchar(4000)
			if ( @search1 != '' )
				Set @search = @search1
			else
				Set @search = @search2

			Set @filtertext = ' and (Text like ''%' + @search + '%'')'

		end else begin

			Set @filtertext = ' and ((Text like ''%' + @search1 + '%'') or (Text like ''%' + @search2 + '%''))'

		end


		Declare @sql nvarchar(4000)

		-- Сначала ограничение (на последней странице меньше записей). 
		Declare @rowcount int			
		Set @sql = 'Select @rc = Count(*) 
					From dbo.[A_Chat_Messages] msg
						Inner join dbo.A_Chat_Members mem on (mem.IdDialog = msg.IdDialog) 
							and (   ((mem.IdUser = @me) and (msg.IdUser = @author)) 
								 or ((mem.IdUser = @author) and (msg.IdUser = @me)))
					Where (1=1)' + @filterdate + @filtertext
		exec sp_executesql @sql, N'@rc int output, @author uniqueidentifier, @me uniqueidentifier, @dt1 datetime, @dt2 datetime', 
						@rc = @rowcount OUTPUT, @author = @opponentid, @me = @authorizeduserid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		-- Корректировка @count 
		if ( @rowcount < @start + @count )
			Set @count = case when ( @rowcount - @start > 0 ) then @rowcount - @start else 0 end

		-- Потом выборка идентификаторов
		if ( @count > 0 ) begin

			Set @sql = 'Insert into #interval ( IdMessage, IdDialog )				
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage, IdDialog
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' msg.IdMessage, msg.IdDialog, msg.DateUTC
							From dbo.[A_Chat_Messages] msg
								Inner join dbo.A_Chat_Members mem on (mem.IdDialog = msg.IdDialog) 
									and (   ((mem.IdUser = @me) and (msg.IdUser = @author)) 
										 or ((mem.IdUser = @author) and (msg.IdUser = @me)))
							Where (1=1)' + @filterdate + @filtertext + '
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@author uniqueidentifier, @me uniqueidentifier, @dt1 datetime, @dt2 datetime', 
						@author = @opponentid, @me = @authorizeduserid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		end 
	
		
		-- Собственно возвращаем сообщения
		Select	m.IdMessage,
				m.IdUser as AuthorId,
				u.Login as AuthorLogin,
				u.Name as AuthorName,
				m.DateUTC,
				m.Text,
				case when ( m.IdUser = @authorizeduserid ) then	--'from me'
							case when (
										(
											select count(*)
											from dbo.[A_Chat_NewMessages] nm
											where nm.IdMessage = i.IdMessage 
												and nm.IdDialog = i.IdDialog
										)
											>=
										(
											select count(*)
											from dbo.[A_Chat_Members] mem
											where mem.IdDialog = i.IdDialog
												and mem.IdUser != @authorizeduserid
										)
									)
									then 1 else 0 
							end
					else null
					end FromMeIsNew,
				case when ( m.IdUser != @authorizeduserid ) then	--'for me'
							case when ( select count(*) 
										from dbo.[A_Chat_NewMessages] 
										where IdMessage = i.IdMessage 
											and IdDialog = i.IdDialog
											and IdUser = @authorizeduserid ) = 0 
									then 0 else 1 
						end
					else null
					end ForMeIsNew
		From #interval as i
			Inner join dbo.[A_Chat_Messages] as m on m.IdMessage = i.IdMessage
			Inner join dbo.[A_Users] u on u.Id = m.IdUser
		Order by m.DateUTC Asc
		
		-- Файлы привязанные
		Select	IdMessage,
				Id,
				FilePath,
				FileSize
		From dbo.[A_Chat_MessageFile]
		Where IdMessage in (select IdMessage from #interval)
		Order by IdMessage
		

		drop table #interval

end
GO