﻿if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_UserRoles_CheckPrivilege' ))
	Drop procedure dbo.A_UserRoles_CheckPrivilege
GO

Create procedure dbo.A_UserRoles_CheckPrivilege
	@privilegecode nvarchar(100),
	@authorizeduserid uniqueidentifier,
	@isenabled int OUTPUT

as
begin
	-- Достает для указанного пользователя разрешение указанной привилегии (по всем текущим ролям)

	-- Ид привилегии
	Declare @privilegeid uniqueidentifier 
	Select @privilegeid = id 
	From A_UserRoles_Rights 
	Where code like @privilegecode
	
	-- По умолчанию запрещено
	Set @isenabled = 0

	-- Выясняем, может доминирующая роль имеет установку, она приоритетна
	Declare @dom int
	Select @dom = Value 
	From A_UserRoles_RightsOfRoles rr
	Where rr.IdRole = @authorizeduserid
		and rr.IdRight = @privilegeid
		
	if ( @dom = 1 ) begin
		Set @isenabled = 1
		return
	end else if ( @dom = 0 ) begin
		Set @isenabled = 0
		return
	end
			
	-- Начинаем смотреть недоменирующие роли, принадлежащие пользователю
	--   Постепенно углубляясь в наследование по тем ролям, у которых привилегия наследуется и есть базовая роль
	Declare @cnt int, @i int
	
	Declare @baseroles table ( id uniqueidentifier, alreadyanalyzed bit, nestlevel int )
	Insert into @baseroles (id, alreadyanalyzed, nestlevel)
		Select IdRole, 0, 0
		From A_UserRoles_RolesOfUsers ru
			Inner join A_UserRoles_Roles r on r.Id = ru.IdRole
		Where (ru.IdUser = @authorizeduserid)
			and (r.Domination = 0)
		
	Select @cnt = @@rowcount, @i = 0
		
	while (@cnt > 0 ) begin
		
		Set @i = @i + 1
		
		-- Если есть хотя бы у одной роли на этом уровне разрешение привилегии - сразу считаем что разрешено
		Select @cnt=Sum(rr.Value)
		From @baseroles r
			Inner join A_UserRoles_RightsOfRoles rr on rr.IdRole = r.Id
		Where (r.alreadyanalyzed = 0)
			and (rr.IdRight = @privilegeid)
		
		if (@cnt > 0) begin	
			Set @isenabled = 1
			return
		end
		
		-- Больше роли этого уровня не анализировать
		Update @baseroles
		Set alreadyanalyzed = 1
		
		-- На следующую итерацию анализа - базовые роли для ролей уже проанализированного уровня, 
		--  у которых привилегия наследуется, есть базовая роль и базовую роль еще не анализировали
		Insert into @baseroles (Id, alreadyanalyzed, nestlevel)
			Select r.BaseRole, 0, @i
			From @baseroles br
				Inner join A_UserRoles_Roles r on r.Id = br.Id
			Where (br.nestlevel = @i - 1)
				And (not exists ( Select IdRight From A_UserRoles_RightsOfRoles rr Where (rr.IdRole = r.Id) and (rr.IdRight = @privilegeid) and (rr.Value in (1,0))))
				And (not exists ( Select Id From @baseroles br2 Where br2.Id = r.BaseRole ))
				And (not r.BaseRole is NULL)
				
		Set @cnt = @@rowcount
	
	end
end
GO

if exists ( Select * from sysobjects where id = object_id ( 'dbo.WO_UserRoles_CheckPrivilege' ))
	Drop procedure dbo.WO_UserRoles_CheckPrivilege
GO

Create procedure dbo.WO_UserRoles_CheckPrivilege
	@privilegecode nvarchar(100),
	@authorizeduserid uniqueidentifier

as
begin
	-- Достает для указанного пользователя разрешение указанной привилегии (по всем текущим ролям)
	
	Declare @en int
	exec dbo.A_UserRoles_CheckPrivilege @privilegecode=@privilegecode, @authorizeduserid = @authorizeduserid, @isenabled = @en OUTPUT
	Select @en
	
end
GO