﻿if exists ( Select * From sysobjects where id= object_id ( 'dbo.A_Stat_Service_FillTempByFilter' ))
	Drop procedure dbo.A_Stat_Service_FillTempByFilter
GO


Create Procedure [dbo].[A_Stat_Service_FillTempByFilter]
	@iduser uniqueidentifier,
	@dstart datetime,
	@dstop datetime,
	@tstart datetime,
	@tstop datetime,
	@lentime datetime,
	@taskid uniqueidentifier = NULL, --Task defined
	@fromcc bit = NULL,
	@operatorid uniqueidentifier = NULL, --Operator defined
	@innerprefix nvarchar(100) = NULL, --Internal Number defined
	@outnumber nvarchar(100) = NULL,  --Out Number defined
	@conntype int = -1,
	@showconference bit = 0,
	@showfailed bit = 0,
	@comment nvarchar ( 4000 ) = NULL,
	@idconn uniqueidentifier = '00000000-0000-0000-0000-000000000000',
	@idchain uniqueidentifier = '00000000-0000-0000-0000-000000000000'
as
begin

----------------------------------------------
------- Подготовка параметров ----------------
----------------------------------------------

	Declare @filtercomment bit
	Set @filtercomment = case when ( @comment IS NULL or @comment like '' ) then 0 else 1 end

	Declare @tstartfloat float  
	Declare @tstopfloat float 
	Set @tstartfloat = Convert ( float, @tstart )  
	Set @tstopfloat  = Convert ( float, @tstop )  
	Set @tstartfloat = @tstartfloat - FLOOR ( @tstartfloat )  
	Set @tstopfloat = @tstopfloat - FLOOR ( @tstopfloat )  
	If ( @tstopfloat <= @tstartfloat )
	    Set @tstopfloat = @tstopfloat + 1

	Declare @lt float
	Set @lt = Convert ( float, @lentime ) - Floor ( Convert ( float, @lentime ) )

	Declare @idempty uniqueidentifier
	Set @idempty = '00000000-0000-0000-0000-000000000000'
	
	Declare @idconfuser uniqueidentifier
	Set @idconfuser = 'CF000000-0000-0000-0000-000000000000'

--------------
--------------

	Declare @intnumbers table ( nid uniqueidentifier, prefix nvarchar ( 50 ) collate database_default, addrtype int, reactid uniqueidentifier )

	if ( @innerprefix is not null ) begin
		
		if exists ( select * from tempdb..sysobjects where id=OBJECT_ID ( 'tempdb..#t_c' ))
			Drop table #t_c
		if exists ( select * from tempdb..sysobjects where id=OBJECT_ID ( 'tempdb..#temp_c' ))
			Drop table #temp_c
		if exists ( select * from tempdb..sysobjects where id=OBJECT_ID ( 'tempdb..#removed_c' ))
			Drop table #removed_c
		Create table #t_c ( nid uniqueidentifier, reactid uniqueidentifier )
		Create table #temp_c ( id uniqueidentifier, oldrid uniqueidentifier, newrid uniqueidentifier )
		Create table #removed_c ( nid uniqueidentifier, reactid uniqueidentifier )

		Insert Into #t_c
			Select np.Id, rr.ReactId
			From A_NumberPlan np
				Inner Join A_NumberPlanAction na on na.NumId = np.Id
				Inner Join A_Rules r on na.ExtraId = r.Id
				Inner Join A_RuleRecords rr on rr.RuleId = r.Id
			Where na.NumType = 0 --только стандартные (расчет отделов по входящим в состав элементам)

		
		Declare @cnt int, @rowcount int, @msg int
		Select @cnt = 0, @rowcount = 1
		
		While ( @rowcount > 0 ) and ( @cnt < 10 )
		begin
			Set @cnt = @cnt + 1
		
			--Выгребаем ссылки на номера @cnt - уровня
			Insert Into #temp_c
				Select t.nid, t.reactid, t1.reactid
				From #t_c t
					 Inner Join #t_c t1 on ( t.Reactid = t1.nId ) and ( t1.nId != t.nId ) and ( t1.reactid != t.nId )
				Where t.ReactId != t.Nid
					and not exists ( select nid from #removed_c r where t.nid = r.nid and t1.reactid = r.reactid )
					and not exists ( select nid from #t_c r where t.nid = r.nid and t1.reactid = r.reactid )
				Group by t.nid, t.reactid, t1.reactid
			
			-- 	
			Insert into #removed_c ( nid, reactid )
				Select t.nid, t.reactid
				From #t_c t
					Inner join #temp_c tmp on t.nid= tmp.id and t.reactid = tmp.oldrid
				Where not exists ( select nid from #removed_c r where t.nid = r.nid and t.reactid = r.reactid )
				Group by t.nid, t.reactid
					
								
			--Удаляем то что будем менять
			Delete From #t_c
				From #temp_c
			Where nid = id and reactid = oldrid


			--вставляем замены
			Insert Into #t_c ( nid, reactid )
				Select id, newrid From #temp_c

			--чистим времянку
			Select @rowcount = COUNT ( * ) 
			From #temp_c
					
			Truncate table #temp_c
		end
			
		-- удаляем остатки ссылок на номера
		Delete From #t_c
			From A_NumberPlan np
		Where reactid = np.Id

		-- Вставляем результат
		Declare @rr table ( ReactId uniqueidentifier, InnerAddressType int )

		Insert into @rr ( ReactId, InnerAddressType )
			Select ReactId, Min ( InnerAddressType )
			From A_RuleRecords rr 
			Group by ReactId

		Insert into @intnumbers ( nid, prefix, addrtype, reactid )
			Select t.nid, np.Prefix, rr.InnerAddressType, t.reactid
			From #t_c t
				Inner join A_NumberPlan np on t.nid = np.Id				
				Inner join @rr rr on rr.ReactId = t.ReactId

	end		

--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
    ----------------------------------------------------
    --- Коммутации -------------------------------------
    ----------------------------------------------------

	-- Временные таблицы для коммутаций
	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Connections_1x1_tmp2' ))
		Drop table #A_Stat_Connections_1x1_tmp2
	Create table #A_Stat_Connections_1x1_tmp2 ( Id uniqueidentifier Primary Key )

	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Connections_1x1_tmp' ))
		Drop table #A_Stat_Connections_1x1_tmp
	Create table #A_Stat_Connections_1x1_tmp ( Id uniqueidentifier PRIMARY KEY, AUSerId uniqueidentifier, BUSerId uniqueidentifier, ALineId uniqueidentifier, BLineId uniqueidentifier )

    -----------------------------------------------
    ------- Начальная выборка коммутаций ----------
    -----------------------------------------------

	if (( @idchain = '00000000-0000-0000-0000-000000000000' ) and ( @idconn = '00000000-0000-0000-0000-000000000000' )) begin

		-- Date & Time Intervals
		-- По идее можно показывать все конференции и коммутации, которые шли в указанный период времени
		--   Но тогда графики нельзя считать аддитивными, поэтому все по точке начала будем собирать
		Insert into #A_Stat_Connections_1x1_tmp 
		Select Id, AUSerId , BUSerId , ALineId , BLineId 
		From A_Stat_Connections_1x1 sc --With ( TabLock )
		Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
			and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) >= @tstartfloat )  
			and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) <= @tstopfloat ) 
			and ( Convert ( float, TimeStop ) - Convert ( float, TimeAnswer ) >= @lt )
			and (( @fromcc is null ) or ( FromCC <= @fromcc ))
			and (@conntype = -1 or (@conntype = 1 and ConnectionType in (4,5)) or (@conntype = 2 and ConnectionType in (1,6,7)) or (@conntype = 3 and ConnectionType in (2,3,8)))

	end else if ( @idchain = '00000000-0000-0000-0000-000000000000' ) begin
		
		Insert into #A_Stat_Connections_1x1_tmp 
		Select Id, AUSerId , BUSerId , ALineId , BLineId 
		From A_Stat_Connections_1x1 sc --With ( TabLock )
		Where ( Id = @idconn )
			and ( Convert ( float, TimeStop ) - Convert ( float, TimeAnswer ) >= @lt )
			and (( @fromcc is null ) or ( FromCC <= @fromcc ))
			and (@conntype = -1 or (@conntype = 1 and ConnectionType in (4,5)) or (@conntype = 2 and ConnectionType in (1,6,7)) or (@conntype = 3 and ConnectionType in (2,3,8)))
	
	end else if ( @idconn = '00000000-0000-0000-0000-000000000000' ) begin
		
		Insert into #A_Stat_Connections_1x1_tmp 
		Select Id, AUSerId , BUSerId , ALineId , BLineId 
		From A_Stat_Connections_1x1 sc --With ( TabLock )
		Where ( IdChain = @idchain )			
			and ( Convert ( float, TimeStop ) - Convert ( float, TimeAnswer ) >= @lt )
			and (( @fromcc is null ) or ( FromCC <= @fromcc ))
			and (@conntype = -1 or (@conntype = 1 and ConnectionType in (4,5)) or (@conntype = 2 and ConnectionType in (1,6,7)) or (@conntype = 3 and ConnectionType in (2,3,8)))
	
	end else begin

		Insert into #A_Stat_Connections_1x1_tmp 
		Select Id, AUSerId , BUSerId , ALineId , BLineId 
		From A_Stat_Connections_1x1 sc --With ( TabLock )
		Where ( IdChain = @idchain )
			and ( Id = @idconn )		
			and ( Convert ( float, TimeStop ) - Convert ( float, TimeAnswer ) >= @lt )
			and (( @fromcc is null ) or ( FromCC <= @fromcc ))
			and (@conntype = -1 or (@conntype = 1 and ConnectionType in (4,5)) or (@conntype = 2 and ConnectionType in (1,6,7)) or (@conntype = 3 and ConnectionType in (2,3,8)))
	end

    --------------------------------------------
    ------- Фильтр по правам контроля ----------
    --------------------------------------------

	Declare @commpermitted table ( Id uniqueidentifier PRIMARY KEY ) -- Id Connection

	------------------------------------
	-- По пользователям
	------------------------------------

	Declare @upermitted table ( Id uniqueidentifier Primary Key )
	Insert into @upermitted
		Select UserB
		From A_UserControls auc
		Where UserA = @iduser

	If not exists ( Select UserB From A_UserControls Where ( UserA = @iduser ) and ( UserB = @iduser )) 
		Insert into @upermitted ( Id ) 
		Values ( @iduser )

	Insert into @commpermitted
		Select sc.ID --auc.UserB   	
		From @upermitted up
			Inner join #A_Stat_Connections_1x1_tmp sc on up.Id = sc.AUserId
	
	Insert into @commpermitted
		Select sc.ID --auc.UserB   	
		From @upermitted up
			Inner join #A_Stat_Connections_1x1_tmp sc on up.Id = sc.BUserId
		Where not exists ( Select id From @commpermitted Where Id = sc.ID)


	------------------------------------
	-- По линиям
	------------------------------------

	Declare @lpermitted table ( Id uniqueidentifier Primary Key )
	Insert into @lpermitted
    	    Select IdLine
    	    From A_UserControlLines
    	    Where ( UserA = @iduser )
          	and ( IdLine != '11111111-1111-1111-1111-111111111111' )

	Declare @canlistenext int       
	Select @canlistenext = Count ( * ) 
	From A_UserControlLines 
	Where ( UserA = @iduser ) 
		and ( IdLine = '11111111-1111-1111-1111-111111111111' )

	if ( @canlistenext > 0 ) begin
    		Insert into @lpermitted
	            Select Id 
        	    From A_ServerExtLines sel
	            Where not exists ( Select id From @lpermitted Where Id = sel.ID )

		Insert into @lpermitted ( Id ) 
		Values ( 'BF000000-0000-0000-0000-000000000000' )		  
	end
     
	Insert into @commpermitted
    	    Select distinct sc.ID            
    	    From @lpermitted l
	          Inner join #A_Stat_Connections_1x1_tmp sc on ( sc.ALineId = l.Id ) or ( sc.AUserId = l.Id )
    	    Where not exists ( Select id From @commpermitted Where Id = sc.ID )

	Insert into @commpermitted            
    	    Select distinct sc.ID            
    	    From @lpermitted l
          	Inner join #A_Stat_Connections_1x1_tmp sc on ( sc.BLineId = l.Id ) or ( sc.BUserId = l.Id )
    	    Where not exists ( Select id From @commpermitted Where Id = sc.ID)

	Delete 
	From #A_Stat_Connections_1x1_tmp 
	Where id not in ( Select Id from @commpermitted )


    ------------------------------------------
    ------- Фильтр по номерам ----------------
    ------------------------------------------

	----------------------------------
	-- Внутренние номера
	----------------------------------

	if ( @innerprefix is not null ) begin
		
		--Users
		Insert into #A_Stat_Connections_1x1_tmp2 
  		    Select Id
		    From #A_Stat_Connections_1x1_tmp sc
				Inner join @intnumbers t on sc.AUserID = t.reactid
		    Where ( addrtype = 0 ) 
				and ( prefix like @innerprefix )

		Insert into #A_Stat_Connections_1x1_tmp2 
		    Select Id
		    From #A_Stat_Connections_1x1_tmp sc
				Inner join @intnumbers t on sc.BUserID = t.reactid
		    Where ( addrtype = 0 ) 
				and ( prefix like @innerprefix )
				and not exists ( Select Id From #A_Stat_Connections_1x1_tmp2 Where id = sc.ID )

		--Lines
		Insert into #A_Stat_Connections_1x1_tmp2 
		    Select Id
		    From #A_Stat_Connections_1x1_tmp sc
				Inner join @intnumbers t on sc.ALineId = t.reactid
		    Where ( addrtype = 1 ) 
				and ( prefix like @innerprefix )
				and not exists ( Select Id From #A_Stat_Connections_1x1_tmp2 Where Id = sc.ID )

		Insert into #A_Stat_Connections_1x1_tmp2 
		    Select Id
		    From #A_Stat_Connections_1x1_tmp sc
				Inner join @intnumbers t on sc.BLineId = t.reactid
		    Where ( addrtype = 1 ) 
				and ( prefix like @innerprefix )
				and not exists ( Select Id From #A_Stat_Connections_1x1_tmp2 Where Id = sc.ID )
		
		Delete 
		From #A_Stat_Connections_1x1_tmp 
		Where Id not in ( Select Id From #A_Stat_Connections_1x1_tmp2 )

		Truncate table #A_Stat_Connections_1x1_tmp2
		
	end 

	---------------------------------------
	-- Внешние номера
	---------------------------------------

	if ( @OutNumber is not null ) begin

		Insert into #A_Stat_Connections_1x1_tmp2 
		    Select Id
		    From A_Stat_Connections_1x1 sc
		    Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
			and ( AOutNumber like '%'+@OutNumber+'%' )

		Insert into #A_Stat_Connections_1x1_tmp2 
		    Select Id
		    From A_Stat_Connections_1x1 sc
		    Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
			and ( BOutNumber like '%'+@OutNumber+'%' )
			and not exists ( Select id From #A_Stat_Connections_1x1_tmp2 Where id = sc.ID )

		Delete 
		From #A_Stat_Connections_1x1_tmp 
		Where id not in ( Select Id From #A_Stat_Connections_1x1_tmp2 )

		Truncate table #A_Stat_Connections_1x1_tmp2

	end

    ----------------------------------------------------
    ------- Фильтр по пользователю ---------------------
    ----------------------------------------------------

	if ( @operatorid is not null ) begin

		Insert into #A_Stat_Connections_1x1_tmp2 
		    Select Id
		    From #A_Stat_Connections_1x1_tmp sc
		    Where AUserId = @operatorid

		Insert into #A_Stat_Connections_1x1_tmp2 
		select Id 
		from #A_Stat_Connections_1x1_tmp sc
		where BUserId = @operatorid
			and not exists ( Select id From #A_Stat_Connections_1x1_tmp2 where id = sc.ID )

		Delete 
		From #A_Stat_Connections_1x1_tmp 
		Where id not in ( Select Id From #A_Stat_Connections_1x1_tmp2 )
		
		Truncate table #A_Stat_Connections_1x1_tmp2
	end

    ------------------------------------------------
    -- Фильтр по комментарию (если установлен) 
    ------------------------------------------------

	if ( @filtercomment > 0 ) begin

		Delete 
		From #A_Stat_Connections_1x1_tmp
		Where id not in ( Select Id From A_Stat_Comments Where Comment like '%' + @comment + '%' )
	end


--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
    ------------------------------------------------
    ------- Неудачные исходящие --------------------
    ------------------------------------------------

    if (( @showfailed = 1 ) and ( @lt = 0 )) begin

	-- Временные таблицы для конференций
	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Failed_tmp' ))
		Drop table #A_Stat_Failed_tmp
	Create table #A_Stat_Failed_tmp ( Id uniqueidentifier PRIMARY KEY, AUSerId uniqueidentifier, ALineId uniqueidentifier )

	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Failed_tmp2' ))
		Drop table #A_Stat_Failed_tmp2
	Create table #A_Stat_Failed_tmp2 ( Id uniqueidentifier PRIMARY KEY )

    ------------------------------------------------
    ------- Начальная выборка неудачных  ----------
    ------------------------------------------------
	
	if (( @idchain = '00000000-0000-0000-0000-000000000000' ) and ( @idconn = '00000000-0000-0000-0000-000000000000' )) begin

		-- По идее можно показывать все конференции и коммутации, которые шли в указанный период времени
		--   Но тогда графики нельзя считать аддитивными, поэтому все по точке начала будем собирать
		Insert into #A_Stat_Failed_tmp
			Select sc.Id, sc.AUserId, sc.ALineId
			From A_Stat_FailedCalls sc
			Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
				and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) >= @tstartfloat )  
				and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) <= @tstopfloat ) 
				and ( Convert ( float, TimeStart ) - Convert ( float, TimeStart ) >= @lt )
	end

    --------------------------------------------
    ------- Фильтр по правам контроля ----------
    --------------------------------------------

	------------------------------------
	-- По пользователям
	------------------------------------

	Insert into @commpermitted
	    Select sc.ID
	    From @upermitted up
		Inner join #A_Stat_Failed_tmp sc on up.Id = sc.AUserId

	------------------------------------
	-- По линиям
	------------------------------------

	Insert into @commpermitted
    	    Select distinct sc.ID            
    	    From @lpermitted l
	          Inner join #A_Stat_Failed_tmp sc on ( sc.ALineId = l.Id ) or ( sc.AUserId = l.Id )
    	    Where not exists ( Select id From @commpermitted Where Id = sc.ID )

	------------------------------------
	-- Удаляем те что нельзя
	------------------------------------

	Delete 
	From #A_Stat_Failed_tmp 
	Where id not in ( Select Id from @commpermitted )

    ------------------------------------------------
    -------- Фильтр по номерам ---------------------
    ------------------------------------------------

	------------------------------------
	-- Внутренние номера
	------------------------------------

	if ( @innerprefix is not null ) begin

		--Users
		Insert into #A_Stat_Failed_tmp2 
  		    Select Id
		    From #A_Stat_Failed_tmp sc
				Inner join @intnumbers t on sc.AUserID = t.reactid
		    Where ( addrtype = 0 ) 
				and ( prefix like @innerprefix )

		--Lines
		Insert into #A_Stat_Failed_tmp2 
		    Select Id
		    From #A_Stat_Failed_tmp sc
				Inner join @intnumbers t on sc.ALineId = t.reactid
		    Where ( addrtype = 1 ) 
				and ( prefix like @innerprefix )
				and not exists ( Select Id From #A_Stat_Failed_tmp2 Where Id = sc.ID )

		
		Delete #A_Stat_Failed_tmp 
		Where Id not in ( Select Id From #A_Stat_Failed_tmp2 )

		Truncate table #A_Stat_Failed_tmp2
	end

	---------------------------------------
	-- Внешние номера
	---------------------------------------

	if ( @OutNumber is not null ) begin

		Insert into #A_Stat_Failed_tmp2 
		    Select Id
		    From A_Stat_FailedCalls sc
		    Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
			and ( AOutNumber like '%'+@OutNumber+'%' )

		Insert into #A_Stat_Failed_tmp2 
		    Select Id
		    From A_Stat_FailedCalls sc
		    Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
			and ( ANumberDialed like '%'+@OutNumber+'%' )
			and not exists ( Select id From #A_Stat_Failed_tmp2 Where id = sc.ID )

		Delete 
		From #A_Stat_Failed_tmp 
		Where id not in ( Select Id From #A_Stat_Failed_tmp2 )

		Truncate table #A_Stat_Failed_tmp2

	end

    ----------------------------------------------------
    ------- Фильтр по пользователю ---------------------
    ----------------------------------------------------

	if ( @operatorid is not null ) begin

		Insert into #A_Stat_Failed_tmp2 
		    Select Id
		    From #A_Stat_Failed_tmp sc
		    Where AUserId = @operatorid


		Delete 
		From #A_Stat_Failed_tmp 
		Where id not in ( Select Id From #A_Stat_Failed_tmp2 )
		
		Truncate table #A_Stat_Failed_tmp2
	end

    ------------------------------------------------
    -- Фильтр по комментарию (если установлен) 
    ------------------------------------------------

	if ( @filtercomment > 0 ) begin

		Delete 
		From #A_Stat_Failed_tmp
		Where id not in ( Select Id From A_Stat_Comments Where Comment like '%' + @comment + '%' )
	  end

    end


--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
    ------------------------------------------------
    ------- Конференции ----------------------------
    ------------------------------------------------

    if ( @showconference = 1 ) begin

	-- Временные таблицы для конференций
	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Conference_tmp' ))
		Drop table #A_Stat_Conference_tmp
	Create table #A_Stat_Conference_tmp ( Id uniqueidentifier PRIMARY KEY, RecordAccess int, Director uniqueidentifier )

	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Conference_tmp2' ))
		Drop table #A_Stat_Conference_tmp2
	Create table #A_Stat_Conference_tmp2 ( Id uniqueidentifier PRIMARY KEY )

    ------------------------------------------------
    ------- Начальная выборка конференций ----------
    ------------------------------------------------

	if (( @idchain = '00000000-0000-0000-0000-000000000000' ) and ( @idconn = '00000000-0000-0000-0000-000000000000' )) begin

		-- По идее можно показывать все конференции и коммутации, которые шли в указанный период времени
		--   Но тогда графики нельзя считать аддитивными, поэтому все по точке начала будем собирать
		Insert into #A_Stat_Conference_tmp
			Select sc.Id, sc.RecordAccess, sc.Director 
			From A_Stat_Conference sc
			Where ( TimeStart >= @dstart ) and ( TimeStart < @dstop ) 
				and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) >= @tstartfloat )  
				and ( Convert ( float, TimeStart ) - FLOOR ( Convert ( float, TimeStart ) ) <= @tstopfloat ) 
				and ( Convert ( float, TimeStop ) - Convert ( float, TimeStart ) >= @lt )
				and (( @conntype = -1 ) or ( @conntype = 4 ))

	end else if ( @idchain = '00000000-0000-0000-0000-000000000000' ) begin

		Insert into #A_Stat_Conference_tmp
			Select sc.Id, sc.RecordAccess, sc.Director 
			From A_Stat_Conference sc
				Inner join A_Stat_Conference_Competitors scc on sc.Id = scc.IdConf
			Where ( scc.IdConn = @idconn )
				and ( Convert ( float, sc.TimeStop ) - Convert ( float, sc.TimeStart ) >= @lt )
				and (( @conntype = -1 ) or ( @conntype = 4 ))

	end else if ( @idconn = '00000000-0000-0000-0000-000000000000' ) begin	
	
		Insert into #A_Stat_Conference_tmp
			Select sc.Id, sc.RecordAccess, sc.Director 
			From A_Stat_Conference sc
				Inner join A_Stat_Conference_Competitors scc on sc.Id = scc.IdConf
			Where ( sc.Id = @idchain )
				and ( Convert ( float, sc.TimeStop ) - Convert ( float, sc.TimeStart ) >= @lt )
				and (( @conntype = -1 ) or ( @conntype = 4 ))
	end else begin

		Insert into #A_Stat_Conference_tmp
			Select sc.Id, sc.RecordAccess, sc.Director 
			From A_Stat_Conference sc
				Inner join A_Stat_Conference_Competitors scc on sc.Id = scc.IdConf
			Where ( sc.Id = @idchain )
				and ( scc.IdConn = @idconn )
				and ( Convert ( float, sc.TimeStop ) - Convert ( float, sc.TimeStart ) >= @lt )
				and (( @conntype = -1 ) or ( @conntype = 4 ))
	end


    --------------------------------------------------------
    ------- Фильтры ----------------------------------------
    --------------------------------------------------------

	------------------------------------------------
	-- Фильтр по комментарию (если установлен) 
	------------------------------------------------

	if ( @filtercomment > 0 ) begin
		Delete 
		From #A_Stat_Conference_tmp
		Where id not in ( Select Id From A_Stat_Comments Where Comment like '%' + @comment + '%' )
	end

	------------------------------------------------
	-- фильтр по правам
	------------------------------------------------

	Insert into #A_Stat_Conference_tmp2
		Select sc.Id From #A_Stat_Conference_tmp sc
			Inner join @upermitted up on sc.Director = up.Id
		Where sc.RecordAccess = 1 --Директор слушает 

	Insert into #A_Stat_Conference_tmp2
		Select sc.Id From #A_Stat_Conference_tmp sc			
			Inner join A_Stat_Conference_Competitors scc on sc.Id = scc.IdConf
			Inner join @upermitted up on scc.IdUser = up.Id
		Where sc.RecordAccess = 2 --Участники слушают
		Group By sc.Id

	Insert into #A_Stat_Conference_tmp2
		Select sc.Id From #A_Stat_Conference_tmp sc
			Inner join A_Stat_Conference_RecordAccessUsers scr on sc.Id = scr.IdConf
			Inner join @upermitted up on scr.IdUser = up.Id
		Where sc.RecordAccess = 3 --Указанные участники слушают
		Group By sc.Id

	if ( @canlistenext > 0 ) begin 

	    -- Тот, у кого есть право на все внешние линии, слушает
	    Insert into #A_Stat_Conference_tmp2
		Select sc.Id From #A_Stat_Conference_tmp sc
			Inner join A_Stat_Conference_Competitors scc on scc.IdConf = sc.Id
		Where scc.IdUser = 'BF000000-0000-0000-0000-000000000000'
			And not exists ( Select id from #A_Stat_Conference_tmp2 where id = sc.Id )
		Group By sc.Id
        end

	----------------------------------
	-- Внутренние номера
	----------------------------------

	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Conference_tmp3' ))
		Drop table #A_Stat_Conference_tmp3
	Create table #A_Stat_Conference_tmp3 ( IdItem uniqueidentifier PRIMARY KEY )

	if ( @innerprefix is not null ) begin

		Insert into #A_Stat_Conference_tmp3 ( iditem )
			Select ReactId
			From @intnumbers t
			Where ( prefix like @innerprefix )
				and ( addrtype in ( 0, 1 ))
	end

    end

--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
--##########################################################################################################
-----------------------------------------------------------
------- Окончательная выборка -----------------------------
-----------------------------------------------------------

	if exists ( select * from tempdb..sysobjects where id=object_id ( 'tempdb..#A_Stat_Result' ))
		Drop table #A_Stat_Result

	--------------------------------
	-- Коммутации
	--------------------------------
	Select sc.*, 
		0 as IsConf, @idempty as Director, sc.TimeStart as CTS, 
		case When ( c.Id is null ) then 0 else 1 end as HasComment
	Into #A_Stat_Result
	From A_Stat_Connections_1x1 sc
		Inner join #A_Stat_Connections_1x1_tmp sct on sc.Id=sct.Id
		Left join A_Stat_Comments c on sc.Id = c.Id

	--------------------------------
	-- Неудачные исходящие
	--------------------------------

   	if (( @showfailed = 1 ) and ( @lt = 0 )) begin

	    Insert into #A_Stat_Result ( Id, IdChain, IdPrev, IdNext,
					TimeStart, TimeAnswer, TimeStop,
					ReasonStart, ReasonStop, StopSide,
					ALineId, BLineId,
					AOutNumber, BOutNumber, 
					ALineNum, BLineNum,
					AUserId, BUserId,
					ANumberDialed, BNumberDialed,
					IsRecorded, ConnectionType,
					AStr, BStr,
					FromCC, IsBetweenUsers, IdRecDir,
					IsConf, Director, CTS, HasComment )

 	    	Select sc.Id, sc.IdChain, @idempty as IdPrev, @idempty as IdNext, 
			sc.TimeStart, sc.TimeStop as TimeAnswer, sc.TimeStop,
			sc.ReasonStart, sc.ReasonFailed, 0 as StopSide, 
			sc.ALineId, sc.BLineId,
			sc.AOutNumber, sc.ANumberDialed as BOutNumber,
			sc.ALineNum, sc.BLineNum,
			sc.AUserId, @idempty as BUserId,
			sc.ANumberDialed, '' as BNumberDialed,
			0 as IsRecorded, 
			21 as ConnectionType,
			sc.AStr, sc.BStr, 
			sc.FromCC,
			case when ( sc.BLineId is NULL or sc.BLineId = @idempty ) then 1 else 0 end as IsBetweenUsers,
			0 as IdRecDir,
			0 as IsConf, @idempty as Director, sc.TimeStart as CTS,
			case When ( c.Id is null ) then 0 else 1 end as HasComment
 
		From A_Stat_FailedCalls sc
			Inner join #A_Stat_Failed_tmp sct on sc.Id = sct.Id
			Left join A_Stat_Comments c on sc.Id = c.Id
	end

	------------------------------------------------
	---- Конференции --------------------------------
	------------------------------------------------

    	if ( @showconference = 1 ) begin

	    Insert into #A_Stat_Result ( Id, IdChain, IdPrev, IdNext,
					TimeStart, TimeAnswer, TimeStop,
					ReasonStart, ReasonStop, StopSide,
					ALineId, BLineId,
					AOutNumber, BOutNumber, 
					ALineNum, BLineNum,
					AUserId, BUserId,
					ANumberDialed, BNumberDialed,
					IsRecorded, ConnectionType,
					AStr, BStr,
					FromCC, IsBetweenUsers, IdRecDir,
					IsConf, Director, CTS, HasComment )

   	    	-- Ид конференции пусть выступает цепочкой, по ней можно будет группировать конференции в статистике
	    	-- А ид коммутации так и остается ид коммутации
	    	Select IdConn, sc.Id as IdChain, @idempty as IdPrev, @idempty as IdNext, 
			scc.TimeStart, scc.TimeStart as TimeAnswer, scc.TimeStop, 
			ReasonStart, ReasonStop, 0 as StopSide, 
			IdLine as ALineId, IdConf as BLineId, 
			OutNumber as AOutNumber, '' as BOutNumber, 
			LineNum as ALineNum, SubString ( cast ( Room as nvarchar ( 10 ) ), 1, 10 ) as BLineNum, 
			IdUser as AUserId, @idconfuser as BUserId, 
			'' as ANumberDialed, '' as BNumberDialed, 
			IsRecorded, case when ( sc.Type = 1 ) then 11/*селектор*/ else 10/*конф*/ end as ConnectionType, 
			StrName as AStr, SubString ( case when ( Name != '' ) then Name else '#' + cast ( Room as nvarchar ( 10 ) ) end, 1, 200 ) as BStr, 
			0 as FromCC, 1 as IsBetweenUsers, IdRecDir, 
			1 as IsConf, sc.Director, sc.TimeStart as CTS,
			case When ( c.Id is null ) then 0 else 1 end as HasComment

		From A_Stat_Conference_Competitors scc
			Inner join A_Stat_Conference sc on sc.Id = scc.IdConf
			Inner join #A_Stat_Conference_tmp2 sct on sc.Id = sct.Id --фильтр по правам
			Left join A_Stat_Comments c on sc.Id = c.Id
			--Inner join @upermitted up on up.Id = scc.IdUser -- чтобы в конференции даже если она разрешена на просмотр не видеть записи неподчиненных

		Where ((( scc.TimeStart <= @dstart ) and ( scc.TimeStop > @dstart ))
			or (( scc.TimeStart < @dstop ) and ( scc.TimeStop >= @dstop ))
			or (( scc.TimeStart <= @dstart ) and ( scc.TimeStop > @dstart ))
			or (( scc.TimeStart >= @dstart ) and ( scc.TimeStop <= @dstop )))
			and ( Convert ( float, scc.TimeStop ) - Convert ( float, scc.TimeStart ) >= @lt )
			and (( @operatorid is null ) or ( scc.IdUser = @operatorid ))
			and (( @OutNumber is null ) or ( scc.OutNumber = @OutNumber ))
			and (( @innerprefix is null ) 
				or ( scc.IdLine in (Select iditem from #A_Stat_Conference_tmp3)) 
				or ( scc.IdUser in (Select iditem from #A_Stat_Conference_tmp3 )))
    	end

	
    ------------------------------------------------
    ---- Финальная выборка -------------------------
    ------------------------------------------------

	Select * 
	From #A_Stat_Result

end
GO
--------------------------------------------------------------------------------------------
if exists ( Select * from oktell_settings..sysobjects where id = object_id ( 'oktell_settings.dbo.A_UserParams_old' ))
	Drop table oktell_settings.dbo.A_UserParams_old
GO


use oktell_settings

if exists ( select * from sysobjects where id = object_id ( 'dbo.A_UserParams' ))
	exec sp_rename 'dbo.A_UserParams', 'A_UserParams_old'
	
use oktell
GO


CREATE TABLE [oktell_settings].[dbo].[A_UserParams]
(
	[IdUser] [uniqueidentifier] NOT NULL Primary Key,
	[IdRole] [uniqueidentifier] NOT NULL,
	[IsOperator] [bit] NOT NULL,
	[CallerId] [nvarchar](20) NOT NULL,
	[IsRedirect] [bit] NOT NULL,
	[IsCC] [bit] NOT NULL Default 0,
	[IsLunch] [bit] NOT NULL Default 0,
	[IsCCManual] [bit] NULL Default 0	
) ON [PRIMARY]
GO


Insert into [oktell_settings].[dbo].[A_UserParams] 
(	[IdUser],
	[IdRole],
	[IsOperator],
	[CallerId],
	[IsRedirect],
	[IsCC],
	[IsLunch],
	[IsCCManual] 
)
	Select 
		[IdUser],
		[IdRole],
		[IsOperator],
		[CallerId],
		[IsRedirect],
		[IsCC],
		[IsLunch],
		0 as IsCCManual	
	From oktell_settings.dbo.[A_UserParams_Old]
GO


if exists ( Select * from oktell_settings..sysobjects where id = object_id ( 'oktell_settings.dbo.A_UserParams_old' ))
	Drop table oktell_settings.dbo.A_UserParams_old
GO


IF  EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[A_UserParams]'))
DROP VIEW [dbo].[A_UserParams]
GO


Create VIEW [dbo].[A_UserParams]
As
	Select *
	FROM oktell_settings.dbo.A_UserParams
GO


--------------------------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallCenter_State_SetUserInCCManual' ))
	Drop procedure dbo.A_CallCenter_State_SetUserInCCManual
GO

Create procedure [dbo].[A_CallCenter_State_SetUserInCCManual]
	@iduser uniqueidentifier,
	@inccmanual	bit
as
begin

	Update A_UserParams
	Set IsCCManual = @inccmanual
	Where IdUser = @iduser
	
end
GO

--------------------------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallCenter_State_SetUserInCC' ))
	Drop procedure dbo.A_CallCenter_State_SetUserInCC
GO

Create procedure [dbo].[A_CallCenter_State_SetUserInCC]
	@iduser uniqueidentifier,
	@incc	bit,
	@inlunch bit = 0
as
begin

	Update A_UserParams
	Set IsCC = @incc,
		IsLunch = @inlunch,
		IsCCManual = 0
	Where IdUser = @iduser
	
end
GO

--------------------------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallCenter_UserStateChange' ))
	Drop procedure dbo.A_CallCenter_UserStateChange
GO

Create procedure [dbo].[A_CallCenter_UserStateChange]
	@start bit,
	@idoperator uniqueidentifier,
	@state int,
	@info nvarchar ( 100 ) = '',
	@selfinfo bit = null,
	@icode int = -1,
	@dtnow datetime = null	
as
begin

	if ( @dtnow is null )
		set @dtnow = GetDate()

	Begin tran AAA

		Update dbo.A_CallCenter_UserStateHistory with ( TabLock, HoldLock )
		Set TimeLeave = @dtnow,
			LengthSec = Cast ( (@dtnow - TimeEnter) as float ) * 86400
		Where OperatorId = @idoperator 
			and State = @state
			and TimeLeave is null


		if ( @start > 0 ) begin
		
			if ( @state in ( 2, 3, 4 ) )
				Update dbo.A_CallCenter_UserStateHistory
				Set TimeLeave = @dtnow,
					LengthSec = Cast ( (@dtnow - TimeEnter) as float ) * 86400
				Where OperatorId = @idoperator 
					and State in ( 2, 3, 4 )
					and TimeLeave is null

			Insert into dbo.A_CallCenter_UserStateHistory
			Values ( NewId ( ), @idoperator, @state, @dtnow, null, null, @info, @selfinfo, @icode )

			if ( @state = 1 )				
				Insert into dbo.A_CallCenter_UserStateHistory
				Values ( NewId ( ), @idoperator, 2, @dtnow, null, null, '', 0, @icode )
				
		end else begin
		
			if ( @state = 0 )
			
				Update dbo.A_CallCenter_UserStateHistory
				Set TimeLeave = @dtnow,
					LengthSec = Cast ( (@dtnow- TimeEnter) as float ) * 86400
				Where OperatorId = @idoperator 
					and TimeLeave is null
					
			else if ( @state = 1 )	
						
				Update dbo.A_CallCenter_UserStateHistory
				Set TimeLeave = @dtnow,
					LengthSec = Cast ( (@dtnow- TimeEnter) as float ) * 86400
				Where OperatorId = @idoperator 
					and TimeLeave is null
					and State != 0

		  end

	Commit Tran AAA
end
GO


--------------------------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallCenter_Temp_UpdateOperatorStates' ))
	Drop procedure dbo.[A_CallCenter_Temp_UpdateOperatorStates]
GO

Create Procedure [dbo].[A_CallCenter_Temp_UpdateOperatorStates]
	@dtnow datetime = null
as 
begin

	set dateformat dmy

	if ( @dtnow is null )	
		set @dtnow = DateAdd ( ss, -30, GetDate() )

	if not exists ( Select * FROM master..sysdatabases where name like 'oktell_cc_temp' )
		return

	Declare @sqltext nvarchar ( 4000 )

	Declare @idempty uniqueidentifier
	Set @idempty = '00000000-0000-0000-0000-000000000000'
	
	-- Всех операторов в таблицу
	Declare @settable table ( Id uniqueidentifier, Name nvarchar ( 500 ) collate database_default)
	Insert into @settable
		Select Id, Name From A_Users u 
			Inner join A_UserParams up on u.Id = up.IdUser
		Where IsOperator = 1

	-- N минут назад
	Declare @dt10m DateTime
	Set @dt10m = DateAdd ( mi, -20, @dtnow )

	-- формируем общую выборку из статистики коммутаций
	Declare @dtsall DateTime
	Select @dtsall = Min ( LastDT ) 
	From [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] oi
		Inner join @settable up on oi.Id = up.Id

	-- Если нет распоследнего - расчитываем за 10 дней.
	if ( @dtsall is null )
		Set @dtsall = DateAdd ( d, -10, @dtnow )

	-- Новое (06.2010)
	-- Если распоследний менее чем 10 минут до текущего момента - считать будем 10 минут.
	if ( @dtsall > @dt10m )
		Set @dtsall = @dt10m
	
	---- Кэш для A_Stat_Connections_1x1
	if exists ( Select * FROM tempdb..sysobjects Where Id = object_id ( 'tempdb..#statconnections' ) )
		Drop Table #statconnections	
	Create table #statconnections 
	( 
		FromCC bit, 
		IsBetweenUsers bit, 
		TimeStart DateTime, 
		TimeAnswer DateTime, 
		TimeStop DateTime, 
		AUserId uniqueidentifier, 
		BUserId uniqueidentifier 
	)

	Set @sqltext = 'Select FromCC, IsBetweenUsers, TimeStart, TimeAnswer, TimeStop, AUserId, BUserId 
		From A_Stat_Connections_1x1 sc 
		Where ( TimeStart > @dtt1 Or TimeStop > @dtt1 Or TimeStop is null ) And ( TimeStart < @dtnow )'
	
	Insert Into #statconnections
		exec sp_executesql @sqltext, N'@dtt1 datetime, @dtnow datetime', @dtsall, @dtnow
	
	---- Кэш для A_CallCenter_UserStateHistory
	if exists ( Select * FROM tempdb..sysobjects Where Id = object_id ( 'tempdb..#ccuserstatehistory' ) )
		Drop Table #ccuserstatehistory
	Create table #ccuserstatehistory 
	(
		OperatorId uniqueidentifier, 
		State int, 
		TimeEnter DateTime, 
		TimeLeave DateTime, 
		Info nvarchar ( 100 ) collate database_default, 
		ICode int 
	)

	Set @sqltext = 'Select OperatorId, State, TimeEnter, TimeLeave, case when selfinfo = 1 then Info else '''' end Info, ICode
		From A_CallCenter_UserStateHistory 
		Where ( TimeEnter > @dtt1 Or TimeLeave > @dtt1 Or TimeLeave is null ) And ( TimeEnter < @dtnow )'

	Insert Into #ccuserstatehistory
		exec sp_executesql @sqltext, N'@dtt1 datetime, @dtnow datetime', @dtsall, @dtnow
	
	
	---- Кэш для A_TaskManager_UserInTask
	if exists ( Select * FROM tempdb..sysobjects Where Id = object_id ( 'tempdb..#userintask' ) )
		Drop Table #userintask
	Create table #userintask 
	( 
		Id uniqueidentifier, 
		UserId uniqueidentifier, 
		TaskId uniqueidentifier, 
		EffortId uniqueidentifier, 
		TimeStart DateTime, 
		TimeStopPrior DateTime, 
		TimeStopWaitCall DateTime, 
		TimeStopBackCall DateTime, 
		TimeStopDialog DateTime, 
		TimeStopPost DateTime 
	)

	Set @sqltext = 'Select Id, UserId, TaskId, Effortid, IsNull ( TimeStart, @dtnow ), IsNull ( TimeStopPrior, @dtnow ), IsNull ( TimeStopWaitCall, @dtnow ), IsNull ( TimeStopBackCall, @dtnow ), IsNull ( TimeStopDialog, @dtnow ), IsNull ( TimeStopPost, @dtnow )
		From A_TaskManager_UserInTask 
		Where ( TimeStart >= @dtt1 or TimeStopPost >= @dtt1 or TimeStopPost is null ) And ( TimeStart < @dtnow )'

	Insert Into #userintask
		exec sp_executesql @sqltext, N'@dtt1 datetime, @dtnow datetime', @dtsall, @dtnow
	
	
	---- Кэш для части предыдущих данных из oktell_cc_temp.dbo.A_Cube_CC_OperatorStates
	Declare @maxid table 
	( 
		IdOperator uniqueidentifier, 
		MaxId int 
	)
		
	if exists ( Select * FROM tempdb..sysobjects Where Id = object_id ( 'tempdb..#tmpos04' ) )
		Drop Table #tmpos04
	Create table #tmpos04 ( IdOperator uniqueidentifier, MaxId int )

	-- Новое (06.2010)
	--   микс со временем, поскольку удалять записи будем все равно..
	Set @sqltext = 'Select IdOperator,  Max ( Id )
		From [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates]
		Where DateTimeStart < @dtm
		Group By IdOperator'

	Insert Into #tmpos04
		exec sp_executesql @sqltext, N'@dtm datetime', @dt10m
	Insert into @maxid
		Select * FROM #tmpos04	

	drop table #tmpos04
		

	-- Распоследнее время для каждого оператора может быть уникальным
	Declare @umaxdts table ( UserId uniqueidentifier, MaxDTS DateTime  )

	Insert into @umaxdts
		Select IdOperator, Max ( DateTimeStop ) 
		From [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] With ( NOLOCK )
		Group By IdOperator

	-- Новое (06.2010)
	-- Будем перерасчитывать не менее 10 минут по любому...
	Update @umaxdts
	Set MaxDTS = @dt10m
	Where MaxDTS > @dt10m


	-- цикл по операторам
	Begin Tran TranUpdateOperatorStates

		Declare ucursor Cursor LOCAL FAST_FORWARD For 
			Select Id, Name 
			From @settable
			
			
		Declare @idoperator uniqueidentifier, @nameoperator nvarchar ( 100 )	


		Open ucursor
		Fetch Next 
			From ucursor 
			Into @idoperator, @nameoperator

		While ( @@fetch_status = 0 ) begin

			Declare @id uniqueidentifier
			Select @id = Id from [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo]
			Where Id = @idoperator
			
			if ( @@rowcount = 0 )
				Insert 
				Into [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] ( Id, Name ) 
				Values ( @idoperator, @nameoperator )
						
			-- определяем время последнего изменения состояния
			Declare @dts DateTime, @dts1 DateTime
			
			Select @dts = MaxDTS
			From @umaxdts
			Where UserId = @idoperator
			
			Select @dts1 = LastDT
			From [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] 
			Where Id = @idoperator
			
			if ( @dts1 > @dts ) or ( @dts is null )
				Set @dts = @dts1

			if ( @dts is null )
				set @dts = '01.01.2000'

			if ( @dts > @dt10m )
				set @dts = @dt10m


			-- обновление производим для операторов не чаще одного раза в минуту
			if ( Cast ( @dtnow - @dts as float ) * 1440 > 1 ) begin		
				
				-- Итоговые непересекающиеся состояния
				/*
				PriorHandle		= 1,
				WaitingCall		= 2,
				BackRing		= 3,
				TalkAbonent		= 4,
				TalkInside		= 5,
				InsideTalk		= 6,
				PostHandle		= 7,
				TalkOther		= 8,
				CCLunch			= 9,
				CCManual		= 12,
				CCReady			= 10,
				System			= 11
				*/
				
				-- Приоритеты (чем меньше, тем выше)
				--   то есть если нет состояния с меньшим приоритетом, подставляется следующее из имеющихся с понижением приоритета
				/*
				UserInTask		= 1,
				Lunch			= 2,
				Commutations	= 3,
				ManualCC		= 4,
				OnlineCC		= 4,
				System			= 5,
				*/

				-----------------------------------------------------
				-- 1. вторая таблица (статусы оператора в КЦ) ----------
				-----------------------------------------------------
				Declare @tempo2 table
				(
					State int not null,
					Priority int,
					DateTimeStart DateTime,
					DateTimeStop DateTime,
					Id int Identity ( 1, 1 ) not null primary key,
					Info nvarchar ( 100 ),
					ICode int
				)

				Delete From @tempo2

				---------------------
				-- 0-в системе, <= 1-в коллцентре, <= ( 2-онлайн, 3-перерыв, 4-ручной режим )
				---------------------
				Insert into @tempo2 ( State, Priority, DateTimeStart, DateTimeStop, Info, ICode )
					Select State, 
						   Case when (State = 3) --Перерыв
									then 2 
								when (State = 4) --Ручной режим
									then 4 
								else 4 --Онлайн состояния
							 end as Priority, 
						   TimeEnter, IsNull ( TimeLeave, @dtnow ) as DateTimeStop, Info, ICode
					From #ccuserstatehistory
					Where OperatorId = @idoperator
						And State in ( 2, 3, 4 )
						And ( TimeEnter > @dts Or TimeLeave > @dts Or TimeLeave is null )
				
				Update @tempo2
				Set State = Case when (State = 3)--Перерыв
									then 9 
								 when (State = 4)--
									then 12
								 else 10--Прочие онлайны (они менее приоритетны)
						      end

				
				-- если есть активное состояние кц - обновляем
				if ( @@rowcount > 0 ) begin	
				
					-- докидываем информацию по присутствию в системе
					Insert into @tempo2 ( State, Priority, DateTimeStart, DateTimeStop, Info, ICode )
						Select 11 as State, 
							   5 as Priority,
							   TimeEnter, IsNull ( TimeLeave, @dtnow ) as DateTimeStop, Info, ICode
						From #ccuserstatehistory
						Where OperatorId = @idoperator
							And State = 0
							And ( TimeEnter > @dts Or TimeLeave > @dts Or TimeLeave is null )

					Update @tempo2			
					Set DateTimeStart = Case When (DateTimeStart < @dts) then @dts else DateTimeStart end,
						DateTimeStop = Case When (DateTimeStop > @dtnow) then @dtnow else DateTimeStop end
					
			
					-----------------------------------------------------
					-- 2. Первая таблица (статусы оператора в задачах) -----
					-----------------------------------------------------
					Declare @tempo1 table
					(
						IdTask uniqueidentifier,
						IsOutput bit,
						IdOperator uniqueidentifier,
						CallResult int,
						State int not null,
						StateCat int,
						DateTimeStart DateTime,
						DateStart DateTime,
						TimeStart DateTime,
						DateTimeStop DateTime,
						LenTime Float,
						IdEffort uniqueidentifier,
						Priority int,
						Id int identity ( 1, 1 ) not null primary key
					)

					Delete From @tempo1
					
					Declare @uito table 
					(
						Id uniqueidentifier,
						UserId uniqueidentifier,
						TaskId uniqueidentifier,	
						EffortId uniqueidentifier,
						TimeStart DateTime,
						TimeStopPrior DateTime,
						TimeStopWaitCall DateTime,
						TimeStopBackCall DateTime,
						TimeStopDialog DateTime,
						TimeStopPost DateTime
					)

					Delete From @uito

					Insert Into @uito
						Select *
						From #userintask
						Where UserId = @idoperator
				
				
					-- Предвызывная обработка (состояние 1, приоритет 1)
					Insert into @tempo1 ( IdTask, IsOutput, IdEffort, IdOperator, Priority, CallResult, State, StateCat, 
							DateTimeStart, DateTimeStop )
						Select TaskId, null, EffortId, @idoperator, 1, null, 1, 0, 
							TimeStart, TimeStopPrior
						From @uito
						Where Cast ( TimeStopPrior - TimeStart as float ) * 86400 > 0
					
					
					-- Ожидание дозвона (состояние 2, приоритет 1)
					Insert into @tempo1 ( IdTask, IsOutput, IdEffort, IdOperator, Priority, CallResult, State, StateCat,
								DateTimeStart, DateTimeStop )
						Select TaskId, null, EffortId, @idoperator, 1, null, 2, 0, 
							TimeStopPrior, TimeStopWaitCall
						From @uito
						Where Cast ( TimeStopWaitCall - TimeStopPrior as float ) * 86400 > 0
					
					
					-- Обратный вызов (состояние 3, приоритет 1)
					Insert into @tempo1 ( IdTask, IsOutput, IdEffort, IdOperator, Priority, CallResult, State, StateCat,
								DateTimeStart, DateTimeStop )
						Select TaskId, null, EffortId, @idoperator, 1, null, 3, 0, 
							TimeStopWaitCall, TimeStopBackCall
						From @uito
						Where Cast ( TimeStopBackCall - TimeStopWaitCall as float ) * 86400 > 0

					
					-- Разговор (состояние 6, приоритет 1)
					Insert into @tempo1 ( IdTask, IsOutput, IdEffort, IdOperator, Priority, CallResult, State, StateCat,
								DateTimeStart, DateTimeStop )
						Select TaskId, null, EffortId, @idoperator, 1, null, 6, 0, 
							TimeStopBackCall, TimeStopDialog
						From @uito
						Where Cast ( TimeStopDialog - TimeStopBackCall as float ) * 86400 > 0

					
					-- Поствызывная обработка (состояние 7, приоритет 1)
					Insert into @tempo1 ( IdTask, IsOutput, IdEffort, IdOperator, Priority, CallResult, State, StateCat,
								DateTimeStart, DateTimeStop )
						Select TaskId, null, EffortId, @idoperator, 1, null, 7, 0, 
							TimeStopDialog, TimeStopPost
						From @uito
						Where Cast ( IsNull ( TimeStopPost, @dtnow ) - IsNull ( TimeStopDialog, @dtnow ) as float ) * 86400 > 0



					-- время начала и время конца по периоду
					Update @tempo1 
					Set DateTimeStart = case when ( DateTimeStart < @dts ) then @dts 
											else DateTimeStart end,
		 				DateTimeStop = case when ( DateTimeStop > @dtnow ) then @dtnow 
											when ( DateTimeStop < @dts ) then @dts 
											else DateTimeStop end

					Delete From @tempo1
					Where DateTimeStart = DateTimeStop


					-- проставляем флаг исх/вх задачи
					Update tmpo
					Set IsOutput = t.IsOutputTask
					From @tempo1 tmpo, 
						A_TaskManager_Tasks t
					Where tmpo.IdTask = t.Id

					-- проставляем результат выполнения задачи
					Update tmpo
					Set CallResult = tic.CallResult
					From @tempo1 tmpo, 
						A_TaskManager_Task_IncomingCalls tic
					Where tmpo.IdEffort = tic.Id

					Update tmpo
					Set CallResult = ste.EResult
					From @tempo1 tmpo, 
						A_TaskManager_SubTaskEfforts ste
					Where tmpo.IdEffort = ste.Id

			
					-----------------------------------------------------
					-- 3. третья таблица (Разговоры) -----------------------
					-----------------------------------------------------
					
					Declare @tempo3 table
					(
						State int not null,
						Priority int,
						DateTimeStart DateTime,
						DateTimeStop DateTime,
						Id int identity ( 1, 1 ) not null primary key
					)
					
					Delete From @tempo3
								
					Insert into @tempo3
						Select 8, 0, TimeStart, isnull ( TimeStop, @dtnow )
						From #statconnections
						Where ( AUserId = @idoperator Or BUserId = @idoperator )
							And ( TimeStart > @dts Or TimeStop > @dts Or TimeStop is null )
							And ( FromCC = 0 )
						

					Update @tempo3
					Set DateTimeStart = Case When DateTimeStart < @dts then @dts else DateTimeStart end,
						Priority = Case When State = 8 then 3 else 0 end
										

					-----------------------------------------------------
					-- Формируем времянку по точкам ---------------------
					-----------------------------------------------------

					-- Select * FROM @tempo1 Order by DateTimeStart
					-- Select * FROM @tempo2 Order by DateTimeStart
					-- Select * From @tempo3 Order by DateTimeStart
				
					Declare @tempo table
					(
						TableN int,
						TableID int,
						State int not null,
						Priority int,
						DT DateTime,
						NK bit
					)

					Delete from @tempo

					Insert into @tempo
						Select 1, Id, State, Priority, DateTimeStart, 0
						From @tempo1

					Insert into @tempo
						Select 1, Id, State, Priority, DateTimeStop, 1
						From @tempo1

					Insert into @tempo
						Select 2, Id, State, Priority, DateTimeStart, 0
						From @tempo2

					Insert into @tempo
						Select 2, Id, State, Priority, DateTimeStop, 1
						From @tempo2

					Insert into @tempo
						Select 3, Id, State, Priority, DateTimeStart, 0
						From @tempo3

					Insert into @tempo
						Select 3, Id, State, Priority, DateTimeStop, 1
						From @tempo3
					

					-- В Tempo2 лежат ICode, но так как состояние перерыва фактически будет по концу коммутации, если оная была
					--   а пересечение с Tempo2 по времени, то нужно добавить в Tempo2 куски из других таблиц, но с их временами,
					--   и расширить предыдущим ICode для пересечения


					-----------------------------------------------------
					-- пробегаем курсором -------------------------------
					-----------------------------------------------------

					Declare @res table ( TableN int, Id int, State int not null, Priority int, DStart DateTime, DStop DateTime )
					Delete From @res

					-- определяем статусы и приоритеты на начало периода расчета
					-- приоритеты: 	0 - коммутации внутри задачи ( попытки )
					--		1 - состояния оператора внутри задачи ( попытки )
					--		2 - перерывы
					--		3 - прочие коммутации
					--		4 - онлайн в КЦ, ручной режим в КЦ
					--		5 - онлайн в системе
					Declare @p0 int, @p1 int, @p2 int, @p3 int, @p4 int, @p5 int
					Declare @s0 int, @s1 int, @s2 int, @s3 int, @s4 int, @s5 int
					Declare @t0 int, @t1 int, @t2 int, @t3 int, @t4 int, @t5 int
					Declare @i0 int, @i1 int, @i2 int, @i3 int, @i4 int, @i5 int

					-- Новое (06.2010)
					--   Не применяем предыдущую историю количеств. Всегда должно в ноль сводиться
					Select @p0 = 0, @p1 = 0, @p2 = 0, @p3 = 0, @p4 = 0, @p5 = 0
					Select @s0 = IsNull ( S0, 0 ), @s1 = IsNull ( S1, 0 ), @s2 = IsNull ( S2, 0 ), @s3 = IsNull ( S3, 0 ), @s4 = IsNull ( S4, 0 ), @s5 = IsNull ( S5, 0 )
					From [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] 
					Where Id = @idoperator
				
					-- формируем сдвоенную таблицу	
					Declare @tempo_ordered table
					(
						Id int identity ( 1, 1 ) not null,
						TableN int,
						TableID int,
						State int not null,
						Priority int,
						DT DateTime,
						NK bit
					)

					Delete From @tempo_ordered

					Insert into @tempo_ordered ( TableN, TableID, State, Priority, DT, NK )
						Select TableN, TableID, State, Priority, DT, NK From @tempo
						Order By DT

					
					Declare @tempo_joined table
					(
						o1 int, 
						o2 int, 
						s1 int not null, 
						p1 int not null, 
						s2 int not null, 
						p2 int not null, 
						dstart datetime, 
						dstop datetime, 
						nk1 bit, 
						nk2 bit, 
						t1 int, 
						t2 int, 
						i1 int, 
						i2 int
					)
					Delete From @tempo_joined
				
					Insert into @tempo_joined 
						Select t1.Id, t2.Id, t1.State as s1, t1.Priority as p1, t2.State as s2, t2.Priority as p2,
							t1.DT as dstart, t2.DT as dstop, t1.NK as nk1, t2.NK as nk2,
							t1.TableN, t2.TableN, t1.TableID, t2.TableID
						From @tempo_ordered t1
							Inner join @tempo_ordered t2 on t1.Id + 1 = t2.Id


					-- курсором бежим и заполняем таблицу результата 
					Declare tcursor Cursor LOCAL FAST_FORWARD For 
						Select o1, o2, s1, p1, s2, p2, dstart, dstop, nk1, nk2, t1, t2, i1, i2 
						From @tempo_joined
						Order by o1						

					Declare @o1 int, @o2 int
					Declare @st1 int, @pr1 int, @st2 int, @pr2 int, @dstart DateTime, @dstop DateTime, @nk1 bit, @nk2 bit, @tb1 int, @tb2 int, @id1 int, @id2 int
					Declare @actualst int, @actualpr int, @actualt int, @actualid int
					Declare @lastst int, @lastpr int, @lastt int, @lastid int, @lastdstart datetime, @lastdstop datetime
					Declare @dontinsert bit, @updatelast bit
					Declare @insertindex int

					set @insertindex = 0
					set @lastt = 0
					set @dontinsert = 1 -- чтобы при втором обновлении (при существующей таблице данных) не вставляло имеющееся
				
					Open tcursor
					Fetch Next 
						From tcursor 
						Into @o1, @o2, @st1, @pr1, @st2, @pr2, @dstart, @dstop, @nk1, @nk2, @tb1, @tb2, @id1, @id2

					While (@@FETCH_STATUS = 0) Begin
		
						--- оцениваем перед началом -----		
						if ( @nk1 = 0 ) begin			
						
							if ( @pr1 = 0 ) begin
			    			
			    				set @p0 = @p0 + 1
				    			set @s0 = @st1				
								set @t0 = @tb1	
								set @i0 = @id1
							
							end else if ( @pr1 = 1 ) begin
							
								set @p1 = @p1 + 1
				    			set @s1 = @st1
								set @t1 = @tb1
								set @i1 = @id1
								
							end else if ( @pr1 = 2 ) begin
							
			    				set @p2 = @p2 + 1
				    			set @s2 = @st1
								set @t2 = @tb1
								set @i2 = @id1
								
							end else if ( @pr1 = 3 ) begin
				    				
				    			set @p3 = @p3 + 1
				    			set @s3 = @st1
								set @t3 = @tb1
								set @i3 = @id1
								
							end else if ( @pr1 = 4 ) begin
			    					
			    				set @p4 = @p4 + 1
				    			set @s4 = @st1
								set @t4 = @tb1
								set @i4 = @id1
								
							end else if ( @pr1 = 5 ) begin
								
								set @p5 = @p5 + 1
								set @s5 = @st1
								set @t5 = @tb1
								set @i5 = @id1
							
							end
						end


						--- производим расчет и модернизацию
						set @dontinsert = 0
						if ( @p0 > 0 ) begin
						
							set @actualst = @s0
							set @actualpr = 0
							set @actualt = @t0
							set @actualid = @i0
						
						end else if ( @p1 > 0 ) begin
						
							set @actualst = @s1
							set @actualpr = 1
							set @actualt = @t1
							set @actualid = @i1

						end else if ( @p2 > 0 )	begin
						
							set @actualst = @s2
							set @actualpr = 2
							set @actualt = @t2
							set @actualid = @i2
						
						end else if ( @p3 > 0 ) begin
						
							set @actualst = @s3
							set @actualpr = 3
							set @actualt = @t3
							set @actualid = @i3
						
						end else if ( @p4 > 0 )begin
						
							set @actualst = @s4
							set @actualpr = 4
							set @actualt = @t4
							set @actualid = @i4
							
						end
				
						-- если не покрывается кусок 4 и 2 уровнями (онлайн и перерыв)
						-- или юзер в офлайне вообще в этом промежутке
						if (( @p4 = 0 ) and ( @p2 = 0 )) or ( @p5 = 0 )
 							set @dontinsert = 1
					
						
						-- разбор состояния на необходимость вставки			
						set @updatelast = 0
						if ( @lastt = 0 or @lastt is null ) begin
							-- если предыдущий был пустым блоком, или самое начало - обновляем величины
							set @updatelast = 1
							
						end else if ( @dontinsert = 0 ) and ( @actualst = @lastst ) begin
							-- если активный блок но статус совпадает с предыдущим блоком - обновляем время конца
							set @lastdstop = @dstop
							
						end	else begin
							-- в противном случае вставляем и обновляем величины
							if ( DateDiff ( ms, @lastdstart, @lastdstop ) > 10 ) 
								Insert Into @res
								Values ( @lastt, @lastid, @lastst, @lastpr, @lastdstart, @lastdstop )
						
							set @updatelast = 1
							
						end
					
						if ( @updatelast = 1 ) begin
							if ( @dontinsert = 1 ) begin
			  				
			  					set @lastt = 0
							
							end else begin
							
								set @lastt = @actualt
								set @lastid = @actualid
								set @lastst = @actualst
								set @lastpr = @actualpr
								set @lastdstart = @dstart
								set @lastdstop = @dstop
								
							end
						end
					
				
						--- оцениваем после вставки (конец) -----
						if ( @nk2 = 1 ) begin
							-- Новое (06.2010)
							--    Условие на неотрицательность

							if ( @pr2 = 0 ) begin
							
								if ( @p0 > 0 ) 
									set @p0 = @p0 - 1 
							
							end else if ( @pr2 = 1 ) begin
								
								if ( @p1 > 0 ) 
									set @p1 = @p1 - 1 
								
							end else if ( @pr2 = 2 ) begin
								
								if ( @p2 > 0 ) 
									set @p2 = @p2 - 1 
								
							end else if ( @pr2 = 3 ) begin
								
								if ( @p3 > 0 ) 
									set @p3 = @p3 - 1 
								
							end else if ( @pr2 = 4 ) begin
							
								if ( @p4 > 0 ) 
									set @p4 = @p4 - 1 
								
							end else if ( @pr2 = 5 ) begin
								
								if ( @p5 > 0 ) 
									set @p5 = @p5 - 1 
								
							end
						end
					
						Fetch Next 
							From tcursor 
							Into @o1, @o2, @st1, @pr1, @st2, @pr2, @dstart, @dstop, @nk1, @nk2, @tb1, @tb2, @id1, @id2
					End


					-- в конце последний промежуток надо втавить, так как алгоритм запаздывающий
					if ( @dontinsert = 0 )
						if ( DateDiff ( ms, @lastdstart, @lastdstop ) > 10 ) 
							Insert Into @res
							Values ( @lastt, @lastid, @lastst, @lastpr, @lastdstart, @lastdstop )

					Close tcursor
					Deallocate tcursor

					
					-------------------------------------------------------------------------------------------------
					-- обновляем данные по таблицам и формируем итоговую по оператору -------------------------------
					-------------------------------------------------------------------------------------------------

					--проверяем наличие записи об операторе в таблице (местами надо поменять с куском обновления и вставки в куб)
					Update [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo]
						Set p0 = @p0, p1 = @p1, p2 = @p2, p3 = @p3, p4 = @p4, p5 = @p5,
							s0 = @s0, s1 = @s1, s2 = @s2, s3 = @s3, s4 = @s4, s5 = @s5,
							LastDT = @dtnow
					Where Id = @idoperator

					Declare @operatorstates table
					(
						Id				int identity ( 1, 1 ) not null primary key,
						IdTask 			uniqueidentifier,
						IsOutput		bit,
						IdOperator 		uniqueidentifier,
						CallResult		int,
						State			int not null,
						DateTimeStart 	DateTime,
						DateStart 		DateTime,
						TimeStart 		DateTime,
						DateTimeStop 	DateTime,
						LenTime 		Float,
						ICode			int,
						IdEffort		uniqueidentifier
					)

					Delete From @operatorstates

					Insert into @operatorstates ( IdOperator, State, DateTimeStart, DateTimeStop, DateStart, TimeStart, LenTime, 
								ICode, IdTask, IsOutput, CallResult, IdEffort )
						Select @idoperator, r.State, r.DStart, r.DStop,
							Cast ( Floor ( Cast ( DStart as Float ) ) as DateTime ),
							Cast ( Cast ( DStart as float ) - Floor ( Cast ( DStart as Float ) ) as DateTime ),				
							Cast ( DStop - DStart as float ) * 86400,
							t2.ICode,
							IdTask, IsOutput, CallResult, IdEffort
						From @res r
							Left join @tempo1 t1 on t1.Id = r.Id and r.TableN = 1
							Left join @tempo2 t2 on t2.State = r.State and t2.DateTimeStart = r.DStart


					-- В Tempo2 лежат ICode, но так как состояние перерыва фактически будет по концу коммутации, если оная была
					--   а пересечение с Tempo2 по времени, то нужно обновить потом ICode там где не проставился, а в Tempo2 есть
					Update os
					Set os.ICode = t2.ICode
					From @operatorstates os, @tempo2 t2
					Where (( os.State = 9 ) and ( os.ICode is NULL ))
						and ( t2.State = 9)
						and ( t2.DateTimeStart < os.DateTimeStart and t2.DateTimeStop > os.DateTimeStart )


		
					-- Код крайней остающейся записи в кубе в настоящее время
					Declare @idt1 int
					Select @idt1 = MaxId
					From @maxid
					Where IdOperator = @idoperator

					-- Новое (06.2010)
					--    удаляем последние 10 минут
					-- i. IdOperator, DateTimeStart
					Delete From [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates]
					Where IdOperator = @idoperator
						--And Id > @idt1
						And DateTimeStart >= @dts

		
					--   Обратно возвращаем на стык значение DateTimeStop
					if ( not @idt1 is null ) begin
					
						Update [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] --With ( UpdLock )
						Set DateTimeStop = @dts,
							LenTime = Cast ( @dts - DateTimeStart as float ) * 86400
						Where Id = @idt1
							And DateTimeStop > @dts
							
					end


					--для склейки результатов выполнения задач (если расчет в середине выполнения - в первые состояния информация о результате не попала)
					Declare @cr table ( idtask uniqueidentifier, ideffort uniqueidentifier, callresult int )
					Insert into @cr ( idtask, ideffort, callresult )
						Select IdTask, IdEffort, CallResult
						From @operatorstates
						Group By IdTask, IdEffort, CallResult

					-- i. IdOperator, IdEffort, IdTask, CallResult
					Update os
					Set CallResult = cr.callresult
					From [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] os, @cr cr
					Where os.IdOperator = @idoperator
						and os.IdEffort = cr.ideffort
						and not ( cr.IdEffort is null )
						and os.CallResult is null

					-- код склейки крайних записей
					Declare @state1 int, @dtstart1 DateTime, @dtstop1 DateTime
					
					if ( not @idt1 is null ) begin
					
						Select @state1 = State, @dtstart1 = DateTimeStart, @dtstop1 = DateTimeStop 
						From [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] --With ( TabLock )
						Where Id = @idt1
						
						Declare @idt2 int, @state2 int, @dtstart2 DateTime, @dtstop2 DateTime
						Select Top 1 @idt2 = Id, @state2 = State, @dtstart2 = DateTimeStart, @dtstop2 = DateTimeStop
						From @operatorstates
						Order By DateTimeStart

						if ( @state1 = @state2 ) and ( @dtstart2 = @dtstop1 ) begin
						
							Update [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] --With ( UpdLock )
							Set DateTimeStop = @dtstop2,
								LenTime = Cast ( @dtstop2 - @dtstart1 as float ) * 86400
							Where Id = @idt1

							Delete 
							From @operatorstates
							Where Id = @idt2
							
			   			end
					end

				
					-- обновление куба
					Insert Into [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] --With ( UpdLock )
							( IdTask, IsOutput, IdOperator, CallResult, State, DateTimeStart, DateStart, TimeStart, DateTimeStop, LenTime, ICode, IdEffort )
						Select IdTask, IsOutput, IdOperator, CallResult, State, DateTimeStart, DateStart, TimeStart, DateTimeStop, LenTime, ICode, IdEffort
						From @operatorstates
										
				end else begin
				
					Update [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo]
					Set LastDT = @dtnow
					Where Id = @idoperator
					
				end

			end
						
			Fetch Next 
				From ucursor 
				Into @idoperator, @nameoperator
		end

		Close ucursor
		Deallocate ucursor

	Commit Tran TranUpdateOperatorStates
	
end
GO

