﻿-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_CacheCreate' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_CacheCreate
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_CacheCreate]
	@tableidx nvarchar(200),
	@tableacindex nvarchar(200),
	@tableopindex nvarchar(200),
	@idfield nvarchar(200),
	@operatorfield nvarchar(200),
	@query nvarchar(4000) = '',
	@table nvarchar(500) = '',
	@taskid uniqueidentifier = '00000000-0000-0000-0000-000000000000'
as
begin
	-- Внутренняя вспомогательная.
	-- Производит создание и/или наполнение БД-кэша таблицы абонентов для задачи с закреплением абонентов за оператором
	
	Declare @sql nvarchar(4000)
	Declare @iscreating bit 
	Set @iscreating = 0
	
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) ) begin
				
		-- cоздаем
		Set @iscreating = 1
		Set @sql = '
			Create table ' + @tableidx + ' ( Id int primary key, AC int, State int, NIdx int, IdOperator uniqueidentifier )
	
			if not exists ( select name from sysindexes where id=object_id ( ''dbo.[' + @tableacindex + ']'' ) and name = ''' + @tableacindex + ''' )
				Create Nonclustered Index [' + @tableacindex + '] ON ' + @tableidx + '
				( AC, State ) With FillFactor = 80	

			
			if not exists ( select name from sysindexes where id=object_id ( ''dbo.[' + @tableopindex + ']'' ) and name = ''' + @tableopindex + ''' )
				Create Nonclustered Index [' + @tableopindex + '] ON ' + @tableidx + '
				( IdOperator ) With FillFactor = 80	
			'

		exec sp_executesql @sql
	
	end
	
	-- Если идет синхронизация, а не создание, то
	--   Добавить те записи, которых нет в кэше
	--   Обновить в кэше IdOperator и State=0 у тех у кого =3
	
	-- Вставляем в цикле по 100000 записей чтобы не переполнить transactionlog
	
	-- Запрос|таблица
	Declare @fromstring nvarchar(4000)
	if ( not @query like '' )
		Set @fromstring = '(' + @query + ') t'
	else
		Set @fromstring = @table + ' t'
		
	-- Формируем запрос для разных условий
	-- Базовая часть
	Set @sql = 'Declare @k int
				Set @k = 0
				'
		
	-- Вставка новых записей
	-- With|WO operators
	if ( @operatorfield like '' ) begin
			
		-- Без операторов
		Set @sql = @sql + '
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Insert into ' + @tableidx + ' ( Id, AC, State, NIdx )				
	 					Select top 100000 [' + @idfield + '], @k, 0, 0
						From ' + @fromstring + '
						Where not exists ( Select [Id] From ' + @tableidx + ' Where [Id] = t.[' + @idfield + '] )
				end
				'
				
	end else begin
		
		-- С операторами
		Set @sql = @sql + '
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Insert into ' + @tableidx + ' ( Id, AC, State, NIdx, IdOperator )				
	 					Select top 100000 [' + @idfield + '], @k, 0, 0, cast ( [' + @operatorfield + '] as uniqueidentifier )
						From ' + @fromstring + '
						Where not exists ( Select [Id] From ' + @tableidx + ' Where [Id] = t.[' + @idfield + '] )
				end
				'
	end
	
	-- State надо обновить по оперативной статистике (20.12.2013)
	if ( @taskid != '00000000-0000-0000-0000-000000000000' ) begin
		Set @sql = @sql + '
				Update cache
				Set [State] = st.NResult
				From ' + @tableidx + ' cache, A_TaskManager_SubTasks st
				Where (st.TaskId = cast(''' + cast ( @taskid as nvarchar(100)) + ''' as uniqueidentifier))
					and (cache.[Id] = st.[IdInList])
				'
	end
	
	-- В режиме синхронизации существующего кэша
	if ( @iscreating = 0 ) begin
		
		-- Обновление привязки к операторам
		if ( not @operatorfield like '' ) begin
			Set @sql = @sql + '
				Set rowcount 100000
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Update c
					Set c.IdOperator = t.[' + @operatorfield + '],
						c.AC = @k
					From ' + @tableidx + ' c, ' + @fromstring + '
					Where (c.Id = t.[' + @idfield + ']) 
						and (c.State != 1)
						and (c.IdOperator != t.[' + @operatorfield + '])
				end
				Set rowcount 0
				'
		end
	
		-- Обновление статусов вновь появившихся абонентов
		Set @sql = @sql + '
				Set rowcount 100000
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Update c
					Set c.State = 0, 
						c.AC = @k
					From ' + @tableidx + ' c, ' + @fromstring + '
					Where (c.Id = t.[' + @idfield + ']) and (c.State = 3)
				end
				Set rowcount 0
				'
	end
	
	exec sp_executesql @sql
				
end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_CacheCreateP' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_CacheCreateP
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_CacheCreateP]
	@tableidx nvarchar(200),
	@tableacindex nvarchar(200),
	@tableopindex nvarchar(200),
	@idfield nvarchar(200),
	@operatorfield nvarchar(200),
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = '',
	@query nvarchar(4000) = '',
	@table nvarchar(500) = '',
	@taskid uniqueidentifier = '00000000-0000-0000-0000-000000000000'
as
begin
	-- Внутренняя вспомогательная.
	-- Производит создание и/или наполнение БД-кэша таблицы абонентов для задачи с закреплением абонентов за оператором

	Declare @sql nvarchar(4000)
	Declare @iscreating bit 
	Set @iscreating = 0

	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) ) begin
		
		-- cоздаем
		Set @iscreating = 1
		Set @sql = '
			Create table ' + @tableidx + ' ( Id int primary key, AC int, State int, NIdx int, IdOperator uniqueidentifier )
	
			if not exists ( select name from sysindexes where id=object_id ( ''dbo.[' + @tableacindex + ']'' ) and name = ''' + @tableacindex + ''' )
				Create Nonclustered Index [' + @tableacindex + '] ON ' + @tableidx + '
				( AC, State ) With FillFactor = 80	

			
			if not exists ( select name from sysindexes where id=object_id ( ''dbo.[' + @tableopindex + ']'' ) and name = ''' + @tableopindex + ''' )
				Create Nonclustered Index [' + @tableopindex + '] ON ' + @tableidx + '
				( IdOperator ) With FillFactor = 80	
			'

		exec sp_executesql @sql
		
	end
	
	-- Если идет синхронизация, а не создание, то
	--   Добавить те записи, которых нет в кэше
	--   Обновить в кэше IdOperator и State=0 у тех у кого =3
	
	-- Вставляем в цикле по 100000 записей чтобы не переполнить transactionlog
	
	-- Запрос|таблица для подстановки в OPENROWSET
	Declare @fromstring nvarchar(4000)
	if ( not @query like '' )
		Set @fromstring = '(' + @query + ')a'
	else
		Set @fromstring = @table
		
	-- Формируем запрос для разных условий
	-- Базовая часть, удаление временной таблицы
	Set @sql = '			
			Declare @k int
			Set @k = 0
				
			if exists ( select * from tempdb..sysobjects where id=object_id(''tempdb..#tcache''))
				Drop table #tcache
			'
	
	-- Времянка из OPENROWSET
	-- With|WO operators
	if ( @operatorfield like '' ) begin		
			
		-- Без операторов
		Set @sql = @sql + '
			Select t.[' + @idfield + '] as idc into #tcache
			From OPENROWSET(''' + @providername + ''',''' + @connectstring + ''', 
				''Select [' + @idfield + '] From ' + @fromstring + ''') t 
			'
	end else begin
			
		-- С операторами
		Set @sql = @sql + '
			Select t.[' + @idfield + '] as idc, t.[' + @operatorfield + '] as idoperator into #tcache
			From OPENROWSET(''' + @providername + ''',''' + @connectstring + ''', 
				''Select [' + @idfield + '], [' + @operatorfield + '] From ' + @fromstring + ''') t
			'
	end				
	
							
	-- Вставка новых записей
	-- With|WO operators
	if ( @operatorfield like '' ) begin			
		
		-- Без операторов
		Set @sql = @sql + '
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Insert into ' + @tableidx + ' ( [Id], [AC], [State], [NIdx] )
		 				Select top 100000 [idc], @k, 0, 0
						From #tcache t
						Where not exists ( Select [Id] From ' + @tableidx + ' Where [Id] = t.[idc] )
				end
				'
				
	end else begin		
		
		-- С операторами
		Set @sql = @sql + '
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Insert into ' + @tableidx + ' ( [Id], [AC], [State], [NIdx], [IdOperator] )				
		 				Select top 100000 [idc], @k, 0, 0, cast ( [idoperator] as uniqueidentifier )
						From #tcache t
						Where not exists ( Select [Id] From ' + @tableidx + ' Where [Id] = t.[idc] )
				end
				'		
	end
	
	-- State надо обновить по оперативной статистике (20.12.2013)
	if ( @taskid != '00000000-0000-0000-0000-000000000000' ) begin
		Set @sql = @sql + '
				Update cache
				Set [State] = st.NResult
				From ' + @tableidx + ' cache, A_TaskManager_SubTasks st
				Where (st.TaskId = cast(''' + cast ( @taskid as nvarchar(100)) + ''' as uniqueidentifier))
					and (cache.[Id] = st.[IdInList])
				'
	end
	
	-- В режиме синхронизации существующего кэша
	if ( @iscreating = 0 ) begin
	
		-- Обновление привязки к операторам
		if ( not @operatorfield like '' )
			Set @sql = @sql + '
				Set rowcount 100000
				Set @k = @k				
				While ( @@rowcount > 0 ) begin
					Update c
					Set c.IdOperator = t.idoperator,
						c.AC = @k
					From ' + @tableidx + ' c, #tcache t
					Where (c.Id = t.[idc]) 
						and (c.State != 1)
						and (c.IdOperator != t.idoperator)
				end
				Set rowcount 0
				'
				
		-- Обновление статусов вновь появившихся абонентов
		Set @sql = @sql + '
				Set rowcount 100000
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Update c
					Set c.State = 0, 
						c.AC = @k
					From ' + @tableidx + ' c, #tcache t
					Where (c.Id = t.[idc]) and (c.State = 3)
				end
				Set rowcount 0
				'
	end
				
	-- Удаление временной таблицы
	Set @sql = @sql + '
				Drop table #tcache
				'				
	
	exec sp_executesql @sql

end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_SyncTable' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_SyncTable
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_SyncTable]
	@taskid uniqueidentifier,
	@table nvarchar ( 200 ),
	@fields nvarchar ( 2000 ),
	@idfield nvarchar ( 200 ) = 'Id',
	@operatorfield nvarchar ( 200 ) = ''
as
begin
	-- Синхронизирует кэш задачи с закреплением абонентов за операторами

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_dIdx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	Declare @tableacindex nvarchar ( 200 )
	Declare @tableopindex nvarchar ( 200 )
	set @tableacindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'
	set @tableopindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_OP'

	-- Добавить те, которых нет в кэше
	-- Обновить на STATE = 3 те, которых нет в таблице
	-- Обновить операторов, если

	-- Create table, Insert absent records, Updating State and IdOperator for existing
	exec A_TaskManager_List_DistrDirectDB_CacheCreate
		@tableidx = @tableidx, 
		@tableacindex = @tableacindex, 
		@tableopindex = @tableopindex, 
		@idfield = @idfield, 
		@operatorfield = @operatorfield, 
		@query = '',
		@table = @table,
		@taskid = @taskid
	

	-- Set STATE = 3 for deleted records
	Declare @sql nvarchar ( 4000 )
	Set @sql = '
		Declare @cnta int
		Select @cnta = Count ( [' + @idfield + '] )
		From ' + @table + '
		
		Declare @cntb int
		Select @cntb = Count ( Id )
		From ' + @tableidx + '

		if ( @cnta < @cntb )
			Update t
			Set State = 3
			From ' + @tableidx + ' t 
			Where  (State != 3)
				and not exists ( Select [' + @idfield + '] From ' + @table + ' Where [' + @idfield + '] = t.[Id] )'

	exec sp_sqlexec @sql
	
end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_SyncQuery' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_SyncQuery
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_SyncQuery]
	@taskid uniqueidentifier,
	@query nvarchar ( 4000 ),
	@fields nvarchar ( 500 ),
	@idfield nvarchar ( 200 ) = 'Id',
	@operatorfield nvarchar ( 200 ) = ''
as
begin
	-- Синхронизирует кэш задачи с закреплением абонентов за операторами

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_dIdx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	Declare @tableacindex nvarchar ( 200 )
	Declare @tableopindex nvarchar ( 200 )
	set @tableacindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'
	set @tableopindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_OP'

	-- Добавить те, которых нет в кэше
	-- Обновить на STATE = 3 те, которых нет в таблице
	-- Обновить операторов, если
	
	-- Create table, Insert absent records, Updating State and IdOperator for existing
	exec A_TaskManager_List_DistrDirectDB_CacheCreate
		@tableidx = @tableidx, 
		@tableacindex = @tableacindex, 
		@tableopindex = @tableopindex, 
		@idfield = @idfield, 
		@operatorfield = @operatorfield, 
		@query = @query,
		@table = '',
		@taskid = @taskid
	
	
	-- Set STATE = 3 for deleted records
	Declare @sql nvarchar ( 4000 )
	Set @sql = '
		Declare @cnta int
		Select @cnta = Count ( [' + @idfield + '] )
		From (' + @query + ')a
		
		Declare @cntb int
		Select @cntb = Count ( Id )
		From ' + @tableidx + '
	
		if ( @cntb != @cnta )
			Update t
			Set State = 3
			From ' + @tableidx + ' t
			Where ( State != 3 )
				and not exists ( Select [' + @idfield + '] From ( ' + @query + ' ) a1 Where [' + @idfield + '] = t.[Id] )
		'

	exec sp_sqlexec @sql
	
end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_SyncTableP' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_SyncTableP
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_SyncTableP]
	@taskid uniqueidentifier,
	@table nvarchar ( 200 ),
	@fields nvarchar ( 2000 ),
	@idfield nvarchar ( 200 ) = 'Id',
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = '',
	@operatorfield nvarchar ( 200 ) = ''
as
begin
	-- Синхронизирует кэш задачи с закреплением абонентов за операторами

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_dIdx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	Declare @tableacindex nvarchar ( 200 )
	Declare @tableopindex nvarchar ( 200 )
	set @tableacindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'
	set @tableopindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_OP'

	
	-- Добавить те, которых нет в кэше
	-- Обновить на STATE = 3 те, которых нет в таблице
	-- Обновить операторов, если

	-- Create table, Insert absent records, Updating State and IdOperator for existing
	exec A_TaskManager_List_DistrDirectDB_CacheCreateP
		@tableidx = @tableidx, 
		@tableacindex = @tableacindex, 
		@tableopindex = @tableopindex, 
		@idfield = @idfield, 
		@operatorfield = @operatorfield, 
		@providername = @providername,
		@connectstring = @connectstring,
		@query = '',
		@table = @table,
		@taskid = @taskid
		

	-- Set STATE = 3 for deleted records
	Declare @sql nvarchar ( 4000 )
	Set @sql = @sql + '
			Declare @cnta int
			Select @cnta = Count ( * ) 
			From #tcache
		
			Declare @cntb int
			Select @cntb = Count ( Id )
			From ' + @tableidx + '

			if ( @cnta < @cntb ) begin
				Update t
				Set State = 3
				From ' + @tableidx + ' t 
				Where (State != 3)
					and not exists ( Select idc From #tcache Where [idc] = t.[Id] )
			end

			Drop table #tcache
			'

	exec sp_executesql @sql
	
end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_SyncQueryP' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_SyncQueryP
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_SyncQueryP]
	@taskid uniqueidentifier,
	@query nvarchar ( 4000 ),
	@fields nvarchar ( 500 ),
	@idfield nvarchar ( 200 ) = 'Id',
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = '',
	@operatorfield nvarchar(200) = ''	
as
begin
	-- Синхронизирует кэш задачи с закреплением абонентов за операторами

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_dIdx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	Declare @tableacindex nvarchar ( 200 )
	Declare @tableopindex nvarchar ( 200 )
	set @tableacindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'
	set @tableopindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_OP'

	-- Добавить те, которых нет в кэше
	-- Обновить на STATE = 3 те, которых нет в таблице
	-- Обновить операторов, если
	
	-- Create table, Insert absent records, Updating State and IdOperator for existing
	exec A_TaskManager_List_DistrDirectDB_CacheCreateP
		@tableidx = @tableidx, 
		@tableacindex = @tableacindex, 
		@tableopindex = @tableopindex, 
		@idfield = @idfield, 
		@operatorfield = @operatorfield, 
		@providername = @providername,
		@connectstring = @connectstring,
		@query = @query,
		@table = '',
		@taskid = @taskid
		
	
	-- Set STATE = 3 for deleted records
	Declare @sql nvarchar ( 4000 )
	Set @sql = @sql + '
			Declare @cnta int
			Select @cnta = Count ( * ) 
			From #tcache
		
			Declare @cntb int
			Select @cntb = Count ( Id )
			From ' + @tableidx + '
	
			if ( @cntb != @cnta ) begin
				Update t
				Set State = 3
				From ' + @tableidx + ' t
				Where (State != 3)
					and not exists ( Select idc From #tcache Where idc = t.[Id] ) 
			end

			Drop table #tcache
			'

	exec sp_executesql @sql
	
end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_GetTable' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_GetTable
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_GetTable]
	@taskid uniqueidentifier,
	@operatorid uniqueidentifier,
	@table nvarchar ( 200 ),
	@fields nvarchar ( 1000 ),
	@idfield nvarchar ( 200 ) = 'Id',
	@operatorfield nvarchar ( 200 ) = ''
as
begin
	-- Производит запрос следующего абонента для совершения звонка из кэша в БД 
	-- При отсутствии кэша производит создание и начальную синхронизацию с таблицей абонентов 
	--   заданной через таблицу на локальном сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_dIdx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'

	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @tableopindex nvarchar ( 200 )
	set @tableopindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_OP'

	Declare @operator nvarchar ( 60 )
	set @operator = '''' + cast ( @operatorid as nvarchar ( 50 ) ) + ''''


	-- создаем таблицу для задачи, если ее нет еще
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) )
		exec A_TaskManager_List_DistrDirectDB_CacheCreate
			@tableidx = @tableidx, 
			@tableacindex = @tableacindex, 
			@tableopindex = @tableopindex, 
			@idfield = @idfield, 
			@operatorfield = @operatorfield, 
			@query = '',
			@table = @table,
			@taskid = @taskid
	

	Declare @sql nvarchar ( 4000 )

	-- Берем значение минимального количества попыток по абонентам
	-- При выборе абонента AC возрастает на 1, но меняется только между 0 и 1, 
	--   поскольку когда минимальный становится 1 - все убавляются на 1.	
	Set @sql = ' 
		Declare @min int
		
		Select @min = Min ( AC ) 
		From ' + @tableidx + '  
		Where (State = 0)
			And (IdOperator = ' + @operator + ')
			
		if (@min > 0) begin
			Update ' + @tableidx + '
			Set AC = AC - @min
			Where (State = 0)
				And (IdOperator = ' + @operator + ')
		
			Set @min = 0
		end
		'
				
	-- Берем первый индекс с минимальным количеством совершенных попыток (по списку)
	--   и достаем из таблицы строки для этого абонента
	Set @sql = @sql + '
		Declare @id int
		
		Select top 1  @Id = Id 
		From ' + @tableidx + ' 
		Where (AC = @min)
			and (State = 0)
			and (IdOperator = ' + @operator + ')

		Update ' + @tableidx + ' 
		Set AC = AC + 1 
		Where Id = @id
		
		Select ' + @fields + ' 
		From ' + @table + '
		Where [' + @idfield + '] = @id
		'
	
	Exec sp_sqlexec @sql
end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_GetQuery' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_GetQuery
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_GetQuery]
	@taskid uniqueidentifier,
	@operatorid uniqueidentifier,
	@query nvarchar ( 4000 ),
	@fields nvarchar ( 2000 ),
	@idfield nvarchar ( 200 ) = 'Id',
	@operatorfield nvarchar ( 200 ) = ''
as
begin
	-- Производит запрос следующего абонента для совершения звонка из кэша в БД 
	-- При отсутствии кэша производит создание и начальную синхронизацию с таблицей абонентов 
	--   заданной через запрос на локальном сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_dIdx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'

	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @tableopindex nvarchar ( 200 )
	set @tableopindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_OP'

	Declare @operator nvarchar ( 60 )
	set @operator = '''' + cast ( @operatorid as nvarchar ( 50 ) ) + ''''


	-- создаем таблицу для задачи, если ее нет еще
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) )
		exec A_TaskManager_List_DistrDirectDB_CacheCreate
			@tableidx = @tableidx, 
			@tableacindex = @tableacindex, 
			@tableopindex = @tableopindex, 
			@idfield = @idfield, 
			@operatorfield = @operatorfield, 
			@query = @query,
			@table = '',
			@taskid = @taskid
		
	
	Declare @sql nvarchar ( 4000 )

	-- Берем значение минимального количества попыток по абонентам
	-- При выборе абонента AC возрастает на 1, но меняется только между 0 и 1, 
	--   поскольку когда минимальный становится 1 - все убавляются на 1.
	Set @sql = ' 
		Declare @min int
		
		Select @min = Min ( AC ) 
		From ' + @tableidx + '  
		Where (State = 0)
			And (IdOperator = ' + @operator + ')
		
		if (@min > 0) begin
			Update ' + @tableidx + '
			Set AC = AC - @min
			Where (State = 0)
				And (IdOperator = ' + @operator + ')
		
			Set @min = 0
		end
		'
		
	-- Берем первый индекс с минимальным количеством совершенных попыток (по списку)
	--   и достаем из таблицы строки для этого абонента
	Set @sql = @sql + '
		Declare @id int
		
		Select top 1 @Id = Id 
		From ' + @tableidx + ' 
		Where (AC = @min)
			and (State = 0)	
			and (IdOperator = ' + @operator + ')

		Update ' + @tableidx + ' 
		Set AC = AC + 1 
		Where Id = @id
		
		Select ' + @fields + '
		From ( ' + @query + ' )a
		Where [' + @idfield + '] = @id
		'

	Exec sp_sqlexec @sql
end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_GetTableExact' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_GetTableExact
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_GetTableExact]
	@taskid uniqueidentifier,
	@operatorid uniqueidentifier,
	@table nvarchar ( 200 ),
	@fields nvarchar ( 1000 ),
	@idfield nvarchar ( 200 ) = 'Id',
	@idinlist int,
	@number nvarchar ( 50 ),
	@operatorfield nvarchar(200) = ''
as
begin
	-- Возвращает данные по конкретному абоненту из кэша в БД 
	-- При отсутствии кэша производит создание и начальную синхронизацию с таблицей абонентов 
	--   заданной через таблицу на локальном сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_dIdx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'

	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @tableopindex nvarchar ( 200 )
	set @tableopindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_OP'

	Declare @operator nvarchar ( 60 )
	set @operator = '''' + cast ( @operatorid as nvarchar ( 50 ) ) + ''''


	-- создаем таблицу для задачи, если ее нет еще
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) )
		exec A_TaskManager_List_DistrDirectDB_CacheCreate 
			@tableidx = @tableidx, 
			@tableacindex = @tableacindex, 
			@tableopindex = @tableopindex, 
			@idfield = @idfield, 
			@operatorfield = @operatorfield, 
			@query = '',
			@table = @table,
			@taskid = @taskid
	
	
	Declare @sql nvarchar ( 4000 )
	
	-- Берем индекс, проверяя условия дополнительно (соответствия оператора и состояния)
	--   и достаем из таблицы строки для этого абонента
	-- При обычной выборке AC возрастает на 1, но меняется только между 0 и 1, 
	--   когда минимальный становится 1 - все убавляют 1. Поэтому здесь при прямом выборе 1 просто поставим
	-- State также не волнует, поскольку прямой выбор абонента оператором
	Set @sql = '
		Declare @id int
		
		Select top 1 @Id = Id 
		From ' + @tableidx + ' 
		Where (IdOperator = ' + @operator + ')
			and (Id = ' + cast ( @idinlist as nvarchar ( 10 ) ) + ')
			--and (State = 0)

		Update ' + @tableidx + ' 
		Set AC = 1 
		Where Id = @id
		
		Select ' + @fields + ' 
		From ' + @table + '
		Where [' + @idfield + '] = @id
		'
	
	Exec sp_sqlexec @sql
end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_GetQueryExact' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_GetQueryExact
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_GetQueryExact]
	@taskid uniqueidentifier,
	@operatorid uniqueidentifier,
	@query nvarchar ( 4000 ),
	@fields nvarchar ( 2000 ),
	@idfield nvarchar ( 200 ) = 'Id',
	@idinlist int,
	@number nvarchar ( 200 ),
	@operatorfield nvarchar ( 200 ) = ''
as
begin
	-- Возвращает данные по конкретному абоненту из кэша в БД 
	-- При отсутствии кэша производит создание и начальную синхронизацию с таблицей абонентов 
	--   заданной через запрос на локальном сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_dIdx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'

	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @tableopindex nvarchar ( 200 )
	set @tableopindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_OP'

	Declare @operator nvarchar ( 60 )
	set @operator = '''' + cast ( @operatorid as nvarchar ( 50 ) ) + ''''


	-- создаем таблицу для задачи, если ее нет еще
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) )
		exec A_TaskManager_List_DistrDirectDB_CacheCreate
			@tableidx = @tableidx, 
			@tableacindex = @tableacindex, 
			@tableopindex = @tableopindex, 
			@idfield = @idfield, 
			@operatorfield = @operatorfield, 
			@query = @query,
			@table = '',
			@taskid = @taskid
	

	Declare @sql nvarchar ( 4000 )

	-- Берем индекс, проверяя условия дополнительно (соответствия оператора и состояния)
	--   и достаем из таблицы строки для этого абонента
	-- При обычной выборке AC возрастает на 1, но меняется только между 0 и 1, 
	--   когда минимальный становится 1 - все убавляют 1. Поэтому здесь при прямом выборе 1 просто поставим
	-- State также не волнует, поскольку прямой выбор абонента оператором
	Set @sql = '
		Declare @id int
		
		Select top 1 @Id = Id 
		From ' + @tableidx + ' 
		Where (IdOperator = ' + @operator + ')
			and (Id = ' + cast ( @idinlist as nvarchar ( 10 ) ) + ')
			--and (State = 0)

		Update ' + @tableidx + ' 
		Set AC = AC + 1 
		Where Id = @id
		
		Select ' + @fields + '
		From ( ' + @query + ' )a
		Where [' + @idfield + '] = @id
		'

	Exec sp_sqlexec @sql
	
end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_GetTableP' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_GetTableP
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_GetTableP]
	@taskid uniqueidentifier,
	@operatorid uniqueidentifier,
	@table nvarchar ( 200 ),
	@fields nvarchar ( 1000 ),
	@idfield nvarchar ( 200 ) = 'Id',
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = '',
	@operatorfield nvarchar ( 200 ) = ''
as
begin
	-- Производит запрос следующего абонента для совершения звонка из кэша в БД 
	-- При отсутствии кэша производит создание и начальную синхронизацию с таблицей абонентов 
	--   заданной через таблицу на стороннем сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_dIdx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'

	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @tableopindex nvarchar ( 200 )
	set @tableopindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_OP'

	Declare @operator nvarchar ( 60 )
	set @operator = '''' + cast ( @operatorid as nvarchar ( 50 ) ) + ''''


	-- создаем таблицу для задачи, если ее нет еще
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) )
		exec A_TaskManager_List_DistrDirectDB_CacheCreateP
			@tableidx = @tableidx, 
			@tableacindex = @tableacindex, 
			@tableopindex = @tableopindex, 
			@idfield = @idfield, 
			@operatorfield = @operatorfield, 
			@providername = @providername, 
			@connectstring = @connectstring,
			@query = '',
			@table = @table,
			@taskid = @taskid
	

	Declare @sql nvarchar ( 4000 )
	
	-- Берем значение минимального количества попыток по абонентам
	-- При выборе абонента AC возрастает на 1, но меняется только между 0 и 1, 
	--   поскольку когда минимальный становится 1 - все убавляются на 1.	
	Set @sql = ' 
		Declare @min int
		
		Select @min = Min ( AC ) 
		From ' + @tableidx + '  
		Where (State = 0)
			And (IdOperator = ' + @operator + ')
			
		if (@min > 0) begin
			Update ' + @tableidx + '
			Set AC = AC - @min
			Where (State = 0)
				And (IdOperator = ' + @operator + ')
		
			Set @min = 0
		end
		'

	-- Берем первый индекс с минимальным количеством совершенных попыток (по списку)
	--   и возвращаем его для организации запроса к исходной удаленной таблице
	Set @sql = @sql + '
		Declare @id int
		
		Select top 1  @Id = Id 
		From ' + @tableidx + ' 
		Where (AC = @min)
			and (State = 0)
			and (IdOperator = ' + @operator + ')

		Update ' + @tableidx + ' 
		Set AC = AC + 1 
		Where Id = @id
		
		Select @id
		'

	-- Через времяночку индекс абонента получаем
	if exists ( select * from tempdb..sysobjects where id=object_id('tempdb..#tx'))
	    Drop table #tx
	Create table #tx ( id int )

	Insert into #tx ( id )
	    Exec sp_executesql @sql

	Declare @id int
	Select @id = id
	From #tx
	
	Drop table #tx

	-- достаем из исходной таблицы строчку с указанным идентификатором	
	Set @sql = '
		Select ' + @fields + '
		From OPENROWSET(''' + @providername + ''',''' + @connectstring + ''', 
			''Select ' + @fields + ' From ' + @table + 
			' Where [' + @idfield + '] = ' + cast ( @id as nvarchar(10))+ ''') t 
		'
	Exec sp_executesql @sql

end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_GetQueryP' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_GetQueryP
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_GetQueryP]
	@taskid uniqueidentifier,
	@operatorid uniqueidentifier,
	@query nvarchar ( 4000 ),
	@fields nvarchar ( 2000 ),
	@idfield nvarchar ( 200 ) = 'Id',
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = '',
	@operatorfield nvarchar ( 200 ) = ''
as
begin
	-- Производит запрос следующего абонента для совершения звонка из кэша в БД 
	-- При отсутствии кэша производит создание и начальную синхронизацию с таблицей абонентов 
	--   заданной через запрос на стороннем сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_dIdx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'

	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @tableopindex nvarchar ( 200 )
	set @tableopindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_OP'

	Declare @operator nvarchar ( 60 )
	set @operator = '''' + cast ( @operatorid as nvarchar ( 50 ) ) + ''''


	-- создаем таблицу для задачи, если ее нет еще
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) )
		exec A_TaskManager_List_DistrDirectDB_CacheCreateP
			@tableidx = @tableidx, 
			@tableacindex = @tableacindex, 
			@tableopindex = @tableopindex, 
			@idfield = @idfield, 
			@operatorfield = @operatorfield, 
			@providername = @providername, 
			@connectstring = @connectstring,
			@query = @query,
			@table = '',
			@taskid = @taskid
	

	Declare @sql nvarchar ( 4000 )

	-- Берем значение минимального количества попыток по абонентам
	-- При выборе абонента AC возрастает на 1, но меняется только между 0 и 1, 
	--   поскольку когда минимальный становится 1 - все убавляются на 1.
	Set @sql = ' 
		Declare @min int
		
		Select @min = Min ( AC ) 
		From ' + @tableidx + '  
		Where (State = 0)
			And (IdOperator = ' + @operator + ')
		
		if (@min > 0) begin
			Update ' + @tableidx + '
			Set AC = AC - @min
			Where (State = 0)
				And (IdOperator = ' + @operator + ')
		
			Set @min = 0
		end
		'
		
	-- Берем первый индекс с минимальным количеством совершенных попыток (по списку)
	--   и возвращаем его для организации запроса к исходной удаленной таблице
	Set @sql = @sql + '
		Declare @id int
		
		Select top 1  @Id = Id 
		From ' + @tableidx + ' 
		Where (AC = @min)
			and (State = 0)	
			and (IdOperator = ' + @operator + ')

		Update ' + @tableidx + ' 
		Set AC = AC + 1 
		Where Id = @id
		
		Select @id
		'

	-- Через времяночку индекс абонента получаем
	if exists ( select * from tempdb..sysobjects where id=object_id('tempdb..#tx'))
	    Drop table #tx
	Create table #tx ( id int )

	Insert into #tx ( id )
	    Exec sp_executesql @sql

	Declare @id int
	Select @id = id
	From #tx
	
	Drop table #tx
	
	-- достаем из исходной таблицы строчку с указанным идентификатором
	Set @sql = '
		Select ' + @fields + '
		From OPENROWSET(''' + @providername + ''',''' + @connectstring + ''', 
			''Select ' + @fields + ' From (' + @query + ')a 
			Where [' + @idfield + '] = ' + cast ( @id as nvarchar(10))+ ''') t 
		'

	Exec sp_executesql @sql
end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_GetTableExactP' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_GetTableExactP
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_GetTableExactP]
	@taskid uniqueidentifier,
	@operatorid uniqueidentifier,
	@table nvarchar ( 200 ),
	@fields nvarchar ( 1000 ),
	@idfield nvarchar ( 200 ) = 'Id',
	@idinlist int,
	@number nvarchar ( 50 ),
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = '',
	@operatorfield nvarchar(200) = ''
as
begin
	-- Возвращает данные по конкретному абоненту из кэша в БД 
	-- При отсутствии кэша производит создание и начальную синхронизацию с таблицей абонентов 
	--   заданной через таблицу на стороннем сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_dIdx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'

	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @tableopindex nvarchar ( 200 )
	set @tableopindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_OP'

	Declare @operator nvarchar ( 60 )
	set @operator = '''' + cast ( @operatorid as nvarchar ( 50 ) ) + ''''

	
	-- создаем таблицу для задачи, если ее нет еще
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) )
		exec A_TaskManager_List_DistrDirectDB_CacheCreateP 
			@tableidx = @tableidx, 
			@tableacindex = @tableacindex, 
			@tableopindex = @tableopindex, 
			@idfield = @idfield, 
			@operatorfield = @operatorfield, 
			@providername = @providername, 
			@connectstring = @connectstring,
			@query = '',
			@table = @table,
			@taskid = @taskid

	
	Declare @sql nvarchar ( 4000 )
	
	-- Берем индекс, проверяя условия дополнительно (соответствия оператора и состояния)
	--   и возвращаем его для организации запроса к исходной удаленной таблице
	-- При обычной выборке AC возрастает на 1, но меняется только между 0 и 1, 
	--   когда минимальный становится 1 - все убавляют 1. Поэтому здесь при прямом выборе 1 просто поставим
	-- State также не волнует, поскольку прямой выбор абонента оператором
	Set @sql = '
		Declare @id int
		
		Select top 1 @Id = Id 
		From ' + @tableidx + ' 
		Where (IdOperator = ' + @operator + ')
			and (Id = ' + cast ( @idinlist as nvarchar ( 10 ) ) + ')
			--and (State = 0)

		Update ' + @tableidx + ' 
		Set AC = 1
		Where Id = @id
		
		Select @id
		'
	
	-- Через времяночку индекс абонента получаем
	if exists ( select * from tempdb..sysobjects where id=object_id('tempdb..#tx'))
	    Drop table #tx
	Create table #tx ( id int )

	Insert into #tx ( id )
	    Exec sp_executesql @sql

	Declare @id int
	Select @id = id
	From #tx
	
	Drop table #tx

	-- достаем из исходной таблицы строчку с указанным идентификатором	
	Set @sql = '
		Select ' + @fields + '
		From OPENROWSET(''' + @providername + ''',''' + @connectstring + ''', 
			''Select ' + @fields + ' From ' + @table + 
			' Where [' + @idfield + '] = ' + cast ( @id as nvarchar(10))+ ''') t 
		'
	Exec sp_executesql @sql
end
GO

-----------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id=object_id ( 'dbo.A_TaskManager_List_DistrDirectDB_GetQueryExactP' ))
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_GetQueryExactP
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_GetQueryExactP]
	@taskid uniqueidentifier,
	@operatorid uniqueidentifier,
	@query nvarchar ( 4000 ),
	@fields nvarchar ( 2000 ),
	@idfield nvarchar ( 200 ) = 'Id',
	@idinlist int,
	@number nvarchar ( 200 ),
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = '',
	@operatorfield nvarchar(200) = ''
as
begin
	-- Возвращает данные по конкретному абоненту из кэша в БД 
	-- При отсутствии кэша производит создание и начальную синхронизацию с таблицей абонентов 
	--   заданной через запрос на стороннем сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_dIdx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'

	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @tableopindex nvarchar ( 200 )
	set @tableopindex = 'idxdTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_OP'

	Declare @operator nvarchar ( 60 )
	set @operator = '''' + cast ( @operatorid as nvarchar ( 50 ) ) + ''''


	-- создаем таблицу для задачи, если ее нет еще
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) )		
		exec A_TaskManager_List_DistrDirectDB_CacheCreateP 
			@tableidx = @tableidx, 
			@tableacindex = @tableacindex, 
			@tableopindex = @tableopindex, 
			@idfield = @idfield, 
			@operatorfield = @operatorfield, 
			@providername = @providername, 
			@connectstring = @connectstring,
			@query = @query,
			@table = '',
			@taskid = @taskid

	
	Declare @sql nvarchar ( 4000 )
	
	-- Берем индекс, проверяя условия дополнительно (соответствия оператора и состояния)
	--   и возвращаем его для организации запроса к исходной удаленной таблице
	-- При обычной выборке AC возрастает на 1, но меняется только между 0 и 1, 
	--   когда минимальный становится 1 - все убавляют 1. Поэтому здесь при прямом выборе 1 просто поставим
	-- State также не волнует, поскольку прямой выбор абонента оператором
	Set @sql = '
		Declare @id int
		
		Select top 1 @Id = Id 
		From ' + @tableidx + ' 
		Where (IdOperator = ' + @operator + ')
			and (Id = ' + cast ( @idinlist as nvarchar ( 10 ) ) + ')
			--and (State = 0)

		Update ' + @tableidx + ' 
		Set AC = 1 
		Where Id = @id
		
		Select @id
		'

	-- Через времяночку индекс абонента получаем
	if exists ( select * from tempdb..sysobjects where id=object_id('tempdb..#tx'))
	    Drop table #tx
	Create table #tx ( id int )

	Insert into #tx ( id )
	    Exec sp_executesql @sql

	Declare @id int
	Select @id = id
	From #tx
	
	Drop table #tx
	
	-- достаем из исходной таблицы строчку с указанным идентификатором
	Set @sql = '
		Select ' + @fields + '
		From OPENROWSET(''' + @providername + ''',''' + @connectstring + ''', 
			''Select ' + @fields + ' From (' + @query + ')a 
			Where [' + @idfield + '] = ' + cast ( @id as nvarchar(10))+ ''') t 
		'

	Exec sp_executesql @sql
	
end
GO
