﻿if exists ( Select * from sysobjects where id=object_id ( 'dbo.A_DB_OptimizeRemove' ))
	Drop procedure dbo.A_DB_OptimizeRemove
GO

Create procedure [dbo].[A_DB_OptimizeRemove]
	@days int = 2
as
begin
	
	-- считаем время границы актуальности
	declare @dt DateTime
	Set @dt = DateAdd ( d, -@days, GetDate () )
	select @dt = Cast ( FLOOR ( Cast ( @dt as float ) ) as DateTime )

	-- переменные
	Declare @logname nvarchar(500)
	Declare @sz int, @msz int
	Declare @curdb nvarchar ( 500 )
	Set @curdb = DB_name()

	-- увеличиваем максимальный размер лога для базы
	Select @logname = name from dbo.sysfiles WHERE (status & 0x40) <> 0 
	Select @sz = size, @msz = maxsize from dbo.sysfiles WHERE (status & 0x40) <> 0
	-- увеличиваем maxsize если он меньше 1500
	if ( @msz != -1 and @msz < 1500 * 128 )
		exec ( N'alter database ' + @curdb + ' modify file(name='+@logname+', maxsize=1500)')
	exec ( N'alter database ' + @curdb + ' set recovery simple' )

	-- 
	If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
	begin
		-- увеличиваем максимальный размер лога для кубовой базы
		Select @logname = name from oktell_cc_temp.dbo.sysfiles WHERE (status & 0x40) <> 0
		Select @sz = size, @msz = maxsize from oktell_cc_temp.dbo.sysfiles WHERE (status & 0x40) <> 0
		if ( @msz != -1 and @msz < 1500 * 128 )
			exec ( N'alter database oktell_cc_temp modify file(name='+@logname+', maxsize=1500)')
		exec ( N'alter database oktell_cc_temp set recovery simple' )

		-- проводим переиндексацию таблиц кубовой базы
		DBCC DBREINDEX ( 'oktell_cc_temp.dbo.A_Cube_CC_EffortConnections' )
		DBCC DBREINDEX ( 'oktell_cc_temp.dbo.A_Cube_CC_OperatorStates' )
		DBCC DBREINDEX ( 'oktell_cc_temp.dbo.A_Cube_CC_CallQueue' )

		-- сжимаем кубовую базу
		DBCC SHRINKDATABASE (oktell_cc_temp)
	
		-- увеличиваем размер лога кубовой базы с выделением места на диске
		Select @logname = name from oktell_cc_temp.dbo.sysfiles WHERE (status & 0x40) <> 0
		Select @sz = size, @msz = maxsize from oktell_cc_temp.dbo.sysfiles WHERE (status & 0x40) <> 0
		if ( @sz < 500 * 128 )
			exec ( N'alter database oktell_cc_temp modify file(name='+@logname+', size=500)')
		exec ( N'alter database oktell_cc_temp set recovery simple' )
	end
			
	-- удаляем засоры из базы
	Delete From dbo.A_CallBackOrders Where CalledDateTime < @dt
	Delete From dbo.A_CallBackRecievers Where not exists ( Select IDconn From dbo.A_CallBackOrders cbo Where cbo.IDconn = dbo.A_CallBackRecievers.IDconn )
	Delete From dbo.A_CallCenter_ControlEventsJournal Where EventDateTime < @dt
	Delete From dbo.A_CallCenter_LastConnected Where CallTime < @dt
	Delete From dbo.A_CallCenter_UserStateHistory Where TimeEnter < @dt		
		
	Delete From dbo.A_Chat_Messages Where DateUTC < @dt
	Delete From dbo.A_Chat_MessageFile Where not exists ( Select IdMessage From dbo.A_Chat_Messages m Where m.IdMessage = dbo.A_Chat_MessageFile.IdMessage )
	Delete From dbo.A_Chat_Dialog Where not exists ( Select Top 1 IdMessage From dbo.A_Chat_Messages m Where m.IdDialog = dbo.A_Chat_Dialog.IdDialog )
	Delete From dbo.A_Chat_Members Where not exists ( Select IdDialog From dbo.A_Chat_Dialog d Where d.IdDialog = dbo.A_Chat_Members.IdDialog )
	Delete From dbo.A_Chat_NewMessages Where not exists ( Select IdMessage From dbo.A_Chat_Messages m Where m.IdMessage = dbo.A_Chat_NewMessages.IdMessage )
	
	Delete From dbo.A_ClientHistory Where CreationDateTime < @dt
	
	Delete From dbo.A_ICQMessages Where date_receive < @dt
	Delete From dbo.A_IM_Messages Where Date_Received < @dt
	
	Delete From dbo.A_Stat_Connections_1x1 Where TimeStart < @dt
	Delete From dbo.A_Stat_Connections_Errors Where TimeStart < @dt
	Delete From dbo.A_Stat_Connections_VoicePerc Where not exists ( Select Id from dbo.A_Stat_Connections_1x1 s Where s.Id = dbo.A_Stat_Connections_VoicePerc.IdConn )	
	Delete From dbo.A_Stat_Conference Where TimeStart < @dt
	Delete From dbo.A_Stat_Conference_Competitors Where not exists ( Select IdConf From A_Stat_Conference c Where c.Id = dbo.A_Stat_Conference_Competitors.IdConf )
	Delete From dbo.A_Stat_Conference_RecordAccessUsers Where not exists ( Select IdConf From A_Stat_Conference c Where c.Id = dbo.A_Stat_Conference_RecordAccessUsers.IdConf )
	Delete From dbo.A_Stat_MissedCalls Where TimeStart < @dt
	Delete From dbo.A_Stat_FailedCalls Where TimeStart < @dt
	Delete From dbo.A_Stat_Comments Where not exists ( Select Id from dbo.A_Stat_Connections_1x1 s Where s.Id = dbo.A_Stat_Comments.Id ) and not exists ( Select Id from dbo.A_Stat_Conference_Competitors c Where c.IdConn = dbo.A_Stat_Comments.Id )
	Delete From dbo.A_Stat_RingTime Where StartDT < @dt	
	Delete From dbo.A_Stat_DTMF Where DT < @dt
	
	Delete From dbo.A_Stat_Talk_Algorythms Where not exists ( Select IdChain From dbo.A_Stat_Connections_1x1 c Where c.IdChain = dbo.A_Stat_Talk_Algorythms.IdConnection )
	Delete From dbo.A_Stat_Talk_Card_Check Where not exists ( Select IdAlgVisit From dbo.A_Stat_Talk_Algorythms a Where a.IdAlgVisit = dbo.A_Stat_Talk_Card_Check.IdAlgVisit )
	Delete From dbo.A_Stat_Talk_Card_Radio Where not exists ( Select IdAlgVisit From dbo.A_Stat_Talk_Algorythms a Where a.IdAlgVisit = dbo.A_Stat_Talk_Card_Radio.IdAlgVisit )
	Delete From dbo.A_Stat_Talk_Card_Text Where not exists ( Select IdAlgVisit From dbo.A_Stat_Talk_Algorythms a Where a.IdAlgVisit = dbo.A_Stat_Talk_Card_Text.IdAlgVisit )	
	Delete From dbo.A_Stat_Talk_StopInfo Where TimeStop < @dt
	Delete From dbo.A_TalkForm_Stat_Values Where TimeSave < @dt
	
	Delete From A_StreamRegistration Where OperationDT < @dt
	
	Delete From dbo.A_TaskManager_SubTaskEfforts Where EDateTime < @dt
	Delete From dbo.A_TaskManager_Task_IncomingCalls Where CallTimeStart < @dt
	Delete From dbo.A_TaskManager_UserInTask Where TimeStart < @dt
	
	if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UserMessageData' )) begin
		Delete From dbo.A_UserMessageData Where SentDate < @dt
		Delete From dbo.A_UserMessageFile Where not exists ( Select Id From dbo.A_UserMessageData d Where d.Id = dbo.A_UserMessageFile.IdMessageData )
		Delete From dbo.A_UserMessageLink Where not exists ( Select Id From dbo.A_UserMessageData d Where d.Id = dbo.A_UserMessageLink.IdMessageData )
	end
	
	Delete From dbo.A_User_NotifyLog Where NotifyDateTime < @dt
	Delete From dbo.A_UserStateHistory Where TimeChange < @dt
	Delete From dbo.A_VoiceMail Where SentTime < @dt
	Delete From dbo.A_VoiceMailReceivers Where not exists ( Select Id From dbo.A_VoiceMail v Where v.Id = dbo.A_VoiceMailReceivers.msgid )
	
	
	
	-- Перестраиваем индексы всех таблиц
	DECLARE tcursor CURSOR FOR 
	Select name FROM sysobjects where xtype = 'U'

	Declare @sql nvarchar(1000)	
	Declare @tab_name varchar(500)

	OPEN tcursor
	FETCH NEXT FROM tcursor INTO @tab_name

	WHILE @@FETCH_STATUS = 0 Begin
		Set @sql='

	Declare @fullname nvarchar(500)

	Select @fullname = ''['' + su.name + ''].['' + s.name + '']''
	From sysobjects s 
		Inner join sysusers su on s.uid = su.uid
	Where s.name like ''' + @tab_name + ''' and OBJECTPROPERTY(id, N''IsTable'') = 1

	DBCC DBREINDEX ( @fullname )'

		Exec sp_executesql @stmt =@sql

		FETCH NEXT FROM tcursor INTO @tab_name
	End

	CLOSE tcursor
	DEALLOCATE tcursor

	/*
	DBCC DBREINDEX ( 'dbo.A_CallCenter_UserStateHistory' )
	DBCC DBREINDEX ( 'dbo.A_CallCenter_ControlEventsJournal' )
	DBCC DBREINDEX ( 'dbo.A_ClientHistory' )
	DBCC DBREINDEX ( 'dbo.A_ICQMessages' )
	DBCC DBREINDEX ( 'dbo.A_SmsTaskManager_SubTasks' )
	DBCC DBREINDEX ( 'dbo.A_SmsTaskManager_SubTaskNumbers' )
	DBCC DBREINDEX ( 'dbo.A_Stat_Conference' )
	DBCC DBREINDEX ( 'dbo.A_Stat_Conference_Competitors' )
	DBCC DBREINDEX ( 'dbo.A_Stat_Conference_RecordAccessUsers' )
	DBCC DBREINDEX ( 'dbo.A_Stat_Connections_1x1' )
	DBCC DBREINDEX ( 'dbo.A_Stat_Connections_Errors' )
	DBCC DBREINDEX ( 'dbo.A_Stat_DTMF' )
	DBCC DBREINDEX ( 'dbo.A_Stat_Talk_Algorythms' )
	DBCC DBREINDEX ( 'dbo.A_Stat_Talk_StopInfo' )
	DBCC DBREINDEX ( 'dbo.A_TaskManager_Task_IncomingCalls' )
	DBCC DBREINDEX ( 'dbo.A_TaskManager_SubTasks' )
	DBCC DBREINDEX ( 'dbo.A_TaskManager_SubTaskNumber' )
	DBCC DBREINDEX ( 'dbo.A_TaskManager_SubTaskEfforts' )
	DBCC DBREINDEX ( 'dbo.A_TaskManager_UserInTask' )
	DBCC DBREINDEX ( 'dbo.A_User_NotifyLog' )
	DBCC DBREINDEX ( 'dbo.A_UserMessages' )
	DBCC DBREINDEX ( 'dbo.A_UserStateHistory' )
	DBCC DBREINDEX ( 'dbo.A_VoiceMail' )
	*/

	-- сжимаем базу
	Set @sql = 'DBCC SHRINKDATABASE ( ' + db_name() + ' )'
	exec sp_executesql @stmt = @sql

	-- увеличиваем размер лога с выделением места на диске
	Select @logname = name from dbo.sysfiles WHERE (status & 0x40) <> 0 
	Select @sz = size, @msz = maxsize from dbo.sysfiles WHERE (status & 0x40) <> 0
	if ( @sz < 500 * 128 )
		exec ( N'alter database ' + @curdb + ' modify file(name='+@logname+', size=500)')
	exec ( N'alter database ' + @curdb + ' set recovery simple' )
end
GO