﻿if exists ( Select * from sysobjects where id = object_id('dbo.WO_Dashboards_Widgets_GetList'))
	Drop procedure dbo.WO_Dashboards_Widgets_GetList
GO

Create PROCEDURE [dbo].[WO_Dashboards_Widgets_GetList]
	@authorizeduserid	uniqueidentifier,
	@sectionid	uniqueidentifier,
	@authorizedobjid	uniqueidentifier = '00000000-0000-0000-0000-000000000000'
as
begin
	
	-- lang
	if exists (select * from tempdb..sysobjects where id = object_id('tempdb..#langtext'))
		drop table #langtext	
	create table #langtext (langkey nvarchar(200) collate database_default, langvalue nvarchar(4000) collate database_default)
	
	insert into #langtext
		exec dbo.[WO_Dashboards_GetLang]
		
	
	if (@authorizeduserid != '00000000-0000-0000-0000-000000000000')
		begin
			
			-- Дашборд какой у виджета?
			Declare @dashboardid uniqueidentifier
			Select @dashboardid = IdDashboard
			From [dbo].[WO_Dashboards_Sections]
			Where Id = @sectionid
			
			-- Проект какой у дашборда?
			Declare @projectid uniqueidentifier	
			Select @projectid = d.IdProject
			From [dbo].[WO_Dashboards_Dashboards] d
			Where d.Id = @dashboardid
					
			-- Привилегия чтения по проекту есть?
			Declare @pcount int
			exec A_Policy_SubjectPrivileges_GetForUserByObject
				@iduser = @authorizeduserid,
				@idobject = @projectid,
				@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818',
				@onlyreturncount = 1,
				@resultcount = @pcount OUTPUT
					
			if ((@pcount = 0) and (@projectid is not null)) 
				RAISERROR ('Access error', 16, 1)
			
			-- Привилегия чтения по дашборду есть?
			Declare @dcount int
			exec A_Policy_SubjectPrivileges_GetForUserByObject
				@iduser = @authorizeduserid,
				@idobject = @dashboardid,
				@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863',
				@onlyreturncount = 1,
				@resultcount = @dcount OUTPUT
				
			if (@dcount = 0) 
				RAISERROR ('Access error', 16, 1)
			
			
			
			
			select	w.Id,
					w.Name,
					w.[Type],
					w.[Source],
					w.Options
			from	[dbo].[WO_Dashboards_Widgets] w
			where	w.IdSection = @sectionid
			
			
			declare @widgets table
				(
					WidgetId uniqueidentifier,
					WidgetFieldCode nvarchar(100) collate database_default,
					WidgetFunctionCode nvarchar(50) collate database_default,
					WidgetFunctionValue nvarchar(50) collate database_default,
					WidgetFieldName nvarchar(250) collate database_default,
					WidgetFieldValue int
				)
			
			if ((select count(*) from [dbo].[WO_Dashboards_Widgets] where IdSection = @sectionid and [Source] in (1,2)) != 0)
				begin
					insert into @widgets (WidgetId, WidgetFieldCode, WidgetFunctionCode, WidgetFunctionValue, WidgetFieldName, WidgetFieldValue)
						select	w.Id WidgetId,
								null WidgetFieldCode,
								null WidgetFunctionCode,
								null WidgetFunctionValue,
								wf.Name WidgetFieldName,
								wf.Value WidgetFieldValue
						from	[dbo].[WO_Dashboards_Widgets] w 
									inner join [dbo].[WO_Dashboards_Widgets_Fields] wf on w.IdFields = wf.IdFields
						where	w.IdSection = @sectionid
								and w.[Source] in (1,2)
				end
			
			if ((select count(*) from [dbo].[WO_Dashboards_Widgets] where IdSection = @sectionid and [Source] in (0)) != 0)
				begin
					insert into @widgets (WidgetId, WidgetFieldCode, WidgetFunctionCode, WidgetFunctionValue, WidgetFieldName, WidgetFieldValue)
						select	w.Id WidgetId,
								wdv.Value WidgetFieldCode,
								wdf.[Function] WidgetFunctionCode,
								wcf.ConstantValue WidgetFunctionValue,
								lt1.langvalue WidgetFieldName,
								1 WidgetFieldValue
						from	[dbo].[WO_Dashboards_Widgets] w 
									inner join [dbo].[WO_Dashboards_Widgets_Constructor_Fields] wcf on w.IdConstructorFields = wcf.IdConstructorFields
										inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] wdv on wdv.Id = wcf.ConstructorValueId
										inner join #langtext lt1 on wdv.Value = lt1.langkey
											left join [dbo].[WO_Dashboards_Widgets_Designer_Function] wdf on wcf.ConstructorFunctionId = wdf.Id
						where	w.IdSection = @sectionid
								and w.[Source] in (0)
						
							union all
						
						select	w.Id WidgetId,
								wda.Axis WidgetFieldCode,
								null WidgetFunctionCode,
								null WidgetFunctionValue,
								lt.langvalue WidgetFieldName,
								0 WidgetFieldValue
						from	[dbo].[WO_Dashboards_Widgets] w 
									inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] wda on w.IdConstructorAxis = wda.Id
										inner join #langtext lt on wda.Axis = lt.langkey
						where	w.IdSection = @sectionid
								and w.[Source] in (0)
				end
			
			
			select	WidgetId,
					WidgetFieldCode,
					WidgetFunctionCode,
					WidgetFunctionValue,
					WidgetFieldName,
					WidgetFieldValue
			from	@widgets
			
		end
	else
		begin
			
			select	w.Id,
					w.Name,
					w.[Type],
					w.[Source],
					w.Options
			from	[dbo].[WO_Dashboards_Widgets] w inner join [dbo].[WO_Dashboards_Sections] s on w.IdSection = s.Id
			where	w.IdSection = @sectionid and s.IdDashboard = @authorizedobjid
			
			
			declare @widgets1 table
				(
					WidgetId uniqueidentifier,
					WidgetFieldCode nvarchar(100) collate database_default,
					WidgetFunctionCode nvarchar(50) collate database_default,
					WidgetFunctionValue nvarchar(50) collate database_default,
					WidgetFieldName nvarchar(250) collate database_default,
					WidgetFieldValue int
				)
			
			if ((select count(*) from [dbo].[WO_Dashboards_Widgets] w inner join [dbo].[WO_Dashboards_Sections] s on w.IdSection = s.Id where w.IdSection = @sectionid and s.IdDashboard = @authorizedobjid and w.[Source] in (1,2)) != 0)
				begin
					insert into @widgets1 (WidgetId, WidgetFieldCode, WidgetFunctionCode, WidgetFunctionValue, WidgetFieldName, WidgetFieldValue)
						select	w.Id WidgetId,
								null WidgetFieldCode,
								null WidgetFunctionCode,
								null WidgetFunctionValue,
								wf.Name WidgetFieldName,
								wf.Value WidgetFieldValue
						from	[dbo].[WO_Dashboards_Widgets] w inner join [dbo].[WO_Dashboards_Sections] s on w.IdSection = s.Id
									inner join [dbo].[WO_Dashboards_Widgets_Fields] wf on w.IdFields = wf.IdFields
						where	w.IdSection = @sectionid
								and w.[Source] in (1,2)
								and s.IdDashboard = @authorizedobjid
				end
			
			if ((select count(*) from [dbo].[WO_Dashboards_Widgets] w inner join [dbo].[WO_Dashboards_Sections] s on w.IdSection = s.Id where w.IdSection = @sectionid and s.IdDashboard = @authorizedobjid and [Source] in (0)) != 0)
				begin
					insert into @widgets1 (WidgetId, WidgetFieldCode, WidgetFunctionCode, WidgetFunctionValue, WidgetFieldName, WidgetFieldValue)
						select	w.Id WidgetId,
								wdv.Value WidgetFieldCode,
								wdf.[Function] WidgetFunctionCode,
								wcf.ConstantValue WidgetFunctionValue,
								lt1.langvalue WidgetFieldName,
								1 WidgetFieldValue
						from	[dbo].[WO_Dashboards_Widgets] w inner join [dbo].[WO_Dashboards_Sections] s on w.IdSection = s.Id
									inner join [dbo].[WO_Dashboards_Widgets_Constructor_Fields] wcf on w.IdConstructorFields = wcf.IdConstructorFields
										inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] wdv on wdv.Id = wcf.ConstructorValueId
										inner join #langtext lt1 on wdv.Value = lt1.langkey
											left join [dbo].[WO_Dashboards_Widgets_Designer_Function] wdf on wcf.ConstructorFunctionId = wdf.Id
						where	w.IdSection = @sectionid
								and w.[Source] in (0)
								and s.IdDashboard = @authorizedobjid
						
							union all
						
						select	w.Id WidgetId,
								wda.Axis WidgetFieldCode,
								null WidgetFunctionCode,
								null WidgetFunctionValue,
								lt.langvalue WidgetFieldName,
								0 WidgetFieldValue
						from	[dbo].[WO_Dashboards_Widgets] w inner join [dbo].[WO_Dashboards_Sections] s on w.IdSection = s.Id
									inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] wda on w.IdConstructorAxis = wda.Id
										inner join #langtext lt on wda.Axis = lt.langkey
						where	w.IdSection = @sectionid
								and w.[Source] in (0)
								and s.IdDashboard = @authorizedobjid
				end
			
			
			select	WidgetId,
					WidgetFieldCode,
					WidgetFunctionCode,
					WidgetFunctionValue,
					WidgetFieldName,
					WidgetFieldValue
			from	@widgets1
		end
	
end
GO