﻿/* */
If not Exists ( Select * From oktell_settings.INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_Policy_ObjectTypes'
                                                              and TABLE_SCHEMA like 'dbo'
                                                              and COLUMN_NAME in ('useProject') )
      Alter Table oktell_settings.dbo.A_Policy_ObjectTypes
      Add useProject bit Default 0
GO

/* */
Update oktell_settings.dbo.A_Policy_ObjectTypes
Set useProject = 0
Where useProject is NULL

Update oktell_settings.dbo.A_Policy_ObjectTypes
Set useProject = 1
Where id in ( 102, 105, 106, 107 )
GO

/* */
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_ObjectTypes' ))
	Drop view dbo.A_Policy_ObjectTypes
GO

Create view dbo.A_Policy_ObjectTypes
as 
	Select * 
	From oktell_settings.dbo.A_Policy_ObjectTypes
GO		

---------------------------------------------------------------------
if exists ( Select * from sysobjects where id= object_id ( 'dbo.A_Policy_ObjectTypes_Save' ))
	Drop procedure dbo.A_Policy_ObjectTypes_Save
GO

Create procedure [dbo].[A_Policy_ObjectTypes_Save]
	@id int,
	@name nvarchar(500),
	@isBasicType bit = 1,
	@useProject bit = 0
as 
begin
	-- Сохраняет тип объекта
	
	Update dbo.A_Policy_ObjectTypes
	Set [name] = @name,
		[isBasicType] = @isBasicType
	Where [id] = @id
	
	if ( @@rowcount = 0 ) begin
		Insert into dbo.A_Policy_ObjectTypes ( [id], [name], [isBasicType], [useProject] )
		Values ( @id, @name, @isBasicType, @useProject )
	end
	
end
GO


-------------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_Modules_ObjectTypes_Create' ))
	Drop procedure dbo.A_Policy_Modules_ObjectTypes_Create
GO	

Create procedure dbo.A_Policy_Modules_ObjectTypes_Create
	@moduleobjectname nvarchar(500),
	@useproject bit = 0
as	
begin
	-- Создает тип объекта (внешний к Oktell) для правовой политики доступа

	Declare @id int
	Set @id = 500
	Select @id = IsNull ( max ( id ) + 1, @id )
	From dbo.A_Policy_ObjectTypes	
	Where id >= 500
	
	Insert into dbo.A_Policy_ObjectTypes ( id, name, isBasicType, useProject )
	Values ( @id, @moduleobjectname, 0, @useproject )
	
	Select @id as id
	
end
GO


------------------------------------------------------------------------------
if exists ( Select * from sysobjects where id= object_id ( 'dbo.A_Policy_SubjectGroups_GetList' ))
	Drop procedure dbo.A_Policy_SubjectGroups_GetList
GO

Create procedure [dbo].[A_Policy_SubjectGroups_GetList]
	@objecttype int = 0
as 
begin

	-- Возвращает список всех именованных субъектов, доступных для назначения прав
	
	
	-- Нужно ли проектные субъекты
	Declare @useproject bit
	Select @useproject = useProject
	From dbo.A_Policy_ObjectTypes
	Where id = @objecttype
	if ( @useproject is NULL )
		Set @useproject = case when ( @objecttype = 0 ) then 1 else 0 end
		
	
	if exists ( Select * From tempdb..sysobjects where id=object_id('tempdb..#subjects'))
		Drop table #subjects
		
	Create table #subjects ( [type] int, [name] nvarchar(500), [id] uniqueidentifier, [extraId] uniqueidentifier, [equalMode] int )
			


	-- Пользователи
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 1, [Name], [Id], null, 2
		From A_Users u
		Where u.IsDeleted = 0
	
	-- Отделы
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 2, [Name], [Id], null, 2
		From A_Groups g	
	
	-- Роли
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 3, r.[Name], r.[Id], null, 2
		From A_UserRoles_Roles r
		Where not exists ( Select Id from A_Users u Where u.Id = r.Id )
			and Domination = 0
	
	
	-- Все операторы	
	Declare @alloperators nvarchar(200)
	Select @alloperators = langvalue 
	From A_LangTexts
	Where langkey like 'alloperators'
	if ( @alloperators is NULL )
		Set @alloperators = N'Все операторы'
		
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 4, @alloperators, st.[groupId], null, 1
		From A_Policy_SubjectTypes st
		Where st.[id] = 4
		
		
	if ( @useproject = 1 ) begin
	
		-- Операторы текущего проекта
		Declare @currentprojectoperators nvarchar(200)
		Select @currentprojectoperators = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectoperators'
		if ( @currentprojectoperators is NULL )
			Set @currentprojectoperators = N'Операторы текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 12, @currentprojectoperators, st.[groupId], null, 1
			From A_Policy_SubjectTypes st
			Where st.[id] = 12
	end
	
	
	---- Операторы задачи
	--Declare @taskoperators nvarchar(200)
	--Select @taskoperators = langvalue 
	--From A_LangTexts
	--Where langkey like 'taskoperators'
	--if ( @taskoperators is NULL )
	--	Set @taskoperators = 'Операторы задачи'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 5, @taskoperators + ' «' + t.[Name] + '»', t.[Id], null, 2
	--	From A_TaskManager_Tasks t
		
		
	---- Операторы проекта
	--Declare @projectoperators nvarchar(200)
	--Select @projectoperators = langvalue 
	--From A_LangTexts
	--Where langkey like 'projectoperators'
	--if ( @projectoperators is NULL )
	--	Set @projectoperators = 'Операторы проекта'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 5, @projectoperators + ' «' + p.[Name] + '»', p.[Id], null, 2
	--	From A_TaskManager_Projects p
	
	
	-- Все супервизоры
	Declare @allsupervisors nvarchar(200)
	Select @allsupervisors = langvalue 
	From A_LangTexts
	Where langkey like 'allsupervisors'
	if ( @allsupervisors is NULL )
		Set @allsupervisors = N'Все супервизоры'
		
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )		
		Select 7, @allsupervisors, st.[groupId], null, 1
		From A_Policy_SubjectTypes st
		Where st.[id] = 7
	
	
	if ( @useproject = 1 ) begin
	
		-- Супервизоры текущего проекта	
		Declare @currentprojectsupervisors nvarchar(200)
		Select @currentprojectsupervisors = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectsupervisors'
		if ( @currentprojectsupervisors is NULL )
			Set @currentprojectsupervisors = N'Супервизоры текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 13, @currentprojectsupervisors, st.[groupId], null, 1
			From A_Policy_SubjectTypes st
			Where st.[id] = 13	
	end
	
	---- Супервизоры задачи
	--Declare @tasksupervisors nvarchar(200)
	--Select @tasksupervisors = langvalue 
	--From A_LangTexts
	--Where langkey like 'tasksupervisors'
	--if ( @tasksupervisors is NULL )
	--	Set @tasksupervisors = 'Супервизоры задачи'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 8, @tasksupervisors + ' «' + t.[Name] + '»', t.[Id], null, 2
	--	From A_TaskManager_Tasks t
		
		
	---- Супервизоры проекта
	--Declare @projectsupervisors nvarchar(200)
	--Select @projectsupervisors = langvalue 
	--From A_LangTexts
	--Where langkey like 'projectsupervisors'
	--if ( @projectsupervisors is NULL )
	--	Set @projectsupervisors = 'Супервизоры проекта'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 9, @projectsupervisors + ' «' + p.[Name] + '»', p.[Id], null, 2
	--	From A_TaskManager_Projects p
	

	---- Владельцы проекта
	--Declare @projectowners nvarchar(200)
	--Select @projectowners = langvalue 
	--From A_LangTexts
	--Where langkey like 'projectowners'
	--if ( @projectowners is NULL )
	--	Set @projectowners = 'Владельцы проекта'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 10, @projectowners + ' «' + p.[Name] + '»', p.[Id], null, 2
	--	From A_TaskManager_Projects p


	-- По другим привилегиями других объектов - ниже
	
	
	-- Все пользователи
	Declare @allusers nvarchar(200)
	Select @allusers = langvalue 
	From A_LangTexts
	Where langkey like 'allusers'
	if ( @allusers is NULL )
		Set @allusers = N'Все пользователи'
		
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )		
		Select 14, @allusers, st.[groupId], null, 1
		From A_Policy_SubjectTypes st
		Where st.[id] = 14


	-- По привилегиям всех объектов - слишком много попадет в список. Пропустим пока.
	-- Только на проекты сошлемся
	
	if ( @useproject = 1 ) begin
		
		Declare @idempty uniqueidentifier
		Set @idempty = '00000000-0000-0000-0000-000000000000'
		
		
		-- Читатели текущего проекта
		Declare @currentprojectviewers nvarchar(200)
		Select @currentprojectviewers = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectviewers'
		if ( @currentprojectviewers is NULL )
			Set @currentprojectviewers = N'Пользователи с правом просмотра текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 101, @currentprojectviewers, @idempty, 'F876E650-7972-475A-AE96-BCCF831AF818', 4
					
						
		-- Редакторы текущего проекта
		Declare @currentprojectredactors nvarchar(200)
		Select @currentprojectredactors = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectredactors'
		if ( @currentprojectredactors is NULL )
			Set @currentprojectredactors = N'Пользователи с правом изменения текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 101, @currentprojectredactors, @idempty, 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3', 4
						
	end
	
	
	-- Итоговый вывод
	Select s.[type], tp.[priority], s.[name], s.[id], s.[extraId], s.[equalMode]
	From #subjects s
		Inner join dbo.A_Policy_SubjectTypePriority tp on tp.[idType] = s.[type]
	Order By tp.[priority], s.[name]
	
end
GO